/*****************************************************************************
 * Mocha
 * Copyright  (c) 2002 Software Center, Samsung Electronics, Inc
 * 599-4 Shinsa-Dong, Kangnam-Gu, Seoul, Korea
 * All right reserved
 *
 * This software is the confidential and proprietary information of Samsung
 * Electronics, Inc ("Confidential Information"). You shall not disclose such
 * Confidential Information and shall use it only in accordance with terms of
 * lecense agreement you  entered into with Samsung Electronics.
 * 
 * This file defines Image GIF CODEC.
 * This Header file contains the sturcture and defintion for the GIF deocoder
 * source file. Structure and constants for the each image properties and 
 * maximum compression values and output structure which contains image data
 *
 *
 * @ Version 3.1
 *		- Added 1 new member in structure GifInfo
 *		- Added ifdef GIF_MOCHA_INTEGRATION for simpler Mocha integration
 *		- Renamed 'struct tagImgBitmap' to 'struct imgBitmap' to prevent
 *		  redefinition of a struct by same name in MOCHA code
 *
 * @ Version 3.2
 *      - Added 6 new members in structure GifInfo to support CR 2
 *      - Added 8 new members in structure ImageParam to support CR 2
 *      - Added 3 new members in structure GifImageInfo to support CR 2
 ****************************************************************************/

#ifndef	__GIF_DECODER_INCLUDED
#define	__GIF_DECODER_INCLUDED

typedef unsigned char tagRGBQUAD;
typedef struct imgBitmap
{
	USHORT				xPos;
	USHORT				yPos;
	UINT				imageWidth;
	UINT				imageHeight;
	BOOL				bChromaKeyFlag; /* Chroma key presence flag */
	ULONG				chromaKey;		/* Packed chroma key information. */
 	UCHAR				disposal;		/* Disposal action to take during */
                                        /*  display of current flag */
	USHORT				delay;			/* Delay before display of*/
                                        /*  next frame */
	BYTE		        inputFlag;      /* User input required befflag used */
	BYTE                interlace;      /* Interlace indicator flag */
    tagRGBQUAD*			pLocalColorMap;	/* Local color palette pointer 	*/
    USHORT              localColorMapSize;
 	UCHAR*				pBits;			/* The complete Decompressed Image */ 
	UCHAR				bitsPerPixel;	/* 1 -- 8 Bits perPixel */
	struct imgBitmap	*pNext;			/* Point to next */
										/*  Image in an Animated GIF */
	UINT				imageCount ;
 	UINT				decodedSize ;
	UINT			origImageWidth ;
	UINT			origImageHeight ;
	UINT			origX ;
	UINT			origY ;
	UINT			expectedWidth ;
	UINT			expectedHeight;
	BOOL            bAGIF;
	ImgResizeInput	resizeMethod ;//added resize option
} ImageParam;
typedef struct imgImage
{
	UCHAR		 cBitMapCount;/* No of Images in an Animated GIF */
	USHORT		 screenWidth;
	USHORT		 screenHeight;
	BYTE         aspectRatio;		
	BYTE         colorResolution;
    BYTE         background;  /* Index into the global color map */  
	USHORT       bitPixel;
	tagRGBQUAD*	 pGlobalColorMap;
    USHORT       globalColorMapSize;
                              /* No of palette entries */
    /**********************/
	ImageParam	 *pHead;      /* Point to the First Image in an Animated GIF */
	ImageParam	 *pTail;      /* Point to the Last Image in an Animated GIF */

	UCHAR		 *pInputImageSize ;	
                              /* Stores the input image size */

	USHORT		loopCount ;	/* This will contain the number of times to repeat
							 * the animation
							 */

	UINT		expectedWidth ;
	UINT		expectedHeight ;
	USHORT		resize ;
	BOOL		memoryFlag ;
	UINT		compositeGIF ;
	BOOL		bLoopCount ;
	ImgInputFlag	inputFlag ;
	ImgResizeInput resizeMethod;//added resize option
} GifInfo;

typedef  struct
{
	BYTE			aspectRatio;		
	BYTE			colorResolution;
	BYTE			background;
	BYTE			inputFlag;
	BYTE			transparent;
	BYTE			interlace;
	USHORT			bitPixel;/* This specifies total No of Bits For RGB */
    UCHAR			*pLastDataInBuf;
	tagRGBQUAD		*pColorMap;
    
    USHORT		    delay;
    UCHAR		    disposal;
    ULONG			chromaKey;
    /*********************/

	UCHAR			*pInputImageSize ;	/* Stores the input image size */

	UINT			imageWidth ;
	UINT			imageHeight ;
	USHORT			resize ;
	BOOL			memoryFlag ;
	UINT			compositeGIF ;
	ImageParam		*pPreviousFrame;

	UCHAR			origRed ;
	UCHAR			origBlue ;
	UCHAR			origGreen ;
	UCHAR			*pLocalColorMap;
	UINT			chromaIndex ;
	BOOL			bChromaKeyFlag ;

	UINT			scaleX ;
	UINT			scaleY ;
	UINT            eWidth ;
	UINT            eHeight;
	UINT            rWidth;
	UINT            rHeight;
	UINT			screenWidth ;
	UINT			screenHeight ;

	tagRGBQUAD		*pGlobalColorMap ;
	BOOL			bBackground ;
	UINT			backgroundColor;
	UINT			backIndex ;
	ImgInputFlag	imgInputFlag;
	ImgResizeInput resizeMethod;//added resize option
} GifImageInfo;

typedef struct gifDecode
{
	GifImageInfo	*pGifImageInfo ;
	ImageParam		*pPreviousFrameOrig;
	unsigned int	offset;
}ImgGIFFrame ;

 

#endif
