/*
 * Mocha 
 * Copyright (c) 2005 IT Center, Samsung Electronics, Inc.
 * All rights reserved.
 *
 * This software is the confidential and proprietary information
 * of Samsung Electronics, Inc. ("Confidential Information").  You
 * shall not disclose such Confidential Information and shall use
 * it only in accordance with the terms of the license agreement
 * you entered into with Samsung Electronics. 
 */
/** 
 * @file AppSndKey.h
 *
 * This file implements the APIs of Application Sound Manager.
 *
 * @author	mobum
 */
/*:Associate with "Application Sound Manager" */

#ifndef _APP_SND_KEY_H_
#define _APP_SND_KEY_H_

/**
 @brief Enum for __AppSndKeyClassIndexType
 */
typedef enum
{
	APP_SND_KEY_TYPE_CLASS_INDEX_MAX		///< max value of key class type
} __AppSndKeyClassIndexType;

/**
 @brief Enum for AppSndKeyClassType
 */
typedef enum
{
	APP_SND_KEY_TYPE_CLASS_NONE		= 0x0000,	///< None
} AppSndKeyClassType;

/**
 @brief Enum for AppSndKeySoundClassType
 */
typedef enum
{
	APP_SND_KEY_SOUND_CLASS_MAX					///< max value of key sound class
} AppSndKeySoundClassType;


/**
 @brief Enum for AppSndKeyPlayStyleType
 */
typedef enum
{
	APP_SND_KEY_TYPE_PLAY_RELEASED	///< Play the key sound when key released
} AppSndKeyPlayStyleType;


/**
 @brief Enum for AppSndKeyAppClassType
 */
typedef enum
{
	APP_SND_KEY_APP_CLASS_MAX		///< max value of class type
} AppSndKeyAppClassType;

typedef	BOOL (*AppSndMuteKeyCallback)(AppSndKeyAppClassType iClass, WmEvent* pEvent, void* pParam);

/**
 @brief Enum for AppSndKeyCallbackInfo
 */
typedef struct
{
	void*					pParam;		///< param
} AppSndKeyCallbackInfo;

/**
 @brief Enum for AppSndKeyClassMuteInfo
 */
typedef struct
{
	APP_SND_DEBUG_STRUCT
} AppSndKeyClassMuteInfo;

/**
 @brief Enum for AppSndKeyClassPlayStyleInfo
 */
typedef struct
{
	APP_SND_DEBUG_STRUCT
} AppSndKeyClassPlayStyleInfo;

/**
 @brief Enum for AppSndKeyPlayStyleInfo
 */
typedef struct
{
	APP_SND_DEBUG_STRUCT
} AppSndKeyPlayStyleInfo;

/**
 @brief Enum for AppSndKeySoundClassInfo
 */
typedef struct
{
	APP_SND_DEBUG_STRUCT
} AppSndKeySoundClassInfo;

/**
 @struct AppSndVibInfo

 @brief Vib information
 */
typedef struct
{
	int		nVibIntensity;
} AppSndKeyUserInfo;

/**
 @struct AppSndKeyInfo

 @brief key sound information
 */  
typedef struct
{
	AppSndKeyType	iKeyType;					///< App Key Code
} AppSndKeyInfo;

#ifdef __cplusplus
extern "C" {
#endif /* __cplusplus */

/*
 * Public Function Prototypes
 */
 
// Local calls
#if defined(_SHP_DLL_APPSNDMGR) || ( ! defined(_SHP_DLL) && ! defined(_SHP_CALL_DLL_APPSNDMGR))

// Mute Key Sound Callback
BOOL	AppSndRegisterMuteKeyCallback	( AppSndKeyAppClassType iAppClass, AppSndMuteKeyCallback callback, void* pParam);
BOOL	AppSndUnregisterMuteKeyCallback	( AppSndKeyAppClassType iAppClass);

/* KeyTone sound */
// KeySound class
BOOL	AppSndLoadKeySoundClass			( void );
BOOL	__AppSndSetKeySoundClass		( AppSndKeyClassType iClassType, AppSndKeySoundClassType iSoundClassType, const char* const pszFile, int nLine );
BOOL	__AppSndSetKeySoundClassByKeyType	( AppSndKeyType iKeyType, AppSndKeySoundClassType iSoundClassType, const char* const pszFile, int nLine );

// Mute/Unmue by Class
BOOL	__AppSndMuteKeySound			( DID did, AppSndKeyClassType iClassType, const char* const pszFile, int nLine );
BOOL	__AppSndUnmuteKeySound			( DID did, AppSndKeyClassType iClassType, const char* const pszFile, int nLine );
BOOL	AppSndGetMutedKeySound			( AppSndKeyClassType iClassType, AppSndMuteType* const pMuteType );
AppSndKeyClassType	AppSndGetMutedKeyClass	( void );
// Mute/Unmue by Key code
BOOL	__AppSndMuteKeySoundByKeyType	( DID did, AppSndKeyType iKeyType, const char* const pszFile, int nLine );
BOOL	__AppSndUnmuteKeySoundByKeyType	( DID did, AppSndKeyType iKeyType, const char* const pszFile, int nLine );
BOOL	AppSndIsMutedKeySoundByKeyType	( AppSndKeyType iKeyType );
// Play style
BOOL	__AppSndSetKeyPlayStyle			( AppSndKeyClassType iClassType, AppSndKeyPlayStyleType iPlayStyle, const char* const pszFile, int nLine );
BOOL	AppSndGetKeyPlayStyle			( AppSndKeyClassType iClassType, AppSndKeyPlayStyleType* const pPlayStyle );
// Play style by Key code
BOOL	__AppSndSetKeyPlayStyleByKeyType( AppSndKeyType iKeyType, AppSndKeyPlayStyleType iPlayStyle, const char* const pszFile, int nLine );
AppSndKeyPlayStyleType	AppSndGetKeyPlayStyleByKeyType	( AppSndKeyType iKeyType );

// Play forced key sound
BOOL	__AppSndPlayKeySound			( AppSndKeyType iKeyType, BOOL bVibration );
BOOL	__AppSndPlayKeySoundEx			( int nKeyCode, BOOL bVibration );

// Vibration
BOOL	AppSndSetKeyVibType				( AppSndKeyType iKeyType, AppSndVibType iVibType );
AppSndVibType	AppSndGetKeyVibType		( AppSndKeyType iKeyType );

BOOL	AppSndSetKeyVibIntensity		( AppSndKeyType iKeyType, int nIntensity );

BOOL	__AppSndMuteKeyVib				( DID did, AppSndKeyType iKeyType, const char* const pszFile, int nLine );
BOOL	__AppSndUnmuteKeyVib			( DID did, AppSndKeyType iKeyType, const char* const pszFile, int nLine );
BOOL	AppSndIsMutedKeyVib				( AppSndKeyType iKeyType );


#endif // defined(_SHP_DLL_APPSNDMGR) || ( ! defined(_SHP_DLL) && ! defined(_SHP_CALL_DLL_APPSNDMGR))

#ifdef __cplusplus
}
#endif /* __cplusplus */

#endif // _APP_SND_KEY_H_
