/*
 * Mocha 
 * Copyright (c) 2005 IT Center, Samsung Electronics, Inc.
 * All rights reserved.
 *
 * This software is the confidential and proprietary information
 * of Samsung Electronics, Inc. ("Confidential Information").  You
 * shall not disclose such Confidential Information and shall use
 * it only in accordance with the terms of the license agreement
 * you entered into with Samsung Electronics. 
 */
/**
 * @file AppSndMgr.h
 *
 * This file implements the APIs of Application Sound Manager.
 *
 * @author	mobum
 */
/*:Associate with "Application Sound Manager" */

#ifndef _APP_SND_MGR_H_
#define _APP_SND_MGR_H_

// Config
#include "AppSndConfig.h"

// Base
#include "AppSndBase.h"

// Base interface
#include "AppSndIBase.h"

// Key
#include "AppSndKey.h"

// Other device
#include "AppSndDev.h"

// Effect sound
#include "AppSndEffect.h"

// Vib
#include "AppSndVib.h"


#ifdef __cplusplus
extern "C" {
#endif /* __cplusplus */

/*
 * Public Function Prototypes
 */
 
// Local calls
#if defined(_SHP_DLL_APPSNDMGR) || ( ! defined(_SHP_DLL) && ! defined(_SHP_CALL_DLL_APPSNDMGR))

// Dispatch
BOOL	AppSndDispatch	( UINT32 action, ULONG param1, ULONG param2, void* pParam3 );

#endif // defined(_SHP_DLL_APPSNDMGR) || ( ! defined(_SHP_DLL) && ! defined(_SHP_CALL_DLL_APPSNDMGR))

/*
 * ECOM Interface
 */
#include "AppSndECOM.h"

extern TAppSndMgr	appSndMgr;

/* External Macro */
#define AppSndMuteKeySound(did, classType)				__AppSndMuteKeySound(did, classType, __FILE__, __LINE__)	///< mute by key class
#define AppSndUnmuteKeySound(did, classType)			__AppSndUnmuteKeySound(did, classType, __FILE__, __LINE__)	///< unmute by key class

#define AppSndMuteKeySoundByKeyType(did, keyType)		__AppSndMuteKeySoundByKeyType(did, keyType, __FILE__, __LINE__)	///< Mute by Key Code
#define AppSndUnmuteKeySoundByKeyType(did, keyType)		__AppSndUnmuteKeySoundByKeyType(did, keyType, __FILE__, __LINE__)	///< Unmute by Key Code

#define AppSndSetKeyPlayStyle(classType, playStyle)	__AppSndSetKeyPlayStyle(classType, playStyle, __FILE__, __LINE__)	///< Play style per class

#define AppSndSetKeyPlayStyleByKeyType(keyType, playStyle)	__AppSndSetKeyPlayStyleByKeyType(keyType, playStyle, __FILE__, __LINE__)	///< Play style per key type

#define AppSndSetKeySoundClass(classType, sndClassType)			__AppSndSetKeySoundClass(classType, sndClassType, __FILE__, __LINE__)	///< Set key sound class
#define AppSndSetKeySoundClassByKeyType(keyType, sndClassType)	__AppSndSetKeySoundClassByKeyType(keyType, sndClassType, __FILE__, __LINE__)	///< Set key sound class

#define AppSndMuteFlipSound()						__AppSndMuteFlipSound(__FILE__, __LINE__)	///< Mute flip sound
#define AppSndUnmuteFlipSound()						__AppSndUnmuteFlipSound(__FILE__, __LINE__)	///< Unmute flip sound

#define AppSndPlayKeySound(keyType)					__AppSndPlayKeySound(keyType, FALSE)	///< Key and Vib
#define AppSndPlayKeySound2(keyType, bVibration)	__AppSndPlayKeySound(keyType, bVibration)	///< Key and Vib

#define AppSndPlayKeySoundEx(keyType)				__AppSndPlayKeySoundEx(keyType, FALSE)	///< Key and Vib
#define AppSndPlayKeySoundEx2(keyType, bVibration)	__AppSndPlayKeySoundEx(keyType, bVibration)	///< Key and Vib

// key Vib
#define AppSndMuteKeyVib(did, keyType)			__AppSndMuteKeyVib(did, keyType, __FILE__, __LINE__)	///< Mute vib
#define AppSndUnmuteKeyVib(did, keyType)		__AppSndUnmuteKeyVib(did, keyType, __FILE__, __LINE__)	///< Unmute vib


#define AppSndMuteEffectSound(did, effectType)		__AppSndMuteEffectSound(did, effectType, __FILE__, __LINE__)	///< Effect
#define AppSndUnmuteEffectSound(did, effectType)	__AppSndUnmuteEffectSound(did, effectType, __FILE__, __LINE__)	///< Effect

#define AppSndMuteEffectSoundEx(did, classType)		__AppSndMuteEffectSoundEx(did, classType, __FILE__, __LINE__)	///< Effect
#define AppSndUnmuteEffectSoundEx(did, classType)	__AppSndUnmuteEffectSoundEx(did, classType, __FILE__, __LINE__)	///< Effect

#define AppSndPlayEffectSound(effectType, callback, pParam)							__AppSndPlayEffectSound(effectType, callback, pParam, TRUE)	///< Effect and Vib
#define AppSndPlayEffectSound2(effectType, callback, pParam, bVibration)			__AppSndPlayEffectSound(effectType, callback, pParam, bVibration)		///< Effect and Vib

#define AppSndPlayEffectSoundEx(effectType, sndType, callback, pParam)					__AppSndPlayEffectSoundEx(effectType, sndType, callback, pParam, TRUE)	///< Effect and Vib
#define AppSndPlayEffectSoundEx2(effectType, sndType, callback, pParam, bVibration)		__AppSndPlayEffectSoundEx(effectType, sndType, callback, pParam, bVibration)	///< Effect and Vib

#ifdef __cplusplus
}
#endif /* __cplusplus */

#endif // _APP_SND_MGR_H_
