/*
 * Samsung Handset Platform
 * Copyright (c) 2000 Software Center, Samsung Electronics, Inc.
 * All rights reserved.
 *
 * This software is the confidential and proprietary information
 * of Samsung Electronics, Inc. ("Confidential Information").  You
 * shall not disclose such Confidential Information and shall use
 * it only in accordance with the terms of the license agreement
 * you entered into with Samsung Electronics. 
 */

/*:Associate with "Event Manager" */

#ifndef _EM_EVENT_HANDLER_H_
#define _EM_EVENT_HANDLER_H_

#ifdef __cplusplus
extern "C" {
#endif /* __cplusplus */

#include "Shp.h"
#include "OsAL.h"
#include "EmEventTypes.h"
#include "EmEventBroker.h"
#include "DllTypes.h"

typedef HObj	HEHGroup;

/*
 * Event handler's execution state
 */
typedef enum
{
	EHS_DESTROYED,
	EHS_CREATED,
	EHS_STARTED,
	EHS_STOPPED
} WmEventHandlerState;


typedef void WmState;
typedef BOOL (*WmHandleEvent)(WmState* pState, WmEvent* pEvent);
typedef struct
{
	UINT16			fromEvent;	/* Event range */
	UINT16			toEvent;
	WmHandleEvent	handleEvent;
} WmEventMapEntry;
typedef WmEventMapEntry		WmEventMap[];

/*
 * Event Handler Object Definition
 */
typedef struct
{
	char				szName[OBJNAME_MAX];
	WmState*			pState;
	WmEventMapEntry*	eventMap;
} WmEventHandler;


typedef BOOL (*WmEventCheckCallback)(WmEvent* pEvent);
typedef void (*WmEventProcessCallback)(WmEvent* pEvent);

/*
 * Internal Event Handler Management
 */
typedef struct
{
	BOOL					bUsed;

	WmEventHandler			eventHandler;
	WmEventHandlerState		state;
	WmHandleEvent			main;			// Single main function
	WmEventProcessCallback	callback;

	DID						did;
	ADDR					sb;				// Static base
} WmEventHandlerInfo;

#ifdef __cplusplus
}
#endif  /* __cplusplus */

#endif // _EM_EVENT_HANDLER_H_
