/*
 * Mocha
 * Copyright (c) 2000-2003 Software Center, Samsung Electronics, Inc.
 * All rights reserved.
 *
 * This software is the confidential and proprietary information
 * of Samsung Electronics, Inc. ("Confidential Information").  You
 * shall not disclose such Confidential Information and shall use
 * it only in accordance with the terms of the license agreement
 * you entered into with Samsung Electronics. 
 */

/*:Associate with "Event Manager" */

#ifndef _EM_EVENT_TYPES_H_
#define _EM_EVENT_TYPES_H_

#ifdef __cplusplus
extern "C" {
#endif /* __cplusplus */

#include "OsAL.h"


/* Event Group */
typedef enum 
{
	EVGRP_MAX
} WmEventGroup;


/* Event Group Boundaries */
#define EVGRP_SYS_BEGIN			0x0000
#define EVGRP_SYS_END			0x01FF
#define EVGRP_GWES_BEGIN		0x0200
#define EVGRP_GWES_END			0x02FF
#define EVGRP_USER_BEGIN		0xF000
#define EVGRP_USER_END			0xFFF0
#define EM_INVALID_EVENT		0xFFFF	/* Invalid value */

#	define BEGIN_EVENT_MAP(eventMap, elemCount)		\
	WmEventMapEntry* _WmGet##eventMap(void)			\
	{												\
		static int _i = 0;							\
		static WmEventMapEntry _map[elemCount+1];	\
		int _elemCount = elemCount;					\
		if (_i > 0) return _map;

#define HANDLE_EVENT(event, eventHandler)			\
		SysAssert(_i < _elemCount);					\
		_map[_i].fromEvent = event;					\
		_map[_i].toEvent = event;					\
		_map[_i++].handleEvent = eventHandler;

#define HANDLE_EVENT_RANGE(from, to, eventHandler)	\
		SysAssert(_i < _elemCount);					\
		_map[_i].fromEvent = from;					\
		_map[_i].toEvent = to;						\
		_map[_i++].handleEvent = eventHandler;
		
#define END_EVENT_MAP()								\
		_map[_i].fromEvent = EM_INVALID_EVENT;		\
		_map[_i].toEvent = EM_INVALID_EVENT;		\
		_map[_i].handleEvent = NULL;				\
		return _map;								\
	}

#	define DECLARE_EVENT_MAP(eventMap)			extern WmEventMapEntry* _WmGet##eventMap(void)
#	define WmGetEventMap(eventMap)				_WmGet##eventMap()

/*
 * Event Types defined by Core
 */
#define EV_ALARM				(EVGRP_SYS_BEGIN + 0x01)

#define EV_EXPOSE				(EVGRP_GWES_BEGIN + 0x07)	/* PEH */
#define EV_KEY_PRESSED			(EVGRP_GWES_BEGIN + 0x10)	/* PWN */
#define EV_NOTIFY				(EVGRP_GWES_BEGIN + 0x14)	/* PEH, PWN */
#define EV_ROTATE				(EVGRP_GWES_BEGIN + 0x19)	/* PWN */
#define EV_TOUCH_PRESSED		(EVGRP_GWES_BEGIN + 0x1B)	/* PWN */
#define EV_TOUCH_RELEASED		(EVGRP_GWES_BEGIN + 0x1F)	/* PWN */
#define EV_TOUCH_MOVE			(EVGRP_GWES_BEGIN + 0x20)	/* PWN */

#define EV_ANYTYPE				(EVGRP_GWES_BEGIN + 0xFF)	/* PEH */

#define DECLARE_HANDLE_EV_EXPOSE(eventHandler)		BOOL eventHandler(WmState* pState, WmEvent* pEvent)
#define DECLARE_HANDLE_EV_PRESSED(eventHandler)		BOOL eventHandler(WmState* pState, WmEvent* pEvent)
#define DECLARE_HANDLE_EV_NOTIFY(eventHandler)		BOOL eventHandler(WmState* pState, WmEvent* pEvent)

#define DECLARE_HANDLE_EV_ROTATE(eventHandler)		BOOL eventHandler(WmState* pState, WmEvent* pEvent)
#define DECLARE_HANDLE_EV_TOUCH_PRESSED(eventHandler)	BOOL eventHandler(WmState* pState, WmEvent* pEvent)
#define DECLARE_HANDLE_EV_TOUCH_RELEASED(eventHandler)	BOOL eventHandler(WmState* pState, WmEvent* pEvent)
#define DECLARE_HANDLE_EV_TOUCH_MOVE(eventHandler)		BOOL eventHandler(WmState* pState, WmEvent* pEvent)
#define DECLARE_HANDLE_EV_ANYTYPE(eventHandler)		BOOL eventHandler(WmState* pState, WmEvent* pEvent)

#define HANDLE_EV_EXPOSE(eventHandler)				HANDLE_EVENT(EV_EXPOSE, eventHandler)
#define HANDLE_EV_PRESSED(eventHandler)				HANDLE_EVENT(EV_KEY_PRESSED, eventHandler)
#define HANDLE_EV_NOTIFY(eventHandler)				HANDLE_EVENT(EV_NOTIFY, eventHandler)
#define HANDLE_EV_ROTATE(eventHandler)				HANDLE_EVENT(EV_ROTATE, eventHandler)
#define HANDLE_EV_TOUCH_PRESSED(eventHandler)		HANDLE_EVENT(EV_TOUCH_PRESSED, eventHandler)
#define HANDLE_EV_TOUCH_RELEASED(eventHandler)		HANDLE_EVENT(EV_TOUCH_RELEASED, eventHandler)
#define HANDLE_EV_TOUCH_MOVE(eventHandler)			HANDLE_EVENT(EV_TOUCH_MOVE, eventHandler)
#define HANDLE_EV_ANYTYPE(eventHandler)				HANDLE_EVENT(EV_ANYTYPE, eventHandler)

#include "OemWmEvents.h"


#define USER_CONTROL_NOTIFY_BASE			0x0400

#define WmTimerCallback						OsTimerCallback
#define WmTimerBindCallback					OsTimerBindCallback
typedef void (*WmAlarmCallback)(HTimer hAlarm, ULONG param, BOOL bPowerOffAlarm, ULONG alarmCount);

typedef struct
{
	UINT16			type;				/* Event Type */
	UINT16			id;					/* Event Sequence Number */
	HEventHandler	hDestEventHandler;	/* Destination Event Handler */
	HObj			hDestWindow;		/* Destination Window */
 	UINT16			keyCode;			/* Key Code like VK_HP_XXX */
	UINT16			state;				/* Key States like VKS_NORMAL, VKS_TIMEOUT, ... */
	int				param;	
	char			dummy[OS_MSG_SIZE - sizeof(int) * 5];	/* Padding To make the structure size to OS_MSG_SIZE] */
} WmKeyEvent;

typedef struct
{
	UINT16			type;				/* Event Type */
	UINT16			id;					/* Event Sequence Number */
	HEventHandler	hDestEventHandler;	/* Destination Event Handler */
	HObj			hDestWindow;		/* Destination Window */
	HObj			hSrcWindow;			/* Source Window */
	UINT16			nType;				/* type of Notification */
	UINT16			nSubType;			/* Sub type of Notification */
	ULONG			param1;				/* Generic Parameters */
	ULONG			param2;
	void*			pParam3;
	BOOL			bRecord;				/* When this field is true, the event is recorded as major event recording. This field is currently controlled
											by EventMgr automatically. */
	char			dummy[OS_MSG_SIZE - sizeof(int) * 9];	/* Padding To make the structure size to OS_MSG_SIZE] */
} WmNotifyEvent;

typedef struct
{
	UINT16			type;				/* Event Type */
	UINT16			id;					/* Event Sequence Number */
	HEventHandler	hDestEventHandler;	/* Destination Event Handler */
	HObj			hDestWindow;		/* Destination Window */
	UINT16			tType;				/* type of touch */
	UINT16			screen;				/* screen id */
	UINT16			tSubType;				/* type of touch */
	INT16			rectX;	
	INT16			rectY;	
	INT16			startX;	
	INT16			startY;	
	INT16			prevX;	            /* previous cursor position x */
	INT16			prevY;	            /* previous cursor position y */
	INT16           moveW;
	INT16           moveH;
	INT16			bPressed:2;
	INT16			bMoved:2;
	INT16			bLongPress:2;
	INT16			bDbPress:2;
	INT16			bFlick:2;
	INT16			bHalfLongPress:2;
} WmTouchEvent;

typedef union
{
	UINT16				type;
	WmKeyEvent			key;
	WmNotifyEvent		notify;
	WmTouchEvent		touch;
} WmEvent;

#ifdef __cplusplus
}
#endif  /* __cplusplus */

#endif // _EM_EVENT_TYPES_H_
