/*
 * Mocha 
 * Copyright (c) 2000-2003 Software Center, Samsung Electronics, Inc.
 * All rights reserved.
 *
 * This software is the confidential and proprietary information
 * of Samsung Electronics, Inc. ("Confidential Information").  You
 * shall not disclose such Confidential Information and shall use
 * it only in accordance with the terms of the license agreement
 * you entered into with Samsung Electronics. 
 */

/*:Associate with "OS Abstraction Layer" */

#ifndef _OSAL_H_
#define _OSAL_H_

#ifdef __cplusplus
extern "C" {
#endif /* __cplusplus */

#include "Shp.h"
#include "OemOsTask.h"

typedef HObj HOsObj;			/* Generic handle for OS objects */
/*
 * OS Objects' Handle
 */
typedef HOsObj HTask;
typedef HOsObj HMsgQueue;
typedef HOsObj HSemaphore;
typedef HOsObj HMutex;
typedef HOsObj HTimer;
typedef HOsObj HEventHandler;



#define OS_MSG_SIZE			40	/* BYTE */


/*
 * Message Structure of OsAL
 */
typedef BYTE OsMessage[OS_MSG_SIZE];

typedef struct tagMSGBOX
{
	BOOL				bUsed;
	OsMessage			msg;
	struct tagMSGBOX*	pMBNext;
} OsMsgBox;

typedef BOOL (*OsMessageCheckCallback)(OsMessage* pMsg);

/*----------
 * Os Object: Generic Class of All OS Objects
 */
typedef struct
{
	BOOL			bUsed;
	char			szName[OBJNAME_MAX+1];
	HTask			hOwnerTask;		/* The task ID which owns this resource */
} OsObj;


/*-----
 * Task
 */
typedef DWORD (*OsTaskStartEntry)(DWORD param);

#define INVALID_TASK_PRIO	(-1)

/*----------
 * Semaphore
 */
#ifndef INFINITE
#	define INFINITE	0xFFFFFFFF
#endif

#define	DISABLED_HMUTEX	-2

/*------
 * Timer
 */
typedef void (*OsTimerCallback)(HTimer hTimer, ULONG param);
typedef void (*OsTimerBindCallback)(HTimer hTimer, ULONG param);

#define TIMER_CALLBACK_ROUTINE OsTimerCallback
#define TIMER_BINDCALLBACK_ROUTINE OsTimerBindCallback

typedef struct
{
	HTimer	hTimer;
	HTask	hTask;
} TimerParam;

#define	TIMER_EXPIRE				0x00

#define _SYS_EVENT_BEGIN		0x0000
#define _SYS_EVENT_END			0x01FF

/*
 * Event Types defined by SYS
 */
#define _SYS_EV_TIMER				(_SYS_EVENT_BEGIN + 0x10)

/*
 * ECOM Interface
 */
#include "OsECOM.h"

extern TOsAL	osAL;

#ifdef __cplusplus
}
#endif /* __cplusplus */

#endif // _OSAL_H_
