/*
 * Samsung Handset Platform
 * Copyright (c) 2000 Software Center, Samsung Electronics, Inc.
 * All rights reserved.
 *
 * This software is the confidential and proprietary information
 * of Samsung Electronics, Inc. ("Confidential Information").  You
 * shall not disclose such Confidential Information and shall use
 * it only in accordance with the terms of the license agreement
 * you entered into with Samsung Electronics. 
 */

/*:Associate with "OS Abstraction Layer" */

#ifndef _OS_ECOM_H_
#define _OS_ECOM_H_

#include "SysECOM.h"

#ifdef __cplusplus
extern "C" {
#endif /* __cplusplus */

/*
 * ECOM Interface
 */
ECOM_INTERFACE(EOsAL)
{
	DECLARE_EBASE()
	
	/* Task */
	HTask		(*CreateTask)(char* szTaskName, int priority, BYTE* pStack, ULONG stackSize,
					OsTaskStartEntry startAddr, DID startDID, DWORD param);
	BOOL		(*StartTask)(HTask hTask);
	BOOL		(*DeleteTask)(HTask hTask);
	HTask		(*GetTaskByName)(char* szTaskName);
	HTask		(*GetCurrentTask)(void);
	const char*	(*GetCurrentTaskName)(void);
	int			(*GetTaskPriority)(HTask hTask);
	BOOL		(*SetTaskPriority)(HTask hTask, int priority);

	/* Message Queue */
	BOOL		(*SendMessage)(HTask hTask, const OsMessage* pMsg);
	BOOL		(*ReceiveMessage)(ULONG timeout, OsMessage* pMsg, UINT* pRemainingMsgCount);
	BOOL		(*WaitMessage)(OsMessageCheckCallback checkRoutine, OsMessage* pMsg);
	BOOL		(*PeekMessage)(OsMessage* pMsg, UINT* pMsgCount);
	BOOL		(*PeekMessageEx)(OsMessage* pMsg, const UINT ith);
	BOOL		(*DeleteMessage)(OsMessage* pMsg, const UINT ith);
	BOOL		(*DeleteMessageByType)(OsMessage* pMsg);

	/* Semaphore */
	HSemaphore	(*CreateSemaphore)(char* szSemName, int count);
	BOOL		(*DeleteSemaphore)(HSemaphore hSem);
	HSemaphore	(*GetSemaphoreByName)(char* szSemName);
	BOOL		(*AcquireSemaphore)(HSemaphore hSem, ULONG timeout);
	BOOL		(*ReleaseSemaphore)(HSemaphore hSem);

	/* Mutex */
	HMutex		(*CreateMutex)(char* szMutexName);
	BOOL		(*DeleteMutex)(HMutex hMutex);
	HMutex		(*GetMutexByName)(char* szMutexName);
	BOOL		(*AcquireMutex)(HMutex hMutex);
	BOOL		(*ReleaseMutex)(HMutex hMutex);

	/* Timer */
	HTimer		(*CreateTimer)(char* szTmName, const char* szCallerFileName, int nLine);
	BOOL		(*StartTimer)(HTimer hTimer, ULONG timeout, OsTimerCallback timerCallback, ULONG param);
	BOOL		(*DeleteTimer)(HTimer hTimer, const char* szCallerFileName, int nLine);
	HTimer		(*GetTimerByName)(char* szTmName);
	BOOL		(*CancelTimer)(HTimer hTimer);
	BOOL		(*IsTimerActive)(HTimer hTimer);
	void		(*Sleep)(int mSecs);

	HTimer		(*CreateTimerEx)(char* szTmName, OsTimerBindCallback bindCallback, const char* szCallerFileName, int nLine);

	/* Os DLL */
	DID			(*GetCurrentDLL)(void);
};

/*
 * Interface Accessor
 */

// Native/DLL Call
#if (defined(_SHP_SIMUL) && defined(_SHP_CALL_DLL_OSALMGR)) || (defined(_SHP_DLL) && ! defined(_SHP_DLL_OSALMGR) && ! defined(_SHP_CALL_DLL_OSALMGR))

#	define OsDispatch		 			(__OsGetVtbl()->Dispatch)

// Task

#	define OsCreateTask				(__OsGetVtbl()->CreateTask)
#	define OsStartTask				(__OsGetVtbl()->StartTask)
#	define OsDeleteTask				(__OsGetVtbl()->DeleteTask)
#	define OsGetTaskByName			(__OsGetVtbl()->GetTaskByName)
#	define OsGetCurrentTask			(__OsGetVtbl()->GetCurrentTask)
#	define OsGetCurrentTaskName		(__OsGetVtbl()->GetCurrentTaskName)
#	define OsGetTaskPriority			(__OsGetVtbl()->GetTaskPriority)
#	define OsSetTaskPriority			(__OsGetVtbl()->SetTaskPriority)

// Message Queue
#	define OsSendMessage			(__OsGetVtbl()->SendMessage)
#	define OsReceiveMessage			(__OsGetVtbl()->ReceiveMessage)
#	define OsWaitMessage			(__OsGetVtbl()->WaitMessage)
#	define OsPeekMessage			(__OsGetVtbl()->PeekMessage)
#	define OsPeekMessageEx			(__OsGetVtbl()->PeekMessageEx)

// Semaphore
#	define OsCreateSemaphore		(__OsGetVtbl()->CreateSemaphore)
#	define OsDeleteSemaphore		(__OsGetVtbl()->DeleteSemaphore)
#	define OsGetSemaphoreByName	(__OsGetVtbl()->GetSemaphoreByName)
#	define OsAcquireSemaphore		(__OsGetVtbl()->AcquireSemaphore)
#	define OsReleaseSemaphore		(__OsGetVtbl()->ReleaseSemaphore)

// Mutex
#	define OsCreateMutex				(__OsGetVtbl()->CreateMutex)
#	define OsDeleteMutex				(__OsGetVtbl()->DeleteMutex)
#	define OsGetMutexByName			(__OsGetVtbl()->GetMutexByName)
#	define OsAcquireMutex			(__OsGetVtbl()->AcquireMutex)
#	define OsReleaseMutex			(__OsGetVtbl()->ReleaseMutex)

// Timer
#	define OsCreateTimer(szTmName)					(__OsGetVtbl()->CreateTimer(szTmName, __FILE__, __LINE__))
#	define OsCreateTimerEx(szTmName, callback)		(__OsGetVtbl()->CreateTimerEx(szTmName, callback, __FILE__, __LINE__))
#	define OsStartTimer				(__OsGetVtbl()->StartTimer)
#	define OsDeleteTimer(hTimer)		(__OsGetVtbl()->DeleteTimer(hTimer, __FILE__, __LINE__))
#	define OsGetTimerByName		(__OsGetVtbl()->GetTimerByName)
#	define OsCancelTimer				(__OsGetVtbl()->CancelTimer)
#	define OsIsTimerActive			(__OsGetVtbl()->IsTimerActive)
#	define OsSleep					(__OsGetVtbl()->Sleep)


// DLL call-stack APIs and Module interfaces	
#	define OsGetCurrentDLL			(__OsGetVtbl()->GetCurrentDLL)
#endif //


/*
 * ECOM Type
 */
typedef struct
{
	DECLARE_ECOM_VTBL(EOsAL)
	UINT32		refCount;

} TOsAL;

#ifdef __cplusplus
}
#endif /* __cplusplus */

#endif // _OS_ECOM_H_
