/*
 * Samsung Handset Platform
 * Copyright (c) 2000 Software Center, Samsung Electronics, Inc.
 * All rights reserved.
 *
 * This software is the confidential and proprietary information
 * of Samsung Electronics, Inc. ("Confidential Information").  You
 * shall not disclose such Confidential Information and shall use
 * it only in accordance with the terms of the license agreement
 * you entered into with Samsung Electronics. 
 */
 
/*:Associate with "Embedded COM" */

#ifndef _SYS_ECOM_H_
#define _SYS_ECOM_H_

#ifdef __cplusplus
extern "C" {
#endif /* __cplusplus */

#include "ShpTypes.h"

typedef UINT32		ECOMCLSID;

#define ECOM_VTBL(name)				name

#define ECOM_INTERFACE(name) \
	struct _##name {\
		struct ECOM_VTBL(name)*		pVtbl;\
	};\
	typedef struct ECOM_VTBL(name) ECOM_VTBL(name);\
	struct ECOM_VTBL(name)

typedef BOOL	(*EcomDispatch)(UINT32 action, ULONG param1, ULONG param2, void* pParam3);

#define DECLARE_EBASE() \
	UINT32	(*AddRef)  (void);\
	UINT32	(*Release) (void);\
	BOOL	(*Dispatch)(UINT32 action, ULONG param1, ULONG param2, void* pParam3);

#define DECLARE_ECOM_VTBL(name)			name* pVtbl##name;		\
										ADDR sb;   

typedef struct
{
	void*	pClass;
	ADDR	sb;
} EcomType;

#define GET_ECOM_PVTBL(p, name)			((struct _##name*)p)->pVtbl
#define SysGetVtbl(compID, compName) (((T##compName*)(pDllBaseEcomFactory[compID-CID_BASE].pClass))->pVtblE##compName)
#define SysGetSb(compID) (((EcomType*)(pDllBaseEcomFactory[compID-CID_BASE].pClass))->sb)


#define INIT_ECOM_VTBL(p, name, vt)		(GET_ECOM_PVTBL(p, name) = (ECOM_VTBL(name)*)&vt)
               
/*
 * EBase Definition
 */
ECOM_INTERFACE(EBase)
{
	DECLARE_EBASE()
};

#define EBaseAddRef(p)				GET_ECOM_PVTBL(p, EBase)->AddRef()
#define EBaseRelease(p)				GET_ECOM_PVTBL(p, EBase)->Release()


/*
 * EComp Interface
 */
ECOM_INTERFACE(EComp)
{
	DECLARE_EBASE()

	BOOL (*Create)(EComp* pEComp, ECOMCLSID clsID, void** ppObj);
	void (*Destroy)(EComp* pEComp);
};

#define ECompAddRef(p)				GET_ECOM_PVTBL(p, EComp)->AddRef()
#define ECompRelease(p)				GET_ECOM_PVTBL(p, EComp)->Release()
#define ECompCreate(p,id,ppo)		GET_ECOM_PVTBL(p, EComp)->Create(id,ppo)
#define ECompDestroy(p)				GET_ECOM_PVTBL(p, EComp)->Destroy()

typedef struct _EcomClass EcomClass;
struct _EcomClass
{
   void*			pData;		// Private data
   EcomClass*		pNextObj;	// Pointer to next class in the list
   ECOMCLSID		clsID;		// Class information
};


typedef struct _EcomComp
{
	DECLARE_ECOM_VTBL(EComp)	
	UINT32		refCount;
	EcomClass*	pObjList;
	BOOL		(*Create)(EComp* pEComp, ECOMCLSID clsID, void** ppObj);
	void		(*Destroy)(EComp* pEComp);
} EcomComp;

typedef struct
{
	ECOMCLSID	clsID;
	void*		pClass;
} EcomFactory;


	extern EcomFactory* pDllBaseEcomFactory;
	extern ADDR dllBaseSb;

#ifdef __cplusplus
}
#endif /* __cplusplus */

#endif // _SYS_ECOM_H_
