/*
 * Mocha
 * Copyright (c) 2001, 2002 Software Center, Samsung Electronics, Inc.
 * All rights reserved.
 *
 * This software is the confidential and proprietary information
 * of Samsung Electronics, Inc. ("Confidential Information").  You
 * shall not disclose such Confidential Information and shall use
 * it only in accordance with the terms of the license agreement
 * you entered into with Samsung Electronics. 
 */
/**
 * @file		ProtoMgr.h
 * @brief		This file implementats the Protocol Manager.
 * @version	1.0
 */

#ifndef _PROTO_MGR_H_
#define _PROTO_MGR_H_

#ifdef __cplusplus
extern "C" {
#endif /* __cplusplus */

#include "Shp.h"
#include "MemMgr.h"
#include "EventMgr.h"
#include "RegMgr.h"
#include "DevPhoneStatus.h"

/*
 * Event handler's name/state/event definition
 */
#define EHN_PROTO						"ProtoMgr"
#define EHN_PROTOIND					"ProtoInd"

#ifdef _SHP_PROTO_MULTI_PS
#define	PROTO_CSD_COUNT_MAX		1
#define	PROTO_PSD_COUNT_MAX		3
#endif

/*
 * Event handlers' declaration
 */
DECLARE_HANDLE_ALL_EVENTS(ProtoHandleEvent);
DECLARE_HANDLE_ALL_EVENTS(ProtoIndHandleEvent);

/*
 * Common data types used for portability across machine architectures
 * Note that this is written for an ARM7 using ARM C at the moment
 * Appropriate modification may need to be done for any other platform
 */
typedef	HObj		HProtocol;
typedef HObj		HSocket;
typedef	HObj		HProtoAccount;

#if !defined(IPADDR_DEFINED)
typedef UINT32 		IPAddress;
#define IPADDR_DEFINED
#endif /* !IPADDR_DEFINED */
typedef UINT32  	Time;
typedef UINT16		TransactionId; 
typedef UINT32		IPV4Addr;
typedef UINT8		IPV6Addr;
typedef UINT8		ServiceCodeAddr;
typedef UINT8 		MSISDNAddr;
typedef UINT16 		Port;
typedef UINT8		APN;

#define PACKED __packed 

#define PROTO_DATA_SERVICE_PSD				YES			/**< Packet switched data service */
#define PROTO_DATA_SERVICE_CSD				NO			/**< Circuit switched data service */
#define PROTO_DATA_SERVICE_MPS				NO			/**< Multi PS service dependent on the Modem */

/* Registry information for Account */
#define PROTO_ACCOUNT_FILE_NAME				"ProtoAccount.ini"
#define PROTO_ACCOUNT_VERSION_NUM			11
#define PROTO_ACCOUNT_COUNT_MAX			20
#define PROTO_ACCOUT_DEFAULT_NAME			"Mocha PS"	/**< Default account name */

/* Registry information for Sttistics */
#define PROTO_STAT_FILE_NAME					"ProtoStat.ini"
#define PROTO_REG_STAT_VERSION				1
#define PROTO_REG_STAT_DEVAULT_VALUE		0
#define PROTO_OPMODE_COUNT_MAX			2
#define PROTO_OPMODE_PS_INDEX				0
#define PROTO_OPMODE_CS_INDEX				1

#define PROTO_ACL_COUNT_MAX			200

#define PROTO_GPRS_SERVICE_CODE1			"*99#"		/**< Request Packet Domain service 'D' which identifies a request to use the Packet Domain service */
#define PROTO_GPRS_SERVICE_CODE2			"*98#"		/**< Request Packet Domain IP service 'D' which identifies a request to use the GPRS with IP (PDP types IP and PPP) */

#define PROTO_INVALID_CONTEXT_ID			(-1)
#define PROTO_INVALID_CONTEXT_INDEX			(-1)

#define PROTO_START_DEFAULT_TIMEOUT			65		/**< In seconds, it gurantees 2 times PDP activation(30s per 1 PDP request). */
#define PROTO_START_MIN_TIMEOUT				30		/**< In seconds, it gurantees 1 time PDP activation specified in the GPRS spec. */
#define PROTO_START_MAX_TIMEOUT				160		/**< In seconds, it gurantees 5 times PDP activation specified in the GPRS spec. */
#define PROTO_STOP_DEFAULT_TIMEOUT			120		/**< In seconds, in case of PS call during MT CS under GPRS, it takes 40s in Madrid. */
#define PROTO_STOP_TIMEOUT_BY_NO_SERVICE		5	/**< In case of NO SERVICE, this timeout will be used */	//[hakbong] for tracking
#define PROTO_SUSPEND_DEFAULT_TIMEOUT		65		/**< In seconds, in case of suspend timeout except by GPRS hold. */
#define PROTO_SUSPEND_GPRSHOLD_TIMEOUT		125		/**< In seconds, in case of suspend timeout only by GPRS hold. */

#define PROTO_INACTIVE_CHECK_DEFAULT_TIMEOUT		1		/**< In seconds, check the inactive time out . */
#define PROTO_DNS_COUNT_MAX					2	/**< Primary and secondary DNS addresses. */

/* These macros are used to use param as two usages. */
#define PROTO_SET_PARAM(device, opMode)		((0xFFFF0000&(device<<16)) | (0x0000FFFF&opMode))	/* Set the param with device and opMode. */
#define PROTO_GET_DEVICE(param)				((param&0xFFFF0000)>>16)							/* Regrieve device from param. */
#define PROTO_GET_OPMODE(param)				(param&0x0000FFFF)								/* Regrieve opMode from param. */

#define PROTO_DEFALT_UP_DOWN_RATE_2G		9600
#define PROTO_DEFALT_UP_DOWN_RATE_2_5G	64000
#define PROTO_DEFALT_UP_DOWN_RATE_3G		128000

/* Network/Bearer/Address Types */			 
typedef enum
{
	ANY_ANY_IPV4 						= 0x00,
	ANY_ANY_IPV6,
	GSM_USSD_MSISDN,
	GSM_SMS_MSISDN,
	IS136_GUTSRDATA_MSISDN,
	IS95CDMA_SMS_MSISDN,
	IS95CDMA_CSD_IPV4,
	IS95CDMA_PACKET_IPV4,
	IS136_CSD_IPV4,
	IS136_PACKET_IPV4,
	GSM_CSD_IPV4,
	GSM_GPRS_IPV4,
	GSM_USSD_IPV4,
	AMPS_CDPD_IPV4,
	PDC_CSD_IPV4,
	PDC_PACKET_IPV4,
	IDEN_SMS_MSISDN,
	IDEN_CSD_IPV4,
	IDEN_PACKET_IPV4,
	PAGER_FLEX_MSISDN,
	PHS_SMS_MSISDN,
	PHS_CSD_IPV4,
	GSM_USSD_SERVICECODE,
	TETRA_SDS_IDSI,
	TETRA_SDS_MSISDN,
	TETRA_PACKET_IPV4,
	ANY_CSD_IPV4,
	ANY_PSD_IPV4,
	ANY_SMS_MSISDN
} ProtoNBA;

typedef enum
{
	PROTO_BEARER_ANY,
	PROTO_BEARER_SMS,
	PROTO_BEARER_MAX	
} ProtoBearerType;

/**
 * @brief	Operation Modes 
 */
typedef enum
{
	OP_MODE_NONE						= 0x00,	/**< Operation mode is not defined. */
	OP_MODE_PS							= 0x01,	/**< Packet switched data service. */
	OP_MODE_CS							= 0x02,	/**< Circuit switched data service. */
	OP_MODE_BT							= 0x04	/**< Data service on Bluetooth */
} ProtoOpMode;

/** 
 * @brief	Protocol Type 
 */
typedef enum
{
	PROTO_TYPE_NONE					= 0x00,	/**< Protocol type is not defined. */
	PROTO_TYPE_IP						= 0x01,	/**< Protocol type is IP. */
	PROTO_TYPE_PPP						= 0x02,	/**< Protocol type is PPP. */
	PROTO_TYPE_IPV6					= 0x03,		/**< Protocol type is IPv6. */
	PROTO_TYPE_USER					= 0x04		/**< User defined protocol type. */
} ProtoType;

/**
 * @brief	Protocol Bearer State 
 */
typedef enum
{
	PROTO_STATE_STOPPED				= 0x00,	/**< (0) Initial state. */
	PROTO_STATE_STARTREQ,						/**< (1) After sening a start request and waiting for a context ID. */
	PROTO_STATE_STARTING,						/**< (2) After receiving the context ID and then waiting for the reply to the start request. */
	PROTO_STATE_STARTED,						/**< (3) Received the reply of start request and data call is started. */
	PROTO_STATE_STOPPING,						/**< (4) After sending a stop request and waiting for the reply to the stop request. */
	PROTO_STATE_STARTHOLD,						/**< (5) During STOPPING, new start request is triggered. */
	PROTO_STATE_STOPHOLD,						/**< (6) During SUSPENDED by GPRS hold, new stop request is triggered. */
	PROTO_STATE_SUSPENDED						/**< (7) Start request was successfull but network is suspend state. */
} ProtoState;

/**
 * @brief	Protocol services error types 
 */
typedef enum
{
	/* Result success */
	PROTO_ERROR_NONE					= 0x0000,
	/* Wireless stack errors */
	PROTO_ERROR_NO_SERVICE,					/**< (1) Under no service state */
	PROTO_ERROR_INVALID_PARAM,					/**< (2) Invalid parameter */
	PROTO_ERROR_INVALID_APN,					/**< (3) Invalid APN */
	PROTO_ERROR_INVALID_OP_MODE,				/**< (4) Invalid operation mode */
	PROTO_ERROR_INVALID_PROTO_TYPE,			/**< (5) Invalid protocol type */
	PROTO_ERROR_INVALID_SERVICE_DOMAIN,		/**< (6) PS triggered in CS only domain */
	PROTO_ERROR_INVALID_PDP_DATA,				/**< (7) PDP address or type is wrong */
	PROTO_ERROR_INVALID_QOS,					/**< (8) Qos Parameter was not accepted by the Network */
	PROTO_ERROR_SESSION_DEACTIVATED,			/**< (9) Session was deactivated */
	PROTO_ERROR_AUTHENTICATION_FAILED,			/**< (10) Failed in authentication process */
	PROTO_ERROR_MOBILE_FAILURE,				/**< (11) Failure by mobile */
	PROTO_ERROR_NETWORK_FAILURE,				/**< (12) Failure by network */
	PROTO_ERROR_TIMEOUT,						/**< (13) Timeout error which includes start and stop timeout */
	PROTO_ERROR_NO_RESOURCE,					/**< (14) Not a fatal error, so can be retried later */
	PROTO_ERROR_INVALID_CONTEXT_ID,			/**< (15) Context ID(call ID) is wrong */
	PROTO_ERROR_MODEM_IN_USE,					/**< (16) Phone is currently in use by another operation by CS in general */
	PROTO_ERROR_INVALID_SIM_STATE,				/**< (17 SIM was marked by the network as invalid for the CS and/or PS domain */
	PROTO_ERROR_SERVICE_NOT_SUBSCRIBED,		/**< (18) If SIM is not subscribed for the requested service, network sends this error cause. */
	PROTO_ERROR_SERVICE_NOT_IMPLEMENTED,		/**< (19) Bearer service is not implemented.*/
	PROTO_ERROR_FDN_NOT_ALLOWED,				/* (20) FDN  is enabled and the number is not listed in the FDN list. */
	PROTO_ERROR_CALL_INCOME,					/**< (21) Stop by network only for CDMA 1x. */
	PROTO_ERROR_NOT_SUPPORT_3GCSD,			/**< (22) Special error code. If application request CSD in 3G, this error code sends to application.*/
	PROTO_ERROR_UNKNOWN,					/**< (23) Unknown protocol error from modem */
	PROTO_ERROR_SI_OFFLINE,					/**< (24) Not accepted by flight offline mode  */
	/**< TCP/IP/PPP stack errors */
	PROTO_ERROR_TCPIP_UP,						/**< TCP/IP stack up failed */
	PROTO_ERROR_TCPIP_DOWN,					/**< TCP/IP stack down failed */
	PROTO_ERROR_PPP_UP,						/**< PPP stack up failed */
	PROTO_ERROR_PPP_DOWN,						/**< PPP stack down failed */
	PROTO_ERROR_WDP_INIT,						/**< WDP init failed */
	/**< ProtoMgr API call errors */
	PROTO_ERROR_ACTIVE_CONNECTIONS,			/**< Active connections still exist. */
	PROTO_ERROR_MAX_CONNECTIONS,			/**< No connection resource */

	PROTO_ERROR_INVALID_HANDLE,				/**< Invalid protocol handle */
	PROTO_ERROR_INVALID_CONTEXT,				/**< Invalid context information. */
	PROTO_ERROR_INVALID_CALLBACK,				/**< Invalid user callback function */
	PROTO_ERROR_INVALID_EHN,					/**< Invalid event handler name */
	PROTO_ERROR_INVALID_EVENTCLASS,			/**< Invalid event class */
	PROTO_ERROR_INVALID_REFTYPE,				/**< Invalid reference type */
	PROTO_ERROR_INVALID_TIMEOUT,				/**< Invalid timeout */
	PROTO_ERROR_MAX_CLIENT,					/**< Reached the maximum client number */
	PROTO_ERROR_MAX_ACCOUNT,					/**< Reached the maximum account number */
	PROTO_ERROR_ACCESS_DENIED,				/**< Access is denied. */
	PROTO_ERROR_REGISTRY_LOAD,				/**< Registry load error. */

	PROTO_ERROR_PCBROWSING_ON,				/**< PC Browsing is on going. (special error type for stealth-h) */
	PROTO_ERROR_REACTIVATION_REQ,			/**< PDP reactivation requested by network because of GGSN rebooting. (special error type for stealth-h)*/
	PROTO_ERROR_NO_PDP_ACTIVATED,			/**< in the MM layer, service request was rejected becase of NO PDP activated.  (special error type for stealth-h) */
	/**<Added new cause codes to support IMS requirement (TD-751)..*/

	PROTO_ERROR_PDG_NOT_REACHABLE,			/**< IKE SA failure after PDG discovery */
	PROTO_ERROR_PDG_AUTH_FAILURE,			/**< IPSec SA failure after IKE SA */

	PROTO_ERROR_DMS_HANDOVER,				/**< DMS data service error by handover */
	PROTO_ERROR_NOSIM,			/**< When No SIM mode is enabled, user can't start PDP */

	PROTO_ERROR_SOCKET_FAILURE,				/**< Socket operation is failed */
	
	PROTO_ERROR_MAX					= 0x0FFF	/**< Set ProtoError with 2 bytes(See UINT16 protoError in ProtoPacket.h). */
} ProtoError;

/**
 * @brief	This type enumerates who triggers bearer start or stop. 
 */
typedef enum
{
	PROTO_INITIATED_BY_NONE			= 0x00,
	PROTO_INITIATED_BY_MOBILE,					/**< Initiated by Mobile. */
	PROTO_INITIATED_NETWORK					/**< Initiated by Network. */
} ProtoInitiateType;

/**
 * @brief	Network suspend reason indicated by QMocha. 
 */
typedef enum
{
	PROTO_SUSPEND_NONE				= -1,		/**< Not suspended */
	PROTO_SUSPEND_NO_SERVICE			= 0x0000,	/**< Suspended by no service */
	PROTO_SUSPEND_FLOW_CONTROL,					/**< Suspended by flow control */
	PROTO_SUSPEND_RAT_CHANGE,						/**< Suspended by RAT change */
	PROTO_SUSPEND_GPRS_HOLD,						/**< Suspended by GPRS hold(CSD is active) */
	PROTO_SUSPEND_MAX					= 0x0FFF		/**< Set ProtoSuspendReason with 2 bytes(See UINT16 suspendReason in ProtoPacket.h). */
} ProtoSuspendReason;

/**
 * @brief	This enumeration is defined only for PS_SVCTYPE 
 */
typedef enum 
{
	PROTO_NETWORK_NONE,			/**< Initial Value before searching network */
	PROTO_NETWORK_2G,				/**< 2G(GSM) network service */
	PROTO_NETWORK_2_5G,			/**< 2.5G(GPRS) network service */
	PROTO_NETWORK_3G				/**< 3G(UMTS) network service */
} ProtoServiceType;

typedef PACKED struct
{
	ProtoServiceType	serviceType;
	UINT32				newGrtUplinkRate;
	UINT32				newMaxUplinkRate;
	UINT32				newGrtDownlinkRate;
	UINT32				newMaxDownlinkRate;
} ProtoNetworkStatusInfo;

#ifdef _SHP_PROTO_MULTI_PS
typedef PACKED struct
{
	int				nUsePsd;
	int				nUseCsd;
} ProtoDataCallInfo;
#endif

typedef struct
{
	UINT16			type;
	UINT16			id;
	HEventHandler	hDestEventHandler;
	HSocket			hSocket;
	int				event;
	int				errorCode;
	ULONG			param;						/* Generic Parameter */
	HProtocol			hProtocol;
} ProtoEvent;

typedef struct
{
	UINT16			type;
	UINT16			id;
	HEventHandler	hDestEventHandler;
	int				event;
	UINT16			opMode;
	UINT16			protoType;
	int				contextID;
	HProtoAccount		hProtoAccount;
	ProtoError		protoError;
	ProtoSuspendReason	suspendReason;
} ProtoNetworkEvent;

/* It has the same elements that ProtoEvent has above, but it is for the FIFO interface by <ProtoPacket.c>. */
typedef PACKED struct
{
	UINT16			type;
	UINT16			id;
	HEventHandler	hDestEventHandler;
	HSocket			hSocket;
	int				event;
	int				errorCode;
	ULONG			param;						/* Generic Parameter */
} ProtoSocketEvent;

/**
 * @brief	
 */
typedef PACKED union
{
	IPV4Addr			ipv4;											/**< Specifies an IPv4 address type. This type has four bytes. */
	IPV6Addr			ipv6[PROTO_IPV6_ADDR_LEN_MAX];					/**< Specifies an IPv6 address type. Tis type as 16 bytes. */
	MSISDNAddr		msisdn[PROTO_MSISDN_ADDR_LEN_MAX*2+1];			/**< Used in SMS only. */
	ServiceCodeAddr	serviceCode[PROTO_SERVCODE_ADDR_LEN_MAX+1];	/**< Presents a service code used in GSM- USSD. */
	APN				apn[PROTO_APN_ADDR_LEN_MAX+1];					/**< Access point name used in GPRS APN */
	MSISDNAddr		szDialNumber[PROTO_DIAL_NUMBER_LEN_MAX+1];		/**< Pnone number in GSM CSD. Only digits and some special characters are allowed (e.g. '+').*/
} Address;

/**
 * @brief	This indicates type of Address of WAP Proxy & Local Address.
 */
typedef enum 
{
	PROTO_ADDRESS_IPV4				= 0x00,		/**< default. An IPv4 address [RFC791] represented in decimal format with dots as delimiters */
	PROTO_ADDRESS_IPV6,						/**< An IPv6 address [RFC2373] represented as hexadecimal numbers with colons as delimiters
												 or as a combination of hexadecimal and decimal numbers with dots and colons as delimiters*/
	PROTO_ADDRESS_E164,						/**< A phone number according to the E164 scheme [GENFORM] */
	PROTO_ADDRESS_ALPHA,						/**< Generic alphanumeric address (as defined by alphanum in [RFC2396]) */
	PROTO_ADDRESS_APN,							/**< A GPRS APN as defined in [GENFORM] */
	PROTO_ADDRESS_SCODE,						/**< A USSD service code as defined in [GENFORM] */
	PROTO_ADDRESS_TETRA_ITSI,					/**< A TETRA SDS address with digits in decimal format [WAPWDP] */
	PROTO_ADDRESS_MAN							/**< A Mobitex MAN address with digits in decimal format [WAPWDP] */
}ProtoAddressType;

/**
 * @brief	Many of the socket functions work in terms of this structure. 
 * This type includes the address type, port name and Internet address(Network, Bearer and Address Type) of a socket.
 */
typedef PACKED struct
{
	ProtoNBA			type;						/**< Address type */
	Address 			address;						/**< Address including IPv4, IPv6, MSISDN, USSD and APN */
	Port				port;						/**< Specifies a network port - 16-bit data. */
} Socket;

/* NAP(Network Access Point) Auth Type */
typedef enum
{
	PROTO_NAPAUTH_NONE			= 0x00,
	PROTO_NAPAUTH_PAP,
	PROTO_NAPAUTH_CHAP		
} ProtoNapAuthType;

/* NAP Auth Information, User ID and password */
typedef PACKED struct
{
	ProtoNapAuthType	type;
	char				szId[PROTO_AUTH_ID_LEN_MAX+1];
	char				szPasswd[PROTO_AUTH_PASSWD_LEN_MAX+1];
} ProtoNapAuthInfo;

/* In GPRS, the default QoS profile is defined in HLR. The SGSN and GGSN control QoS in GPRS, but mainly in the SGSN. */
/* Indicates the importance of the packet with regard to discarding it in case of problems and degradatioin of QoS when necessary. */
typedef enum
{
	PROTO_QOS_PRECED_CLASS_SUBSCRIBED	= 0x00,
	PROTO_QOS_PRECED_CLASS_HIGH,
	PROTO_QOS_PRECED_CLASS_NORMAL,
	PROTO_QOS_PRECED_CLASS_LOW,
	PROTO_QOS_PRECED_CLASS_RESERVED		= 0x0FFFFFFF
} ProtoQosPrecedClass;

/*  The transfer delay includes the uplink radio channel access or downlink radio channel scheduling delay, the radio channel transit delay, and GPRS network transit delay. */
typedef enum
{
	PROTO_QOS_DELAY_CLASS_SUBSCRIBED		= 0x00,
	PROTO_QOS_DELAY_CLASS_1,
	PROTO_QOS_DELAY_CLASS_2,
	PROTO_QOS_DELAY_CLASS_3,
	PROTO_QOS_DELAY_CLASS_4,
	PROTO_QOS_DELAY_CLASS_RESERVED		= 0x0FFFFFFF
} ProtoQosDelayClass;

/**
 * @brief	Specifies the mode of operation for various error detection and recovery protocols, how securely the data should be delivered. 
 */
typedef enum
{
	PROTO_QOS_REL_CLASS_SUBSCRIBED		= 0x00,
	PROTO_QOS_REL_CLASS_1,							/**< Acknowledged GTP, LLC, and RLC; Protected data */
	PROTO_QOS_REL_CLASS_2,							/**< Unacknowledged GTP; Acknowledged LLC and RLC, Protected data */
	PROTO_QOS_REL_CLASS_3,							/**< Unacknowledged GTP and LLC; Acknowledged RLC, Protected data */
	PROTO_QOS_REL_CLASS_4,							/**< Unacknowledged GTP, LLC, and RLC, Protected data */
	PROTO_QOS_REL_CLASS_5,							/**< Unacknowledged GTP, LLC, and RLC, Unprotected data */
	PROTO_QOS_REL_CLASS_RESERVED			= 0x0FFFFFFF
} ProtoQosRelClass;

/*  Defines the maximul allowed transfer rate. */
typedef enum
{
	PROTO_QOS_PEAK_TPUT_SUBSCRIBED		= 0x00,
	PROTO_QOS_PEAK_TPUT_1KB,
	PROTO_QOS_PEAK_TPUT_2KB,
	PROTO_QOS_PEAK_TPUT_4KB,
	PROTO_QOS_PEAK_TPUT_16KB,
	PROTO_QOS_PEAK_TPUT_32KB,
	PROTO_QOS_PEAK_TPUT_64KB,
	PROTO_QOS_PEAK_TPUT_128KB,
	PROTO_QOS_PEAK_TPUT_256KB,
	PROTO_QOS_PEAK_TPUT_RESERVED			= 0x0FFFFFFF
} ProtoQosPeakThroughput;

/*  Defines long term averate transfer rate. */
typedef enum
{
	PROTO_QOS_MEAN_TPUT_SUBSCRIBED		= 0x00,
	PROTO_QOS_MEAN_TPUT_100B,
	PROTO_QOS_MEAN_TPUT_200B,
	PROTO_QOS_MEAN_TPUT_1KB,
	PROTO_QOS_MEAN_TPUT_2KB,
	PROTO_QOS_MEAN_TPUT_5KB,
	PROTO_QOS_MEAN_TPUT_10KB,
	PROTO_QOS_MEAN_TPUT_20KB,
	PROTO_QOS_MEAN_TPUT_50KB,
	PROTO_QOS_MEAN_TPUT_100KB,
	PROTO_QOS_MEAN_TPUT_200KB,
	PROTO_QOS_MEAN_TPUT_500KB,
	PROTO_QOS_MEAN_TPUT_1MB,
	PROTO_QOS_MEAN_TPUT_2MB,
	PROTO_QOS_MEAN_TPUT_5MB,
	PROTO_QOS_MEAN_TPUT_10MB,
	PROTO_QOS_MEAN_TPUT_20MB,
	PROTO_QOS_MEAN_TPUT_50MB,
	PROTO_QOS_MEAN_TPUT_BEST,
	PROTO_QOS_MEAN_TPUT_RESERVED			= 0x0FFFFFFF
} ProtoQosMeanThroughput;

/*  The enum type for traffic class */
typedef enum
{
	PROTO_QOS_TRAFFIC_CLASS_SUBSCRIBED	= 0x00,
	PROTO_QOS_TRAFFIC_CLASS_CONVERSATION,
	PROTO_QOS_TRAFFIC_CLASS_STREAMING,
	PROTO_QOS_TRAFFIC_CLASS_INTERACTIVE,
	PROTO_QOS_TRAFFIC_CLASS_BACKGROUND,
	PROTO_QOS_TRAFFIC_CLASS_RESERVED		= 0x0FFFFFFF
} ProtoQosTrafficClass;

/*  The enum type for Maximum ul/dl bit rates  and gurarenteed ul/dl bit rates */
typedef enum
{
	PROTO_QOS_BIT_RATE_SUBSCRIBED			= 0x00,
	PROTO_QOS_BIT_RATE_0KB,
	PROTO_QOS_BIT_RATE_8KB,
	PROTO_QOS_BIT_RATE_16KB,
	PROTO_QOS_BIT_RATE_32KB,
	PROTO_QOS_BIT_RATE_64B,
	PROTO_QOS_BIT_RATE_128KB,
	PROTO_QOS_BIT_RATE_256KB,
	PROTO_QOS_BIT_RATE_1024KB,
	PROTO_QOS_BIT_RATE_2048KB,
	PROTO_QOS_BIT_RATE_RESERVED			= 0x0FFFFFFF
} ProtoQosMaxDlBitrate, ProtoQosMaxUlBitrate, ProtoQosGrtDlBitrate, ProtoQosGrtUlBitrate;

/*  The enum type for Max SDU size */
typedef enum
{
	PROTO_QOS_MAX_SDU_SIZE_SUBSCRIBED	= 0x00,
	PROTO_QOS_MAX_SDU_SIZE_1500			= 1500,
	PROTO_QOS_MAX_SDU_SIZE_RESERVED		= 0x0FFFFFFF
} ProtoQosMaxSDUSize;

/* The enum type for QOS Delivery Order Type */
typedef enum
{
	PROTO_QOS_DLVORDER_SUBSCRIBED		= 0x00,
	PROTO_QOS_DLVORDER_YES,
	PROTO_QOS_DLVORDER_NO,
	PROTO_QOS_DLVORDER_RESERVED			= 0x0FFFFFFF
} ProtoQosDlvOrder; 

/**
 * @brief	The enum type for QOS SDU Error Ratio  (per 3GPP TS24.008 10.5.6.5)
 */
typedef enum
{
	PROTO_QOS_SDU_ERR_SUBSCRIBED			= 0x00,	/**< Subscribed                  */
	PROTO_QOS_SDU_ERR_1E2,     						/**< 1E-2 */
	PROTO_QOS_SDU_ERR_7E3,							/**< 7E-3 */
	PROTO_QOS_SDU_ERR_1E3,							/**< 1E-3 */
	PROTO_QOS_SDU_ERR_1E4,							/**< 1E-4 */
	PROTO_QOS_SDU_ERR_1E5,							/**< 1E-5 */
	PROTO_QOS_SDU_ERR_1E6,							/**< 1E-4 */
	PROTO_QOS_SDU_ERR_1E1,							/**< 1E-1 */
	PROTO_QOS_SDU_ERR_RESERVED			= 0x0FFFFFFF
} ProtoQosSDUErrorRatio;

/**
 * @brief	The enum type for QOS RES BER 
 */
typedef enum
{
	PROTO_QOS_RES_BER_SUBSCRIBED			= 0x00,	/**< Subscribed                  */
	PROTO_QOS_RES_BER_5E2,							/**< 5E-2 */
	PROTO_QOS_RES_BER_1E2,							/**< 1E-2 */
	PROTO_QOS_RES_BER_5E3,							/**< 5E-3 */
	PROTO_QOS_RES_BER_4E3,							/**< 4E-3 */
	PROTO_QOS_RES_BER_1E3,							/**< 1E-3 */
	PROTO_QOS_RES_BER_1E4,							/**< 1E-4 */
	PROTO_QOS_RES_BER_1E5,							/**< 1E-5 */
	PROTO_QOS_RES_BER_1E6,							/**< 1E-6 */
	PROTO_QOS_RES_BER_6E8,							/**< 6E-8 */
	PROTO_QOS_RES_BER_RESERVED			= 0x0FFFFFFF
} ProtoQosResBer;

/**
 * @brief	The enum type for QOS Erroneous SDU Delivery Options 
 */
typedef enum
{
	PROTO_QOS_SDU_DLVR_SUBSCRIBED		= 0x00,	/**< Subscribed */
	PROTO_QOS_SDU_DLVR_NODETECT,      				/**< No detection */
	PROTO_QOS_SDU_DLVR_YES,      					/**< Erroneous SDU is delivered */
	PROTO_QOS_SDU_DLVR_NO,      						/**< Erroneous SDU not delivered*/
	PROTO_QOS_SDU_DLVR_RESERVED			= 0xFFFF	/**< force max to 0xffff so that  enum is defined as a word */
} ProtoQosSDUDlvOption;

/* The enum type for QOS Transfer delay */
typedef enum
{
	PROTO_QOS_TRANS_DELAY_SUBSCRIBED		= 0x00,
	PROTO_QOS_TRANS_DELAY_RESERVED		= 0x0FFFFFFF
} ProtoQosTransDelay;

/* The enum type for QOS Traffic priority */
typedef enum
{
	PROTO_QOS_TRAFFIC_PRIORITY_SUBSCRIBED	= 0x00,
	PROTO_QOS_TRAFFIC_PRIORITY_RESERVED	= 0x0FFFFFFF
} ProtoQosTrafficPriority;

/*  The enum type for call type */
typedef enum
{
	PROTO_CALLTYPE_ANALOG					= 0x00,	/* default */
	PROTO_CALLTYPE_ISDN,		
	PROTO_CALLTYPE_V_120,
	PROTO_CALLTYPE_V_110,
	PROTO_CALLTYPE_X_31,
	PROTO_CALLTYPE_RESERVED				= 0x0FFFFFFF
} ProtoCallType;

/*  The enum type for call speed */
typedef enum
{
	PROTO_CALLSPEED_AUTO					= 0x00,
	PROTO_CALLSPEED_9600,
	PROTO_CALLSPEED_14400,
	PROTO_CALLSPEED_19200,
	PROTO_CALLSPEED_57600,
	PROTO_CALLSPEED_RESERVED				= 0x0FFFFFFF
} ProtoCallSpeed;

/*  The struct type for Gprs Qos Info */
typedef PACKED struct
{
	ProtoQosPrecedClass		precedClass;
	ProtoQosDelayClass		delayClass;
	ProtoQosRelClass			relClass;
	ProtoQosPeakThroughput	peakThroughput;
	ProtoQosMeanThroughput	meanThroughput;
} ProtoGprsQosInfo;

/**
 * @brief	 The struct type for Cdma Qos Info 
 */
typedef PACKED struct
{
	UINT32       serviceOption;		/**< CDMA service option. See cm_call_cmd_info_cdma_s_type in <Cmi.h>. */
	UINT32       forwardMuxOption;	/**< Forward link multiplex option */
	UINT32       reverseMuxOption;	/**< Reverse link multiplex option */
	UINT32       nMSDormant;		/**< Dormant Time */
	BOOL         bBSDormant;		/**< BS Dormant Switch (True, False) */ 
} ProtoCdmaQosInfo;

/**
 * @brief	The structure type for the Umts QoS Information
 */
typedef PACKED struct
{
	ProtoQosPrecedClass		precedClass;		/**< Precedence class */
	ProtoQosDelayClass		delayClass;		/**< Delay class */
	ProtoQosRelClass			relClass;			/**< Reliability class */
	ProtoQosPeakThroughput	peakThroughput;	/**< Peak throughput */
	ProtoQosMeanThroughput	meanThroughput;	/**< Mean throughput */
	ProtoQosTrafficClass 		trafficClass;		/**< Traffic Class */
	ProtoQosMaxDlBitrate 		maxDlBitrate;		/**< Maximum UL bitrate */
	ProtoQosMaxUlBitrate		maxUlBitrate;		/**< Maximum DL bitrate */
	ProtoQosGrtDlBitrate		grtDlBitrate;		/**< Guaranteed UL bitrate */
	ProtoQosGrtUlBitrate		grtUlBitrate;		/**< Guaranteed DL bitrate */
	ProtoQosDlvOrder			dlvOrder;			/**< SDU delivery order */
	ProtoQosMaxSDUSize		maxSDUSize;		/**< Maximum SDU size */
	ProtoQosSDUErrorRatio		sduError;			/**< SDU error ratio list index */
	ProtoQosResBer			resBer;			/**< Residual biterr list index */
	ProtoQosSDUDlvOption		sduDlvOpt;		/**< Delivery of erroneous SDU*/
	ProtoQosTransDelay		transDelay;		/**< Transfer Delay */
	ProtoQosTrafficPriority		trafficPriority;		/**< Traffic handling priority */
} ProtoPsQosInfo;

/**
 * @brief	The structure type for cs qos info
 */
typedef PACKED struct
{
	BOOL					bTransparent;		/**< Transparent or nontransparent */
	ProtoCallType				callType;			/**< Call type */
	ProtoCallSpeed			callSpeed;		/**< Call speed */
} ProtoCsQosInfo;

/**
 * @brief	The structure type for the QoS Information that depends on the ProtoOpMode or ProtoNBA
 * Currently GPRS and CDA QoS are not considered.
 */
typedef PACKED struct
{
	PACKED union
	{
		ProtoGprsQosInfo		gprsQosInfo;		/**< GPRS QoS info */
		ProtoCdmaQosInfo		cdmaQosInfo;		/**< CDMA QoS info */
		ProtoPsQosInfo		psQosInfo;		/**< PS QoS info */
		ProtoCsQosInfo		csQosInfo;		/**< CS QoS info */
	} qosInfoUnion;
} ProtoNetQosInfo;

/**
 * @brief	Types for inactive type
 */
typedef enum
{
	PROTO_INACTIVE_DATA,					/**< No incomming/outgoing data */
	PROTO_INACTIVE_SOCEKT,				/**< No opened socket */
	PROTO_INACTIVE_MAX = 0x0FFF
} ProtoInactiveType;

/* Types for bearer interface type */
typedef enum
{
	PROTO_IFTYPE_DOMAIN				= 0x01,
	PROTO_IFTYPE_P2P,
	PROTO_IFTYPE_SOCKET,
	PROTO_IFTYE_PLUGIN
} ProtoInterfaceType;

/* Protocol Type ID, which supports filtering. */
typedef enum
{
	PROTO_APP_TYPE_NONE				= 0x01,
	PROTO_APP_TYPE_WAP					= 0x02,
	PROTO_APP_TYPE_HTTP				= 0x04,
	PROTO_APP_TYPE_OBEX				= 0x08,
	PROTO_APP_TYPE_ALL					= (PROTO_APP_TYPE_NONE|PROTO_APP_TYPE_WAP|PROTO_APP_TYPE_HTTP)
} ProtoAppType;

/**
 * @brief	Protocol network information; Each Handle can share a same network information.. 
 */
typedef struct
{
	BOOL					bState;				/**< TRUE if network is available, or FALSE otherwise.*/
	ProtoOpMode 				opMode;				/**< Specifiles an operation mode */
	ProtoType				protoType;			/**< Specifiles a protocol type*/
	HObj					hBearerAccount;		/**< Specific bearer account handle to access (WLAN, LAN, Bluetooth, or other wireless/wired bearer) */

	ProtoNBA 				nbaType;				/**< Specifies a network/bearer/address type.*/
	Address					napAddr;				/**< Network Access Point Name*/
	Address					napDialNumber;
	ProtoAddressType			napAddrType;			/**< Network Accress Point Name Type*/
	BOOL					bStaticIpAddr;			/**< Static or Dynamic IP Address*/
	Address					localAddr;			/**< Local address*/
	BOOL					bStaticDnsAddr;		/**< Static or Dynamic DNS Address*/
	Address					dnsAddr[PROTO_DNS_COUNT_MAX];					/**< Dns address for 2 entry*/
	Address					pcscfaddr;			/**< P-CSCF address*/

	ProtoNetQosInfo			qosInfo;				/**< Qos param*/
	ProtoNapAuthInfo			authInfo;				/**< NAP Auth param*/

	ProtoAppType				protoAppType;		/**< Application protocol type(WAP, HTTP, OBEX etc)*/
	char						szGatewayAddr[PROTO_IPV4_STR_LEN_MAX+1];	/**< Gateway address with a port number for WAP*/
	char						szProxyAddr[PROTO_IPV4_STR_LEN_MAX+1];		/**< Proxy address with a port number for HTTP*/
	char						szHomeURL[PROTO_HOME_URL_LEN_MAX+1];	/**< Browser Home URL or MMS server URL*/
	BOOL					bSecureConnection;	/**< Secure connection which uses WTLS for WAP*/
	BOOL					bInactiveData;		/**< inactive Data*/
	ULONG					inactiveDataTime;		/**< inactive Data time*/
	BOOL					bInactiveSocket;		/**< inactive Socket */
	ULONG					inactiveSocketTime;	/**< inactive Socket time*/

	char						szAccountName[PROTO_NETINFO_NAME_LEN_MAX+1];	/**< Account name*/
	char						szAppName[PROTO_APP_NAME_LEN_MAX+1];			/**< Application name*/
} ProtoNetInfo;

/**
 * @brief	Mobile Station network infomation to send to or receive from the wireless stack. 
 */
typedef PACKED struct
{
	Address				napAddr;				/**< Network Access Point Name */
	Address				napDialNumber;		/**< DialNumber*/
	HProtoAccount			hProtoAccount;
	BOOL				bStaticIpAddr;			/**< Static or Dynamic Address*/
	BOOL				bValidLocalAddr;		/**< Valid local IP address?*/
	IPV4Addr				localAddr;			/**< Local address*/
	BOOL				bStaticDnsAddr;		/**< DNS present?*/
	BOOL				bValidDnsAddr1;		/**< Valid primary DNS address?*/
	IPV4Addr				dnsAddr1;			/**< Primary DNS address*/
	BOOL				bValidDnsAddr2;		/**< Valid secondary DNS address?*/
	IPV4Addr				dnsAddr2;			/**< Secondary DNS address*/
	ProtoNetQosInfo		qosInfo;				/**< Quqlity Of Service param*/
	ProtoNapAuthInfo		authInfo;				/**< NAP Auth param*/
} ProtoStartNetworkInfo;

/* Modem Service State */
typedef enum
{
	PROTO_MODEMSERVICE_PULLEDOUT		= 0x00,
	PROTO_MODEMSERVICE_PLUGGEDIN,
	PROTO_MODEMSERVICE_CONNECTING,
	PROTO_MODEMSERVICE_CONNECTED,
	PROTO_MODEMSERVICE_DISCONNECTING,
	PROTO_MODEMSERVICE_DISCONNECTED
} ProtoModemServiceState;

/* Modem Service Info */
typedef PACKED struct
{
	PhoneStatusDevType		devType;
	ProtoOpMode				opMode;
	PhsPcBrowsingType	 	serviceState;
} ProtoModemServiceInfo;

/**
 * @brief	 PS/CS data call statistics 
 */
typedef struct
{
	UINT32 	lastCallDuration;					/**< Last call time in seconds */
	UINT32 	lastSentDataSize;					/**< Last sent data size in bytes */
	UINT32 	lastRcvdDataSize;					/**< Last received data size in bytes */
	UINT32 	totalCallDuration;					/**< Total call time in seconds*/
	UINT32 	totalSentDataSize;					/**< Total send data size in bytes */
	UINT32 	totalRcvdDataSize;					/**< Total received data size */
	BOOL	bContinuous;					/**< specify this connection establish time. */
} ProtoStatInfo;

/**
 * @brief	Type of data call statistics info 
 */
typedef enum
{
	PROTO_STAT_ALL						= 0x00,		/**< all statistics types*/
	PROTO_STAT_CALL_DURATION,						/**< Call duration (Last + Total)*/
	PROTO_STAT_SENT_SIZE,								/**< Sent size (Last + Total)*/
	PROTO_STAT_RCVD_SIZE,								/**< Received size (Last + Total)*/
	PROTO_STAT_LAST_CALL_DURATION,					/**< Call duration	(Last)*/
	PROTO_STAT_LAST_SENT_SIZE,						/**< Sent size (Last)*/
	PROTO_STAT_LAST_RCVD_SIZE,						/**< Received size (Last)*/
	PROTO_STAT_TOTAL_CALL_DURATION,					/**< Call duration (Total)*/
	PROTO_STAT_TOTAL_SENT_SIZE,						/**< Sent size (Total)	*/
	PROTO_STAT_TOTAL_RCVD_SIZE						/**< Received size (Total)	*/
} ProtoStatType;

/**
 * @brief	Protocol event class to specify the client's usage 
 */
typedef enum
{
	PROTO_BEARER_CLASS_NONE			= 0x00,
	PROTO_EVENT_CLASS_BEARER		= 0x01,			/**< Client is used for bearer management. */
	PROTO_EVENT_CLASS_MODEM			= 0x02,			/**< Client is used for modem service management. */
} ProtoEventClass;

/* Context state */
typedef enum
{
	PROTO_CONTEXT_STATE_NULL			= 0x00,
	PROTO_CONTEXT_STATE_USED,
	PROTO_CONTEXT_STATE_CLOSING
} ProtoContextState;

/**
 * @brief	This structure manages context ID and its index.
 */
typedef struct
{
	ProtoContextState		contextState;			/**< Context information state */
	ProtoState			protoState;			/**< Protocol state defined in ProtoState. */
	HProtoAccount			hProtoAccount;		/**< Protocol account handle */
	int					contextID;			/**< Each entity has an unique context ID. */
	BOOL				bServiced;			/**< Is serviced or not? */
	HTimer				hContextTimer;		/**< Each context has each timer to process start/stop timeout. */
	ProtoError			protoError;			/**< Protocol error defined in ProtoError */
	ProtoSuspendReason	suspendReason;		/**< Suspend reason defined in ProtoSuspendReason */

	UINT32				lastStartTime;			/**< Time that this context is started */
	UINT32				lastStopTime;			/**< Time that this context is stopped */
	UINT32 				lastSentDataSize;		/**< Last sent data size in bytes */
	UINT32 				lastRcvdDataSize;		/**< Last received data size in bytes */

	BOOL				bInactiveData;			/**< inactive Data */
	ULONG				inactiveDataLastTime;	/**< inactive Data time */
	BOOL				bInactiveSocket;			/**< inactive Socket */
	ULONG				inactiveSocketLastTime;	/**< inactive Socket time */

	ProtoNetInfo			netInfo;				/**< Each context has each network information. */
	ProtoNetworkStatusInfo	netStatusInfo;
} ProtoContextInfo;

/* User defined bearer indication callback */
typedef void (*ProtoNetworkCallback)(ProtoEvent* pProtoEvent);

/* Client state */
typedef enum
{
	PROTO_CLIENT_STATE_NULL			= 0x00,
	PROTO_CLIENT_STATE_OPENING,
	PROTO_CLIENT_STATE_USED,
	PROTO_CLIENT_STATE_CLOSING,
	PROTO_CLIENT_STATE_CLOSED
} ProtoClientState;

/**
 * @brief	Client information to receive the bearer related information;e.g., start or stop. 
 */
typedef struct
{
	ProtoClientState		clientState;				/**< Client state in ProtoClientState */
	ProtoEventClass		eventClass;				/**< The usage of this client */
	int					contextInfoIndex;		/**< Specifies the ProtoContextInfo's zero-based index. */
	HProtoAccount			hProtoAccount;			/**< Specifies the Account handle */
	HEventHandler		hEventHandler;			/**< Each client should one of the event handler or callback function. */
	ProtoNetworkCallback	callback;					/**< User callback function to receive network information. */
	DID					did;						/**< DLL ID */
	char					szAppName[PROTO_APP_NAME_LEN_MAX+1];	/**< Application name */
	BOOL				bBound[PROTO_CLIENT_COUNT_MAX];		/**< Bind handle per each protocol handle */
	HProtoAccount			hBoundProtoAccount;		/**<  Bind Account handle per each handle */
	int					boundContextInfoIndex;			/**< 20070705, Youna, Bind CII per each handle */

} ProtoClientInfo;


/**
 * @brief	 Network events used for test only with TkShell 
 */
typedef enum
{
	PROTO_INJECT_EVENT_NONE						= 0x00,
	PROTO_INJECT_EVENT_NETWORK_START_REQIND,			/**< __ProtoStartReqNetworkInd*/
	PROTO_INJECT_EVENT_NETWORK_START_CNF,			/**< __ProtoStartNetworkCnf*/
	PROTO_INJECT_EVENT_NETWORK_START_IND,			/**< __ProtoStartNetworkInd*/
	PROTO_INJECT_EVENT_NETWORK_STOP_CNF,				/**< __ProtoStopNetworkCnf*/
	PROTO_INJECT_EVENT_NETWORK_STOP_IND,				/**< __ProtoStopNetworkInd*/
	PROTO_INJECT_EVENT_NETWORK_SUSPEND_IND,			/**< __ProtoSuspendNetworkInd*/
	PROTO_INJECT_EVENT_NETWORK_RESUME_IND,			/**< __ProtoResumeNetworkInd*/
	PROTO_INJECT_EVENT_SOCKET_READ,					/**< Socket read event - SOCKET_EVENT_READ*/
	PROTO_INJECT_EVENT_SOCKET_WRITE,					/**< Socket write event - SOCKET_EVENT_WRITE*/
	PROTO_INJECT_EVENT_SOCKET_ACCEPT,				/**< Socket accept event - SOCKET_EVENT_ACCEPT*/
	PROTO_INJECT_EVENT_SOCKET_CONNECT,				/**< Socket connect event - SOCKET_EVENT_CONNECT*/
	PROTO_INJECT_EVENT_SOCKET_CLOSE,					/**< Socket close event - SOCKET_EVENT_CLOSE*/
	PROTO_INJECT_EVENT_MODEM_SERVICE,					/**< Modem service event for the PC Browsing*/
	PROTO_INJECT_EVENT_PCSYNC_SERVICE,					/**< PC Sync service event for the PC Browsing*/
	PROTO_INJECT_EVENT_DATACALL_SERVICE				/**< Data call service event for the PC Browsing*/
} ProtoInjectEvent;

/*
 * Event Definition
 */
#define PROTO_BEARER_NONE			(EVGRP_PROTO_BEGIN + 0x00)
#define PROTO_BEARER_STARTING		(EVGRP_PROTO_BEGIN + 0x01)
#define PROTO_BEARER_STARTED			(EVGRP_PROTO_BEGIN + 0x02)
#define PROTO_BEARER_STOPPING		(EVGRP_PROTO_BEGIN + 0x03)
#define PROTO_BEARER_STOPPED			(EVGRP_PROTO_BEGIN + 0x04)
#define PROTO_BEARER_SUSPEND		(EVGRP_PROTO_BEGIN + 0x05)
#define PROTO_BEARER_RESUME		(EVGRP_PROTO_BEGIN + 0x06)
#define PROTO_BEARER_NETSTATUSUPDATED	(EVGRP_PROTO_BEGIN + 0x07)
#define PROTO_BEARER_AVAILABLE	(EVGRP_PROTO_BEGIN + 0x08)
#define PROTO_BEARER_UNAVAILABLE	(EVGRP_PROTO_BEGIN + 0x09)

#define PROTO_MODEM_SERVICE		(EVGRP_PROTO_BEGIN + 0x0A)
#define PROTO_PCSYNC_SERVICE		(EVGRP_PROTO_BEGIN + 0x0B)
#define PROTO_DATACALL_SERVICE		(EVGRP_PROTO_BEGIN + 0x0C)
#define PROTO_CONNECTIVITY_SERVICE	(EVGRP_PROTO_BEGIN + 0x0D)

#define PROTO_NETWORK_EVENT		(EVGRP_PROTO_BEGIN + 0x10)
#define PROTO_SOCKET_ASYNC		(EVGRP_PROTO_BEGIN + 0x11)
#define PROTO_HTTP_ASYNC			(EVGRP_PROTO_BEGIN + 0x12)						   
#define PROTO_WAP_ASYNC			(EVGRP_PROTO_BEGIN + 0x13)
#define PROTO_DNS_ASYNC			(EVGRP_PROTO_BEGIN + 0x14)
#define PROTO_CONN_ASYNC			(EVGRP_PROTO_BEGIN + 0x15)
#define PROTO_MMS_ASYNC			(EVGRP_PROTO_BEGIN + 0x16)
#define PROTO_DIAG_ASYNC			(EVGRP_PROTO_BEGIN + 0x17)
#define PROTO_HTTP_ADDCHUNK		(EVGRP_PROTO_BEGIN + 0x18)
#define PROTO_WAP_PUSH_ASYNC		(EVGRP_PROTO_BEGIN + 0x19)
#define PROTO_IP_PUSH_ASYNC			(EVGRP_PROTO_BEGIN + 0x1A)
#define PROTO_DNS_REQUEST			(EVGRP_PROTO_BEGIN + 0x1B)
#define PROTO_SERVER_ASYNC			(EVGRP_PROTO_BEGIN + 0x1C)

#define PROTO_WLAN_EVENT			(EVGRP_PROTO_BEGIN + 0x23)
#define PROTO_PDG_EVENT			(EVGRP_PROTO_BEGIN + 0x24)
#define PROTO_NDSIP_EVENT			(EVGRP_PROTO_BEGIN + 0x25)

/* Event type for Connection Manager */
#define PROTO_CONN_MGR				(EVGRP_PROTO_BEGIN + 0x40)
/* This is used by applications to receive the socket event. */
#define PROTO_SOCKET_USER			(EVGRP_PROTO_BEGIN + 0x50)
/* Added by Sanghamitra - WTLS callback */ // FIXME. WTLS must be defined.
#define	PROTO_WTLS_CERT_VERIFY_CALLBACK		(EVGRP_PROTO_BEGIN + 0x51)

#define DECLARE_HANDLE_PROTO_BEARER_STARTING(eventHandler)			BOOL eventHandler(WmState* pState, WmEvent* pEvent)
#define DECLARE_HANDLE_PROTO_BEARER_STARTED(eventHandler)			BOOL eventHandler(WmState* pState, WmEvent* pEvent)
#define DECLARE_HANDLE_PROTO_BEARER_STOPPING(eventHandler)			BOOL eventHandler(WmState* pState, WmEvent* pEvent)
#define DECLARE_HANDLE_PROTO_BEARER_STOPPED(eventHandler)			BOOL eventHandler(WmState* pState, WmEvent* pEvent)
#define DECLARE_HANDLE_PROTO_BEARER_SUSPEND(eventHandler)		BOOL eventHandler(WmState* pState, WmEvent* pEvent)
#define DECLARE_HANDLE_PROTO_BEARER_RESUME(eventHandler)		BOOL eventHandler(WmState* pState, WmEvent* pEvent)
#define DECLARE_HANDLE_PROTO_MODEM_SERVICE(eventHandler)		BOOL eventHandler(WmState* pState, WmEvent* pEvent)

/* Event map macros */
#define HANDLE_PROTO_BEARER_STARTING(eventHandler)					HANDLE_EVENT(PROTO_BEARER_STARTING, eventHandler)
#define HANDLE_PROTO_BEARER_STARTED(eventHandler)					HANDLE_EVENT(PROTO_BEARER_STARTED, eventHandler)
#define HANDLE_PROTO_BEARER_STOPPING(eventHandler)					HANDLE_EVENT(PROTO_BEARER_STOPPING, eventHandler)
#define HANDLE_PROTO_BEARER_STOPPED(eventHandler)					HANDLE_EVENT(PROTO_BEARER_STOPPED, eventHandler)
#define HANDLE_PROTO_BEARER_SUSPEND(eventHandler)				HANDLE_EVENT(PROTO_BEARER_SUSPEND, eventHandler)
#define HANDLE_PROTO_BEARER_RESUME(eventHandler)					HANDLE_EVENT(PROTO_BEARER_RESUME, eventHandler)
#define HANDLE_PROTO_MODEM_SERVICE(eventHandler)				HANDLE_EVENT(PROTO_MODEM_SERVICE, eventHandler)

/* Define to compile async dns support */
// WLAN interface
#include "ProtoWLAN.h"
// PDG interface
#include "ProtoPDG.h"

#include "OemProtoMgr.h"



/*
 * Access point interface
 */
#include "ProtoAccount.h"

/*
 * ECOM Interface
 */
#include "ProtoECOM.h"

#ifdef __cplusplus
}
#endif  /* __cplusplus */

#endif // _PROTO_MGR_H_

