/** 
* Samsung Handset Platform
* Copyright (c) 2006 Samsung Electronics, Inc.
* All rights reserved. 
*
* This software is the confidential and proprietary information
* of Samsung Electronics, Inc. ("Confidential Information").  You
* shall not disclose such Confidential Information and shall use
* it only in accordance with the terms of the license agreement
* you entered into with Samsung Electronics.
*/
/*
*  This file defines the porting layer API's for the BAL framework.
*  @author Naveen Kumar SG(naveensg@samsung.com)
*  @author Sneha Bhat(sneha.bhat@samsung.com)
*/

#ifndef _SFBAL_H
#define _SFBAL_H

//#include "SFBalMain.h"
#include "SFBalDef.h"

#ifdef __cplusplus
extern "C" {
#endif /* __cplusplus */

/****************************************************************************************************/
/*									Memory API's										*/
/**************************************************************************************************/ 

/**
*Function Name              :BalMemAlloc       
*Function Description      :This function returns a pointer to a block of memory of at least the number of bytes 
*                                    specified by the Size parameter.
*			             
*@param              memSize             [in] Specifies the size, in bytes, of memory to allocate
*
*/
#ifndef SFB_DEBUG_MEMORY
void*
SFBalMemAllocEx(UINT32 memSize);

#define SFBalMemAlloc MemAlloc

void*
SFBalMemAllocWithPreCheck(UINT32 memSize);

/**
*Function Name              :SFBalMemFree       
*Function Description      :The function frees block of memory specified .
*			             
*@param             memPtr             [in] Specifies the location to be freed
*
*/
void
SFBalMemFree(void *memPtr);


/**
*Function Name              :SFBalMemCalloc       
*Function Description      :This function allocates memory and then initializes it .
*
*@param             count                  [in] Specifies the no elements for which the size has to be allocated.
*@param             memSize            [in] Specifies the size, in bytes, of memory to allocate
*
*/
void*
SFBalMemCallocEx(UINT32 count , UINT32 memSize);

#define SFBalMemCalloc MemCalloc

/**
*Function Name              :SFBalMemRealloc       
*Function Description      :This function shall change the size of the memory object pointed to by ptr to the size specified by size.
*
*@param             memPtr              [in] Specifies the location whose size has to be reallocated.
*@param             memSize            [in] Specifies the size, in bytes, of memory to allocate.
*
*/
void*
SFBalMemReallocEx(void *memPtr , UINT32 memSize);
#define SFBalMemRealloc MemReAlloc

/** This API checks for SHP platfom memory status and returns TRUE if it is running out of memory*/
BOOL
SFBalCheckPlatformOOM();

#else
	#ifdef __cplusplus
extern "C" {
#endif /* __cplusplus */

void*
SFBalMemAllocDebug(UINT32 memSize, char *pcFileName,UINT32 uiLineNumber);

void*
SFBalMemCallocDebug(UINT32 count , UINT32 memSize, char* pcFileName, UINT32 uiLineNumber);

void*
SFBalMemReallocDebug(void *memPtr , UINT32 memSize, char* pcFileName, UINT32 uiLineNumber);

void
SFBalMemFreeDebug(void *memPtr, char* pcFileName, UINT32 uiLineNumber);

#define SFBalMemAlloc(uiSize)				SFBalMemAllocDebug(uiSize, __FILE__, __LINE__)
#define SFBalMemFree(pvPtr)					SFBalMemFreeDebug(pvPtr, __FILE__, __LINE__)
#define SFBalMemCalloc(uiCount , uiSize)	SFBalMemCallocDebug(uiCount , uiSize, __FILE__, __LINE__)
#define SFBalMemRealloc(pvPtr , uiSize)		SFBalMemReallocDebug(pvPtr , uiSize, __FILE__, __LINE__)
#ifdef __cplusplus
} //extern "C"
#endif /* __cplusplus */
#endif
/****************************************************************************************************/
/*									File System API's										*/
/****************************************************************************************************/ 

/**
*Function Name              :SFBalFileCreate       
*Function Description      :This function shall create a file with the specified name and in the specified mode.
*
*@param             fileName          [in] Specifies the filename 
*@param             mode              [in] Specifies the mode in which it has to be opened
*
*/

int SFBalMemGetStatEx();

BalFile
SFBalCreateFile(const char* fileName , BalFileMode mode);


/**
*Function Name              :SFBalFileOpen       
*Function Description      :This function shall open a file with the specified name and in the specified mode.
*
*@param             fileName          [in] Specifies the filename 
*@param             mode              [in] Specifies the mode in which it has to be opened.
*
*/
BalFile
SFBalOpenFile(const char * filename, UINT32 mode); 


/**
*Function Name              :SFBalFileClose       
*Function Description      :This function shall close a file with the specified name.
*
*@param             fd          [in] Specifies the file pointer which refers to the file that has to be closed.
*
*/
BOOL
SFBalCloseFile(BalFile fd);


/**
*Function Name              :SFBalFileWrite       
*Function Description      :This function writes a block of data to the stream.
*
*@param             fd          [in] Specifies the file pointer which refers to the file that has to be written.
*@param             buf        [in] Specifies the buffer which has to be written to the file
*@param             size       [in] Specifies the size of data that has to be written.
*
*/
BOOL 
SFBalWriteFile(BalFile fd,  void * buf, UINT32 size);


/**
*Function Name              :SFBalFileRead       
*Function Description      :This function copies bufLen items of data  from the named input stream into an array pointed to by readBytes
*
*@param             fd        	      [in] Specifies the file pointer which refers to the file that has to be read.
*@param             buf                 [in] Specifies the buffer which has to be written to the file.
*@param             bufLen            [in] Specifies the length of the buffer that had to be read.
*@param             readBytes       [out] Specifies the location where the data has to be stored.
*
*/
BOOL
SFBalReadFile(BalFile fd , void* buf , UINT32 bufLen , UINT32 *readBytes);


/**
*Function Name              :SFBalFileRemove       
*Function Description      :This function remove the specified file.
*
*@param             fileName        	  [in] Specifies the file name to be removed.
*
*/
BOOL
SFBalRemoveFile(const char* fileName);


/**
*Function Name              :SFBalFileSeek       
*Function Description      :This function remove the specified file.
*
*@param             fd        	       [in] Specifies the file pointer .
*@param             position           [in] an integer which specifies the origin position
*@param             offset              [in] Specifies the number of bytes from where the position indicator should be placed
*
*/
BOOL
SFBalSeekFile(BalFile fd, INT32 position , INT32 offset);


/**
*Function Name              :SFBalFileTell       
*Function Description      :
*
*@param             fd         [in] Specifies the file pointer .
*
*/
INT32
SFBalTellFile(BalFile fd);


/**
*Function Name              :SFBalFileGetAttributes       
*Function Description      :This function reads file information such as file size, attribute, and creation date.
*
*@param             fd         [in] Specifies the file pointer .
*@param             attr       [out] Points to the memory area that receives the file-attribute data.
*/
BOOL
SFBalGetFileAttributes(const char* fileName , SFBalFileAttributes* attr);

/**
*Function Name              :SFBalIsFileExist       
*Function Description      :This function true if the file exists.
*
*@param             fileName         [in] Specifies the filename .
*/
BOOL
SFBalIsFileExist(const char* fileName);


/**
*Function Name              :SFBalFileGetSize       
*Function Description      :Returns the file size
*
*@param             fileName         [in] Specifies the filename .
*/
UINT32
SFBalGetFileSize(const char* fileName);
/*GET EXTN FROM URL OR FROM FILEPATH*/


/**
*Function Name              :SFBalGetFileExt       
*Function Description      :Returns the file extension
*
*@param             szPathName         [in] Specifies the path .
*@param             pszExt                 [out] contains the extension of the file
*@param             nBufLen               [in] 
*/
void
SFBalGetFileExt(const char* szPathName, char* pszExt, int nBufLen);


/**
*Function Name              :SFBalGetFileName       
*Function Description      :Returns the file name
*
*@param             szPathName         [in] Specifies the path .
*@param             pszName              [out] contains the name of the file
*@param             nBufLen               [in] 
*/
void
SFBalGetFileName(const char* szPathName, char* pszName, int nBufLen);


/**
*Function Name              :SFBalFileRename       
*Function Description      :Renames the fpecified file with the new name given
*
*@param             oldFileName         [in] Specifies the filename that has to be changed .
*@param             newFileName       [out] new file name
*/
BOOL
SFBalRenameFile(const char* oldFileName , const char* newFileName);


/**
*Function Name              :SFBalFileCopy       
*Function Description      :This function copies an existing file to a new file
*
*@param             srcFileName         [in] Specifies the source filename  .
*@param             dstFileName         [out] Specifies the source filename
*@param             ifExist                  [in] 
*/
BOOL
SFBalCopyFile(const char* srcFileName , const char* dstFileName, BOOL ifExist);



void
SFBalFmMakeTempName(SFBalFmStorageType type, const char* szExt, char* pName );

/*void
SFBalFmMakeTempNameWithName( BrPlFmStorageType type, const char* szName, const char* szExt, char* pName );

void
SFBalFmGetNameFromFolderPath(const char* szFilePath, char* szName);

void
SFBalFmGetFolderFromPath(const char* szFilePath, char* szFolderName);

void
SFBalFmGetFolderFromPathEx(const char* szFilePath, char* szFolderName);*/

void
SFBalFmGetNameFromPath(const char* szFilePath, char* szName);

void
SFBalFmGetNameWithoutExtFromPathEx(const char* szFilePath, char* szName);

void
SFBalFmGetNameFromPathEx(const char* szFilePath, char* szName);

void
SFBalFmGetActualPathFromFile(const char* szFilePath, SFBalFmStorageType storageType, char* szActualPath);

int
SFBalExtractNameFromURL( const char* szURL, char* szName, int iBufSize );

int
SFBalGetValidFilename( char* szDest, char* szSrc, int destSize );

//BOOL SFBalIsValidPathName( char* szSrc );

int SFBalGetLastError();


/****************************************************************************************************/
/*									Directories API's										*/
/****************************************************************************************************/ 

/**
*Function Name              :SFBalDirCreate       
*Function Description      :This function creates a directory with the specified name.
*
*@param             dirName         [in] Specifies the directory name to be created .
*/
BOOL
SFBalCreateDir(const char* dirName);

/**
*Function Name              :SFBalDirOpen       
*Function Description      :This function opens a directory with the specified name.
*
*@param             dirName         [in] Specifies the directory name to be opened.
*/
BalDir
SFBalOpenDir(const char* dirName);


/**
*Function Name              :SFBalDirClose       
*Function Description      :This function closes a directory with the specified name.
*
*@param             hDir         [in] Specifies the directory to be closed.
*/
BOOL
SFBalCloseDir(BalDir hDir);

/**
*Function Name              :SFBalDirRemove       
*Function Description      :This function removes a directory with the specified name.
*
*@param             dirName         [in] Specifies the directory name to be removed.
*/
BOOL
SFBalRemoveDir(const char* dirName);

/**
*Function Name              :SFBalDirRemoveRecursive       
*Function Description      :
*
*@param             dirName         [in] Specifies the directory name to be removed.
*/
void
SFBalRemoveRecursiveDir(const char* dirName);

/**
*Function Name              :SFBalDirRead       
*Function Description      :
*
*@param             hDir    [in]
*/
BOOL
SFBalReadDir(BalDir  hDir, SFBalDirEntry* entry);


BOOL
SFBalFmIsExistDir(const char* szDirName);

BOOL
SFBalFmDeleteDir(const char* szDirName);

/****************************************************************************************************/
/*									Time related API's										*/
/****************************************************************************************************/ 

/**
*Function Name              :SFBalGetTime       
*Function Description      :This function returns the current time since 1/1/BASE_YEAR UTC in seconds
*
*@param            None   
*/
UINT32
SFBalGetTime();


/**
*Function Name              :SFBalGetTimeUTC       
*Function Description      :This function returns the current time since 1/1/BASE_YEAR UTC in seconds
*
*@param            None   
*/
UINT32
SFBalGetTimeUTC();



/****************************************************************************************************/
/*									         Timer									*/
/****************************************************************************************************/ 

/*Handle for the timer*/



#define	SFBalCreateTimer(name)											SFBalCreateTimer_macro(name)			
#define   SFBalDeleteTimer(BalTimer)										SFBalDeleteTimer_macro(BalTimer)
#define   SFBalStartTimer(BalTimer, timeout, timerCallback, param) 			SFBalStartTimer_macro(BalTimer, timeout, timerCallback, param)
#define   SFBalPauseTimer(BalTimer)										SFBalPauseTimer_macro(BalTimer)
#define   SFBalIsTimerActive(BalTimer)									SFBalIsTimerActive_macro(BalTimer)		 
#define   SFBalDateTimeToSeconds										SFBalDateTimeToSeconds_macro	
#define   SFBalSecondsToDateTime										SFBalSecondsToDateTime_macro
#define	SFBalDateToDays										 		SFBalDateToDays_macro     
#define	SFBalTimerCallback												SFBalTimerCallback_macro	 



/****************************************************************************************************/
/*									Event Management API's										*/
/****************************************************************************************************/ 


/**
*Function Name              :SFBalPostEvent       
*Function Description      :This function posts an event.
*
*@param            None   
*/
BOOL
SFBalPostEvent();



/****************************************************************************************************/
/*									Logging/Debugging related										*/
/****************************************************************************************************/ 

/*Code can be found at SFBalDef.h. 
For deugging Webcore modules use DI_WEBCORE, or for core modules like cache, RM etc use DI_BROWSER
For Exception errors like failure cases use BAL_EXCEPTION
*/
void
SFBalDebug( UINT32 code, const char* pDebugMsg, ... );

void
SFBalDebugEx( UINT32 code, const char* pDebugMsg, ... );

void
SFBalDebugPrint(UINT32 code, const char* pDebugMsg);

void
SFBalAssertDebug(const char* pDebugMsg, ...);

void
SFBalPrintDumpData(const void* in_svData, int in_dataLen);

void
SFBalDebugOpenScreenShot(void);

void
SFBalDebugCloseScreenShot(void);

void
SFBalDebugSaveScreenShot(void);

void
SFBalDebugTrigerScreenShot(void);

char * SFBalGetUserAgent(); // Basavaraj - 1st Apr 08

char * SFBalGetAppName(); // Romit
char * SFBalGetAppVersion(); // Romit


#define SFBalRequireEx	SFBalRequireEx_macro


/**
 * Initialize BAL layer
 */
BOOL
SFBalInitialize();

/**
 * Finalize BAL layer
 */
BOOL
SFBalFinalize();

char* SFBBalGetMimeFromExt(const char* szExt);

BOOL SFBalIsUTF8(const char * szPathName);

INT32 SFBalIsValidEntryName(const char * szDirectoryEntryName);

BOOL SFBalIsValidPathDepth(const char * szPathName, SFBalFileType entryType, UINT32 * uPathDepth);

BOOL SFBalIsValidPathName(const char * szPathName, SFBalFileType entryType);

// Zafar ECOM
char* 		SFBStrdup(const char *pszStr);
char* 		SFBItoa(int n, char* str, int b);
int 		SFBStrnicmp(const char* dst, const char* src, size_t n);
int 		SFBIsspace(int ch);
ULONG		SFBSysGetTick();
BOOL		SFBalIsLcdHorizontal(int ith);
Diagnose	SFBalGetDiagnoseInfo(int iComp);
int			SFBalGetDebugLevel (void);
BOOL		SFBalSetEditFieldInputTypeCategory( int hWin, int inputTypeCategory, BOOL bEnable );


BOOL     SFBalSetEditFieldInputAlphabetMode( int hWin, int  alphabetMode);
BOOL     SFBalGetEditFieldInputAlphabetMode( int hWin, int * pAlphabetMode);
BOOL     SFBalGetEditAreaInputAlphabetMode( int hWin, int * pAlphabetMode);
BOOL	SFBalWsSetVeEnableCR(int hWin , int bUseCR );

#ifdef __cplusplus
} //extern "C"
#endif /* __cplusplus */

#endif


