/* 
 * Copyright (c) 2007 SISO, Samsung Electronics, Inc.
 * All rights reserved.
 *
 * This software is the confidential and proprietary information
 * of Samsung Electronics, Inc. ("Confidential Information").  You
 * shall not disclose such Confidential Information and shall use
 * it only in accordance with the terms of the license agreement
 * you entered into with Samsung Electronics. 
 *
 */
 /*
  /* FILE :
 *  This file defines the porting layer API's for the the components ProtoMgr(Data Call management).
 *
 * @name		SFBalRmProto.h [SHP PROTO MGR interface]
 * @author		Ancil K George (ancilgeorge@samsung.com)
				Basavaraj P S (basavaraj.ps@samsung.com)
				Anshul Saxena (a.saxena@samsung.com)
				Rohit P Shetty (rohit.shetty@samsung.com)
 */
#ifndef _SFBAL_RMPROTO_H_
#define _SFBAL_RMPROTO_H_

	/*INCLUDES*/
//#include <ProtoMgr.h>
#include "SFBalMain.h"
	/*MACROS*/ 

#ifdef __cplusplus
extern "C" {
#endif /* __cplusplus */


#define	SFB_PROTO_CONTEXT_COUNT_MAX	PROTO_CONTEXT_COUNT_MAX

	/*DATA TYPES*/

typedef enum
{
	SFB_NETWORK_STATE_STOPPED,
	SFB_NETWORK_STATE_STOPPING,
	SFB_NETWORK_STATE_STARTING,
	SFB_NETWORK_STATE_STARTED,
	SFB_NETWORK_STATE_DISCONNECT_OTHER,	//sumdra 2008.06.18 Need for handling case other app is already using PDP.
}SFBNetState;


/*Defines similar to ProtoMgr*/
typedef HObj SFBalHProtocol;
typedef HObj SFBalHProtoAccount;
typedef HObj SFBalHProtoClient;


/* Protocol Type ID */
typedef enum
{
	SFB_PROTO_APP_TYPE_NONE				= 0x01,
	SFB_PROTO_APP_TYPE_WAP				= 0x02,
	SFB_PROTO_APP_TYPE_HTTP				= 0x04,
	SFB_PROTO_APP_TYPE_OBEX				= 0x08,
	SFB_PROTO_APP_TYPE_ALL				= (SFB_PROTO_APP_TYPE_NONE|SFB_PROTO_APP_TYPE_WAP|SFB_PROTO_APP_TYPE_HTTP)
} SFBalProtoAppType;



typedef struct
{

	SFBalProtoAppType		protoAppType;	
	char						szGatewayAddr[BAL_PROTO_IPV4_STR_LEN_MAX+1];	// Gateway address with a port number for WAP
	char						szProxyAddr[BAL_PROTO_IPV4_STR_LEN_MAX+1];	

	

}SFBalProtoNetInfo;



/* Function Declarations */
typedef enum
{
	SFBAL_PROTOCB_STARTED,
	SFBAL_PROTOCB_STOPPED,
	SFBAL_PROTOCB_STOPPING,
	SFBAL_PROTOCB_STARTING,
	SFBAL_PROTOCB_NOTIFY_DISCONNECT_OTHER_PDP, // For handling case that other app is using PDP - Sumdra
	SFBAL_PROTOCB_CONFIRM_DISCONNECT_OTHER_PDP,
	SFBAL_PROTOCB_INDICATOR,   // For 3G Indicator - Sumdra
}SFBalProtoCBEvent;
typedef void (*SFBalProtoCallback)( SFBalHProtoClient hClient, SFBalProtoCBEvent nEvent,  int nParm1, int nParam2, void *pParam );

/**
Fucntion: SFBalProtoRegisterClient

* This function is called to register RM Module to ProtoMgr
* It is the common function used for both HTTP and WSP connection types.

* @param	szAppName [in] Application name
* @return	Returns the protocol handle if success or SFB_INVALID_OBJ if fail
*/

SFBalHProtoClient SFBalProtoRegisterClient (const char *szAppName, SFBalProtoCallback fpProtoCB );



/**
Fucntion: SFBalProtoUnregisterClient

* This function is called to unregister RM Module to ProtoMgr
* It is the common function used for both HTTP and WSP connection types.

*/

BOOL SFBalProtoUnregisterClient ( SFBalHProtoClient hClient );


/**
Fucntion: SFBalProtoGetProtoAccountHandle

* This function is called to get the ProtoAccount from phone settings
* It is the common function used for both HTTP and WSP connection types.
* @return	Returns ProtoAccount Handle if success else SFB_INVALID_OBJ

*/

SFBalHProtoAccount SFBalProtoGetProtoAccountHandle (SFBalHProtoClient hClient);


/**
Fucntion: SFBalProtoSetNetworkInfo

* This function is called to set the Network Info by passing input param hProtoAccount and SFBalHProtocol to ProtoMgr
* It is the common function used for both HTTP and WSP connection types.
* @param	hProtoAccount [in] connection ID to protoMngr
* @param	SFBalHProtocol [in] protocolhandle to protoMngr
* @return	Returns TRUE if success or FALSE if fail

*/

BOOL SFBalProtoSetNetworkInfo ( SFBalHProtoClient hClient, SFBalHProtoAccount hProtoAccount);



/**
Fucntion: SFBalProtoGetNetworkInfo

* This function is called to get the Network Info by passing input param SFBalHProtocol to ProtoMgr
* It is the common function used for both HTTP and WSP connection types.
* @param	SFBalHProtocol [in] protocolhandle to protoMngr
* @return	Returns TRUE if success or FALSE if fail

*/

SFBalProtoNetInfo* SFBalProtoGetNetworkInfo ( SFBalHProtoClient hClient );



/**
* Restart Linger timer
*/
BOOL SFBalRestartLingerTimer( SFBalHProtoClient hClient );

/**
Fucntion: SFBalSetLingerTimer

* This function is called to change linger time duration
* @param	hClient [in] protocolhandle to protoMngr
* @param	lingertime [in] duration which want to change.
* @return	Returns TRUE if success or FALSE if fail

*/

BOOL SFBalSetLingerTimer( SFBalHProtoClient hClient, ULONG lingertime );

/**
Fucntion: SFBalProtoStartNetwork

* This function is called to start Bearer connection by passing input param SFBalHProtocol to ProtoMgr
* It is the common function used for both HTTP and WSP connection types.
* @param	hProtocol [in]  protocol handle
* @return	Returns TRUE if success or FALSE if fail

*/

BOOL SFBalProtoStartNetwork ( SFBalHProtoClient hClient );

/**
Fucntion: SFBalProtoStartNetwork

* This function is called to stop the Bearer connection by passing input param hProtocol to ProtoMgr
* It is the common function used for both HTTP and WSP connection types.
* @param	hProtocol [in]  protocol handle
* @return	Returns TRUE if success or FALSE if fail

*/

BOOL SFBalProtoStopNetwork (SFBalHProtoClient hClient);


/**
Fucntion: SFBalProtoEventIndication

* This function is handles the protocol events sent by the underlying protocol manager
* It is the common function used for both HTTP and WSP connection types.
* @param	SFBalProtoEvent [in]  SFB protocol event (which is common for SHP and Linux)
* @return	Returns TRUE if success or FALSE if fail
*/

//BOOL SFBalProtoEventIndication(SFBalProtoEvent* pProtoEvent);

/**
Fucntion: SFBalIsNetworkConnected

* This function is to check wheather Bearer Network is connected
* @return	Returns TRUE if success or FALSE if fail
*/

BOOL SFBalProtoIsNetworkConnected(SFBalHProtoClient hClient);

/**
Fucntion: SFBalProtoIsNetworkConnecting

* This function is to check wheather Bearer Network is connecting
* @return	Returns TRUE if success or FALSE if fail
*/
BOOL SFBalProtoIsNetworkConnecting(SFBalHProtoClient hClient);

/**
Fucntion: SFBalProtoSetupNetwork

* This function is to setup Bearer Network
* @return	Returns SFB_RM_OK if success or SFB_INVALID_OBJ if fail
*/
SFBResult SFBalProtoSetupNetwork(SFBalHProtoClient hClient, SFBalHProtoAccount hAccount);

///**
//Fucntion: SFBalProtoNotifyStarted
//
//* This function is called BrPlProtoEventHandler when BEARER_STARTED
//* @return	Returns SFB_RM_OK if success or SFB_INVALID_OBJ if fail
//*/
//SFBResult SFBalProtoNotifyStarted(int account);
//
///**
//Fucntion: SFBalProtoNotifyStoped
//
//* This function is called BrPlProtoEventHandler when BEARER_STOPPED
//* @return	Returns SFB_RM_OK if success or SFB_INVALID_OBJ if fail
//*/
//SFBResult SFBalProtoNotifyStoped(int account, int err);
//
///**
//Fucntion: SFBalProtoNotifyStopping
//
//* This function is called BrPlProtoEventHandler when BEARER_STOPPING
//* @return	Returns SFB_RM_OK if success or SFB_INVALID_OBJ if fail
//*/
//SFBResult SFBalProtoNotifyStopping(int account, int err);
typedef enum
{
	SFBAL_PROTOEVT_STARTING,
	SFBAL_PROTOEVT_STARTED,
	SFBAL_PROTOEVT_STOPPED,
	SFBAL_PROTOEVT_STOPPING,
}SFBalProtoEventType;

typedef struct
{
	SFBalProtoEventType nEvent;
	SFBalHProtoAccount hAccount;
	SFBalHProtocol hProtocol; // in order to support mutiple clients, hProtocol should be passed by platform
	int nError;
}SFBalProtoEvent;

SFBResult SFBalProtoNotifyEvent( SFBalProtoEvent *pEvent  );

/**
Fucntion: SFBalProtoGetProtocolHandle

* This function is called to get the protocol Handle from protoMgr
* It is the common function used for both HTTP and WSP connection types.
* @return	Returns Protocol Handle if success else SFB_INVALID_OBJ

*/
SFBalHProtocol SFBalProtoGetProtocolHandle (SFBalHProtoClient hClient);


SFBalProtoAppType	SFBalProtoGetAppType(SFBalHProtoClient hClient);


char* SFBalGetHTTPProxyAddr(SFBalHProtoClient hClient);

#ifdef SFB_RM_WSP
char* SFBalGetWapGatewayAddress(SFBalHProtoClient hClient);

BOOL SFBalIsSecureConnection(SFBalHProtoClient hClient);
#endif

//NATARAJ 16th MAY for the bug SFB00001083 
char * SFBalGetHostName(SFBalHProtoClient hClient);

// This function is for handling after receiving user confirmation when other net application is using PDP
// Add by sumdra. 2008.06.18
BOOL
SFBalProtoDisconnectOtherNet(SFBalHProtoClient hProtoClient, BOOL bConfirm);

BOOL SFBalCancelLingerTimer( SFBalHProtoClient hClient );

void SFBalChangeLcdMode(SFBalHProtoClient hClient, BOOL bIsLcdMode);

#ifdef __cplusplus
}
#endif /* __cplusplus */

#endif //_SFBAL_PROTO_H_