/*
* ==============================================================================
*  Name        : BrowserSettings.hpp
*  Part of     : Browser Control.
*  Description : Implements browser control.
*  Version     :
*
*    Copyright (c) 2007,
*    All rights reserved.
* ==============================================================================
*/

#ifndef __BROWSERSETTINGS_H__
#define __BROWSERSETTINGS_H__

#include "BrowserControl_DataTypes.h" 
#include "CMMDefs.h"

#define BROWSER_SETTINGS_COOKIES_DEFAULT_VALUE					1
#define BROWSER_SETTINGS_CACHEDEFAULT_VALUE  					1
#define BROWSER_SETTINGS_SMARTLINK_DEFAULT_VALUE				1
#define BROWSER_SETTINGS_ANIMATION_DISPLAY_DEFAULT_VALUE		1
#define BROWSER_SETTINGS_IMAGE_SAVE_MODE_DEFAULT_VALUE			1
#define BROWSER_SETTINGS_SPLASH_HTML_DEFAULT_VALUE				1
#define BROWSER_SETTINGS_SSR_DEFAULT_VALUE						1
#define BROWSER_SETTINGS_VIRTUAL_CURSOR_DEFAULT_VALUE			1
#define BROWSER_SETTINGS_SPROXYINFO_AUTH_TYPE_DEFAULT_VALUE		HTTP
#define BROWSER_SETTINGS_SPROXYINFO_PORT_DEFAULT_VALUE			80

#define BROWSER_SETTINGS_IAPHANDLE_DEFAULT_VALUE				0

#define BROWSER_SETTINGS_CACHE_SIZEDEFAULT_VALUE				256
	
#define BROWSER_SETTINGS_SPROXYINFO_SERVER_NAME_DEFAULT_VALUE	""
#define BROWSER_SETTINGS_SPROXYINFO_USER_NAME_DEFAULT_VALUE		""
#define BROWSER_SETTINGS_SPROXYINFO_PASSWORD_DEFAULT_VALUE		""

class BrowserSettings CMM_MODIFIED
{
public:
	//set settings
	static ESTATUS SetHttpProxy(const SProxyInfo* aProxyInfo);
	static ESTATUS SetIAP(const IAPHANDLE aSelectedIapHandle);
	static ESTATUS SetCacheSize(const unsigned int  aCacheSize);
	
	static ESTATUS SetStatus_Cookies(const bool aStatus);
	static ESTATUS SetStatus_Cache(const bool aStatus);
	static ESTATUS SetStatus_SmartLink(const bool aStatus);
	static ESTATUS SetStatus_AnimationDisplay(const bool aStatus);
	static ESTATUS SetStatus_ImageSaveMode(const bool aStatus);
	static ESTATUS SetStatus_SplashHTML(const bool aStatus);
	static ESTATUS SetStatus_SSR(const bool aStatus);
	static ESTATUS SetStatus_VirtualCursor(const bool aStatus);
	
	//get settings
	static ESTATUS GetHttpProxy(SProxyInfo* aProxyInfo);
	static ESTATUS GetIAP(IAPHANDLE* aSelectedIapHandle);
	static ESTATUS GetCacheSize(unsigned int * aCacheSize);
	
	static bool GetStatus_Cookies();
	static bool GetStatus_Cache();
	static bool GetStatus_SmartLink();
	static bool GetStatus_ImageDisplay();
	static bool GetStatus_AnimationDisplay();
	static bool GetStatus_JavaScript();
	static bool GetStatus_ImageSaveMode();
	static bool GetStatus_SplashHTML();
	static bool GetStatus_SSR();
	static bool GetStatus_VirtualCursor();
protected:
	BrowserSettings();							//to avoid constructing the object
	~BrowserSettings();
	 SProxyInfo mProxyInfo;						//proxy settings
	 unsigned int  mCacheSize;					//cache size
	 unsigned int  mIapHandle;
	 bool mStatus_Cookies;						//Enable/Disable Cookies
	 bool mStatus_Cache;						//Enable/Disable Cache
	 bool mStatus_SmartLink;					//Enable/Disable SmartLinks
	 bool mStatus_AnimationDisplay;				//Enable/Disable Animation Display
	 bool mStatus_ImageSaveMode;				//Enable/Disable Image saving
	 bool mStatus_SplashHTML;					//Enable/Disable Splash HTML
	 bool mStatus_SSR;							//Enable/Disable SSR. If SSR is disabled then it is Desktop view
	 bool mStatus_VirtualCursor;
	
	//the one and only instance
	static BrowserSettings m_browserSettings;
};

#endif //__BROWSERSETTINGS_HPP__


