/*
* ==============================================================================
*  Name        : BrowserControl.cpp
*  Part of     : Browser Control.
*  Description : Defines a browser control interface.
*  Version     :
*
*    Copyright (c) 2007,
*    All rights reserved.
* ==============================================================================
*/

// bs0425.lee 
#include "platform.h"


#include "BrowserControl.hpp"
#if defined ( SHP_SUPPORT_WML )
#include "config.h"
#include "webkitshppage.h"
#endif
//extern function to set the browser control handle with the event handler.
#ifdef __SHP__
extern "C"
{
    extern void registerBrCtrlWithEventHandler(const HANDLE brCtrlHandle);
}
#endif

#ifdef SANITY_CHECK
GSList* BrowserControl::handleList = NULL;

gboolean BrowserControl::IsValidHandle(HANDLE aBrowserHandle)
{
    GSList *pHandle = g_slist_find( handleList, (void*)aBrowserHandle);
    if (pHandle)
        return true;
    else
        return false;
}
#endif //_DEBUG

//------------------------------------------------------------------------
// WebKit initialize and finalize APIs
//------------------------------------------------------------------------
ESTATUS BrowserControl::BrowserControl_WebKit_Initialize(HWIN hMainWin)
{
	WEBKIT_INITIALIZE(hMainWin);
	return ESuccess;
}

ESTATUS BrowserControl::BrowserControl_WebKit_Finalize()
{
	WEBKIT_FINALIZE();
	return ESuccess;
}
GlobalHandler aGlobalHandler;
//------------------------------------------------------------------------
// Creation and Destruction API
//------------------------------------------------------------------------

ESTATUS BrowserControl::BrowserControl_Init(HANDLE*& aBrowserHandle)
{
    *aBrowserHandle = (HANDLE)(new BrowserControl());
    if(*aBrowserHandle)
    {
#ifdef SANITY_CHECK
        BrowserControl::handleList = g_slist_prepend( BrowserControl::handleList, aBrowserHandle);
#endif // SANITY_CHECK
        return ESuccess;
    }
    else
        return EFail;
}

ESTATUS BrowserControl::BrowserControl_Finalize(HANDLE aBrowserHandle)
{
    if(aBrowserHandle)
    {

#ifdef SANITY_CHECK
        BrowserControl::handleList = g_slist_prepend( BrowserControl::handleList, aBrowserHandle);
#endif // SANITY_CHECK

		delete ((BrowserControl *)aBrowserHandle);
		aBrowserHandle = NULL;
    }
	return ESuccess;
}


ESTATUS BrowserControl::BrowserControl_Create(BrowserWidget** aWidget)
{
    mPage = (WebKitPage*)WEBKIT_PAGE_NEW();
    *aWidget = (BrowserWidget *) mPage;
    return ESuccess;
}

ESTATUS BrowserControl::BrowserControl_Delete()
{
#ifdef __SHP__
    WEBKIT_PAGE_FINALIZE(mPage);
#endif
    return ESuccess;
}


//------------------------------------------------------------------------
// Settings API
//------------------------------------------------------------------------

ESTATUS BrowserControl::BrowserControl_SetHwnd(const HWIN hWin)
{
#ifdef __SHP__
    WEBKIT_PAGE_SET_HWIN( mPage, hWin );
#else
#endif
    return ESuccess;
}

ESTATUS BrowserControl::BrowserControl_SetHttpProxy(const SProxyInfo* aProxyInfo)
{
	WEBKIT_PAGE_SET_HTTPPROXY(mPage,aProxyInfo);
    return ESuccess;
}

ESTATUS BrowserControl::BrowserControl_SetIAP(const IAPHANDLE aSelectedIapHandle)
{
    WEBKIT_PAGE_SET_IAP(mPage,aSelectedIapHandle);
	return ESuccess;
}

ESTATUS BrowserControl::BrowserControl_SetCacheSize(const guint aCacheSize)
{
	WEBKIT_PAGE_SET_CACHE_SIZE(mPage,aCacheSize);
	return ESuccess;
}

ESTATUS BrowserControl::BrowserControl_SetMinimumFontSize(const int  aSettingsValue)
{
	WEBKIT_PAGE_SET_MINIMUM_FONT_SIZE(mPage,aSettingsValue);
  	return ESuccess;
}

ESTATUS BrowserControl::BrowserControl_SetMinimumLogicalFontSize(const int aSettingsValue)
{
	WEBKIT_PAGE_SET_MINIMUM_LOGICAL_FONT_SIZE(mPage,aSettingsValue);
	return ESuccess;
}

ESTATUS BrowserControl::BrowserControl_SetDefaultFontSize(const int  aSettingsValue)
{
	WEBKIT_PAGE_SET_DEFAULT_FONT_SIZE(mPage,20);
	return ESuccess;
}

ESTATUS BrowserControl::BrowserControl_SetDefaultFixedFontSize(const int  aSettingsValue)
{
	WEBKIT_PAGE_SET_DEFAULT_FIXED_FONT_SIZE(mPage,aSettingsValue);
	return ESuccess;
}

ESTATUS BrowserControl::BrowserControl_SetDefaultTextEncodingName(const gchar *aSettingsValue)
{
	WEBKIT_PAGE_SET_DEFAULT_TEXT_ENCODING_NAME(mPage,aSettingsValue);
	return ESuccess;
}

ESTATUS BrowserControl::BrowserControl_SetFtpDirectoryTemplatePath(const gchar *aSettingsValue)
{
	WEBKIT_PAGE_SET_FTP_DIRECTORY_TEMPLATE_PATH(mPage,aSettingsValue);
	return ESuccess;
}
ESTATUS BrowserControl::BrowserControl_SetUserStyleSheetLocation(const gchar *aSettingsValue)
{
	WEBKIT_PAGE_SET_USER_STYLE_SHEET_LOCATION(mPage,aSettingsValue);
	return ESuccess;
}
ESTATUS BrowserControl::BrowserControl_SetStandardFontFamily(const gchar *aSettingsValue)
{
	WEBKIT_PAGE_SET_STANDARD_FONT_FAMILY(mPage,aSettingsValue);
	return ESuccess;
}
ESTATUS BrowserControl::BrowserControl_SetFixedFontFamily(const gchar *aSettingsValue)
{
	WEBKIT_PAGE_SET_FIXED_FONT_FAMILY(mPage,aSettingsValue);
	return ESuccess;
}
ESTATUS BrowserControl::BrowserControl_SetSerifFontFamily(const gchar *aSettingsValue)
{
	WEBKIT_PAGE_SET_SERIF_FONT_FAMILY(mPage,aSettingsValue);
	return ESuccess;
}
ESTATUS BrowserControl::BrowserControl_SetSansSerifFontFamily(const gchar *aSettingsValue)
{
	WEBKIT_PAGE_SET_SANS_SERIF_FONT_FAMILY(mPage,aSettingsValue);
	return ESuccess;
}
ESTATUS BrowserControl::BrowserControl_SetCursiveFontFamily(const gchar *aSettingsValue)
{
	WEBKIT_PAGE_SET_CURSIVE_FONT_FAMILY(mPage,aSettingsValue);
	return ESuccess;
}
ESTATUS BrowserControl::BrowserControl_SetFantasyFontFamily(const gchar *aSettingsValue)
{
	WEBKIT_PAGE_SET_FANTASY_FONT_FAMILY(mPage,aSettingsValue);
	return ESuccess;
}
ESTATUS BrowserControl::BrowserControl_SetEditableLinkBehavior(const WebCore::EditableLinkBehavior aSettingsValue)
{
	WEBKIT_PAGE_SET_EDITABLE_LINK_BEHAVIOR(mPage,aSettingsValue);
	return ESuccess;
}
ESTATUS BrowserControl::BrowserControl_SetProperty(const EBrowserProperty aBrowserProperty,
                                                    const gboolean aSettingValue)
{
	ESTATUS result =(ESTATUS) WEBKIT_PAGE_SET_PROPERTY( mPage,aBrowserProperty,aSettingValue );
	if ( EFail == result )
		BrowserControl::SetLastError(EInvalidInputParameter);
	return result;
	
}
ESTATUS BrowserControl::BrowserControl_GetHttpProxy(SProxyInfo* aProxyInfo)
{
	WEBKIT_PAGE_GET_HTTPPROXY(mPage,aProxyInfo);
	return ESuccess;
}

ESTATUS BrowserControl::BrowserControl_GetIAP(IAPHANDLE* aSelectedIapHandle)
{
	WEBKIT_PAGE_GET_IAP(mPage,aSelectedIapHandle);
	return ESuccess;
}

ESTATUS BrowserControl::BrowserControl_GetCacheSize(guint* aCacheSize)
{
	WEBKIT_PAGE_GET_CACHE_SIZE(mPage,aCacheSize) ;
	return ESuccess;
}



ESTATUS BrowserControl::BrowserControl_GetMinimumFontSize(int* aSettingsValue)
{
	WEBKIT_PAGE_GET_MINIMUM_FONT_SIZE(mPage,aSettingsValue);
	return ESuccess;
}

ESTATUS BrowserControl::BrowserControl_GetMinimumLogicalFontSize(int* aSettingsValue)
{
	WEBKIT_PAGE_GET_MINIMUM_LOGICAL_FONT_SIZE(mPage,aSettingsValue);
	return ESuccess;
}

ESTATUS BrowserControl::BrowserControl_GetDefaultFontSize(int* aSettingsValue)
{
	WEBKIT_PAGE_GET_DEFAULT_FONT_SIZE(mPage,aSettingsValue);
	return ESuccess;
}
ESTATUS BrowserControl::BrowserControl_GetDefaultFixedFontSize(int* aSettingsValue)
{
	WEBKIT_PAGE_GET_DEFAULT_FIXED_FONT_SIZE(mPage,aSettingsValue);
	return ESuccess;
}

#ifdef SFB_VISUAL_SUPPORT
ESTATUS BrowserControl::BrowserControl_VisualHistory(WmBitmap** pWmBitmap,void* pUserData)
{
	WEBKIT_PAGE_VISUAL_HISTORY(mPage,pWmBitmap,pUserData);
    return ESuccess;
}
#endif
//tahseen

ESTATUS BrowserControl::BrowserControl_GetDefaultTextEncodingName(gchar* aSettingsValue,gulong* aSize)
{
	ESTATUS result = EFail;
	if(!aSize)
	{
		BrowserControl::SetLastError(ENullInputParameter);
		return result;
	}
	gulong aReturnedSize = *aSize;
	WEBKIT_PAGE_GET_DEFAULT_TEXT_ENCODING_NAME(mPage,aSettingsValue,&aReturnedSize);

	if (aSettingsValue)
	{
		if( *aSize == aReturnedSize )
		{
			result = ESuccess;
		}
		else
		{
			*aSize = aReturnedSize;
			BrowserControl::SetLastError(EInsufficientBuffer);
		}
	}
	else
	{
		BrowserControl::SetLastError(ENullInputParameter);
		*aSize = aReturnedSize;
	}
	return result;

}

ESTATUS BrowserControl::BrowserControl_GetFtpDirectoryTemplatePath(gchar* aSettingsValue,gulong* aSize)
{
	ESTATUS result = EFail;
	if(!aSize)
	{
		BrowserControl::SetLastError(ENullInputParameter);
		return result;
	}
	gulong aReturnedSize = *aSize;
	WEBKIT_PAGE_GET_FTP_DIRECTORY_TEMPLATE_PATH(mPage,aSettingsValue,&aReturnedSize);

	if (aSettingsValue)
	{
		if( *aSize == aReturnedSize )
		{
			result = ESuccess;
		}
		else
		{
			*aSize = aReturnedSize;
			BrowserControl::SetLastError(EInsufficientBuffer);
		}
	}
	else
	{
		BrowserControl::SetLastError(ENullInputParameter);
		*aSize = aReturnedSize;
	}
	return result;

}


ESTATUS BrowserControl::BrowserControl_GetUserStyleSheetLocation(gchar* aSettingsValue,gulong* aSize)
{
	ESTATUS result = EFail;
	if(!aSize)
	{
		BrowserControl::SetLastError(ENullInputParameter);
		return result;
	}
	gulong aReturnedSize = *aSize;
	WEBKIT_PAGE_GET_USER_STYLE_SHEET_LOCATION(mPage,aSettingsValue,&aReturnedSize);

	if (aSettingsValue)
	{
		if( *aSize == aReturnedSize )
		{
			result = ESuccess;
		}
		else
		{
			*aSize = aReturnedSize;
			BrowserControl::SetLastError(EInsufficientBuffer);
		}
	}
	else
	{
		BrowserControl::SetLastError(ENullInputParameter);
		*aSize = aReturnedSize;
	}
	return result;
}


ESTATUS BrowserControl::BrowserControl_GetStandardFontFamily(gchar* aSettingsValue, gulong* aSize)
{
	ESTATUS result = EFail;
	if(!aSize)
	{
		BrowserControl::SetLastError(ENullInputParameter);
		return result;
	}
	gulong aReturnedSize = *aSize;
	WEBKIT_PAGE_GET_STANDARD_FONT_FAMILY(mPage,aSettingsValue,&aReturnedSize);

	if (aSettingsValue)
	{
		if( *aSize == aReturnedSize )
		{
			result = ESuccess;
		}
		else
		{
			*aSize = aReturnedSize;
			BrowserControl::SetLastError(EInsufficientBuffer);
		}
	}
	else
	{
		BrowserControl::SetLastError(ENullInputParameter);
		*aSize = aReturnedSize;
	}
	return result;
}

ESTATUS BrowserControl::BrowserControl_GetFixedFontFamily(gchar* aSettingsValue,gulong* aSize)
{
	ESTATUS result = EFail;
	if(!aSize)
	{
		BrowserControl::SetLastError(ENullInputParameter);
		return result;
	}
	gulong aReturnedSize = *aSize;
	WEBKIT_PAGE_GET_FIXED_FONT_FAMILY(mPage,aSettingsValue,&aReturnedSize);

	if (aSettingsValue)
	{
		if( *aSize == aReturnedSize )
		{
			result = ESuccess;
		}
		else
		{
			*aSize = aReturnedSize;
			BrowserControl::SetLastError(EInsufficientBuffer);
		}
	}
	else
	{
		BrowserControl::SetLastError(ENullInputParameter);
		*aSize = aReturnedSize;
	}
	return result;
}

ESTATUS BrowserControl::BrowserControl_GetSerifFontFamily(gchar* aSettingsValue
														,gulong* aSize)
{
	ESTATUS result = EFail;
	if(!aSize)
	{
		BrowserControl::SetLastError(ENullInputParameter);
		return result;
	}
	gulong aReturnedSize = *aSize;
	WEBKIT_PAGE_GET_SERIF_FONT_FAMILY(mPage,aSettingsValue,&aReturnedSize);

	if (aSettingsValue)
	{
		if( *aSize == aReturnedSize )
		{
			result = ESuccess;
		}
		else
		{
			*aSize = aReturnedSize;
			BrowserControl::SetLastError(EInsufficientBuffer);
		}
	}
	else
	{
		BrowserControl::SetLastError(ENullInputParameter);
		*aSize = aReturnedSize;
	}
	return result;
}

ESTATUS BrowserControl::BrowserControl_GetSansSerifFontFamily(gchar* aSettingsValue,gulong* aSize)
{
	ESTATUS result = EFail;
	if(!aSize)
	{
		BrowserControl::SetLastError(ENullInputParameter);
		return result;
	}
	gulong aReturnedSize = *aSize;
	WEBKIT_PAGE_GET_SANS_SERIF_FONT_FAMILY(mPage,aSettingsValue,&aReturnedSize);

	if (aSettingsValue)
	{
		if( *aSize == aReturnedSize )
		{
			result = ESuccess;
		}
		else
		{
			*aSize = aReturnedSize;
			BrowserControl::SetLastError(EInsufficientBuffer);
		}
	}
	else
	{
		BrowserControl::SetLastError(ENullInputParameter);
		*aSize = aReturnedSize;
	}
	return result;
}

ESTATUS BrowserControl::BrowserControl_GetCursiveFontFamily(gchar* aSettingsValue,gulong* aSize)
{
	ESTATUS result = EFail;
	if(!aSize)
	{
		BrowserControl::SetLastError(ENullInputParameter);
		return result;
	}
	gulong aReturnedSize = *aSize;
	WEBKIT_PAGE_GET_CURSIVE_FONT_FAMILY(mPage,aSettingsValue,&aReturnedSize);

	if (aSettingsValue)
	{
		if( *aSize == aReturnedSize )
		{
			result = ESuccess;
		}
		else
		{
			*aSize = aReturnedSize;
			BrowserControl::SetLastError(EInsufficientBuffer);
		}
	}
	else
	{
		BrowserControl::SetLastError(ENullInputParameter);
		*aSize = aReturnedSize;
	}
	return result;
}

ESTATUS BrowserControl::BrowserControl_GetFantasyFontFamily(gchar* aSettingsValue,gulong* aSize)
{
	ESTATUS result = EFail;
	if(!aSize)
	{
		BrowserControl::SetLastError(ENullInputParameter);
		return result;
	}
	gulong aReturnedSize = *aSize;
	WEBKIT_PAGE_GET_FANTASY_FONT_FAMILY(mPage,aSettingsValue,&aReturnedSize);

	if (aSettingsValue)
	{
		if( *aSize == aReturnedSize )
		{
			result = ESuccess;
		}
		else
		{
			*aSize = aReturnedSize;
			BrowserControl::SetLastError(EInsufficientBuffer);
		}
	}
	else
	{
		BrowserControl::SetLastError(ENullInputParameter);
		*aSize = aReturnedSize;
	}
	return result;
}
	
ESTATUS BrowserControl::BrowserControl_GetEditableLinkBehavior(WebCore::EditableLinkBehavior* aSettingsValue)
{
	WEBKIT_PAGE_GET_EDITABLE_LINK_BEHAVIOR(mPage,aSettingsValue);
	return ESuccess;
}



ESTATUS BrowserControl::BrowserControl_GetProperty(const EBrowserProperty aBrowserProperty,
                                                        gboolean* aSettingValue)
{
	ESTATUS result = (ESTATUS) WEBKIT_PAGE_GET_PROPERTY(mPage, aBrowserProperty, aSettingValue);
	if ( EFail == result )
		BrowserControl::SetLastError(EInvalidInputParameter);
	return result;
}

//------------------------------------------------------------------------
// Page API
//------------------------------------------------------------------------
ESTATUS BrowserControl::BrowserControl_SwitchTab()
{
	WEBKIT_PAGE_SWITCH_TAB(mPage);
	return ESuccess;
}

ESTATUS BrowserControl::BrowserControl_LoadUrl(const gchar* aUrl)
{
    WEBKIT_PAGE_OPEN (mPage, aUrl);
    return ESuccess;
}

ESTATUS BrowserControl::BrowserControl_LoadFile(const gchar* aPath)
{
    return ESuccess;
}

ESTATUS BrowserControl::BrowserControl_LoadData(const gchar* aData)
{
    WEBKIT_PAGE_LOAD_HTML_STRING(mPage, aData, "");
    return ESuccess;
}

ESTATUS BrowserControl::BrowserControl_PostUrl(const gchar* aUrl,
                                                const gchar* aContentType,
                                                const gchar* aPostData,
                                                const gchar* aBoundary)
{
    return ESuccess;
}

ESTATUS BrowserControl::BrowserControl_GetPageUrl(gchar* aUrl, unsigned long* aSize)
{
	WEBKIT_PAGE_GET_URL(mPage, aUrl, aSize);
    return ESuccess;
}

ESTATUS BrowserControl::BrowserControl_GetPageTitle(gchar* aTitle, unsigned long* aSize)
{
	WEBKIT_PAGE_GET_TITLE(mPage, aTitle, aSize);
	return ESuccess;
}

ESTATUS BrowserControl::BrowserControl_GetPageMimeType(gchar* aType, gulong* aSize)
{
	WEBKIT_PAGE_GET_MIME_TYPE(mPage, aType, aSize);
	return ESuccess;
}
ESTATUS BrowserControl::BrowserControl_GetPageSize(gulong* aSize)
{
	WEBKIT_PAGE_GET_SIZE(mPage, aSize);
	return ESuccess;
}

ESTATUS BrowserControl::BrowserControl_GetPageContent(gchar* aContent, unsigned long* aSize)
{
    return ESuccess;
}

ESTATUS BrowserControl::BrowserControl_GetPageLastModifiedTime(gchar* aTime, gulong* aSize)
{
	WEBKIT_PAGE_GET_LAST_MODIFIED_TIME(mPage, aTime, aSize);
	return ESuccess;
}

ESTATUS BrowserControl::BrowserControl_GetSelectedElementType(EPageItemType* aType)
{
	WEBKIT_PAGE_GET_SELECTED_ELEMENT_TYPE(mPage,aType);
    return ESuccess;
}

/**
 *   bs0425.lee  modified for touch events 
 */

ESTATUS BrowserControl::BrowserControl_GetSelectedElementTypeByPoint(EPageItemType* aType, int x, int y )
{
	WEBKIT_PAGE_GET_SELECTED_ELEMENT_TYPE_BY_POINT(mPage,aType, x, y);
    return ESuccess;
}

 /**
 *   bs0425.lee  modified for touch events 
 */


ESTATUS BrowserControl::BrowserControl_GetSelectedElementInfo(EPageItemType aType, ElementInfoBase* aInfo)
{
	WEBKIT_PAGE_GET_SELECTED_ELEMENT_INFO(mPage, aType, aInfo);
	return ESuccess;
}

ESTATUS BrowserControl::BrowserControl_GetElementTypeUnderPoint(const guint aX,
                                                                const guint aY,
                                                                EPageItemType* aType)
{
    return ESuccess;
}

ESTATUS BrowserControl::BrowserControl_GetItemCount(const EPageItemType aType, guint* aCount)
{
	WEBKIT_PAGE_GET_ITEM_COUNT(mPage, aType, aCount);
    return ESuccess;
}

// CELUNITE_SAVE_Change_Begin
ESTATUS BrowserControl::BrowserControl_SavePage(const gchar* aPath)
{
    WEBKIT_PAGE_SAVE (mPage , aPath);
    return ESuccess;
}
// CELUNITE_SAVE_Change_End

ESTATUS BrowserControl::BrowserControl_GetImages(bool bVisibleOnly, ImageSaveInfo*** imageList)
{
	WEBKIT_GET_IMAGES (mPage , bVisibleOnly, imageList);
	return ESuccess;
}

ESTATUS BrowserControl::BrowserControl_GetImageCount(bool bVisibleOnly, guint* imageCount)
{
	*(imageCount) = WEBKIT_GET_IMAGE_COUNT (mPage , bVisibleOnly);
	return ESuccess;
}

ESTATUS BrowserControl::BrowserControl_SaveElement(const gchar* aPath)
{
	WEBKIT_ELEMENT_SAVE (mPage , aPath);
	return ESuccess;
}

/**
 *   bs0425.lee  modified for touch events 
 */

ESTATUS BrowserControl::BrowserControl_SaveElementByPoint(const gchar* aPath, int pointX, int pointY)
{
	WEBKIT_ELEMENT_SAVE_BY_POINT (mPage , aPath,pointX,pointY);
	return ESuccess;
}
/**
 *   bs0425.lee  modified for touch events 
 */

ESTATUS BrowserControl::BrowserControl_Reload()
{
    WEBKIT_PAGE_RELOAD(mPage);
	return ESuccess;
}

ESTATUS BrowserControl::BrowserControl_StopLoading()
{
	WEBKIT_PAGE_STOP_LOADING(mPage);
    return ESuccess;
}

ESTATUS BrowserControl::BrowserControl_Find(const SFindInfo* aFindInfo)
{
#ifdef _SFB_SEARCH_STRING_IN_PAGE //Jan-16 Vamsi/Pramod
    if(WEBKIT_PAGE_FIND_STRING(mPage, aFindInfo) > 0)
		return EFail;
	else
		return ESuccess;
#else
	return ESuccess;
#endif
}

#ifdef SFB_RSS_LINK //vamsi Jan 28 Rss Link Tag Support for SFB.
ESTATUS BrowserControl::BrowserControl_GetRssLinks(SRssLink** pRssLink, int* rssCount, gchar** url)
{
	WEBKIT_PAGE_GET_RSS_LINKS(mPage, pRssLink, rssCount, url);
    return ESuccess;
}
#endif

ESTATUS BrowserControl::BrowserControl_FindNext(const SFindInfo* aFindInfo)
{
    return ESuccess;
}

ESTATUS BrowserControl::BrowserControl_CanGoBack(gboolean* aRet)
{
    *aRet = WEBKIT_PAGE_CAN_GO_BACKWARD(mPage);
	return ESuccess;
}

ESTATUS BrowserControl::BrowserControl_CanGoForward(gboolean* aRet)
{
    *aRet = WEBKIT_PAGE_CAN_GO_FORWARD(mPage);
	return ESuccess;
}

ESTATUS BrowserControl::BrowserControl_GoBack()
{
    WEBKIT_PAGE_GO_BACKWARD(mPage);
    return ESuccess;
}

ESTATUS BrowserControl::BrowserControl_GoForward()
{
    WEBKIT_PAGE_GO_FORWARD(mPage);
    return ESuccess;
}

ESTATUS BrowserControl::BrowserControl_ZoomIn(const guint aScalePercent)
{
	WEBKIT_PAGE_ZOOMIN(mPage, aScalePercent);
    return ESuccess;
}

ESTATUS BrowserControl::BrowserControl_ZoomOut(const guint aScalePercent)
{
	WEBKIT_PAGE_ZOOMIN(mPage, aScalePercent);
    return ESuccess;
}

ESTATUS BrowserControl::BrowserControl_GetZoom(guint* aScalePercent)
{
	*aScalePercent = WEBKIT_PAGE_GET_ZOOMFACTOR(mPage);
    return ESuccess;
}

ESTATUS BrowserControl::BrowserControl_CreatePageThumbnail(guint width, guint height, const gchar *thumbPath)
{
    return ESuccess;
}

ESTATUS BrowserControl::BrowserControl_SetTextSize(guint size)
{
    return ESuccess;
}

//------------------------------------------------------------------------
// Error Handling Interface
//------------------------------------------------------------------------

ESTATUS BrowserControl::BrowserControl_SetErrorHandler(const EErrorType aType,
                                                        const ErrorHandler aHandler)
{
    return ESuccess;
}

void BrowserControl::BrowserControl_GetLastError(ERRORCODE *aErrorCode)
{
    *aErrorCode = lastError;

}


//------------------------------------------------------------------------
// Observers API
//------------------------------------------------------------------------

ESTATUS BrowserControl::BrowserControl_SetLoadHandler(const LoadHandler aHandler)
{
    return ESuccess;
}


//------------------------------------------------------------------------
// General API
//------------------------------------------------------------------------

ESTATUS BrowserControl::BrowserControl_GetVersion(gchar* aVersionInfo, gulong* aBufferSize)
{
    return ESuccess;
}

ESTATUS BrowserControl::BrowserControl_GetCertificateInfo(SCertInfo* aCertInfo)
{
    return ESuccess;
}

ESTATUS BrowserControl::BrowserControl_GetBrowserParameter(const EBrowserParameters aBrowserParameter,
                                                            gchar* aParameterValue,
                                                            gulong* aBufferSize)
{
    return ESuccess;
}

ESTATUS BrowserControl::BrowserControl_SetBrowserParameter(const EBrowserParameters aBrowserParameter,
                                                            const gchar* aParameterValue)
{
    return ESuccess;
}

#ifdef _SFB_PAGE_PREVIEW	//vamsi Mar 03 - Page Overview
ESTATUS BrowserControl::BrowserControl_SetPreview()
{
#ifdef __SHP__
	webkit_shp_page_set_preview(mPage);
#endif
    return ESuccess;
}

ESTATUS BrowserControl::BrowserControl_IsPreview(bool* bPreview)
{
#ifdef __SHP__
	webkit_shp_page_is_preview(mPage, bPreview);
#endif
    return ESuccess;
}
#endif

ESTATUS BrowserControl::BrowserControl_Paint( void* pUserData)
{
#ifdef __SHP__

	#ifndef  SFB_PROGRESSIVE_RENDERING
		if (mPage->isLoadingDone)
	#endif
		WEBKIT_PAGE_PAINT(mPage, pUserData);
#else
#endif
    return ESuccess;
}

ESTATUS BrowserControl::BrowserControl_HandleKeyEvent( int keyCode ,  bool isKeyUp)
{
#ifdef __SHP__
    WEBKIT_PAGE_KEY_EVENT(mPage, keyCode,  isKeyUp);
#endif
    return ESuccess;
}

/**
 *   bs0425.lee  Handle for touch events 
 */

ESTATUS BrowserControl::BrowserControl_HandleMouseEvent( int recX, int recY, int MouseEventType)
{
    WEBKIT_PAGE_MOUSE_EVENT(mPage, recX, recY, MouseEventType);
    
    return ESuccess;
}

ESTATUS BrowserControl::BrowserControl_PaneScrollBy( int recX, int recY)
{
    WEBKIT_PAGE_SCROLL_BY(mPage, recX, recY);

    return ESuccess;
}

gboolean BrowserControl::BrowserControl_IsFocusedByPoint( int pointX, int pointY)
{
    gboolean result = TRUE;

    result=WEBKIT_PAGE_IS_FOCUSED_BYPOINT(mPage, pointX, pointY);

    return result;
}

gboolean BrowserControl::BrowserControl_IsLink( int recX, int recY)
{
    gboolean result = FALSE;

    result=WEBKIT_PAGE_IS_LINKED_ELEMENT(mPage, recX, recY);

    return result;
}

gboolean BrowserControl::BrowserControl_IsImageElement( int recX, int recY)
{
    gboolean result = FALSE;

    result=WEBKIT_PAGE_IS_IMAGE_ELEMENT(mPage, recX, recY);

    return result;
}


ESTATUS BrowserControl::BrowserControl_GetSelectedElementInfoByPoint(EPageItemType aType, ElementInfoBase* aInfo, int pointX, int pointY)
{
	WEBKIT_PAGE_GET_ELEMENT_INFO_BYPOINT(mPage, aType, aInfo, pointX, pointY);
	return ESuccess;
}


/**
 *   bs0425.lee  Handle for touch events 
 */




#ifdef __SHP__
//------------------------------------------------------------------------
// Notification Handlers functions
//------------------------------------------------------------------------

void BrowserControl::BrowserControl_SetProgressHandler(ProgressHandler aProgressHandler)
{
	if(mPage)
		mPage->m_progressHandler = aProgressHandler;
}

void BrowserControl::BrowserControl_SetTitleChangedHandler(TitleChangedHandler aTitleChangedHandler)
{
	if(mPage)
		mPage->m_titleChangedHandler = aTitleChangedHandler;
}

void BrowserControl::BrowserControl_SetLinkHoveringHandler(LinkHoveringHandler aLinkHoveringHandler)
{
	if(mPage)
		mPage->m_linkHoveringHandler = aLinkHoveringHandler;
}

void BrowserControl::BrowserControl_SetStatusbarTextChangedHandler(StatusbarTextChangedHandler aStatusbarTextChangedHandler)
{
	if(mPage)
		mPage->m_statusbarTextChangedHandler = aStatusbarTextChangedHandler;
}

void BrowserControl::BrowserControl_SetIconLoadedHandler(IconLoadedHandler aIconLoadedHandler)
{
	if(mPage)
		mPage->m_iconLoadedHandler = aIconLoadedHandler;
}

void BrowserControl::BrowserControl_SetSelectionChangedHandler(SelectionChangedHandler aSelectionChangedHandler)
{
	if(mPage)
		mPage->m_selectionChangedHandler = aSelectionChangedHandler;
}
void BrowserControl::BrowserControl_SetUrlSchemeHandler(UrlSchemeHandler aUrlSchemeHandler)
{
	aGlobalHandler.m_urlschemehandler = aUrlSchemeHandler;
}
void BrowserControl::BrowserControl_SetUIStateHandler(UIStateHandler aUIStateHandler)
{
	aGlobalHandler.m_uistatehandler = aUIStateHandler;		
}
#endif

#if defined SHP_SUPPORT_WML
void BrowserControl::BrowserControl_FetchRequest (void)
{
	WEBKIT_PAGE_SEND_NEW_FETCH_REQUEST(mPage);
}
#endif //SHP_SUPPORT_WML
//------------------------------------------------------------------------
// Internal functions
//------------------------------------------------------------------------

void BrowserControl::SetLastError(ERRORCODE aErrorCode)
{
    lastError = aErrorCode;
}
#if defined ( SHP_SUPPORT_WML )
int BrowserControl::BrowserControl_WmlItemCount()
{
	ASSERT (mPage) ;
	return WEBKIT_DOCUMENT_GET_COUNT(mPage) ;
}

int BrowserControl::BrowserControl_WmlDoElement( int ID) 
{
	ASSERT (mPage) ;
	return WEBKIT_DOCUMENT_WMLDOEVENT(mPage, ID) ;
}
#endif//SHP_SUPPORT_WML
ERRORCODE BrowserControl::lastError = _ESuccess;

//vamsi relayout
void BrowserControl::BrowserControl_ReLayout() 
{
	ASSERT (mPage) ;
	WEBKIT_PAGE_RELAYOUT(mPage) ;
}

