/*
* ==============================================================================
*  Name        : BrowserControl_History_Interface.h
*  Part of     : Browser Control.
*  Description : Defines a browser control history interface.
*  Version     :
*
*    Copyright (c) 2007,
*    All rights reserved.
* ==============================================================================
*/

#ifndef __BROWSERCONTROL_HISTORY_INTERFACE_H__
#define __BROWSERCONTROL_HISTORY_INTERFACE_H__

#include "BrowserControl_DataTypes.h" 

#ifdef __cplusplus
extern "C"
{
#endif
/**
 * @defgroup History History related APIs
 */

//------------------------------------------------------------------------
// History API:
//------------------------------------------------------------------------

/**
*  BrowserControl_GetHistoryHandle
* @brief Purpose : Get an handle to the history.
* @param[in] HANDLE aBrowserHandle : Handle to the BrowserControl.
* @param[out] HANDLE* aHistoryHandle : Handle to the Browser history.
* @return ESTATUS : ESUCCESS if History handle could be retrieved, else EFAIL.
* Exact reason of the failure can be queried using BrowserControl_GetLastError().
* @ingroup History
*/
WEBKIT_SHP_API ESTATUS BrowserControl_GetHistoryHandle(HANDLE aBrowserHandle, HANDLE* aHistoryHandle);
/**
*  BrowserControl_GetHistoryIteratorHandle
* @brief Purpose : Get an handle to the history.
* @param[in] HANDLE aHistoryHandle : Handle to the Browser history.
* @param[out] HANDLE* aHistoryIteratorHandle : Handle to the History Iterator.
* @return ESTATUS : ESUCCESS if History Iterator handle could be retrieved, else EFAIL.
* Exact reason of the failure can be queried using BrowserControl_GetLastError().
* @ingroup History
*/
WEBKIT_SHP_API ESTATUS BrowserControl_GetHistoryIteratorHandle(HANDLE aHistoryHandle, HANDLE* aHistoryIteratorHandle);
/**
*  BrowserControl_GetHistoryCount
* @brief Purpose : Retrieves the count of items in history.
* @param[in] HANDLE aHistoryHandle : Handle to the history.
* @param[out] unsigned int* aCount : Count of items in history.
* @return ESTATUS : ESUCCESS if Count could be retrieved, else EFAIL.Exact reason
* of the failure can be queried using BrowserControl_GetLastError().
* @ingroup History
*/
WEBKIT_SHP_API ESTATUS BrowserControl_GetHistoryCount(HANDLE aHistoryHandle, guint* aCount);
/**
*  BrowserControl_ClearHistory
* @brief Purpose : Clears the history.
* @param[in] HANDLE aHistoryHandle : Handle to the history.
* @return ESTATUS : ESUCCESS if history could be cleared, else EFAIL.Exact reason
* of the failure can be queried using BrowserControl_GetLastError().
* @ingroup History
*/
WEBKIT_SHP_API ESTATUS BrowserControl_ClearHistory(HANDLE aHistoryHandle);
/**
*  BrowserControl_SetMaxHistoryCount
* @brief Purpose : Set the max limit for the number of urls that can be maintained in the history.
* @param[in] HANDLE aBrowserHandle : Handle to the history.
* @param[in] unsigned int aCount : Max number of urls that can be maintained in history.
* @return ESTATUS : ESUCCESS if the count could be set, else EFAIL.Exact reason
* of the failure can be queried using BrowserControl_GetLastError().
* @ingroup History
*/
WEBKIT_SHP_API ESTATUS BrowserControl_SetMaxHistoryCount(HANDLE aHistoryHandle, const guint aCount);
/**
*  BrowserControl_ClearForwardHistory
* @brief Purpose : Clears the history list in the forward direction from the
*  currently loaded page.
* @param[in] HANDLE aHistoryHandle : Handle to the history.
* @return ESTATUS : ESUCCESS if the clear operation could be done, else EFAIL.
* Exact reason of the failure can be queried using BrowserControl_GetLastError().
* @ingroup History
*/
WEBKIT_SHP_API ESTATUS BrowserControl_ClearForwardHistory(HANDLE aHistoryHandle);

/**
*  BrowserControl_ClearBackwardHistory
* @brief Purpose : Clears the history list in the backward direction from the
*  currently loaded page.
* @param[in] HANDLE aHistoryHandle : Handle to the history.
* @return ESTATUS : ESUCCESS if the clear operation could be done, else EFAIL.
* Exact reason of the failure can be queried using BrowserControl_GetLastError().
* @ingroup History
*/
WEBKIT_SHP_API ESTATUS BrowserControl_ClearBackwardHistory(HANDLE aHistoryHandle);

/**
*  BrowserControl_DeleteHistoryItem
* @brief Purpose : Deletes the history item from the history list at the specified 
*  index
* @param[in] HANDLE aHistoryHandle : Handle to the history.
* @param[in] const guint aIndex : Index of the history item to be deleted.
* @return ESTATUS : ESUCCESS if the item could be deleted, else EFAIL.
* Exact reason of the failure can be queried using BrowserControl_GetLastError().
* @ingroup History
*/
WEBKIT_SHP_API ESTATUS	BrowserControl_DeleteHistoryItem(HANDLE aHistoryHandle,const guint aIndex);

#ifdef __cplusplus
} //extern "C"
#endif

#endif //__BROWSERCONTROL_HISTORY_INTERFACE_H__
