
#include "config.h"

#ifdef SFB_WIDGET
#include "kjs_window.h"
#include "error_object.h"
#include "lookup.h"
#include "operations.h"
#include <stdio.h>
#include <wtf/Assertions.h>
#include <wtf/HashSet.h>
#include "shp.h"
#include "GWES.h"
//#include "JSDOMWindow.h"
#include "widget_object.h"
#include "widget_object.lut.h"
#include <algorithm> // for std::min
using namespace WebCore;

namespace KJS {


// ------------------------------ WidgetPrototype ----------------------------

const ClassInfo WidgetPrototype::info = {"widget", &WidgetPrototype::info, &widgetTable};

/* Source for widget_object.lut.h
@begin widgetTable 500
  maximizeWindow		    widgetProtoFuncMaximiseWindow         DontEnum|Function 0
  setPreferenceForKey		widgetProtoFuncsetPreferenceKey       DontEnum|Function 0
  preferenceForKey			widgetProtoFunPreferenceForKey        DontEnum|Function 0
  openURL					widgetProtoFuncOpenUrl				  DontEnum|Function 0
  getLanguage				widgetProtoFuncGetLanguage            DontEnum|Function 0
  getIsNetworkAvailable		widgetProtoFuncGetIsNetworkAvailable  DontEnum|Function 0
  resizeWindow				widgetProtoFuncResizeWindow           DontEnum|Function 0
  notify					widgetProtoFuncErrorNotify			  DontEnum|Function 0
  setScroll					widgetProtoFuncSetScroll			  DontEnum|Function 0
  getMccMnc					widgetProtoFuncGetMccMnc			  DontEnum|Function 0

@end
*/ 
WidgetPrototype::WidgetPrototype(ExecState*, ObjectPrototype* objProto)
  : WidgetInstance(objProto)
{
	
}

JSValue* widgetProtoFuncsetPreferenceKey(ExecState* exec, JSObject* thisObj, const List& args)
{
    String data = args[0]->toString(exec);
	String key = args[1]->toString(exec);

	Frame* frame = Window::retrieveActive(exec)->impl()->frame();
	if(!frame)
		return jsUndefined();
	 frame->page()->chrome()->client()->setPreferenceForKeyInfo(data,key);//setPreferenceForKeyInfo(key,data);
	
	return jsUndefined();
}

JSValue* widgetProtoFuncOpenUrl(ExecState* exec, JSObject* thisObj, const List& args)
{
   	int ec = 0;

	Frame* frame = Window::retrieveActive(exec)->impl()->frame();
	if(!frame)
		return jsUndefined();
	WebCore::KURL url = frame->loader()->completeURL(DeprecatedString(args[0]->toString(exec)));
	String urlString = url.string();
	//frame->loader()->load(ResourceRequest(url));
	frame->page()->chrome()->client()->openUrl(urlString);
	return jsUndefined();
}

JSValue* widgetProtoFunPreferenceForKey(ExecState* exec, JSObject* thisObj, const List& args)
{
	String key = args[0]->toString(exec);
	Frame* frame = Window::retrieveActive(exec)->impl()->frame();
	if(!frame)
		return jsUndefined();
	char *data = frame->page()->chrome()->client()->preferenceforkeyInfo(key);//setPreferenceForKeyInfo(key,data);
	if( data == 0 )
		return jsNull();

	int len = (1 + AcStrlen(data))*3;

	MCHAR *ptestMChar = NULL;

	ptestMChar = ( MCHAR *)MemCalloc(1 ,len*sizeof(MCHAR));
	
	WmConvert2PCode( ptestMChar , len , data );


	String mcharString(((wchar_t *)ptestMChar),WmStrlen(ptestMChar));

	JSValue* retStr = jsString(mcharString);
	// This is allocated by MemAlloc, so only use MemFree here
	MemFree(data) ;
	if (ptestMChar)
		MemFree(ptestMChar);
	return retStr ;
}

JSValue* widgetProtoFuncMaximiseWindow(ExecState* exec, JSObject* thisObj, const List& args)
{
	String url = args[0]->toString(exec);
	bool b = true;
	Frame* frame = Window::retrieveActive(exec)->impl()->frame();
	if(!frame)
		return jsUndefined();
	b = frame->page()->chrome()->client()->setWidgetURL(url);
	// Currently returning true by default.
	// TODO : Need to modify. Widget App setWidgetURL should return a bool value
	return jsBoolean(b); 
}

JSValue* widgetProtoFuncGetLanguage(ExecState* exec, JSObject* thisObj, const List& args)
{	
	Frame* frame = Window::retrieveActive(exec)->impl()->frame();
	if(!frame)
		return jsUndefined();
	const char* lang = frame->page()->chrome()->client()->getLanguage();
	return jsString(lang);
}

JSValue* widgetProtoFuncGetIsNetworkAvailable(ExecState* exec, JSObject* thisObj, const List& args)
{
	Frame* frame = Window::retrieveActive(exec)->impl()->frame();
	if(!frame)
		return jsUndefined();
	bool b = frame->page()->chrome()->client()->getIsNetworkAvailable();
	return jsBoolean(b);
}

JSValue* widgetProtoFuncResizeWindow(ExecState* exec, JSObject* thisObj, const List& args)
{
	int width = args[0]->toInteger(exec);
	int height = args[1]->toInteger(exec);
	Frame* frame = Window::retrieveActive(exec)->impl()->frame();
	if(!frame)
		return jsUndefined();
	frame->page()->chrome()->client()->resizeWindow(width,height);
	return jsUndefined();
}

JSValue* widgetProtoFuncErrorNotify(ExecState* exec, JSObject* thisObj, const List& args)
{
	int errType = args[0]->toInteger(exec);
	Frame* frame = Window::retrieveActive(exec)->impl()->frame();
	if(!frame)
		return jsUndefined();
	frame->page()->chrome()->client()->errorNotify(errType);
	return jsUndefined();
}

JSValue* widgetProtoFuncSetScroll(ExecState* exec, JSObject* thisObj, const List& args)
{
	bool scroll = args[0]->toBoolean(exec);
	Frame* frame = Window::retrieveActive(exec)->impl()->frame();
	if(!frame)
		return jsUndefined();
	frame->page()->chrome()->client()->setScroll(scroll);
	return jsUndefined();
}

JSValue* widgetProtoFuncGetMccMnc(ExecState* exec, JSObject* thisObj, const List& args)
{
	Frame* frame = Window::retrieveActive(exec)->impl()->frame();
	if(!frame)
		return jsUndefined();
	const char *data = frame->page()->chrome()->client()->getMccMnc();
	if( data == 0 )
		return jsNull();
	JSValue *retStr = jsString(data);
	// This is allocated by MemAlloc, so only use MemFree here
	void* pMem = (void *)data;
	MemFree(pMem) ;
	return retStr ;
}

// ECMA 15.4.4


// ------------------------------ WidgetObjectImp -------------------------------

WidgetObjectImp::WidgetObjectImp(ExecState* exec, FunctionPrototype* funcProto, WidgetPrototype* widgetProto)
    : InternalFunctionImp(funcProto, widgetProto->classInfo()->className)
{
       putDirect(exec->propertyNames().prototype, widgetProto, DontEnum|DontDelete|ReadOnly);

       putDirect(exec->propertyNames().length, jsNumber(1), ReadOnly|DontDelete|DontEnum);
}

bool WidgetObjectImp::implementsConstruct() const
{
    return true;
}


JSObject* WidgetObjectImp::construct(ExecState* exec, const List& args)
{
    if (args.size() == 1 && args[0]->isNumber()) 
	{
        uint32_t n = args[0]->toUInt32(exec);
        if (n != args[0]->toNumber(exec))
            return throwError(exec, RangeError, "Error in Widget.");
     
    }
}

JSValue* WidgetObjectImp::callAsFunction(ExecState* exec, JSObject*, const List& args)
{
      return construct(exec,args);
}

bool WidgetPrototype::getOwnPropertySlot(ExecState* exec, const Identifier& propertyName, PropertySlot& slot)
{
    return getStaticFunctionSlot<JSObject>(exec, &widgetTable, this, propertyName, slot);
}

void WidgetPrototype::getPropertyNames(ExecState* exec, PropertyNameArray& propertyNames)
{
	JSObject::getPropertyNames(exec, propertyNames);
}
}
#endif