# Author: Prashant Nevase
# 
# Maintained by: Pushparajan Vijayakumar (rajan.vp@samsung.com)
#
# SISO, Copyright 2008.
###############################################################################

include SFBConfig.gmk

###############################################################################
#
#  Section 1: Toolchain settings (Compiler, Linker, Archiver)
#
###############################################################################

# WARNING: Overriden from Config.gmk
# C ARM compilers
CC := armcc --thumb
LD := armlink
AR := armar
FROMELF := fromelf
ifeq ($(findstring TRUE, $(STANDALONE_BUILD)), TRUE)
SANDBOXPL   := perl $(MAINDIR)/tools/sandbox.pl#
FULLDEPENDS := $(APPSBROWSER)/_shp_depend_uniq
endif

###############################################################################
#
#  Section 2: Compiler, Linker and Binary generater arguments
#
###############################################################################

###############################
# Binary related arguments
###############################

DID=00111104# Set Webkit's DID here
BINNAME := DLL$(DID).DLL#
DIFNAME := DIF$(DID).DIF#
FROMELF_ARGS := --nodebug --no_comment_section --elf -o#

###############################
# Compiler arguments
###############################


# CFLAGS taken from SFBConfig.gmk
#CFLAGS = -c -W --cpu ARM926EJ-S --apcs /noswst/interwork -g --dwarf2 -Ospace -O2 --littleend --thumb --cpp -W --friend_injection --no_parse_templates --no_dep_name --using_std --diag_warning 20

# Remove Compiler include from CFLAGS

ifeq ($(MAKEOPTIONS), NO_COMPILER_INCLUDES)
CFLAGS =  $(OUT) $(CPU) $(SHP_CFLAGS) $(APCS) $(CODE) $(PRAGMA) $(CHK) $(MSG) $(OPT) $(ZC)#
endif

# enabling Debug CFLAGS
#CFLAGS += $(DBG)

# core team's fix
CFLAGS += --bss_threshold=0#

# General flags
DFLAGS := $(DEPEND)
ERRFLAGS := --errors

# WARNING: Overriden from Config.gmk
# C++ ARM compilers

CXX := armcpp --thumb
CXXFLAGS := $(CFLAGS) --cpp -W --friend_injection --no_parse_templates --no_dep_name --using_std --diag_warning 20

###########################
# Linker arguments
###########################

ENTRY = --entry DllBaseInit#
SCLFILE= $(MAINDIR)/Webkit.scl#
MAP= --map#
INFO= --info sizes,totals,unused#
VERB    = --verbose#  
LIST= --list $(BINDIR)/$(TARGET)_Webkit.map#
SYM= --symbols --symdefs $(BINDIR)/$(TARGET).sym#
SCATTER = --scatter $(SCLFILE)#   

OBJ_CMD       = -o#          # Command line option to specify output filename

EXETYPE    = elf


# ARM Libraries to be linked, Assumes ARMLIB is set
CLIB    = "$(RVCT22LIB)"/armlib/c_t__un.l  # ARM embedded ANSI C library
FLIB    = "$(RVCT22LIB)"/armlib/g_t_p.l#        # ARM embedded software floating 
                                               # point library
MLIB    = "$(RVCT22LIB)"/armlib/m_t_pu.l#       # ARM embedded software math 

ARMLIBS :=  $(CLIB) $(FLIB) $(MLIB)

# MODIFY HERE FOR EXTRA LINKER ARGUMENTS
LFLAGS = $(SCATTER) --datacompressor off  $(MAP) $(INFO) $(ENTRY) $(LIST) $(SYM)

###############################################################################
#
#  Section 3: Configuration settings - Check for Debug/Release mode
#
###############################################################################

ifeq ($(CURRENT_CONFIG), RELEASE)
	export OUTPUTDIR=objRelease
else
	export OUTPUTDIR=objDebug
endif


###############################################################################
#
#  Section 4: Platform directory tweak - Reads all the include paths from SFBConfig.gmk
#
###############################################################################

# make all paths to absolute paths
SHPROOT := $(subst \,/,$(SHPROOT))
ifneq ($(NET_COMPILE), yes)
SFB_SHPAPP_DIR := $(SHPROOT)\\ShpApp2
else
SFB_SHPAPP_DIR := $(ROOTDIR)\\ShpApp2
endif

ifneq ($(NET_COMPILE), yes)
OEM_H_DIR := $(subst $(SHP_BASE_DIR),$(SHPROOT)\\SHP3,$(OEM_H_DIR))
OEM_H_DIR := $(subst ./Target6280, $(SHPROOT)\\ShpTarget\\Target6280, $(OEM_H_DIR))
OEM_H_DIR := $(subst ../SMP6280, $(SHPROOT)\\SMP6280, $(OEM_H_DIR)) 

SYS_H_DIR := $(subst $(SHP_BASE_DIR),$(SHPROOT)\\SHP3,$(SYS_H_DIR))
SYS_H_DIR := $(subst ../ShpApp2,$(SFB_SHPAPP_DIR),$(SYS_H_DIR))
SYS_H_DIR := $(subst ./Target6280, $(SHPROOT)\\ShpTarget\\Target6280, $(SYS_H_DIR))
SYS_H_DIR := $(subst ../SMP6280, $(SHPROOT)\\SMP6280, $(SYS_H_DIR)) 

GWES_H_DIR := $(subst $(SHP_BASE_DIR),$(SHPROOT)\\SHP3,$(GWES_H_DIR))
GWES_H_DIR := $(subst ../ShpApp2,$(SFB_SHPAPP_DIR),$(GWES_H_DIR))
GWES_H_DIR := $(subst ./Target6280, $(SHPROOT)\\ShpTarget\\Target6280, $(GWES_H_DIR))
GWES_H_DIR := $(subst ../SMP6280, $(SHPROOT)\\SMP6280, $(GWES_H_DIR)) 

APP_H_DIR := $(subst $(SHP_BASE_DIR),$(SHPROOT)\\SHP3,$(APP_H_DIR))
APP_H_DIR := $(subst ../ShpApp2,$(SFB_SHPAPP_DIR),$(APP_H_DIR))
APP_H_DIR := $(subst ./Target6280, $(SHPROOT)\\ShpTarget\\Target6280, $(APP_H_DIR))
APP_H_DIR := $(subst ../SMP6280, $(SHPROOT)\\SMP6280, $(APP_H_DIR)) 
APP_H_DIR := $(subst ../ShpBrowser, $(SHPROOT)\\ShpBrowser, $(APP_H_DIR)) 
else

OEM_H_DIR := $(subst $(SHP_BASE_DIR),$(ROOTDIR)\\SHP3,$(OEM_H_DIR))
OEM_H_DIR := $(subst ./Target6280, $(ROOTDIR)\\ShpTarget\\Target6280, $(OEM_H_DIR))
OEM_H_DIR := $(subst ../SMP6280, $(ROOTDIR)\\SMP6280, $(OEM_H_DIR)) 

SYS_H_DIR := $(subst $(SHP_BASE_DIR),$(ROOTDIR)\\SHP3,$(SYS_H_DIR))
SYS_H_DIR := $(subst ../ShpApp2,$(SFB_SHPAPP_DIR),$(SYS_H_DIR))
SYS_H_DIR := $(subst ./Target6280, $(ROOTDIR)\\ShpTarget\\Target6280, $(SYS_H_DIR))
SYS_H_DIR := $(subst ../SMP6280, $(ROOTDIR)\\SMP6280, $(SYS_H_DIR)) 

GWES_H_DIR := $(subst $(SHP_BASE_DIR),$(ROOTDIR)\\SHP3,$(GWES_H_DIR))
GWES_H_DIR := $(subst ../ShpApp2,$(SFB_SHPAPP_DIR),$(GWES_H_DIR))
GWES_H_DIR := $(subst ./Target6280, $(ROOTDIR)\\ShpTarget\\Target6280, $(GWES_H_DIR))
GWES_H_DIR := $(subst ../SMP6280, $(ROOTDIR)\\SMP6280, $(GWES_H_DIR)) 

APP_H_DIR := $(subst $(SHP_BASE_DIR),$(ROOTDIR)\\SHP3,$(APP_H_DIR))
APP_H_DIR := $(subst ../ShpApp2,$(SFB_SHPAPP_DIR),$(APP_H_DIR))
APP_H_DIR := $(subst ./Target6280, $(ROOTDIR)\\ShpTarget\\Target6280, $(APP_H_DIR))
APP_H_DIR := $(subst ../SMP6280, $(ROOTDIR)\\SMP6280, $(APP_H_DIR)) 
APP_H_DIR := $(subst ../ShpBrowser, $(ROOTDIR)\\ShpBrowser, $(APP_H_DIR)) 


endif
###############################################################################
#
#  Section 5: Common includes to all the projects under build system
#
###############################################################################

ifneq ($(NET_COMPILE), yes)
BROWSER_INCLUDES := \
	-I"$(SHPROOT)\\ShpBrowser\\H" \
	-I"$(SHPROOT)\\ShpBrowser\\Application\\H" \
	-I"$(SHPROOT)\\ShpBrowser\\Platform\\H" \
	-I"$(SHPROOT)\\ShpBrowser\\Core\\H" \
	-I"$(SHPROOT)\\ShpBrowser\\Solutions\\sfb\\bal\\h" \
	-I"$(SHPROOT)\\ShpBrowser\\Solutions\\sfb\\Utility\\h" \
	-I"$(SHPROOT)\\ShpBrowser\\Plugin\\h" \
	-I"$(SHPROOT)\\ShpBrowser\\Webcontrol\\H" \
	-I"$(SHPROOT)\\ShpBrowser\\UrlLoader\\H" \
	-I"$(SHPROOT)\\ShpBrowser\\WebControl\\src\\SFB" \
	-I"$(SHPROOT)\\SHP3\\Security\\PKI\\CertMgr\\H" \
	-I"$(SHPROOT)\\SHP3\\AVMS\\ImgCodec"	\
	-I"$(SHPROOT)\\SHP3\\AppSvc\\Drm\\OMADRM\\H"	\
	-I"$(SHPROOT)\\shpGWES\\H\\CoreH"	\
	-I"$(SHPROOT)\\shpGWES\\H\\WinSetH"	\
	-I"$(SHPROOT)\\shpGWES\\AppMgr\\H"	\
	-I"$(SHPROOT)\\shpGWES\\RsrcMgr\\H"	\
	-I"$(SHPROOT)\\SHP3\\AVMS\\ImgCodec"
else
BROWSER_INCLUDES := \
	-I"$(ROOTDIR)\\ShpBrowser\\H" \
	-I"$(ROOTDIR)\\ShpBrowser\\Application\\H" \
	-I"$(ROOTDIR)\\ShpBrowser\\Platform\\H" \
	-I"$(ROOTDIR)\\ShpBrowser\\Core\\H" \
	-I"$(ROOTDIR)\\ShpBrowser\\Solutions\\sfb\\bal\\h" \
	-I"$(ROOTDIR)\\ShpBrowser\\Solutions\\sfb\\Utility\\h" \
	-I"$(ROOTDIR)\\ShpBrowser\\Plugin\\h" \
	-I"$(ROOTDIR)\\ShpBrowser\\Webcontrol\\H" \
	-I"$(ROOTDIR)\\ShpBrowser\\UrlLoader\\H" \
	-I"$(ROOTDIR)\\ShpBrowser\\WebControl\\src\\SFB" \
	-I"$(ROOTDIR)\\SHP3\\Security\\PKI\\CertMgr\\H" \
	-I"$(ROOTDIR)\\SHP3\\AVMS\\ImgCodec"	\
	-I"$(ROOTDIR)\\SHP3\\AppSvc\\Drm\\OMADRM\\H"	\
	-I"$(ROOTDIR)\\shpGWES\\H\\CoreH"	\
	-I"$(ROOTDIR)\\shpGWES\\H\\WinSetH"	\
	-I"$(ROOTDIR)\\shpGWES\\AppMgr\\H"	\
	-I"$(ROOTDIR)\\shpGWES\\RsrcMgr\\H"	\
	-I"$(ROOTDIR)\\SHP3\\AVMS\\ImgCodec"
endif


ifneq ($(NET_COMPILE), yes)
SHP_INCLUDES = \
	-I"$(SHPROOT)\\ShpTarget\\H" \
	-I"$(SHPROOT)\\ShpTarget\\OEM\\H" \
	$(OEM_H_DIR) \
       	$(SYS_H_DIR) \
	$(GWES_H_DIR) \
	$(APP_H_DIR)  \
	$(BROWSER_INCLUDES)
else
SHP_INCLUDES = \
	-I"$(ROOTDIR)\\ShpTarget\\H" \
	-I"$(ROOTDIR)\\ShpTarget\\OEM\\H" \
	$(OEM_H_DIR) \
       	$(SYS_H_DIR) \
	$(GWES_H_DIR) \
	$(APP_H_DIR)  \
	$(BROWSER_INCLUDES)
endif

###############################################################################
#
#  Section 6: Common Defines - Defines for all the projects
#
###############################################################################

COMMON_DEFINES = -DPACKED=__packed \
	-DHAVE_CONFIG_H \
	-D__STD_C \
       	-DUSE_NATIVE_WIDGET_SUPPORT \
	-D__CMM__ \
	-DSHP_SUPPORT_WML \
	-DWEBCORE_WCSS_INTEGRATION \
	-DUSE_SYSTEM_MALLOC \
	-DUSE_FAST_MALLOC \
	-DENABLE_XPATH \
	-DCELUNITE_PORT \
	-DSMART_FOCUSING \
	-DWBXML_SUPPORT \
	-DSUPPORT_TOUCH_SCREEN \
	-DSUPPORT_FOUR_WAY_NAVIGATION \
	-DSFB_SUPPORT_IMAGE_ZOOM \
	-DHTTP_POST_UPLOAD_SUPPORT \
	-DBUILDING_WEBKIT \
	-D__SHP__ \
	-D_SFB_SEARCH_STRING_IN_PAGE	\
	-DSFB_PROGRESSIVE_RENDERING \
	-DENABLE_FOCUS_RING_DRAWING \
	-DSMART_FOCUSING \
	-DSUPPORT_NP_PLUGIN \
	-DSUPPORT_AUTO_FORM_FILL \
	-D_SFB_PAGE_PREVIEW \
	-DSFB_SSR_RENDERING \
	-DSFB_GLOBAL_STATIC_HANDLING \
	-DMULTIPLE_INSTANCE_SUPPORT \
	-DSFB_BROWSER_NTOUCH_UI \
	-D_SFB_BACKINGSTORE \
	-DSFB_GWES_TRANSFORM \
	-DSFB_WIDGET \
	-DSFB_UPDATE_URL_BAR \
	-DUSE_SHP_TEMPLATE_MIN_MAX \
	-DSUPPORT_SMART_TOUCH_FOCUSING \
	-DJS_MULTIPLE_WINDOW_SUPPORT \
        -DUSE_SMALL_SCREEN_LAYOUT \
        -DUSE_LIST_BOX_SUPPORT \
	-DUSE_NATIVE_WIDGET_SUPPORT \
	-DJASMINE_PERFORMANCE_TWEAKS

# Flags related ONLY to ECOM must be added below		 
ifeq ($(findstring -D_JASMINE_ECOM, $(TARGET_BUILD_SHP_OPTION)), -D_JASMINE_ECOM)
COMMON_DEFINES += -D_SHP_DLL_WEBCONTROL -D_SHP_DLL -DUSING_URLLOADER -D_JASMINE_ECOM -D_SHP_MY_DID=0x$(DID)

COMMON_DEFINES += -D_SFBAL_SUPPORT_VECTOR_FONT -DMULTIPLE_INSTANCE_SUPPORT -DSUPPORT_NP_PLUGIN
endif

# Added below flag to undefine CUST rex include dirs. REQUIRED
COMMON_DEFINES += -UCUST_H

# Undefine the following flags 
COMMON_DEFINES += -USUPPORT_VIRTUAL_CURSOR -U_SFB_PAGE_PREVIEW
