#!/usr/local/bin/perl -w

#!/usr/bin/perl

#
#
# Last updated: $Date$
# Author:  $Author$
# Revision: $Rev$
#
# Using the perl script:
#
# perl parser.pl <vcproj file location>
#
# Changelog:
#
# 21/09/07: 
# Changed logic to extract include path only from Debug|Win32
# Fixed error: "Filter less vcproj file generated wrongly"
# Added a comment generation logic for Makefile
# Logic added to get unique elements in the FILTERLIST
# Fixed bug: "Filter less vcproj generate empty make file rules"
# Added Macro definition, updated the time and username generation
# Fixed bug in Filter. Added Name attribute also
# Added a feature to add "" to the include path if not exists
#

use XML::Parser;
use File::Basename;

# Makefile property declarations
$VCFILEPATH_ATT = "RelativePath";
$VCCONFIG_NAME = "Debug";
$COMPILER = "armcc";
$LOADER = "armlink";
$ARCHIVER = "armar";
$RMCMD = "/bin/rm -f";

# Input filename
my $file = shift;
die "Can't find file \"$file\""
  unless -f $file;

# Output filename
my $output = shift;

# Generate the makefile on the same path as vcproj file
# Extract the project name, path, extension from filepath
($project, $path, $extens) = fileparse($file, qr/\.[^.]*/);

# Initialise the output files
$path .= "\\";
$prebuild = "prebuild.bat";
$postbuild = "postbuild.bat";

# create object
my $parser = new XML::Parser(ErrorContext => 2);

$parser->setHandlers(Char => \&char_handler,
			Start => \&start_handler);

# parse XML file
$parser->parsefile($file);

# Create files
# BUG: Will not work with already exisiting gmk files
open (MAKEFILE, ">".$output);
if($PRESCRIPTDATA)
{
open (PRESCRIPT, ">".$prebuild);
}
if($POSTSCRIPTDATA)
{
open (POSTSCRIPT, ">".$postbuild);
}

# Get the environment variables
$SHPSDKPATH = $ENV{'ShpSdkPath'};
$SHPWEBKITOUTPUT = $ENV{'ShpWebkitOutputDir'};
$SHPWEBKITLIBS = $ENV{'ShpWebkitLibrariesDir'};

#Change WebKitOutputDir (if exist) to system OutputDir
#$INCLUDEDIRS =~ s/\$\(WebKitOutputDir\)/$SHPWEBKITOUTPUT/g;
#$INCLUDEDIRS =~ s/\$\(WebKitLibrariesDir\)/$SHPWEBKITLIBS/g;
$SHPSDKPATH =~ s/\\/\\\\/g;
$SHPWEBKITOUTPUT =~ s/\\/\\\\/g;
$SHPWEBKITLIBS =~ s/\\/\\\\/g;

#Replace the ShpDirs with environment variable
$INCLUDEDIRS =~ s/\$\(ShpWebKitOutputDir\)/$SHPWEBKITOUTPUT/g;
$INCLUDEDIRS =~ s/\$\(ShpSdkPath\)/$SHPSDKPATH/g;
$INCLUDEDIRS =~ s/\$\(ShpWebKitLibrariesDir\)/$SHPWEBKITLIBS/g;
$INCLUDEDIRS =~ s/\$\(ProjectDir\)//g;

# Get Date
($Second, $Minute, $Hour, $Day, $Month, $Year, $WeekDay, $DayOfYear, $IsDST) = localtime(time);
$Year += 1900;
$Month++;
$date = "$Day/$Month/$Year $Hour:$Minute";

# Logic to extract unique filter names

@FILTER_ARR = split(/ /,$FILTERLIST);
foreach my $elem ( @FILTER_ARR )
{
	next if $seen{ $elem }++;
	push @UNIQUE_FILTER, $elem;
}
$FILTERLIST = join( " ", @UNIQUE_FILTER);

@VPATH_ARR = split(/\:/,$VPATH);
foreach my $elem ( @VPATH_ARR )
{
	next if $seen{ $elem }++;
	push @UNIQUE_VPATH, $elem;
}
$VPATH = join( ":", @UNIQUE_VPATH);

# Remove Shp header file include path 
$INCLUDEDIRS =~ s/ShpApp2\\\\Target\\\\C501\\\\H\\\\Main/ShpTarget\\H/g;

# Remove _SHP_SIMUL Define from MACROS
$MACROS =~ s/-D_SHP_SIMUL//g;

# Add PACKED=_packed for RVCT compile
#$MACROS .= "-DPACKED=__packed";

# Adding common defines
$MACROS .= "\$\(COMMON_DEFINES\)";

# Set Output dir
$OUTPUTDIR = $project;

# Set lib name
$LIBNAME = $project .".a";

# Write makefile

print MAKEFILE "# Makefile for $file\n";
print MAKEFILE "# Created on $date\n";
print MAKEFILE "# Author: ".getlogin()."\n";
print MAKEFILE "# SISO, Copyright 2008.\n";
print MAKEFILE "############################################################################### \n\n";
print MAKEFILE "include \$\(MAINDIR\)/mkrules.inc\n\n";
print MAKEFILE "# Target specific variables\n";
print MAKEFILE "############################################################################### \n\n";
print MAKEFILE "TARGET := ".$LIBNAME."\n";
print MAKEFILE "TARGET_BUILD_PATH := \$\(TOPOUTPUTDIR\)/".$OUTPUTDIR."\n\n";

print MAKEFILE "# Source Search paths\n";
print MAKEFILE "############################################################################### \n\n";
print MAKEFILE "VPATH :=".$VPATH."\n\n";

print MAKEFILE "# Object files \n";
print MAKEFILE "############################################################################### \n\n";
print MAKEFILE "$FILELIST\n";
$FULL="OBJECTS := ";
foreach(@UNIQUE_FILTER)
{
	$FULL.="\$\(".$_."_OBJECTS\) \\\n\t";
}
print MAKEFILE $FULL." \n\n";

print MAKEFILE "# Include paths \n";
print MAKEFILE "############################################################################### \n\n";
# add SHP_INCLUDES to include dir
$INCLUDEDIRS .= "\t\$(SHP_INCLUDES)";
print MAKEFILE "INCLUDES := ".$INCLUDEDIRS."\n\n";

print MAKEFILE "# Conditional defines\n";
print MAKEFILE "############################################################################### \n\n";
print MAKEFILE "DEFINES := ".$MACROS."\n\n";

print MAKEFILE "# Build rules\n";
print MAKEFILE "############################################################################### \n\n";
print MAKEFILE "include \$\(MAINDIR\)/targetrules.inc\n";

# Write pre-build script
#print PRESCRIPT "\n";

#Change WebKitOutputDir (if exist) to system OutputDir

#$SCRIPTDATA =~ s/\$\(WebKitOutputDir\)/$ENV{'ShpWebkitOutputDir'}/g;
#$SCRIPTDATA =~ s/\$\(WebKitLibrariesDir\)/$ENV{'ShpWebkitLibrariesDir'}/g;

# TODO: Need a good regexp to get the ProjectDir
#$projectdir = $ENV{'ShpWebkitHome'}."\/WebCore\/";
#$SCRIPTDATA =~ s/\$\(ProjectDir\)//g;

$PRESCRIPTDATA =~ s/\$\(ProjectDir\)//g;
$PRESCRIPTDATA =~ s/\$\(ConfigurationName\)/Debug/g;
$PRESCRIPTDATA =~ s/\$\(WebKitConfigSuffix\)/_debug/g;

$POSTSCRIPTDATA =~ s/\$\(ProjectDir\)//g;

if($PRESCRIPTDATA)
{
 print PRESCRIPT $PRESCRIPTDATA;
}

if($POSTSCRIPTDATA)
{
 print POSTSCRIPT $POSTSCRIPTDATA;
}


############################
# Handlers for XML parser  #
############################


sub start_handler
{
my $p = shift;
my $el = shift;

# $p - Expat Parser
# $el - Element
while (@_)
{
	my $att = shift;
	my $data = shift;

	# $att - Attribute
	# $data - Value

        if ($el eq "VisualStudioProject" and $att eq "Name")
        {
            $PROJECTNAME = $data;
        }
	if ($el eq "Filter" and $att eq "Name")
	{
		# Append the filter named variable to the FILELIST
		$FILELIST .= "\n\n$data\_OBJECTS += ";

		# Extract the filter list to store in MODULES variable
		$FILTERLIST .= $data." ";	
	}
	#print "		Attribute: $att Data: $data\n ";
	if ($att eq $VCFILEPATH_ATT) {
		if($data =~ /\s*\.c[a-z]*/)
		{
			$data =~ s/\\/\//g;
			if(!defined($FILTERLIST))
			{
				$FILELIST.= "OBJECTS += ";
				$FILTERLIST = "";
			}
			# BUG: Files which are filter less are getting added under the last filter
			# Extract the object names instead of CPP file names
			($filename, $fpath, $fextens) = fileparse($data, qr/\.[^.]*/);
                        $fpath =~ s/\$\(ProjectName\)/$PROJECTNAME/g;
			$VPATH .= $fpath.":";
			$FILELIST .= "\$\(TARGET_BUILD_PATH\)\/".$filename. ".o \\\n\t";
		}
	}
	if ($el eq "Configuration" and $att eq "Name")
	{
		if($data eq "Debug|Win32")
		{
			print $data;
			$FLAG_CONF = 1;
		}
	}

	# Read under the Debug | win32 configuration the following attributes
	if( $att eq "AdditionalIncludeDirectories" and $FLAG_CONF == 1)
	{
		@INCLIST = split(/;/, $data);
		foreach (@INCLIST) 
		{
			$_ =~ s/\\/\\\\/g;
			$_ =~ s/\$\(ConfigurationName\)/$VCCONFIG_NAME/g;
                        # ignore includepaths with \shp3\ on it
                        if($_ =~ m/\\[sS][hH][Pp]3\\/)
                        {
                            next;
                        }
                        
                        # ignore includepaths with \shptarget\ on it
                        if($_ =~ m/\\[sS][hH][Pp][tT][aA][rR][gG][eE][tT]\\/)
                        {
                            next;
                        }

                        # ignore includepaths with \shpapp2\ on it
                        if($_ =~ m/\\[sS][hH][Pp][aA][pP][pP]2\\/)
                        {
                            next;
                        }

                        # ignore includepaths with \shprsrc\ on it
                        if($_ =~ m/\\[sS][hH][Pp][rR][sS][rR][cC]\\/)
                        {
                            next;
                        }

                        #check if includepath covered with double quotes if not add it
			if ($_ =~ m/".*"/)
			{
				$INCLUDEDIRS .= "\t-I".$_. " \\\n";
			}
			else
			{
				$INCLUDEDIRS .= "\t-I\"".$_. "\" \\\n";
			}
		}
	}

	if( $att eq "PreprocessorDefinitions" and $FLAG_CONF == 1)
	{
		@DEFINELIST = split(/;/, $data);
		foreach (@DEFINELIST) 
		{
			$MACROS .= "-D".$_. " ";
		}
	}
	if( $el eq "Tool" and $att eq "Name" and $FLAG_CONF ==1 and $data eq "VCPreBuildEventTool")
	{
		$FLAG_PRESCRIPT = 1;
	}

	if( $el eq "Tool" and $att eq "Name" and $FLAG_CONF ==1 and $data eq "VCPostBuildEventTool")
	{
		$FLAG_POSTSCRIPT = 1;
		# make FLAG_CONF when finishing with configuration tag
		$FLAG_CONF = 0;		
	}

	if ( $att eq "CommandLine" and $FLAG_PRESCRIPT == 1)
	{
		$PRESCRIPTDATA = $data;
		$FLAG_PRESCRIPT = 0;
	}

	if ( $att eq "CommandLine" and $FLAG_POSTSCRIPT == 1)
	{
		$POSTSCRIPTDATA = $data;
		$FLAG_POSTSCRIPT = 0;
	}

}
}

sub end_handler
{
	my $p = shift;
	my $el = shift;
}

sub char_handler
{

}



