/*
 * This file is part of the DOM implementation for KDE.
 *
 * Copyright (C) 2005 Apple Computer, Inc.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public License
 * along with this library; see the file COPYING.LIB.  If not, write to
 * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA 02110-1301, USA.
 *
 */
#ifndef QualifiedName_h
#define QualifiedName_h

#include "AtomicString.h"

#if PLATFORM(SHP)
#include "CMMDefs.h"
#endif // PLATFORM(SHP)

namespace WebCore {


typedef Vector<String> HTMLSrcString ;


class QualifiedName 
    CMM_MODIFIED
{
public:
    class QualifiedNameImpl : public RefCounted<QualifiedNameImpl> 
        CMM_MODIFIED_ADD
    {
    public:
        QualifiedNameImpl(const AtomicString& p, const AtomicString& l, const AtomicString& n)
            : RefCounted<QualifiedNameImpl>(0)
            , m_prefix(p)
            , m_localName(l)
            , m_namespace(n)
        {
        }

#ifdef SFB_GLOBAL_STATIC_HANDLING
        void reInit()
        {
            m_prefix.reInitString();
            m_localName.reInitString();
            m_namespace.reInitString();
        }
#endif
        AtomicString m_prefix;
        AtomicString m_localName;
        AtomicString m_namespace;
    };

    QualifiedName(const AtomicString& prefix, const AtomicString& localName, const AtomicString& namespaceURI);
    ~QualifiedName();
#ifdef QNAME_DEFAULT_CONSTRUCTOR
    QualifiedName() : m_impl(0) { }
#endif

    QualifiedName(const QualifiedName&);
    const QualifiedName& operator=(const QualifiedName&);

#ifdef SFB_GLOBAL_STATIC_HANDLING

#ifndef QNAME_DEFAULT_CONSTRUCTOR
    QualifiedName() : m_impl(0) { }
#endif

    void reInit()
    {
        m_impl = NULL;
    }
#endif

    bool operator==(const QualifiedName& other) const { return m_impl == other.m_impl; }
    bool operator!=(const QualifiedName& other) const { return !(*this == other); }

    bool matches(const QualifiedName& other) const { return m_impl == other.m_impl || (localName() == other.localName() && namespaceURI() == other.namespaceURI()); }

    bool hasPrefix() const { return m_impl->m_prefix != nullAtom; }
    void setPrefix(const AtomicString& prefix);

    const AtomicString& prefix() const { return m_impl->m_prefix; }
    const AtomicString& localName() const { return m_impl->m_localName; }
    const AtomicString& namespaceURI() const { return m_impl->m_namespace; }

    String toString() const;
    bool toString(HTMLSrcString &htmlSrc) const;

    QualifiedNameImpl* impl() const { return m_impl; }
    
    // Init routine for globals
    static void init();

private:
    void ref() { m_impl->ref(); }
    void deref();
    
    QualifiedNameImpl* m_impl;
};

#ifndef WEBCORE_QUALIFIEDNAME_HIDE_GLOBALS
extern const QualifiedName anyName;
inline const QualifiedName& anyQName() { return anyName; }
#endif

inline bool operator==(const AtomicString& a, const QualifiedName& q) { return a == q.localName(); }
inline bool operator!=(const AtomicString& a, const QualifiedName& q) { return a != q.localName(); }
inline bool operator==(const QualifiedName& q, const AtomicString& a) { return a == q.localName(); }
inline bool operator!=(const QualifiedName& q, const AtomicString& a) { return a != q.localName(); }

}
#endif
