/*
 * Copyright (C) 1999 Lars Knoll (knoll@kde.org)
 *           (C) 1999 Antti Koivisto (koivisto@kde.org)
 * Copyright (C) 2004, 2005, 2006, 2007 Apple Inc. All rights reserved.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public License
 * along with this library; see the file COPYING.LIB.  If not, write to
 * the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 *
 */
#if defined(SHP_SUPPORT_WML)
#include "config.h"
#include "WMLCardElement.h"

#include "HTMLDocument.h"
#include "HTMLElementFactory.h"
#include "HTMLNames.h"
#include "RenderObject.h"

#include "Frame.h"
#include "FrameLoader.h"
#include "HistoryItem.h"
#include "Page.h"
#include "WMLDocument.h"
#include "WMLDoElement.h"
#include "WMLDocument.h"

namespace WebCore {

using namespace HTMLNames;

WmlCardElement::WmlCardElement(Document* doc)
:HTMLElement(cardTag, doc)
,m_bIsEventAlreadyHandled(false)
{
}

WmlCardElement::~WmlCardElement()
{
}

String WmlCardElement::newcontext ()
{
	return getAttribute(newcontextAttr) ;
}

String WmlCardElement::ordered ( )  const
{
    return getAttribute(orderedAttr);
}

String WmlCardElement::title()const
{
    String title;
    String* domStr = document()->frame()->getVariableReplacedString(getAttribute(titleAttr).domString());
    if(domStr)
	    title = domStr->copy();
    
    return title;
}

String WmlCardElement::onenterbackward( )const
{
    return getAttribute(onenterbackwardAttr);
}

String WmlCardElement::onenterforward ( )const
{
    return getAttribute(onenterforwardAttr);
}

String WmlCardElement::ontimer ( )const
{
    return getAttribute(ontimerAttr);
}

void WmlCardElement::setontimer ( const String &value ) 
{
	setAttribute(ontimerAttr, value);
}

RenderObject* WmlCardElement::createRenderer(RenderArena* arena, RenderStyle* style)
{
	RenderObject *obj = HTMLElement::createRenderer(arena, style);

	WmlCardElement *pCard = static_cast<WmlCardElement *>(obj->node());
	if(pCard->id() == document()->getActiveCardId())
	{
		obj->setShouldRender(true);
		document()->setCardTitle(title());
	}
	else
	{
		obj->setShouldRender(false);
	}

	return obj;
}

void WmlCardElement::handleEvent(EventType eventType, bool bForwardNavigation, bool & bIsEventHandled)
{
	if(m_bIsEventAlreadyHandled)
	{
		return; //We reached loop while processing event, eg: card1 -> card2 -> card3 -> card1
		        //Report error to Frame loader client. For now just returning.
	}
	if(document()->getActiveCardId() != id())
	{
		return; //This card is not the correct recipent of the event, so do not process
	}
	

	if(eventType == RefreshTaskEnterEvent)
	{
		//Pause the timers and commit the timer values
		for(Node *pChild = firstChild();pChild;pChild = pChild->nextSibling())
		{
			if(pChild->hasTagName(timerTag))
			{
				pChild->handleEvent(eventType,bForwardNavigation, bIsEventHandled);
			}
		}
		m_bIsEventAlreadyHandled = false;
		return;
	}
	if(eventType == RefreshTaskExitEvent)
	{
		//restart the timers with the committed values
		for(Node *pChild = firstChild();pChild;pChild = pChild->nextSibling())
		{
			if(pChild->hasTagName(timerTag))
			{
				pChild->handleEvent(eventType,bForwardNavigation, bIsEventHandled);
			}
		}
		m_bIsEventAlreadyHandled = false;
		return;
	}
	else
	{
		
		//new code
		m_bIsEventAlreadyHandled = true;

		if(NavigationEvent == eventType)
		{
		    if(newcontext() == "true")
			{
				((WMLDocument *)document())->BrowserContext()->ResetBrowserContext();
			}
			
			if((!(onenterforward().isNull())) && bForwardNavigation)
			{
				bIsEventHandled = true;
			}
			if((!(onenterbackward().isNull())) && !bForwardNavigation)
			{
				bIsEventHandled = true;
			}
		}
		
		else if((TimerEvent == eventType) &&
			    (!ontimer().isNull()))
		{
			bIsEventHandled = true;
		}
			

		//Check for onenterforward and onenterbackward here
		//In case of card, eventType will be NavigationEvent only, no need to check
		String url;
		if((!ontimer().isNull())&& eventType == TimerEvent)
		{
			url = ontimer();
		}
		else if(bForwardNavigation)
		{
			url = onenterforward();
		}
		else
		{
			url = onenterbackward();
			if (!url.isEmpty())
				bForwardNavigation = true ;
		}

		if(!url.isNull())
		{
			int pos	= url.reverseFind('#');
			if(pos != -1)
			{
				String cardId = url.substring(pos+1);
				
				//create a History item with url of next card, and then set new card id.
				//RefPtr<HistoryItem> item = document()->frame()->page()->mainFrame()->loader()->createHistoryItemTree(document()->frame(), true);
				//item->setURLString(url);
				//document()->frame()->page()->backForwardList()->addItem(item);
				String completeUrl = "#";
				//completeUrl.append(document()->getActiveCardId());
				completeUrl.append(cardId);
				completeUrl = document()->completeURL(completeUrl);
				document()->setURL(completeUrl.deprecatedString());
				if(document()->frame()->loader()->isForwardNavigation())
				{
					((WMLDocument *)(document()))->BrowserContext()->addHistoryItem(completeUrl);
				}
				// now that the entry to the history has been made replace the card id
				document()->setActiveCardId(cardId);
			}
			else
			{
				String empty ;
				document()->frame()->loader()->urlSelected(document()->completeURL(url),empty, NULL, false, true);
			}
		}
		//Call handle event for all children of card
		for(Node *pChild = firstChild();pChild;pChild = pChild->nextSibling())
		{
			pChild->handleEvent(eventType,bForwardNavigation, bIsEventHandled);
		}
	}
}

void WmlCardElement::attach()
{
	if (attached())
		return ;

	if (document()->getActiveCardId() == id())	{
		HTMLCollection *collection = new HTMLCollection(this, HTMLCollection::DoTag) ;
		WMLDocument *wmldoc = static_cast<WMLDocument *>(document()) ;
		//NodeMap *nodeMap = wmldoc->getDoElementMap () ;
		//NodeMap::const_iterator end = nodeMap->end() ;
		//Node *endNode = 0 ;

		/*if (nodeMap->size() > 0)
			endNode = static_cast<Node *>(end->second) ;*/

		for(Node *node = collection->firstItem(); node; \
								node = collection->nextItem())	{
#if 0
			/* check whether we have any stuff that is already displayed here.
			in that case we need to remove that one and display another
			*/
			bool elementsRemoved = true ;
			WMLDoElement *wmlDoElem = static_cast<WMLDoElement *>(node) ;
			String name = wmlDoElem->name() ;
			String typeValue = wmlDoElem->type () ;
			WMLDoElement *pNode = 0;
			for (NodeMap::const_iterator it = nodeMap->begin(); endNode && it->second != endNode ; ++it)	{
				pNode = static_cast<WMLDoElement *>(it->second) ;
				ASSERT (pNode) ;
				if ( ((!name.isEmpty()) && name == pNode->name()) ||
					(name.isEmpty() && !typeValue.isEmpty() && typeValue == pNode->type() ) )	{
					Node *parent = pNode->parent() ;
					if (parent && parent->hasTagName(templateTag))
						wmldoc->removeDoElement (it->first) ;
				}
			}
#endif
			wmldoc->setDoElement (node) ;
		}
		/* here i get the wmlDoElement, present in the card, i need to add this to the document.
		Document API calls the frame API and it inturn calls the frameLoaderClient API.
		FrameLoaderClient passes the control to the UI using browserControl APIs
		I am not calling the same at the deck level bcos it adds all DoTag guys from 
		all the card guys
		*/
		delete collection ;
	}
	HTMLElement::attach() ;
}

void WmlCardElement::detach()
{
	HTMLElement::detach() ;
}

} //WebCore
#endif

