/*
 * This file is part of the DOM implementation for KDE.
 *
 * Copyright (C) 1999 Lars Knoll (knoll@kde.org)
 *           (C) 1999 Antti Koivisto (koivisto@kde.org)
 *           (C) 2001 Dirk Mueller (mueller@kde.org)
 * Copyright (C) 2004, 2005, 2006 Apple Computer, Inc.
 *           (C) 2006 Alexey Proskuryakov (ap@nypop.com)
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public License
 * along with this library; see the file COPYING.LIB.  If not, write to
 * the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 *
 */
#if defined(SHP_SUPPORT_WML) 

#include "config.h"
#include "WMLOptionElement.h"

#include "CSSStyleSelector.h"
#include "Document.h"
#include "EventNames.h"
#include "ExceptionCode.h"
#include "HTMLNames.h"
#include "HTMLSelectElement.h"
#include "RenderMenuList.h"
#include "Text.h"
#include <wtf/Vector.h>

#include "config.h"
#include "HTMLAnchorElement.h"

#include "CSSHelper.h"
#include "Document.h"
#include "Event.h"
#include "EventHandler.h"
#include "EventNames.h"
#include "Frame.h"
#include "FrameLoader.h"
#include "HTMLImageElement.h"
#include "HTMLNames.h"
#include "KeyboardEvent.h"
#include "MouseEvent.h"
#include "MutationEvent.h"
#include "RenderFlow.h"
#include "RenderImage.h"
#include "ResourceRequest.h"
#include "SelectionController.h"
#include "Settings.h"
#include "UIEvent.h"

#include "EventTargetNode.h"
#include "Event.h"
#include "EventTarget.h"
#include "EventListener.h"
#include "BackForwardList.h"
#include "Page.h"
#include "HistoryItem.h"
#include "FrameView.h"

#include "WmlCardElement.h"
#include "WMLSelectElement.h"
#include "WMLDocument.h"

namespace WebCore {

using namespace HTMLNames;
using namespace EventNames;

WMLOptionElement::WMLOptionElement(Document* doc, HTMLFormElement* f)
    : HTMLOptionElement(doc, f)
{
}

bool WMLOptionElement::checkDTD(const Node* newChild)
{
	return newChild->isTextNode() || newChild->hasTagName(scriptTag) || newChild->hasTagName(oneventTag);
}

void WMLOptionElement::setSelectedState(bool selected)
{
	//commenting below code as we have to handle events,
	//no matter whether option is already selected or not.
	//if (m_selected == selected)
    //   return;

	//Here, we need to do
	//1.Update the value for the variable of select tag in WMLBrowserContext.
	//  varible can be attribute value of name,iname etc.
	//2.Check for onpick attribute value and go to that url if present.
	//  Call children so that if any onevent with type="onpick", tasks inside it can be done
	WMLSelectElement *pSelect = getSelect();
	String selectNameAttr = pSelect->getAttribute(nameAttr) ;
	String selectInameAttr = pSelect->iName();
	//If selected, either Single or Multiple selection list
	if(selected)
	{
		if(!selectInameAttr.isEmpty())
		{
			//String varName = pSelect->iName();
			/*String varValue = getAttribute(valueAttr);
			//as option value is vdata, get replaced string
			varValue = String (((WMLDocument *)document())->getVariableReplacedString (varValue.impl(), WMLBrowserContext::ENoEscape)) ;
			*/
			//using varValue find the option index
			String varValue = valueAttribute() ;
			int index = pSelect->getOptionIndex(varValue);
			varValue = varValue.number(index);
			updateVariableValue(selectInameAttr, varValue, pSelect);
		}
		if(!selectNameAttr.isEmpty())
		{
			//String varName = pSelect->name();
			/*String varValue = getAttribute(valueAttr);
			//as option value is vdata, get replaced string
			varValue = String (((WMLDocument *)document())->getVariableReplacedString (varValue.impl(), WMLBrowserContext::ENoEscape)) ;
			*/
			updateVariableValue(selectNameAttr, valueAttribute(), pSelect);
		}
	}
	else if(!selected && pSelect->multiple()) //Deselection in multi select list
	{
		String varValue = valueAttribute();
		//Remove the option value that is getting deselected, from variable value and update the value in context
		//In case of single selection list, option value overwrites existing value, so need to take care
		if(!selectInameAttr.isEmpty())
		{
			//String varName = pSelect->getAttribute(inameAttr);
			//as option value is vdata, get replaced string
			//varValue = String (((WMLDocument *)document())->getVariableReplacedString (varValue.impl(), WMLBrowserContext::ENoEscape)) ;
			//using varValue find the option index
			//String varValue = valueAttribute();
			int index = pSelect->getOptionIndex(varValue);
			varValue = varValue.number(index);
			removeOptionFromVariableValue(selectInameAttr, varValue);
		}
		if(!selectNameAttr.isEmpty()) //check for name attr
		{
			//String varName = pSelect->getAttribute(nameAttr);
			//varValue = String (((WMLDocument *)document())->getVariableReplacedString (varValue.impl(), WMLBrowserContext::ENoEscape)) ;
			removeOptionFromVariableValue(selectNameAttr, varValue);
		}
	}

	if(!pSelect->m_recalcListItems && !pSelect->m_inRestore) //When recalculation of list items is in progress, events should not be processed.
	{
		//When option is selected need to check if any events to be handled
		if((selected ) ||   //Single/multiple select: selecting option
		   (!selected && getSelect()->multiple())) //Multiple select: deselecting option,in this case also need to process onpick event
		{
			String pickAttrValue = onpick();
			//Do the task, when onpick attribute is present for option.
			if(!(pickAttrValue.isNull()))
			{
				//Get onpick attribute value of option
 				String url = parseURL(onpick());
				//Get target url to which we want to navigate to
				String targetUrl = document()->completeURL(url);

				//It may be link like #card3, remove first character, get card id, card3 in this case

				int pos = url.find('#');
				if(pos == -1) // # not found in url, means it can be normal url like google.com
				{
					String empty ;
					document()->frame()->loader()->urlSelected(targetUrl, empty, 0, false, true);
					m_selected = selected;
					setChanged();
					return;
				}
				else if(pos == 0) // first char is '#', so it is local card
				{
					url.remove(0,1);
				}
				else   //For case like samepage.wml#card2, we should just set active id.
				{
					//Get current url from document
					String docUrl = document()->url();
					String docUrlPrefix = "";  //holds document url excluding card id
					int revpos;
					if((revpos = docUrl.reverseFind('#')) == -1)
					{
						docUrlPrefix = docUrl;
					}
					else
					{
						docUrlPrefix = docUrl.substring(0,revpos);
					}

					String targetUrlPrefix = "";  //holds target url excluding card id
					if((revpos = targetUrl.reverseFind('#')) == -1)
					{
						targetUrlPrefix = targetUrl.substring(0,revpos);
					}
					else
					{
						targetUrlPrefix = targetUrl;
					}

					if(docUrlPrefix != targetUrlPrefix) //Check whether request is with in the page or not
					{
						String empty ;
						document()->frame()->loader()->urlSelected(targetUrl, empty, 0, false, true);
						m_selected = selected;
						setChanged();
						return;
					}
					else //card in same page, set active id
					{
						url = url.substring(pos+1);
					}
				}
				

				//set active id first and start from event handling from that card
				document()->setActiveCardId(url);
				document()->frame()->loader()->processEventsAndFindDesiredCard(NavigationEvent, true);//Passing true as it is forward nav
			}
			else  //onpick attrib is not present for option, process handleEvent for children like onevent.
			{
     			//When an option is selected, if it has any onevent with type="onpick", we need to do the tasks inside onevent.
				//Call children handle event to process onevent if any
				for(Node *pChild = firstChild();pChild; pChild = pChild->nextSibling())
				{
					bool bIsEventHandled = false;
					pChild->handleEvent(PickEvent,true, bIsEventHandled);//Passing true as it is forward navigation.
				}
			}
		}
	}

	m_selected = selected;
    setChanged();
}

String WMLOptionElement::onpick() const
{
	String value ;
	String *temp = document()->frame()->getVariableReplacedString(getAttribute(onpickAttr).domString()) ;
	if (temp)
		value = temp->copy() ;
	return value ;
}

String WMLOptionElement::valueAttribute() const
{
	String value ;
	String *temp = document()->frame()->getVariableReplacedString(getAttribute(valueAttr).domString());
	if (temp)
		value = temp->copy() ;
	return value ;
}

void WMLOptionElement::defaultEventHandler(Event* ev)
{

}

WMLSelectElement* WMLOptionElement::getSelect() const
{
    Node* select = parentNode();
    while (select && !select->hasTagName(selectTag))
        select = select->parentNode();
    return static_cast<WMLSelectElement*>(select);
}

void WMLOptionElement::removeOptionFromVariableValue(String varName, String varValue) const
{
	String currentValue = ((WMLDocument *)document())->BrowserContext()->GetVariableValue(varName);
	if(!(currentValue.isEmpty()))
	{
		String newValue = "";
		int pos = currentValue.find(varValue);
		if(pos == 0)
		{
			if(currentValue.find(';') != -1)
			{
				newValue = currentValue.substring(pos + varValue.length() + 1) ;	
			}
			else
			{
				newValue = "";
			}
		}
		else if(pos != -1)
		{
			newValue = currentValue.substring(0,pos-1) + currentValue.substring(pos+varValue.length());
		}
		else
		{
			newValue = "";
		}
		((WMLDocument *)document())->BrowserContext()->SetVariableValue(varName,newValue);
	}
}

void WMLOptionElement::updateVariableValue(String varName, String varValue, WMLSelectElement *pSelect) const
{
		if(pSelect->multiple())
		{
			//Append all values seperated by ';'. Update this value into context
			String currentValue = ((WMLDocument *)document())->BrowserContext()->GetVariableValue(varName);
			if(!(currentValue.isEmpty()))
			{
				currentValue.append(';');
			}
			currentValue.append(varValue);
			((WMLDocument *)document())->BrowserContext()->SetVariableValue(varName,currentValue);
		}
		else
		{
			((WMLDocument *)document())->BrowserContext()->SetVariableValue(varName,varValue);
		}	
}
} // namespace
# endif