/*
 * Copyright (C) 1999 Lars Knoll (knoll@kde.org)
 *           (C) 1999 Antti Koivisto (koivisto@kde.org)
 *           (C) 2000 Simon Hausmann <hausmann@kde.org>
 * Copyright (C) 2003, 2006, 2007 Apple Inc. All rights reserved.
 *           (C) 2006 Graham Dennis (graham.dennis@gmail.com)
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public License
 * along with this library; see the file COPYING.LIB.  If not, write to
 * the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */
#if defined(SHP_SUPPORT_WML)
#include "config.h"
#include "WMLPrevElement.h"

#include "CSSHelper.h"
#include "Document.h"
#include "Event.h"
#include "EventHandler.h"
#include "EventNames.h"
#include "Frame.h"
#include "FrameLoader.h"
#include "HTMLNames.h"
#include "WMLAnchorElement.h"
#include "Page.h"

namespace WebCore {

using namespace HTMLNames;
using namespace EventNames;

WMLPrevElement::WMLPrevElement(Document *doc)
:HTMLAnchorElement(prevTag, doc )
{
}

WMLPrevElement::~WMLPrevElement()
{
}

RenderObject *WMLPrevElement::createRenderer(RenderArena* arena, RenderStyle* style)
{
	RenderObject *obj = HTMLAnchorElement::createRenderer (arena, style) ;

	//for some miguided code here... :) for attributes
	Node *parent = parentNode() ;
	ASSERT (parent) ;
	while (parent)
	{
		if (parent->hasTagName(anchorTag)) // may be we need to add several tags here
		{
			WMLAnchorElement *anchor = static_cast<WMLAnchorElement *>(parent) ;
			anchor->setChildAttributes (this) ;
			break ;
		}
		else
			parent = parent->nextSibling () ;
	}
	return obj ;
}


void WMLPrevElement::defaultEventHandler(Event* evt)
{
	if (m_isLink && evt->type() == clickEvent) {
		evt->setDefaultHandled();
	}
	else {
		HTMLAnchorElement::defaultEventHandler(evt)	;
	}
}

void WMLPrevElement::handleEvent(EventType eventType, bool bForwardNavigation, bool & bIsEventHandled)
{
	//Get the previous card or page from history, display it 

	//Call children handleEvent for setvar etc
	for(Node *pChild = firstChild();pChild;pChild = pChild->nextSibling())
	{
		pChild->handleEvent(eventType,bForwardNavigation, bIsEventHandled);
	}
	document()->frame()->page()->goBack() ;
}

}
#endif