/*
 * Copyright (C) 1999 Lars Knoll (knoll@kde.org)
 *           (C) 1999 Antti Koivisto (koivisto@kde.org)
 *           (C) 2000 Simon Hausmann <hausmann@kde.org>
 * Copyright (C) 2003, 2006, 2007 Apple Inc. All rights reserved.
 *           (C) 2006 Graham Dennis (graham.dennis@gmail.com)
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public License
 * along with this library; see the file COPYING.LIB.  If not, write to
 * the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */
#if defined ( SHP_SUPPORT_WML )
#include "config.h"
#include "WMLAnchorElement.h"

#include "CSSHelper.h"
#include "Document.h"
#include "Event.h"
#include "EventHandler.h"
#include "EventNames.h"
#include "Frame.h"
#include "FrameLoader.h"
#include "HTMLNames.h"
#include "Page.h"
#include "HistoryItem.h"
#include "FrameView.h"
#include "WMLGoElement.h"
#include "WMLPrevElement.h"
#include "WMLRefreshElement.h"

namespace WebCore {

using namespace HTMLNames;
using namespace EventNames;


WMLAnchorElement::WMLAnchorElement(Document *doc)
:HTMLAnchorElement(anchorTag, doc )
,m_type(NoneTAG)
{

}

WMLAnchorElement::~WMLAnchorElement()
{

}

void WMLAnchorElement::setChildAttributes (Node *child) 
{
	//AtomicString attribute =  child->getAttribute(hrefAttr) ;
    //if (attr->name() == hrefAttr) {
	if (child->hasTagName(goTag)) {
		WMLGoElement *goElement = static_cast<WMLGoElement *>(child) ;
		m_isLink = true;
		setHref (goElement->href()) ;
		setChanged();
		m_type = GoTAG ;
	}
	else if (child->hasTagName(prevTag)) {
		WMLPrevElement *prevElement = static_cast<WMLPrevElement *>(child) ;
		BackForwardList* bfList = document()->frame()->page()->backForwardList();
		HistoryItem* item = bfList->backItem();
		String url ;
		if (item)
			url = item->urlString() ;
		else
			url  = "about:bank" ;
		m_isLink = true;
		//now i have to get the element from the server
		setHref (url) ;
		m_type = PrevTAG ;
	}
	else if (child->hasTagName(refreshTag)) {
		WMLAnchorElement *anchorElement = static_cast<WMLAnchorElement *>(child) ;
		m_isLink = true;
		//now i have to get the element from the server
		String hrf("about:blank");
		setHref (hrf) ;
		
		m_type = RefreshTAG ;
	}
}

void WMLAnchorElement::defaultEventHandler(Event* evt)
{
	if (m_isLink && evt->type() == clickEvent) {
		switch (m_type)	{
		case GoTAG:
		{
			/*evt->setDefaultHandled();
			String url = parseURL(getAttribute(hrefAttr));
	        if (!evt->defaultPrevented() && document()->frame())
				document()->frame()->loader()->urlSelected(document()->completeURL(url), \
												getAttribute(targetAttr), evt, false, true);*/
			//Get go child, and call go defaultEventHandler
			Node *pChild = firstChild();
			while(pChild)
			{
				if(pChild->hasTagName(goTag))
				{
					WMLGoElement *pGoNode = static_cast<WMLGoElement *>(pChild);
					pGoNode->defaultEventHandler(evt);
					break;
				}
				pChild = pChild->nextSibling();
			}
			evt->setDefaultHandled();
		}
		break ;
		case PrevTAG:
			for(Node *pChild = firstChild();pChild;pChild = pChild->nextSibling())
			{
				bool bIsEventHandled = false;
				pChild->handleEvent(NavigationEvent, false, bIsEventHandled);
			}
			//document()->frame()->page()->goBack() ;
			evt->setDefaultHandled();
			break ;
		case RefreshTAG:
			{
				Node *pChild = firstChild();
				while(pChild)
				{
					if(pChild->hasTagName(refreshTag))
					{
						WMLRefreshElement *pRefreshNode = static_cast<WMLRefreshElement *>(pChild);
						pRefreshNode->defaultEventHandler(evt);
						break;
					}
					pChild = pChild->nextSibling();
				}
			}
			break;
		case NoneTAG:
			HTMLAnchorElement::defaultEventHandler(evt)	;
			break ;
		
		}
	}
	else {
		HTMLAnchorElement::defaultEventHandler(evt)	;
	}
}



}
#endif//SHP_SUPPORT_WML
