#ifdef SUPPORT_AUTO_FORM_FILL

#include "AutoFormFill.h"
#include "CString.h"
#include "ShpJasmine.h"

#define NAME_VALUE_SEPARATOR	13
#define INVALID_POS				-1

namespace WebCore {

AutoFormFill* AutoFormFill::m_autoFormFill = NULL;

void AutoFormFill::Init()
{
	if(!m_autoFormFill)
	{
		m_autoFormFill = new AutoFormFill();
		//To read the data from the file and insert into Pagemap
		SFBalFSReadAutoFormFill((AddEntryToPageMap)ConstructEntry);
		m_autoFormFill->m_fnAddAutoFormFill = NULL;
		m_autoFormFill->m_fnUpdateAutoFormFill = NULL;
	}
}

void AutoFormFill::Finalize()
{
	if(m_autoFormFill)
	{
		//Write data to persistent
		m_autoFormFill->WriteToFile(m_autoFormFill->m_pageMap);
		m_autoFormFill->ClearMap(m_autoFormFill->m_pageMap);
		ClearTransaction();
		delete m_autoFormFill;
		m_autoFormFill = NULL;
	}
}

void AutoFormFill::RegisterHandlers(cbAddAutoFormFill fnAdd, cbUpdateAutoFormFill fnUpdate)
{
	if(m_autoFormFill)
	{
		m_autoFormFill->m_fnAddAutoFormFill = fnAdd;
		m_autoFormFill->m_fnUpdateAutoFormFill = fnUpdate;
	}
}

bool AutoFormFill::ConstructEntry(char* url, char* form, char* nameValuePairs)
{
	bool result = false;

	String sUrl(url);
	String sForm(form);
	String sElementMap(nameValuePairs);

	ElementMapType elementMap;
	Vector<String> nameValueList;
	Vector<String> nameValue;
	UChar separator = 13;

	sUrl = sUrl.lower();
	sForm = sForm.lower();
	
	nameValueList = sElementMap.split(separator);

	for(int i = 0; i < nameValueList.size(); i++)
	{
		nameValue = nameValueList[i].split('='); 

		if(nameValue.size() == 2)
			elementMap.add(nameValue[0],nameValue[1]);
	}

	if(m_autoFormFill && elementMap.size() > 0)
	{
		result = m_autoFormFill->AddTempElementMapToPageMap(m_autoFormFill->m_pageMap, sUrl, sForm, elementMap);

	}

	return result;
}

void AutoFormFill::DeleteData()
{
	if(m_autoFormFill)
	{
		//clear hashmaps
		m_autoFormFill->ClearMap(m_autoFormFill->m_pageMap);
	}
}

void AutoFormFill::StartTransaction(const String& url, const String& form)
{
	String url_Tiny = MakeTinyURL(url);

	String url_lower = url_Tiny.lower();
	String form_lower = form.lower();

	if(m_autoFormFill)
	{
		m_autoFormFill->m_tempURL = url_lower;
		m_autoFormFill->m_tempForm = form_lower;
	}
}

void AutoFormFill::AddElement(const String& elementName, const String& elementValue)
{
	if(m_autoFormFill)
	{
		m_autoFormFill->m_tempElementMap.add(elementName, elementValue.lower());
		m_autoFormFill->m_isDirty = true;
	}
}

bool AutoFormFill::ShouldAskUser()
{
	bool result = false;

	if(m_autoFormFill && m_autoFormFill->m_isDirty)
	{
		result = true;

		if(true == m_autoFormFill->m_pageMap.contains(m_autoFormFill->m_tempURL))
		{
			FormMapType formMap = m_autoFormFill->m_pageMap.get(m_autoFormFill->m_tempURL);

			if(true == formMap.contains(m_autoFormFill->m_tempForm))
			{
				ElementMapVectorType elementMapVector = formMap.get(m_autoFormFill->m_tempForm);

				ElementMapTypeIterator e = m_autoFormFill->m_tempElementMap.begin();
										
				for(int i = 0; i < elementMapVector.size(); i++)
				{
					if((elementMapVector[i].contains(e->first)) && 
						(e->second == elementMapVector[i].get(e->first)))
					{
						result = false;
						break;
					}
				}
			}
		}

	}

	return result;
}

void AutoFormFill::CommitTransaction()
{
	if(m_autoFormFill)
	{
		if(m_autoFormFill->m_isDirty)
		{
			bool add = m_autoFormFill->AddTempElementMapToPageMap(m_autoFormFill->m_pageMap,
										m_autoFormFill->m_tempURL, m_autoFormFill->m_tempForm, 
										m_autoFormFill->m_tempElementMap);

			ElementMapTypeIterator e = m_autoFormFill->m_tempElementMap.begin();
			String nameValuePairs;
			UChar separator[2];
			separator[0] = NAME_VALUE_SEPARATOR;
			separator[1] = 0;

			for(; e != m_autoFormFill->m_tempElementMap.end(); ++e)
			{
				nameValuePairs += e->first;
				nameValuePairs += "=";
				nameValuePairs += e->second;
				nameValuePairs += String(separator);
			}
			
			if(add)
			{
				//call add callback
				if(m_autoFormFill->m_fnAddAutoFormFill)
					(*m_autoFormFill->m_fnAddAutoFormFill)(m_autoFormFill->m_tempURL.latin1().mutableData(),
														m_autoFormFill->m_tempForm.latin1().mutableData(),
														nameValuePairs.latin1().mutableData());
			}
			else
			{
				//call update callback
				if(m_autoFormFill->m_fnUpdateAutoFormFill)
					(*m_autoFormFill->m_fnUpdateAutoFormFill)(m_autoFormFill->m_tempURL.latin1().mutableData(),
														m_autoFormFill->m_tempForm.latin1().mutableData(),
														nameValuePairs.latin1().mutableData());
			}

			m_autoFormFill->m_isDirty = false;
		}
	}
}

bool AutoFormFill::AddTempElementMapToPageMap(PageMapType& pageMap, const String& url, const String& form, ElementMapType elementMapToAdd)
{
	bool result = true;

	if(false == pageMap.contains(url)) // URL not found
	{
		ElementMapVectorType elementMapVector;
		elementMapVector.append(elementMapToAdd);

		FormMapType formMap;
		formMap.add(form, elementMapVector);

		pageMap.add(url, formMap);
	}
	else
	{
		FormMapType formMap = pageMap.get(url);
		if(false == formMap.contains(form))		// Form does not exist
		{
			ElementMapVectorType elementMapVector;
			elementMapVector.append(elementMapToAdd);
			formMap.add(form, elementMapVector);
		}
		else
		{
			ElementMapVectorType elementMapVector = formMap.get(form);

			ElementMapTypeIterator it = elementMapToAdd.begin();
			ElementMapTypeIterator e = elementMapToAdd.begin();

			for(it = elementMapToAdd.begin(); it != elementMapToAdd.end(); ++it)
			{
				e = it;
			}
				

			for(int i = 0; i < elementMapVector.size(); i++)
			{
				if((elementMapVector[i].contains(e->first)) && 
					(e->second == elementMapVector[i].get(e->first)))
				{
					// Update data
					elementMapVector[i] = elementMapToAdd;
					result = false;
					break;
				}
			}

			if(true == result) // ElementMap not found. So add
			{
				elementMapVector.append(elementMapToAdd);
			}

			formMap.set(form, elementMapVector);
		}

		pageMap.set(url, formMap);
	}

	return result;
}

void AutoFormFill::ClearTransaction()
{
	if(m_autoFormFill)
	{
		m_autoFormFill->m_tempElementMap.clear();
		m_autoFormFill->m_isDirty = false;
	}
}

//this API is for auto form fill
//String AutoFormFill::Get(const String& url, const String& form, const String& elementName)
String AutoFormFill::GetValue(const String& elementName)
{
	String elementValue;

	//String url_lower = url.lower();
	//String form_lower = form.lower();

	String url_lower = m_autoFormFill->m_tempURL;
	String form_lower = m_autoFormFill->m_tempForm;

	if(m_autoFormFill)
	{
		FormMapType formMap = m_autoFormFill->m_pageMap.get(url_lower);

		if(!formMap.isEmpty())
		{
			ElementMapVectorType elementmapvector = formMap.get(form_lower);

			if( (!elementmapvector.isEmpty()) &&  1 == elementmapvector.size()) // If only one entry exist then AutoFormFill works
				elementValue = elementmapvector[0].get(elementName); // Return the first element value
		}
	}

	return elementValue;
}

Vector<String> AutoFormFill::GetAllValues(const String& elementName)
{
	return GetAll(m_autoFormFill->m_tempURL, m_autoFormFill->m_tempForm,elementName);
}

// Get all values of a particular element
Vector<String> AutoFormFill::GetAll(const String& url, const String& form, const String& elementName)
{
	Vector<String> elementValues;
	String elementValue;

	String url_lower = url.lower();
	String form_lower = form.lower();

	if(m_autoFormFill)
	{
		FormMapType formMap = m_autoFormFill->m_pageMap.get(url_lower);

		if(!formMap.isEmpty())
		{
			ElementMapVectorType elementmapvector = formMap.get(form_lower);

			for(int i = 0; i < elementmapvector.size(); i++)
			{
				elementValue = elementmapvector[i].get(elementName);
				if(!elementValue.isEmpty())
					elementValues.append(elementValue);
			}
		}
	}

	return elementValues;
}


ElementMapType AutoFormFill::GetValuesOfAllElement(const String& elementName, const String& elementValue)
{
	return GetElementMap(m_autoFormFill->m_tempURL, m_autoFormFill->m_tempForm, elementName, elementValue);
}

ElementMapType AutoFormFill::GetElementMap(const String& url, const String& form, const String& elementName, const String& elementValue)
{
	ElementMapType elementMap;

	String url_lower = url.lower();
	String form_lower = form.lower();
	String elementValue_lower = elementValue.lower();

	FormMapType formMap = m_autoFormFill->m_pageMap.get(url_lower);
	ElementMapVectorType elementMapVector = formMap.get(form_lower);

	for(int i = 0; i < elementMapVector.size(); i++)
	{
		if((elementMapVector[i].contains(elementName)) && 
			(elementValue_lower == elementMapVector[i].get(elementName)))
		{
			elementMap = elementMapVector[i];
			break;
		}
	}

	return elementMap;
}

//This API is for Writing data to persistent
void AutoFormFill::WriteToFile(PageMapType& pageMap)
{
	for(PageMapTypeIterator pageIterator = pageMap.begin(); pageIterator != pageMap.end(); ++pageIterator)
	{
		for(FormMapTypeIterator formIterator = pageIterator->second.begin(); formIterator != pageIterator->second.end(); ++formIterator)
		{
			ElementMapVectorType elementMapVector = formIterator->second;
			
			for(int i = 0; i < elementMapVector.size(); i++)
			{
				ElementMapType elementMap = elementMapVector[i];
				ElementMapTypeIterator e = elementMap.begin();
				String nameValuePairs;
				UChar separator[2];
				separator[0] = NAME_VALUE_SEPARATOR;
				separator[1] = 0;

				for(; e != elementMap.end(); ++e)
				{
					nameValuePairs += e->first;
					nameValuePairs += "=";
					nameValuePairs += e->second;
					nameValuePairs += String(separator);
				}
			
				SFBalFSWriteAutoFormFill( pageIterator->first.latin1().mutableData(), formIterator->first.latin1().mutableData(), nameValuePairs.latin1().mutableData());
			}
		}
	}
}

void AutoFormFill::ClearMap(PageMapType& pageMap)
{
	for(PageMapTypeIterator pageIterator = pageMap.begin(); pageIterator != pageMap.end(); ++pageIterator)
	{
		for(FormMapTypeIterator formIterator = pageIterator->second.begin(); formIterator != pageIterator->second.end(); ++formIterator)
		{
			ElementMapVectorType elementMapVector = formIterator->second;

			for(int i = 0; i < elementMapVector.size(); i++)
			{
				elementMapVector[i].clear();
			}

			formIterator->second.clear();
		}

		pageIterator->second.clear();
	}

	pageMap.clear();
}


bool AutoFormFill::IsElementTypeSupported(const String& elementType)
{
	bool result = false;
	String elementType_lower = elementType.lower(); 

	if (elementType_lower == "text" || elementType_lower == "password")
		result = true;

	return result;
}

// Removes data after the last "/" in a string. e.g.
// https://www.google.com/accounts/ServiceLogin?service=mail&passive=true&rm=false&continue=http%3A%2F%2Fmail.google.com%2Fmail%2F%3Fui%3Dhtml%26zy%3Dl&bsv=1k96igf4806cy&ltmpl=default&ltmplcache=2
// is Converted to 
// https://www.google.com/accounts/

String AutoFormFill::MakeTinyURL(String str)
{
	String TinyURL;

	int iPos = str.reverseFind('/');

	for(int i=0; i<=iPos; i++)
		TinyURL.append(str[i]);

	return TinyURL;
}

#ifdef SFB_GLOBAL_STATIC_HANDLING
void initAutoFormFill()
{
    AutoFormFill::Init();
}

void deInitAutoFormFill()
{
    AutoFormFill::Finalize();
}
#endif

} //namespace WebCore

#endif //SUPPORT_AUTO_FORM_FILL
