#ifdef SUPPORT_AUTO_FORM_FILL
#ifndef __AUTOFORMFILL_H__
#define __AUTOFORMFILL_H__

#include "StringHash.h"
#include "../../BrowserControl/interface/inc/types.h"
#include "Vector.h"

#if PLATFORM(SHP)
#include "CMMDefs.h"
#endif // PLATFORM(SHP)

namespace WebCore {

//Maps
typedef HashMap<String, String>						ElementMapType;
typedef Vector<ElementMapType>						ElementMapVectorType;

typedef HashMap<String, ElementMapVectorType>		FormMapType;
typedef HashMap<String, FormMapType>				PageMapType;

//Iterators
typedef HashMap<String, String>::iterator				ElementMapTypeIterator;
typedef HashMap<String, ElementMapVectorType>::iterator	FormMapTypeIterator;
typedef HashMap<String, FormMapType>::iterator			PageMapTypeIterator;

//Temp till persistent storage is implemented in BAL
typedef  int(*cbAddAutoFormFill) (char*, char*, char*);
typedef  int(*cbUpdateAutoFormFill) (char*, char*, char*);

class AutoFormFillHelper;

class AutoFormFill
    CMM_MODIFIED
{
public:
	static void Init();
	static void Finalize();

	static void RegisterHandlers(cbAddAutoFormFill fnAdd,
									cbUpdateAutoFormFill fnUpdate);
	
	static bool ConstructEntry(char* url, char* form, char* elementMapString);

	static void DeleteData();

	static void StartTransaction(const String& url, const String& form);
	static void AddElement(const String& elementName, const String& elementValue);
	
	//This API is for AutoFormFill
	static String GetValue(const String& elementName);

	//This APIs are for AutoComplete
	static Vector<String> GetAllValues(const String& elementName);
	static Vector<String> GetAll(const String& url, const String& form, const String& elementName);

	static ElementMapType GetValuesOfAllElement(const String& elementName, const String& elementValue);
	static ElementMapType GetElementMap(const String& url, const String& form, const String& elementName, const String& elementValue); //Useful to retrive Password 

	static bool	ShouldAskUser();
	static void CommitTransaction();
	static void ClearTransaction();

	static bool IsElementTypeSupported(const String& ElementType);
	//This API is for Writing data to persistent
	static void WriteToFile(PageMapType& pageMap);


private:
	bool AddTempElementMapToPageMap(PageMapType& pageMap, const String& url, const String& form, ElementMapType elementMap);
	void ClearMap(PageMapType& pageMap);
	static String MakeTinyURL(String str);
	
	cbAddAutoFormFill		m_fnAddAutoFormFill;
	cbUpdateAutoFormFill	m_fnUpdateAutoFormFill;

	//temp record to be added
	ElementMapType			m_tempElementMap;
	String					m_tempURL;
	String					m_tempForm;
	bool					m_isDirty;

	PageMapType				m_pageMap;
	static AutoFormFill*	m_autoFormFill; //one and only instance

	friend class			AutoFormFillHelper;
};

} //namespace webcore

#endif //__AUTOFORMFILL_H__

#endif //SUPPORT_AUTO_FORM_FILL