/*
* ==============================================================================
*  Name         : VirtualCursor.h
*  Part of      : WEBCORE / Page
*  Description  : Scroll Adjustment implemented inline with GTK adjustment
*  Date         : February 29, 2008
*  Author       : Prashant Nevase
* ==============================================================================
*/

#ifndef __VCS_ADJUSTMENT_H__
#define __VCS_ADJUSTMENT_H__

#if PLATFORM(SHP)
#include "CMMDefs.h"
#endif // PLATFORM(SHP)

#ifdef __cplusplus
extern "C"
{
#endif

namespace WebCore {

class CVCSAdjustment;
typedef void (*cbVCSHandler)(CVCSAdjustment*, void*);

enum EChangedValueType
{
	CHANGED_VALUE_SAME,
	CHANGED_VALUE_INCREMENTED,
	CHANGED_VALUE_DECREMENTED
};

//Virtual Cursor Scroll Adjustment class
class CVCSAdjustment
    CMM_MODIFIED
{
public:
	CVCSAdjustment(	double value,
					double lower,
					double upper,
					double step_increment,
					double page_increment,
					double page_size
					):
					m_value(value),
					m_lower(lower),
					m_upper(upper),
					m_step_increment(step_increment),
					m_page_increment(page_increment),
					m_page_size(page_size),
					m_changedValueType(CHANGED_VALUE_SAME),
					m_AdjustmentChangedHandler(0),
					m_ValueChangedHandler(0),
					m_owner(0)
	{}

	void setHandlers(cbVCSHandler adjustmentChangedHandler, cbVCSHandler valueChangedHandler, void* owner)
	{
		m_AdjustmentChangedHandler = adjustmentChangedHandler;
		m_ValueChangedHandler = valueChangedHandler;
		m_owner = owner;
	}

	void removeHandlers()
	{
		m_AdjustmentChangedHandler = 0;
		m_ValueChangedHandler = 0;
		m_owner = 0;
	}

	double getValue() { return m_value; }
	void setValue(double value)
	{ 
		m_value = (value > m_upper)? m_upper : value;
		m_value = (m_value < m_lower)? m_lower : m_value;
	}

	double getLower() { return m_lower; }
	
	void setLower(double lower)
	{
		m_lower = lower;
		m_value = (m_value < m_lower)? m_lower : m_value;
	}

	double getUpper() { return m_upper; }
	
	void setUpper(double upper)
	{
		m_upper = upper;
		m_value = (m_value > m_upper)? m_upper : m_value;
	}

	double getStep_increment() { return m_step_increment; }
	void setStep_increment(double step_increment) { m_step_increment = step_increment; }
	double getPage_increment() { return m_page_increment; }
	void setPage_increment(double page_increment) { m_page_increment = page_increment; }
	double getPage_size() { return m_page_size; }
	void setPage_size(double page_size) { m_page_size = page_size; }
	EChangedValueType getChangedValueType() { return m_changedValueType; }
	void setChangedValueType(EChangedValueType changedValueType) { m_changedValueType = changedValueType; }

	void clampPage(double lower, double upper) {}
	
	void adjustmentChanged()
	{
		if(m_AdjustmentChangedHandler)
			(*m_AdjustmentChangedHandler)(this, m_owner);
	}

	void valueChanged()
	{
		if(m_ValueChangedHandler)
			(*m_ValueChangedHandler)(this, m_owner);
	}

private:
	double m_value;
	double m_lower;
	double m_upper;
	double m_step_increment;
	double m_page_increment;
	double m_page_size;
	EChangedValueType m_changedValueType;

	cbVCSHandler m_AdjustmentChangedHandler;
	cbVCSHandler m_ValueChangedHandler;
	void* m_owner;
};

} //namespace WebCore

#ifdef __cplusplus
} //extern "C"
#endif

#endif //__VIRTUAL_CURSOR_SCROLL_ADJUSTMENT_H__