/*
    Copyright (C) 2007 Krzysztof Kowalczyk <kkowalczyk@gmail.com>
    Copyright (C) 2004, 2005, 2006 Nikolas Zimmermann <wildfox@kde.org>
                  2004, 2005, 2006 Rob Buis <buis@kde.org>
                  2005, 2007 Apple Inc. All Rights reserved.
                  2007 Alp Toker <alp.toker@collabora.co.uk>

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.

    You should have received a copy of the GNU Library General Public License
    aint with this library; see the file COPYING.LIB.  If not, write to
    the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
    Boston, MA 02111-1307, USA.
*/

#include "config.h"
#include "Path.h"

#include "AffineTransform.h"
#include "ShpPath.h"
#include "NotImplemented.h"
#include "PlatformString.h"

#include "IntRect.h"


namespace WebCore {

static bool inpoly(Vector<FloatPoint>& vect, const FloatPoint& point);
static bool inEllipse(FloatRect& rect, const FloatPoint& point);

Path::Path()
    : m_path(new ShpPath())
{
}

Path::~Path()
{
    delete m_path;
}

Path::Path(const Path& other)
    : m_path(new ShpPath())
{
	m_path->m_linePointVect = other.m_path->m_linePointVect;
	m_path->m_pathRect = other.m_path->m_pathRect;
	m_path->m_pathEllipse = other.m_path->m_pathEllipse;
}


Path& Path::operator=(const Path& other)
{
    if (&other == this)
        return *this;

	m_path->m_linePointVect = other.m_path->m_linePointVect;
	m_path->m_pathRect = other.m_path->m_pathRect;
    m_path->m_pathEllipse = other.m_path->m_pathEllipse;
    return *this;
}

void Path::clear()
{
    notImplemented();
	m_path->m_pathRect = FloatRect(IntRect(0, 0, 0, 0));
	m_path->m_linePointVect.clear();
	m_path->m_pathEllipse = FloatRect(IntRect(0, 0, 0, 0));
}

bool Path::isEmpty() const
{

    notImplemented();

	return false;
}

void Path::translate(const FloatSize& p)
{
    notImplemented();
}

void Path::moveTo(const FloatPoint& p)
{
    m_path->m_linePointVect.append(p);
}

void Path::addLineTo(const FloatPoint& p)
{
   m_path->m_linePointVect.append(p);
}

void Path::addRect(const FloatRect& rect)
{
    m_path->m_pathRect = rect;
}


void Path::addQuadCurveTo(const FloatPoint& controlPoint, const FloatPoint& point)
{
    notImplemented();
}

void Path::addBezierCurveTo(const FloatPoint& controlPoint1, const FloatPoint& controlPoint2, const FloatPoint& controlPoint3)
{
    notImplemented();
}

void Path::addArc(const FloatPoint& p, float r, float sa, float ea, bool clockwise)
{
    notImplemented();
}

void Path::addArcTo(const FloatPoint& p1, const FloatPoint& p2, float radius)
{
    // FIXME: cairo_arc_to not yet in cairo see cairo.h
    // cairo_arc_to(m_cr, p1.x(), p1.y(), p2.x(), p2.y());
    notImplemented();
}

void Path::addEllipse(const FloatRect& rect)
{
    m_path->m_pathEllipse = rect;
}

void Path::closeSubpath()
{
    notImplemented();
}

FloatRect Path::boundingRect() const
{
    double x0, x1, y0, y1;
	x0 = x1 = y0 = y1 = 0;

    notImplemented();

    return m_path->m_pathRect;
}

bool Path::contains(const FloatPoint& point, WindRule rule) const
{
     return (m_path->m_pathRect.contains(point) || 
            inpoly(m_path->m_linePointVect, point) ||
            inEllipse(m_path->m_pathEllipse, point));
}

void Path::apply(void* info, PathApplierFunction function) const
{
    notImplemented();
}

void Path::transform(const AffineTransform& trans)
{
    notImplemented();
}

String Path::debugString() const
{
    String string = "";
    notImplemented();
    return string;
}

bool inpoly(Vector<FloatPoint>& poly, const FloatPoint& point)
{
     float xnew,ynew;
     float xold,yold;
     float x1,y1;
     float x2,y2;
     int i;
     bool inside = false;

	float xt = point.x();
	float yt = point.y();

	 if (poly.size() < 3) {

          return false;
     }

	 int npoints = poly.size();

	 xold = ((FloatPoint) poly[npoints-1]).x();
     yold=  ((FloatPoint) poly[npoints-1]).y();

     for (i=0 ; i < npoints ; i++) {

          xnew=((FloatPoint) poly[i]).x();
          ynew=((FloatPoint) poly[i]).y();
          if (xnew > xold) {
               x1=xold;
               x2=xnew;
               y1=yold;
               y2=ynew;
          }
          else {
               x1=xnew;
               x2=xold;
               y1=ynew;
               y2=yold;
          }
          if ((xnew < xt) == (xt <= xold)         /* edge "open" at left end */
           && ((long)yt-(long)y1)*(long)(x2-x1)
            < ((long)y2-(long)y1)*(long)(xt-x1)) {
               inside=!inside;
          }
          xold=xnew;
          yold=ynew;
     }
     return(inside);
}

bool inEllipse(FloatRect& rect, const FloatPoint& point)
{
    // Get the center, semiminor axis and semiminor axis of the ellipse
    float a = rect.width() / 2;
    float b = rect.height() / 2;
    float centerX = rect.x() + a;
    float centerY = rect.y() + b;

    //Determine and normalize quadrant.
    float dx = fabs(centerX - point.x());
    float dy = fabs(centerY - point.y());

    //Shortcut
    if( dx > a || dy > b ) {
      return false;
    }

    // Use the equation of ellipse
    return ( ((dx * dx) / (a * a)) + ((dy * dy) / (b * b)) ) <= 1;
}

} // namespace WebCore
