/*
 * Copyright (C) 2006 Apple Computer, Inc.
 *
 * Portions are Copyright (C) 2001-6 mozilla.org
 *
 * Other contributors:
 *   Stuart Parmenter <stuart@mozilla.com>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Alternatively, the contents of this file may be used under the terms
 * of either the Mozilla Public License Version 1.1, found at
 * http://www.mozilla.org/MPL/ (the "MPL") or the GNU General Public
 * License Version 2.0, found at http://www.fsf.org/copyleft/gpl.html
 * (the "GPL"), in which case the provisions of the MPL or the GPL are
 * applicable instead of those above.  If you wish to allow use of your
 * version of this file only under the terms of one of those two
 * licenses (the MPL or the GPL) and not to allow others to use your
 * version of this file under the LGPL, indicate your decision by
 * deletingthe provisions above and replace them with the notice and
 * other provisions required by the MPL or the GPL, as the case may be.
 * If you do not delete the provisions above, a recipient may use your
 * version of this file under any of the LGPL, the MPL or the GPL.
 */

#include "config.h"
#include "OtherImageDecoder.h"
#include <assert.h>

#ifdef __CMM__
#include <allocs.h>
#endif
#include "ShpJasmine.h"
#include "Cache.h"

#define SFB_IMAGE_DECODER_FILE_NAME			"/User/Br/SFB/tempdecode.jpg"
#define EXTRA_MEMORY_LARGE_IMAGES			(4U)
#define EXTRA_MEMORY_SMALL_IMAGES			(2U)
#define USE_FILE_JPEG_DECODE_API

BOOL
SFBCheckBufferSize( ImgCodecType m_imageType,ImgImage* pImage, unsigned char* pEncodedData,ULONG dataSize)
{
	unsigned int imageHeight = 0, imageWidth = 0, nof = 0;
	
	// Get the memory required by the image decoder
	unsigned requiredSize = ImgGetMemoryRequired (pEncodedData, 
													dataSize, 
													&imageWidth,	
													&imageHeight, 
													&nof);
	
	if(MAX_IMAGE_SIZE <= requiredSize)
		{
				return FALSE;
		}

	// The ImgGetMemoryRequired do not calculate the exact value.
	// So multiply it by some additional factor
	// Note: the value 320*240 has been taken form ImgJpgDecodeImage line no 245
	if( m_imageType == IMG_DEC_GIF ||
		(m_imageType == IMG_DEC_JPEG &&
		((imageWidth * imageHeight) >= (320 * 240))))
	{
		requiredSize *= EXTRA_MEMORY_LARGE_IMAGES;
	}
		else
	{
		requiredSize *= EXTRA_MEMORY_SMALL_IMAGES;
	}

	// Do not decode the image if memory is low or the image is too large
	if((MAX_MEM_USAGE * CMM_TOTAL_HEAP_SIZE) 
		> (CMM_CURRENTLY_ALLOCATED_HEAP_SIZE + requiredSize))
	{
		// Decode only if a contigious block of requiredSize is available
		if(SFBalMemGetStatEx() >= requiredSize)
		{
			return TRUE;
		}
	}
			return FALSE;
}

namespace WebCore {

OtherImageDecoder::OtherImageDecoder()
{
    m_image = (ImgImage*)malloc( sizeof(ImgImage));
	AcMemset (m_image, 0, sizeof(ImgImage));
}

OtherImageDecoder::~OtherImageDecoder()
{
	if (m_image)
	{
		ImgFreeImage(m_image);
		free(m_image);
		m_image = NULL;
	}

}

// Take the data and store it.
void OtherImageDecoder::setData(SharedBuffer* data, bool allDataReceived)
{
    if (m_failed)
        return;

    // Cache our new data.
    ImageDecoder::setData(data, allDataReceived);

    // Decode since all data has arrived
    if(allDataReceived)
    {
        if(m_frameBufferCache.isEmpty())
            m_frameBufferCache.resize(1);
        decode();
    }
}

// Whether or not the size information has been decoded yet.
bool OtherImageDecoder::isSizeAvailable() const
{
    return m_sizeAvailable;
}

RGBA32Buffer* OtherImageDecoder::frameBufferAtIndex(size_t index)
{
    // we must have a single frame
    if (index)
        return NULL;

    // decode if the frame is not already decoded
    if (m_frameBufferCache.isEmpty())
    {
        m_frameBufferCache.resize(1);
        // Decode this frame.
        decode();
    }

    RGBA32Buffer& frame = m_frameBufferCache[0];
    return &frame;
}
#ifdef USE_FILE_JPEG_DECODE_API
void OtherImageDecoder::decode(bool sizeOnly)
{
	HFile		hFile;
	char		fileName[50];
	ImgImage* file_m_image = NULL;
	ImgImage* copy_m_image = NULL;
	if (m_failed)
        return;

	if(SFBCheckBufferSize( m_imgCodecType,m_image, ( unsigned char* )m_data->data(),m_data->size())!=TRUE)
    {
        MemoryManager::SetStatus(CMM_CHECKRESULT_COLLECTING);
		return ;
    }
	else
	{
		// Decode the image using buffer API's
		//Added bStatus to put a chcek whether image is decoded or not.
		BOOL bStatus = ImgDecodeImage( m_imgCodecType, m_image, (unsigned char*)m_data->data(), m_data->size() );

		// The data was not decoded using buffer API's. We shall try to decode using file API's
		//if bStatus is false, image is not decoded using file API's
		if((!m_image->pHead || bStatus == false) &&  m_imgCodecType == IMG_DEC_JPEG )
		{
			// If the data is not decoded, the data is put into a file and is decoded using the file API's- Hema
			int imageWidth=0,imageHeight=0,nof=0,Imagesize=0;

			AcStrcpy( fileName, SFB_IMAGE_DECODER_FILE_NAME );
			//Creating a file to temporarirly save the image so that it can be decoded using the file API's.
			hFile = FmCreateFile( fileName,FM_WRITE | FM_TRUNCATE );
			if(hFile == INVALID_HOBJ)
				return;
			
			/*Todo. Add a check here to see if the memory is available in file system to write this buffer to file*/
			if(!FmWriteFile(hFile, ( unsigned char* )m_data->data(), m_data->size()))
			{
				FmCloseFile(hFile);
				return;
			}
			FmCloseFile(hFile);
		
			Imagesize= ImgGetMemoryRequired ((unsigned char*)m_data->data(), m_data->size(), ( unsigned int* )&imageWidth,( unsigned int* )&imageHeight,( unsigned int* )&nof );

			if(Imagesize > 0)
			{	// Decode the image using file API's	
				if(ImgDecodeFileEx( fileName, &file_m_image, imageWidth, imageHeight,IMG_RGB_OUTPUT,IMG_RESIZE_MAINTAIN_ASPECT_RATIO))
				{
					//Copying the Image attributes into copy_m_image. 
					//copy_m_image will contain file_m_image: If the image is decoded from file API's
			        if(file_m_image)
					copy_m_image = file_m_image;
					//making bStatus as true, as image is decoded now, using file API's
					bStatus = true;
				}
				//The copy_m_image will contain m_image:if image is not decoded even after using file API's
				else if(m_image)
				copy_m_image = m_image;
									
			}
			FmRemoveFile(fileName);
			hFile = INVALID_HOBJ;
							
		}

		//Putting the value of m_image into copy_m_image if the image is decoded using buffer 
		//or if the image was not decoded using file API's due to insufficient memory.
		else if(m_image)
		copy_m_image = m_image;

		//If the data is not decoded  with file API's as well as Buffer API's.- Hema
		if((copy_m_image == NULL || bStatus == false)  || !copy_m_image->pHead)
				 return;

		// Set the image attributes
		// update the size
		if(copy_m_image->width && copy_m_image->height && !m_sizeAvailable)
		{
			m_size.setWidth( copy_m_image->width );
			m_size.setHeight( copy_m_image->height );
			m_sizeAvailable = true;
		}

		// Add the raw data to the image structure
		m_frameBufferCache[0].setBytes((unsigned char*)copy_m_image->pHead->pbmBitmap->pBits);

		// Inform that decode is complete
		m_frameBufferCache[0].setStatus( RGBA32Buffer::FrameComplete );

		m_frameBufferCache[0].setBitsPerPixel( copy_m_image->pHead->pbmBitmap->bitsPerPixel);
		m_frameBufferCache[0].setUseChromakey( copy_m_image->pHead->pbmBitmap->bChromaKeyFlag);
		m_frameBufferCache[0].setChromakey( copy_m_image->pHead->pbmBitmap->chromaKey );
		m_frameBufferCache[0].ensureHeight(copy_m_image->pHead->pbmBitmap->height);
		m_frameBufferCache[0].setAlphaChannel((unsigned short*)copy_m_image->pHead->pbmBitmap->pAlphaChannel);
		m_frameBufferCache[0].setCodecType(m_imgCodecType);


		// Set the frame rect
		IntRect frameRect(copy_m_image->pHead->x, copy_m_image->pHead->y,
					  copy_m_image->pHead->pbmBitmap->width,
					  copy_m_image->pHead->pbmBitmap->height);
		m_frameBufferCache[0].setRect( frameRect );
	}
}
#else //USE_FILE_JPEG_DECODE_API

void OtherImageDecoder::decode(bool sizeOnly)
{
    if (m_failed)
        return;
	if(SFBCheckBufferSize( m_imgCodecType,m_image, ( unsigned char* )m_data->data(),m_data->size())!=TRUE)
    {
        MemoryManager::SetStatus(CMM_CHECKRESULT_COLLECTING);
		return ;
    }
	else
	{
    // Decode the image
	ImgDecodeImage( m_imgCodecType, m_image, (unsigned char*)m_data->data(), m_data->size() );

    // The data was not decoded
    if(!m_image->pHead)
        return;

    // Set the image attributes
    // update the size
    if(m_image->width && m_image->height && !m_sizeAvailable)
    {
        m_size.setWidth( m_image->width );
        m_size.setHeight( m_image->height );
        m_sizeAvailable = true;
    }

    // Add the raw data to the image structure
    m_frameBufferCache[0].setBytes((unsigned char*)m_image->pHead->pbmBitmap->pBits);

    // Inform that decode is complete
    m_frameBufferCache[0].setStatus( RGBA32Buffer::FrameComplete );

    m_frameBufferCache[0].setBitsPerPixel( m_image->pHead->pbmBitmap->bitsPerPixel);
    m_frameBufferCache[0].setUseChromakey( m_image->pHead->pbmBitmap->bChromaKeyFlag);
    m_frameBufferCache[0].setChromakey( m_image->pHead->pbmBitmap->chromaKey );
    m_frameBufferCache[0].ensureHeight(m_image->pHead->pbmBitmap->height);
	m_frameBufferCache[0].setAlphaChannel((unsigned short*)m_image->pHead->pbmBitmap->pAlphaChannel);
	m_frameBufferCache[0].setCodecType(m_imgCodecType);


    // Set the frame rect
    IntRect frameRect(m_image->pHead->x, m_image->pHead->y,
                  m_image->pHead->pbmBitmap->width,
                  m_image->pHead->pbmBitmap->height);
    m_frameBufferCache[0].setRect( frameRect );
	}

}
#endif //USE_FILE_JPEG_DECODE_API

} // namespace WebCore

// End of File
