/*
 * Copyright (C) 2006 Apple Computer, Inc.  All rights reserved.
 * Copyright (C) 2006 Michael Emmel mike.emmel@gmail.com
 * Copyright (C) 2007 Alp Toker <alp.toker@collabora.co.uk>
 * Copyright (C) 2007 Holger Hans Peter Freyther
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1.  Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 * 2.  Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 * 3.  Neither the name of Apple Computer, Inc. ("Apple") nor the names of
 *     its contributors may be used to endorse or promote products derived
 *     from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY APPLE AND ITS CONTRIBUTORS "AS IS" AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL APPLE OR ITS CONTRIBUTORS BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#include "config.h"
#include "SimpleFontData.h"

#include "FloatRect.h"
#include "Font.h"
#include "FontCache.h"
#include "FontDescription.h"
#include "GlyphBuffer.h"
#include "NotImplemented.h"
#include "GWES.h"

#define LINE_GAP  (2)

namespace WebCore {

void SimpleFontData::platformInit()
{
	HFNT hFont = m_font.getFontHandle();
	int height = WmGetFontMaxHeight(hFont);

	m_descent = WmGetFontDescender( hFont, height );
	m_ascent = WmGetFontAscender( hFont, height );
	m_lineSpacing = m_ascent + m_descent + LINE_GAP;
	m_lineGap = LINE_GAP;

	int	width;
	MCHAR mszText[3];

	WmConvert2PCode (mszText, sizeof(mszText), " ");
	WmGetTextExtentN(hFont, mszText, 1, &width, &height);
    m_spaceWidth =  width;
	WmConvert2PCode (mszText, sizeof(mszText), "x");
	WmGetTextExtentN(hFont, mszText, 1, &width, &height);
	m_xHeight = height;
}

void SimpleFontData::platformDestroy()
{
    delete m_smallCapsFontData;
	//m_smallCapsFontData = NULL;
}

SimpleFontData* SimpleFontData::smallCapsFontData(const FontDescription& fontDescription) const
{
    if (!m_smallCapsFontData) {
        FontDescription desc = FontDescription(fontDescription);
        desc.setComputedSize(0.70f*fontDescription.computedSize());
        desc.setSpecifiedSize(fontDescription.computedSize());
        const FontPlatformData* pdata = new FontPlatformData(desc, desc.family().family());
        m_smallCapsFontData = new SimpleFontData(*pdata);
		delete pdata;
		pdata = NULL;
    }
    return m_smallCapsFontData;
}

bool SimpleFontData::containsCharacters(const UChar* characters, int length) const
{
	notImplemented();
    return false;
}

void SimpleFontData::determinePitch()
{
	m_treatAsFixedPitch = m_font.isFixedPitch();
}

float SimpleFontData::platformWidthForGlyph(Glyph glyph) const
{
	int	width, height;
	HFNT hFont = m_font.getFontHandle();
	WmGetTextExtentN(hFont, (MCHAR *) &glyph, 1, &width, &height);

    return width;
}

float SimpleFontData::platformHeightForGlyph(Glyph glyph) const
{
	int	width, height;
	HFNT hFont = m_font.getFontHandle();
	WmGetTextExtentN(hFont, (MCHAR *) &glyph, 1, &width, &height);

    return height;
}

void SimpleFontData::setFont(HGC* cr) const
{
    ASSERT(cr);
    m_font.setFont(cr);
}

}
