#if defined(USE_NATIVE_WIDGET_SUPPORT) && defined(USE_LIST_BOX_SUPPORT)

#include "NativeListBox.h"
#include "IntRect.h"
#include "Document.h"
#include "Frame.h"
#include "Page.h"
#include "FocusController.h"
#include "EventHandler.h"
#include "PlatformKeyboardEvent.h"
#include "FrameView.h"
#include "FontPlatformData.h"

#ifdef SFB_GWES_TRANSFORM
#include "ChromeClient.h"
#endif

#include "ShpJasmine.h"
#include "CString.h"

namespace WebCore {

/* event handlers for List box */
DECLARE_EVENT_MAP( WkNativeListBoxEventMap );

DECLARE_HANDLE_EV_PRESSED(WkNativeListBoxEventKeyPressed);
DECLARE_HANDLE_EV_TOUCH_PRESSED(WkNativeListBoxEventTouchPressed);

BEGIN_EVENT_MAP( WkNativeListBoxEventMap, 2)
	HANDLE_EV_PRESSED(WkNativeListBoxEventKeyPressed)
	HANDLE_EV_TOUCH_PRESSED(WkNativeListBoxEventTouchPressed)
END_EVENT_MAP()

static int iListIndex = -1;
BOOL WkNativeListBoxEventTouchPressed(WmState* pState, WmEvent* pEvent)
{
	bool fEventAbsorbed = false;

	if(TRUE == pEvent->touch.bPressed)
	{
		NativeListBox* pNativeListBox = NativeListBox::getNativeListBoxInstance();
		if(pNativeListBox && pNativeListBox->isActivated())
		{
			HFNT  ListFont[MAX_LISTEX_LINE][MAX_LISTEX_COLUMN] = {0, };
			COLOR ListColor[MAX_LISTEX_LINE][MAX_LISTEX_COLUMN] = {0, };

			COLOR ItemBgColor = WHITE;

			HWIN hWinList = pNativeListBox->GetWindowHandle();

			if(WmIsWindowValid (hWinList))
			{
				//int iListIndex = -1;
				if(INVALID_DEPTH != SFBalGwesGetListExTouchPressedItemIndex(hWinList, pEvent->touch.startX, pEvent->touch.startY, &iListIndex))
				{
					SFBalGwesGetListExColumnForm(hWinList, ListFont, ListColor, ListColor, iListIndex);
					if(FALSE == pNativeListBox->GetItemValue(iListIndex)) // Not selected. Change the colour of selected item. Remember selection
					{
						ListColor[0][0] = ListColor[0][1] = ListColor[1][0] = ListColor[1][1] = RED;
						ItemBgColor	= NAVY;
						pNativeListBox->SetItemValue(iListIndex, TRUE);
					}
					else // Already selected. So deselect. Clear selection info
					{
						ListColor[0][0] = ListColor[0][1] = ListColor[1][0] = ListColor[1][1] = BLACK;
						pNativeListBox->SetItemValue(iListIndex, FALSE);
						ItemBgColor	= WHITE;
					}

					SFBalGwesSetListExColumnForm(hWinList, ListFont, ListColor, ListColor, ONE_DEPTH, iListIndex);
					SFBalGwesSetListExFocusItemBgColor(hWinList, ItemBgColor);
					fEventAbsorbed = true;
				}
			}
		}
	}

	return TRUE;
}

BOOL WkNativeListBoxEventKeyPressed(WmState* pState, WmEvent* pEvent)
{
	bool fEventAbsorbed = false;

	switch ( pEvent->key.keyCode )
	{
		case VK_HP_RIGHTEASY:
		case VK_HP_LEFTEASY:
		{
			NativeListBox* pNativeList = NativeListBox::getNativeListBoxInstance();
			if(pNativeList && pNativeList->isActivated())
			{
				pNativeList->unsetFocusedNode();
				pNativeList->deactivate();
				fEventAbsorbed = true;
			}
			break;
		}
	}

	if(!fEventAbsorbed)
	{
		WmCallNextKeyPressedHandler(pState, pEvent);
	}

	return TRUE;
}

static NativeListBox* g_pNativeListBox = NULL;
NativeListBox* NativeListBox::getNativeListBoxInstance()
{
	if(NULL == g_pNativeListBox)
	{
		g_pNativeListBox = new NativeListBox();
	}

	return g_pNativeListBox;
}

void NativeListBox::destroyNativeListBoxInstance()
{
	NativeListBox* pNativeListBox = NativeListBox::getNativeListBoxInstance();
	if(pNativeListBox && pNativeListBox->isActivated())
	{
		pNativeListBox->deactivate();
	}
	delete g_pNativeListBox;
	g_pNativeListBox = NULL;
}

NativeListBox::NativeListBox() :
	m_handleWin(INVALID_HOBJ),
	m_pDocument(NULL),
	m_pSelectElement(NULL),
	m_nodeID(Node::ID_NONE)
{

}

NativeListBox::~NativeListBox()
{
	if(m_handleWin != INVALID_HOBJ)
	{
		deactivate();
	}
}

void NativeListBox::create(const NativeListBoxAttrib& attrib)
{
	m_pDocument = attrib.m_pDocument;
	m_nodeID = attrib.m_nodeID;
	_createListBox(attrib);
}

void NativeListBox::_createListBox(const NativeListBoxAttrib& attrib)
{
	int iListCount = 0;
	IntRect rect(attrib.m_rect);

	m_pDocument->frame()->view()->scrollTo(rect);
	rect = m_pDocument->frame()->view()->contentsToWindow(rect);
	if(0 > rect.x())
	{
		m_pDocument->frame()->view()->scrollBy(rect.x(), 0);
		rect.setX(0);
	}

	int visibleWidth = m_pDocument->frame()->view()->visibleWidth();
	int visibleHeight = m_pDocument->frame()->view()->visibleHeight();

#if defined(SFB_GWES_TRANSFORM)
	float zoomFactor = m_pDocument->page()->chrome()->client()->getZoomFactor();

	int frameRectX = m_pDocument->frame()->view()->contentsX();
	int frameRectY = m_pDocument->frame()->view()->contentsY();
	IntRect viewPortRect = m_pDocument->page()->chrome()->client()->viewportRect();

	rect.setX((rect.x() - (viewPortRect.x() - frameRectX)) * zoomFactor);
	rect.setY((rect.y() - (viewPortRect.y() - frameRectY)) * zoomFactor);

	rect.setWidth(rect.width() * zoomFactor);
	rect.setHeight(rect.height() * zoomFactor);

#endif
	int height= rect.height();
	int width = rect.width();

#if defined(SFB_GWES_TRANSFORM)

	if(rect.x() > 0)
	{
		if( (width + rect.x() ) > visibleWidth)
		{
			width = visibleWidth - rect.x() ;
		}
	}
	else
	{
		if(width > visibleWidth)
		{
			width = visibleWidth;
		}
	}
#else

	if( (width + rect.x() ) > visibleWidth)
	{
		width = visibleWidth - rect.x() ;
	}
#endif

	if( (height + rect.y() ) > visibleHeight)
	{
		height=visibleHeight - rect.y();
	}

	iListCount = attrib.m_pSelectElement->listItems().size();

	if((iListCount>0) && (width > 0) &&(height > 0) &&(rect.height()>0) && (rect.width()>0))
	{
		int hWin = INVALID_HOBJ;
		int hGC = INVALID_HOBJ;
		getDocument()->frame()->page()->chrome()->client()->getWindowHandles(&hWin,&hGC);

		if(WmIsWindowValid (hWin ))
		{
			m_handleWin = SFBalGwesCreateListBox(hWin,LISTEX_STYLE_NONE | LISTEX_STYLE_1TOUCH_EXECUTE, rect.x(), rect.y(), width, height,WmGetEventMap(WkNativeListBoxEventMap));

			RenderStyle* style = attrib.m_pStyle;
			FontDescription fontDesc = style->fontDescription();
			FontPlatformData fontData(fontDesc, fontDesc.family().family());
			WmSetWindowFont(m_handleWin, fontData.getFontHandle());
		}
	}

	m_nodeID = Node::ID_SELECT;
}

void NativeListBox::addItem(String value, unsigned index, bool bEnable)
{
	if (!value)
	{
		value.append(" ");
	}

	if(WmIsWindowValid(m_handleWin))
	{
		SFBalGwesAddListItemText(m_handleWin, value.latin1().data());
	}
}

void NativeListBox::SetCountOfItems(int iCount)
{
	m_ItemValues.resize(iCount);
}

BOOL NativeListBox::GetItemValue(int iIndex)
{
	return m_ItemValues[iIndex];
}

void NativeListBox::SetItemValue(int iIndex, BOOL bValue)
{
	m_ItemValues[iIndex] = bValue;
}

void NativeListBox::setCurSelItem(int index)
{
	SFBalGwesSetListExFocusItem(m_handleWin, ONE_DEPTH, index);
}

HWIN NativeListBox::GetWindowHandle()
{
	return m_handleWin;
}

void NativeListBox::activate(const NativeListBoxAttrib& attrib)
{
	if(WmIsWindowValid (m_handleWin))
	{
		m_pSelectElement = attrib.m_pSelectElement;
		
		String m_State;
		getElement()->saveState(m_State);

		HFNT  ListFont[MAX_LISTEX_LINE][MAX_LISTEX_COLUMN] = {0, };
		COLOR ListColor[MAX_LISTEX_LINE][MAX_LISTEX_COLUMN] = {0, };
		COLOR ItemBgColor;

		int iListCount = SFBalGwesGetListExAllItemCount(m_handleWin);

		for(int i=0; i < iListCount; i++)
		{
			SFBalGwesGetListExColumnForm(m_handleWin, ListFont, ListColor, ListColor, i);

			if('X' == m_State[i])
				m_ItemValues[i] = TRUE;
			else
				m_ItemValues[i] = FALSE;

			if(TRUE == m_ItemValues[i]) // To display selected items initially
			{
				ListColor[0][0] = ListColor[0][1] = ListColor[1][0] = ListColor[1][1] = RED;
				ItemBgColor	= NAVY;
				SFBalGwesSetListExColumnForm(m_handleWin, ListFont, ListColor, ListColor, ONE_DEPTH, i);
			}
			else
			{
				ItemBgColor = WHITE;
			}

			SFBalGwesSetListExFocusItemBgColor(m_handleWin, ItemBgColor);
		}

		WmSetFocusWindow(m_handleWin);
		WmShowWindow(m_handleWin);
	}
}


void NativeListBox::deactivate()
{
	if(WmIsWindowValid(m_handleWin))
	{
		int hWin = INVALID_HOBJ;
		int hGC = INVALID_HOBJ;

		for(int i=0; i < m_ItemValues.size(); i++)
		{
			m_pSelectElement->setListSelectedIndex(i, m_ItemValues[i], true);
		}

		if(m_pDocument && m_pDocument->frame())
		{
			m_pDocument->frame()->page()->chrome()->client()->getWindowHandles(&hWin,&hGC);
		}

		WmDestroyWindow(m_handleWin);

		m_handleWin = INVALID_HOBJ;
		m_pDocument = NULL;
		m_pSelectElement = NULL;
		m_nodeID    = Node::ID_NONE;

		if(INVALID_HOBJ != hWin)
		{
			WsDrawMainSoftKey(WmGetParentWindow(hWin));
			WsShowMainSoftKey(WmGetParentWindow(hWin));
			WmDrawWindow(hWin, TRUE );
			WmShowWindow(hWin);
		}
	}
}

bool NativeListBox::isActivated() const
{
	return WmIsWindowValid(m_handleWin);
}

void NativeListBox::unsetFocusedNode()
{
	if(m_pDocument && m_pDocument->frame())
	{
		if(isActivated())
		{
			if(m_pDocument->frame()->view())
			{
				FrameView* view = m_pDocument->frame()->view();

				while (view->parent())
				{
					view = static_cast<FrameView*>(view->parent());
				}
				view->update();
			}
		}

		m_pDocument->page()->focusController()->setFocusedNode(0, m_pDocument->frame());
	}
}

void NativeListBox::SetSelectedIndex(bool bSelect)
{
	ASSERT(INVALID_HOBJ != m_handleWin);
	NativeListBox* pNativeListBox = NativeListBox::getNativeListBoxInstance();
	if(pNativeListBox && pNativeListBox->isActivated())
	{
		int selectedIndex = -1;
		if(SFBalGwesGetListExFocusedItem(m_handleWin, &selectedIndex))
		{
			getElement()->setListSelectedIndex(selectedIndex, bSelect, true);

			if(isActivated() && !getElement()->usesMenuList())
			{
				getElement()->onChange();
			}
		}
	}
}


#ifdef SFB_GLOBAL_STATIC_HANDLING
void initNativeListBoxGlobals()
{
    g_pNativeListBox = NULL;
	iListIndex = -1;
}
#endif

}  // namespace WebCore


#endif //#if defined(USE_NATIVE_WIDGET_SUPPORT) && defined(USE_LIST_BOX_SUPPORT)

