/*
* ============================================================================
*  Name     : WebKitBridge.cpp
*  Part of     : WebKit / BrowserView
*
*  Description: Client side bridge implementation
* ============================================================================
*/


//INCLUDES
#include "WebControl.h"
#include "WebKitBridge.h"
#include "WebKitLoader.h"
#include "ResourceLoader.h"
#include "CString.h"
#include "UrlRequestInfo.h"
#include "PlatformString.h"


//CONSTANTS
//MACROS

//DATATYPES

// ----------------------------------------------------------------------------
// WebKitBridge::WebKitBridge()
// Default constructor for WebKitBridge
// ----------------------------------------------------------------------------
// <TODO> : Remove the parameters from the constructor
// since WebKitBridge can access WebControl
WebKitBridge::WebKitBridge()	
    {
	m_loader = new WebKitLoader(this);
    }

// ----------------------------------------------------------------------------
// WebKitBridge::~WebKitBridge()
// WebKitBridge class Destructor
// ----------------------------------------------------------------------------
WebKitBridge::~WebKitBridge()
    {
    // Delete all memory that we own
    delete m_loader;
    }


// ----------------------------------------------------------------------------
// WebKitBridge::LoadPage()
// Loads the main page
// ----------------------------------------------------------------------------
void WebKitBridge::LoadPage( WebCore::ResourceLoader* _resourceLoader,
                                WebCore::ResourceRequest* _request )
{
    UrlRequestInfo* urlRequestInfo = new UrlRequestInfo();

	const WebCore::String m_String = _request->url().string();

//	m_loader->SetPageUrl( static_cast <String &>(_request->url().string()));
	m_loader->SetPageUrl(m_String );
	
	urlRequestInfo->SetUrl( _request->url().string().latin1().data() );
	urlRequestInfo->SetCurrentUrl( _request->url().string().latin1().data() );

	if( "GET" == _request->httpMethod() )
    {
        urlRequestInfo->SetLoadMethod( URL_GET );
    }
    else if("POST" == _request->httpMethod())
    {
        urlRequestInfo->SetLoadMethod( URL_POST );

        if(_request->httpBody())
        {
        urlRequestInfo->SetPostDataStr( _request->httpBody()->flattenToString() );

#if HTTP_POST_UPLOAD_SUPPORT
		urlRequestInfo->SetPostDataElemets( _request->httpBody());
#endif
    }
    }

	// Set the request headers
    urlRequestInfo->SetRequestHeaders(_request->httpHeaderFields());

    // set the cache mode
    urlRequestInfo->SetCacheMode( _request->cachePolicy() );

    // start loading
    m_loader->LoadPage( urlRequestInfo, _resourceLoader );
}

// ----------------------------------------------------------------------------
// WebKitBridge::LoadResource()
// Loads a sub-resource
// ----------------------------------------------------------------------------
void WebKitBridge::LoadResource( WebCore::ResourceLoader* _resourceLoader,
                                    const WebCore::ResourceRequest* _request )
{
    UrlRequestInfo* requestInfo = new UrlRequestInfo();

    // Set the main page url
	requestInfo->SetUrl( m_loader->PageUrl().latin1().data() );
    // Set the current url
	requestInfo->SetCurrentUrl( (const char *) _request->url().string().ascii().data() );

	if("POST" == _request->httpMethod())
	{
		requestInfo->SetLoadMethod(URL_POST);

		if(NULL != _request->httpBody())
			requestInfo->SetPostDataStr(_request->httpBody()->flattenToString());
	}

    requestInfo->SetTopLevel( false );
   		
    // Set the request headers
	requestInfo->SetRequestHeaders( _request->httpHeaderFields() );

    // Set the cache mode
#ifdef SFB_WIDGET
	WebControl::WebControl* iControl = WebControl::kit(GetWebCoreBridge()->GetFrame()->page());
	
	if(equalIgnoringCase(iControl->GetInstanceName(),"WIDGET"))
	{
		//Setting Cache Mode as "ReloadIgnoringCacheData" for Widgets - Fix for CQ 3977 
		requestInfo->SetCacheMode( ReloadIgnoringCacheData );
	}
	else
#endif //SFB_WIDGET
	{
    requestInfo->SetCacheMode( _request->cachePolicy() );
	}

    // load the resource
    m_loader->LoadResource( requestInfo, _resourceLoader );
}

// ----------------------------------------------------------------------------
// WebKitBridge::CancelTransaction()
// Cancels a particular transaction for the given resource loader
// ----------------------------------------------------------------------------
void WebKitBridge::CancelTransaction( WebCore::ResourceLoader* _resourceLoader )
{
    m_loader->CancelTransaction( _resourceLoader );
}

// ----------------------------------------------------------------------------
// WebKitBridge::CancelAll()
// Cancels a particular transaction for the given resource loader
// ----------------------------------------------------------------------------
void WebKitBridge::CancelAll( )
{
    m_loader->CancelAll();
}

// ----------------------------------------------------------------------------
// WebKitBridge::RedirectTransaction()
// Redirects the contents to the given content listener
// ----------------------------------------------------------------------------
eErrorCode WebKitBridge::RedirectTransaction( WebCore::ResourceLoader* _resourceLoader, 
                                               int* _transportHandle )
{
    return m_loader->RedirectTransaction( _resourceLoader, _transportHandle );
}

// ----------------------------------------------------------------------------
// WebKitBridge::UpdateMetaCache()
// Update meta Cache
// ----------------------------------------------------------------------------
void WebKitBridge::UpdateMetaCache( const char* szUrl, 
									const char* szMetaField, 
									const char* szMetaFieldContent )
{
	m_loader->UpdateMetaCache( szUrl, szMetaField, szMetaFieldContent );
}

// ----------------------------------------------------------------------------
// WebKitBridge::SetCookie()
// Update meta cookie
// ----------------------------------------------------------------------------
void WebKitBridge::SetCookie( const char* szURL, 
							  const char* szContent)
{
	m_loader->SetCookie( szURL, szContent );
}

// ----------------------------------------------------------------------------
// WebKitBridge::GetCookie()
// Update meta cookie
// ----------------------------------------------------------------------------
char* WebKitBridge::GetCookie( const char* szUrl)
{
	return m_loader->GetCookie( szUrl );
}

// ----------------------------------------------------------------------------
// WebKitBridge::GetCookieEnabled()
// Get Cookie enable status
// ----------------------------------------------------------------------------
BOOL WebKitBridge::GetCookieEnabled()
{
	return m_loader->GetCookieEnabled();
}

// End of File

