#ifndef __WEBKITBRIDGE_H__
#define __WEBKITBRIDGE_H__

//INCLUDES
#include "BridgeTypes.h"
#include "WebCoreBridge.h"
#include "CMMDefs.h"

//CONSTANTS

//MACROS

//DATATYPES

// FORWARD DECLARATIONS
class WebKitLoader;

class WebKitBridge  : public IWebClientBridge
    CMM_MODIFIED_ADD
    {
    public: //  Constructors and destructor

        /**
        * Constructor.
        */
        WebKitBridge();

        /**
        * destructor.
        */
        ~WebKitBridge();

    public: // IWebClientBridge Methods

        /**
        * LoadPage
        * Public Class Method
        * Purpose - Loads the main page
        * @param ResourceLoader* _resourceLoader
        * @param ResourceRequest* _request
        * @return void
        */
        virtual void LoadPage( WebCore::ResourceLoader* _resourceLoader,
                                WebCore::ResourceRequest* _request );

        /**
        * LoadResource
        * Public Class Method
        * Purpose - Loads a sub-resource
        * @param const ResourceLoader* _resourceLoader
        * @param const ResourceRequest* _request
        * @return void
        */
        virtual void LoadResource( WebCore::ResourceLoader* _resourceLoader,
                                    const WebCore::ResourceRequest* _request );

        /**
        * CancelTransaction
        * Public Class Method
        * Purpose - Cancel a particular transaction for the given resource loader
        * @param _resourceLoader : The resource loader whose transaction is to be canceled
        * @return void
        */
        void CancelTransaction( WebCore::ResourceLoader* _resourceLoader );

        /**
        * CancelAll
        * Public Class Method
        * Purpose - Cancel all outstanding requests
        * @return void
        */
        void CancelAll( );

        /**
        * GetWebCoreBridge
        * Public Class Method
        * Purpose - Returns the WebCoreBridge
        * @return WebCoreBridge
        */
        WebCoreBridge*  GetWebCoreBridge() const { return m_webCoreBridge; }

        /**
        * SetWebCoreBridge
        * Public Class Method
        * Purpose - Stores the WebCoreBridge
        * @param _webCoreBridge : The bridge instance to be stored
        * @return void
        */
        void SetWebCoreBridge( WebCoreBridge* _webCoreBridge ) { m_webCoreBridge = _webCoreBridge; }

        /**
        * RedirectTransaction
        * Public Class Method
        * Purpose - Redirects the contents to the given content listener
        * @param _resourceLoader : The resource loader whose transaction is to be redirected
		* @param _transportHandle : This is the out parameter, returns transport handle
        * @return eErrorCode
        */
        eErrorCode RedirectTransaction( WebCore::ResourceLoader* _resourceLoader, 
										int* _transportHandle);

		/*
		 * Update meta Cache
		 * @param
         * @return
		 */
		void UpdateMetaCache( const char* szUrl, const char* szMetaField , const char* szMetaFieldContent);

		/*
		 * Update meta cookie
		 * @param
         * @return
		 */
		void SetCookie( const char* szUrl, const char* szContent);

		/*
		 * Update meta cookie
		 * @param
         * @return
		 */
		char* GetCookie( const char* szUrl);

		/*
		 * Update meta cookie
		 * @param
         * @return
		 */
		BOOL GetCookieEnabled();


	private:    // Class Data
        //
        WebCoreBridge*         m_webCoreBridge;       // not-owned
        //
        WebKitLoader*          m_loader;              // owned
        
    }; // class WebKitBridge
    
#endif  //__WEBKITBRIDGE_H__

