/*
* ==============================================================================
*  Name        : WebKitLoader.cpp
*  Part of     : WebKit / BrowserView
*  Description :
*  Version     : 3.1
* ==============================================================================
*/


// INCLUDE FILES
#include "ShpJasmine.h"
#include "types.h"
#include "WebKitLoader.h"
#include "WebKitBridge.h"
#include "LoadListeners.h"
#include "WebCoreBridge.h"
#include "RequestDispatcher.h"
#include "ContentDispatcher.h"
#include "CString.h"
#include "WebControl.h"
/* Gnana modified for Http-equiv cache and cookie*/


//#include "WebKitShpPage.h"
//#include "WebKitShpPrivate.h"
/*end of modification- Gnana*/

#ifdef __CMM__
#include <AnsiCLib.h>
# else 
#include "Shp.h"
# endif


#ifdef __CMM__
#include <allocs.h>
#endif

#include "WebDefs.h" // WebControl Error Code

// EXTERNAL DATA STRUCTURES

// EXTERNAL FUNCTION PROTOTYPES

// CONSTANTS


// MACROS

// LOCAL CONSTANTS AND MACROS

// MODULE DATA STRUCTURES

// LOCAL FUNCTION PROTOTYPES
int __ConvertToWebControlError( eErrorCode error );
// FORWARD DECLARATIONS

// ============================= LOCAL FUNCTIONS ===============================
// ----------------------------------------------------------------------------
// int __ConvertToWebControlError
//
// This function converts Error to SFB WebControl Error.
// ----------------------------------------------------------------------------
int __ConvertToWebControlError( eErrorCode error ){
	// [todo]
	// need to covert to webcontrol error
	// refer to error codes of WebControl in WebDefs.h
	int errorType = WebControl::ERROR_UNKNOWN;
	switch(error)
	{
	case ERR_INVALID_URL:
		errorType = WebControl::ERROR_INVALID_URL;
		break;
	case ERR_OK:
		errorType = WebControl::HRESULT_URL_SCHEME_HANDLED;
		break;	
	}
	return errorType;
}
// ============================ MEMBER FUNCTIONS ===============================

// -----------------------------------------------------------------------------
// WebKitLoader::WebKitLoader
// C++ default constructor
// -----------------------------------------------------------------------------
//
WebKitLoader::WebKitLoader( WebKitBridge* _webKitBridge)
    : m_webKitBridge( _webKitBridge )      
    {
    m_contentDispatcher = new ContentDispatcher();
    m_requestDispatcher = new RequestDispatcher( this, m_contentDispatcher);

    // Set the main page url to about:blank
    m_pageUrl = "about:blank";
    }

// Destructor
WebKitLoader::~WebKitLoader()
    {
    // delete resource loader dispatchers first
    delete m_requestDispatcher;
    if( m_contentDispatcher )
        {
        m_contentDispatcher->Deref();
        }
    }

// -----------------------------------------------------------------------------
// WebKitLoader::LoadPage
// Public Class Method
// Request to load a url
// -----------------------------------------------------------------------------
eErrorCode WebKitLoader::LoadPage( UrlRequestInfo* _urlRequestInfo,
                                    WebCore::ResourceLoader* _resourceLoader )
    {
    _urlRequestInfo->SetTopLevel( true );

    IContentLoader* urlLoader = NULL;
	WebControl::WebControl* iControl = WebControl::kit(GetWebKitBridge()->GetWebCoreBridge()->GetFrame()->page());

    // Dispatch request
	eErrorCode status = m_requestDispatcher->Dispatch( _urlRequestInfo, 
                                                        &urlLoader,
                                                        _resourceLoader->isMainResourceLoader(),
														iControl->GetStateChangeInfo());
	

    if( ERR_OK == status && urlLoader )
        {
        // new load
        int transId( _urlRequestInfo->TransId() );

        // Create a page load listner and associate it with the dispatcher
		PageLoadListener* pageLoadListener = new PageLoadListener( this, _urlRequestInfo->Url()  );

        // Let the listner hold the webcore resourceloader
        pageLoadListener->SetResourceLoader( _resourceLoader );

        m_contentDispatcher->SetContentListener( transId, pageLoadListener);
        }
	// convert to SFB Webcontrol error message to be displayed on UI and stop fetching invalid URL continuously
	else 
		{
			int iStatus = __ConvertToWebControlError( status );
			WebCore::ResourceError r( "", iStatus, _urlRequestInfo->Url(), "" );
			_resourceLoader->didFail(r);		
		}

	return status;
    }

// ----------------------------------------------------------------------------
// WebKitLoader::LoadResource
//
// Loads URL and returns the handle to resource
// ----------------------------------------------------------------------------
eErrorCode WebKitLoader::LoadResource( UrlRequestInfo* _urlRequestInfo,
                                        WebCore::ResourceLoader* _resourceLoader )
    {
    eErrorCode status = ERR_OK;

    IContentLoader* urlLoader = NULL;
	WebControl::WebControl* iControl = WebControl::kit(GetWebKitBridge()->GetWebCoreBridge()->GetFrame()->page());

    // Dispatch request
	status = m_requestDispatcher->Dispatch( _urlRequestInfo, 
                                            &urlLoader,
                                            _resourceLoader->isMainResourceLoader(),
                                           	iControl->GetStateChangeInfo());

    if( ERR_OK == status && urlLoader )
        {
        int transId( _urlRequestInfo->TransId() );
        ResourceLoadListener* resLoadListener = new ResourceLoadListener( this );

        // webcore resource loader
        resLoadListener->SetResourceLoader( _resourceLoader );
        resLoadListener->SetTransactionId( transId );
        m_contentDispatcher->SetContentListener( transId, resLoadListener );
        }
    return status;
    }

// ----------------------------------------------------------------------------
// WebKitLoader::CancelTransaction
//
// Cancel a particular transaction for the given Transaction Id
// ----------------------------------------------------------------------------
void WebKitLoader::CancelTransaction( int _transactionId, eErrorCode _error )
    {
    m_requestDispatcher->CancelTransaction( _transactionId, _error );
    }

// ----------------------------------------------------------------------------
// WebKitLoader::CancelTransaction
//
// Cancel a particular transaction for the given resource loader
// ----------------------------------------------------------------------------
void WebKitLoader::CancelTransaction( const WebCore::ResourceLoader* _resourceLoader )
    {
    // Find the transaction entry
    TransactionEntry* trans = m_contentDispatcher->FindTransactionEntry( _resourceLoader );
    if( trans )
        {
        m_requestDispatcher->CancelTransaction( trans->m_transactionId, ERR_CANCEL );
        }
    }

/**
 * redirect the data stream of transID to IContentListener
 */
eErrorCode WebKitLoader::RedirectTransaction( WebCore::ResourceLoader* _resLoader, 
                                                int* _transportHandle )
    {
    // Find the transaction entry
    TransactionEntry* trans = m_contentDispatcher->FindTransactionEntry( _resLoader );
    eErrorCode status = ERR_CANCEL;
    if( trans )
        {
	    status = m_requestDispatcher->RedirectTransaction( trans->m_transactionId, _transportHandle );
        }

    return status;
    }

// ----------------------------------------------------------------------------
// WebKitLoader::CancelAll
//
// Cancel all outstanding requests
// ----------------------------------------------------------------------------
void WebKitLoader::CancelAll()
    {
    m_requestDispatcher->CancelAllLoaders( ERR_CANCEL );
    }


/*Gnana Modification for HTTP-Equiv for cache and cookie */
#if PLATFORM(SHP)

WebControl::WebControl* gpWebControl = NULL; //temp hack Gnana

//-----------------------------------------------------------------------------
// WebKitLoader::UpdateMetaCache 
// Update meta Cache
//-----------------------------------------------------------------------------
void WebKitLoader::UpdateMetaCache( const char* szUrl,
								    const char* szMetaField,
									const char* szMetaFieldContent)
{
	XMLFrame* pFrame = GetWebKitBridge()->GetWebCoreBridge()->GetFrame();
	WebControl::WebFrame *pWebFrame = WebControl::kit( pFrame );
	WebControl::WebControl* pWebControl = pWebFrame->GetWebControl();
	
	SFBUpdateCacheMetaData ( SFBRmGetCacheStorage((SFBRmHClient)pWebControl->GetLoadGroup()),
							 (char*)szUrl,
							 (char*)szMetaField,
							 (char*)szMetaFieldContent);
}

static void SFBClientHandleCookie ( SFBRmHClient hClient,
											SFBRmReqId reqID,
											void* appData,
											char* szHostName,
											BOOL bCookieExist,
											char* szCookieString,
                                                                                        void* pCookie,
											SFBCookieCallbackResponse fpCallbackResponse
											)
{
	gpWebControl->ReceivedCookieAcceptMethod( hClient, reqID, szHostName, bCookieExist, szCookieString, pCookie, fpCallbackResponse );
}


//-----------------------------------------------------------------------------
// WebKitLoader::SetCookie 
// Set Cookie
//-----------------------------------------------------------------------------
void WebKitLoader::SetCookie( const char* szURL,
							  const char* szContent)
{
	XMLFrame* pFrame = GetWebKitBridge()->GetWebCoreBridge()->GetFrame();
	WebControl::WebFrame *pWebFrame = WebControl::kit( pFrame );
	WebControl::WebControl* pWebControl = pWebFrame->GetWebControl();
	
	gpWebControl = pWebControl;//Temp hack Gnana in the case of UI POPUP.

	SFBSetJavaScriptCookie ( (SFBCookieCallback)SFBClientHandleCookie,
							 -1,
							 (SFBRmHClient)pWebControl->GetLoadGroup(),
							 NULL,
							 SFBRmGetCookieStorage((SFBRmHClient)pWebControl->GetLoadGroup()),
							 szURL,
							 szContent);
}

//-----------------------------------------------------------------------------
// WebKitLoader::GetCookie 
// Get Cookie
//-----------------------------------------------------------------------------
char* WebKitLoader::GetCookie(const char* szUrl)
{
	XMLFrame* pFrame = GetWebKitBridge()->GetWebCoreBridge()->GetFrame();	 
	WebControl::WebFrame *pWebFrame = WebControl::kit( pFrame );
	WebControl::WebControl* pWebControl = pWebFrame->GetWebControl();
	char* szCookie = SFBGetJavaScriptCookie (SFBRmGetCookieStorage((SFBRmHClient)pWebControl->GetLoadGroup()),szUrl);
	
	return szCookie;
}

//-----------------------------------------------------------------------------
// WebKitLoader::GetCookieEnabled 
// Get Cookie status
//-----------------------------------------------------------------------------
BOOL WebKitLoader::GetCookieEnabled()
{
	XMLFrame* pFrame = GetWebKitBridge()->GetWebCoreBridge()->GetFrame();	 
	WebControl::WebFrame *pWebFrame = WebControl::kit( pFrame );
	WebControl::WebControl* pWebControl = pWebFrame->GetWebControl();
	
	BOOL bCookieStatus = SFBGetCookieEnable(SFBRmGetCookieStorage((SFBRmHClient)pWebControl->GetLoadGroup()));	
	
	return bCookieStatus;
}

#endif
/*end of modification - gnana*/
//  End of File