#ifndef WEBKITLOADER_H
#define WEBKITLOADER_H

//  INCLUDES
#include "BridgeTypes.h"
#include "ResourceRequest.h"
#include "CMMDefs.h"

// CONSTANTS

// Default user agent string to be used if we are not able to get the string from the platform.

// MACROS

// DATA TYPES

// FUNCTION PROTOTYPES

// FORWARD DECLARATIONS
class UrlRequestInfo;
class RequestDispatcher;
class ContentDispatcher;
class WebKitBridge;
class IContentListener;
namespace WebCore 
    {
    class ResourceLoader;
    };
// CLASS DECLARATION
/**
*
*  @lib resLoader.lib
*  @since 3.0
*/
class WebKitLoader CMM_MODIFIED
    {
    public:  // Constructors and destructor

        /**
        * Constructor
        */
        WebKitLoader( WebKitBridge* _webKitBridge);

        /**
        * Destructor
        */
        virtual ~WebKitLoader();

    public: // new functions

        /**
        * LoadPage
        * Load the url
        * @param UrlRequestInfo* _urlRequestInfo
        * @param ResourceLoader* _resourceLoader
        * @return error code
        */
        eErrorCode LoadPage( UrlRequestInfo* _urlRequestInfo,
							WebCore::ResourceLoader* _resourceLoader
							 );

        /**
        * StartLoadingResource
        * Public Class Method
        * Purpose
        * @param UrlRequestInfo* _urlRequestInfo
        * @param ResourceLoader* _resourceLoader
        * @return Error Code
        */
        eErrorCode LoadResource( UrlRequestInfo* _urlRequestInfo,
                                    WebCore::ResourceLoader* _resourceLoader );

        /**
        * CancelTransaction
        * Cancel a particular transaction for the given Transaction Id
        * @param _transactionId : Id of the transaction to be canceled
        * @param _error : Reson to cancel the transaction
        * @return void
        */
        void CancelTransaction( int _transactionId, eErrorCode _error );

        /**
        * CancelTransaction
        * Cancel a particular transaction for the given resource loader
        * @param const _resourceLoader : The resource loader whose transaction is to be canceled
        * @return void
        */
        void CancelTransaction( const WebCore::ResourceLoader* _resourceLoader );

		/**
		 * redirect the data stream of transID to IContentListener
		 */
		eErrorCode RedirectTransaction( WebCore::ResourceLoader* _resLoader, 
                                            int* _transportHandle );

        /**
        * CancelAll
        * Cancel all outstanding requests
        * @return void
        */
        void CancelAll( );

        /**
        * SetPageUrl
        *
        * @param
        * @return
        */
        void SetPageUrl( const WebCore::String& _responseUrl )
        {
            m_pageUrl = _responseUrl;
        }

        /**
        * PageUrl
        *
        * @param
        * @return
        */
        WebCore::String PageUrl() { return m_pageUrl; }

        
        /**
        * Return WebKitBridge
        * Public Class Method
        * @return bridge
        */
        WebKitBridge* GetWebKitBridge() { return m_webKitBridge; }
        
/* Gnana modified for Http-equiv for cache and cookie*/
#if PLATFORM(SHP)
		/*
		 * Update meta Cache
		 * @param
         * @return
		 */
		void UpdateMetaCache( const char* szUrl,
							  const char* szMetaField,
							  const char* szMetaFieldContent);

		/*
		 * Set Cookie
		 * @param
         * @return
		 */
		void SetCookie( const char* szURL,
			            const char* szContent);

		/*
		 * Get Cookie
		 * @param
         * @return
		 */
		char* GetCookie( const char* szUrl);	
		
		/*
		 * Get cookie enabled
		 * @param
         * @return
		 */
		BOOL GetCookieEnabled();
#endif
/*end of modification - gnana */

	
    private: // data
        //
        ContentDispatcher*      m_contentDispatcher;    // owned
        //
        RequestDispatcher*      m_requestDispatcher;    // owned
        //
        WebKitBridge*           m_webKitBridge;         // not owned
        //
        WebCore::String         m_pageUrl;
        
    }; // Class WebKitLoader
    
#endif      // WEBKITLOADER_H

// End of File

