/*
* ==============================================================================
*  Name        : ContentDownloadListener.cpp
*  Part of     : WebKit / BrowserView
*  Description : 
*/
#ifdef SUPPORT_CONTENT_DOWNLOADER


// INCLUDE FILES
#include "types.h"
#include "LoadListeners.h"
#include "UrlResponseInfo.h"
#include "WebKitLoader.h"
#include "CString.h"

//#include "SFBCdWebkitCallback.h"//sfb_re_eng_cd

// EXTERNAL DATA STRUCTURES

// EXTERNAL FUNCTION PROTOTYPES

// CONSTANTS

// MACROS

// LOCAL CONSTANTS AND MACROS

// MODULE DATA STRUCTURES

// LOCAL FUNCTION PROTOTYPES

// FORWARD DECLARATIONS

// ============================= LOCAL FUNCTIONS ===============================

// ============================ MEMBER FUNCTIONS ===============================

// -----------------------------------------------------------------------------
// ContentDownloadListener::ContentDownloadListener
// C++ default constructor 
// -----------------------------------------------------------------------------
//
ContentDownloadListener::ContentDownloadListener( WebKitLoader* _webKitLoader,
                                                    int _transactionId,
                                                    eContentDownloadType _cdType)
    : m_transId( _transactionId ),
      m_CDType( _cdType ),
	  m_transportHandle(0),
      m_webKitLoader( _webKitLoader )
    {
    }


// Destructor
ContentDownloadListener::~ContentDownloadListener()
    {
    }


// -----------------------------------------------------------------------------
// ContentDownloadListener::Headers
//
//
// -----------------------------------------------------------------------------
//
eErrorCode ContentDownloadListener::Headers(
    int _transactionId,
    UrlResponseInfo* _response )
    {
	m_transportHandle = (int)_response->GetTransportHandle();
    // Notify contentDownloader for headers in case of direct download
    if( CD_DIRECT_DOWNLOAD == m_CDType )
        {
        WebCore::String respHdr = _response->HeaderInfo()->ResponseHeadersString();

		//sfb_re_eng_cd
      /*  SFBCdWebkitDirectDnldNotify( m_transportHandle, 
                                     _response->HeaderInfo()->RequestUrl(), 
                                     ( char* )respHdr.latin1().data(), 
                                     respHdr.length(),this );*/
        }
    return ERR_OK;
    }

// -----------------------------------------------------------------------------
// ContentDownloadListener::Response
//
//
// -----------------------------------------------------------------------------
//
void ContentDownloadListener::Response(
    int _transactionId,
    UrlResponseInfo* _response )
    {
    // Notify the content downloader for data
    switch( m_CDType )
        {
        case CD_DIRECT_DOWNLOAD :
            {
			//sfb_re_eng_cd
            //SFBCdWebkitDirectDnldDataCB( m_transportHandle, 
            //                                ( char* )_response->Body(), 
            //                                _response->BodyLength(), false );
            break;
            }
        case CD_OMA_DOWNLOAD :
        case CD_MIDP_DOWNLOAD :
            {
            WebCore::String tempStr( (char *) _response->Body() );
            m_collatedData.append( tempStr );
            break;
            }
        }
    }

// -----------------------------------------------------------------------------
// ContentDownloadListener::Complete
//
//
// -----------------------------------------------------------------------------
//
void ContentDownloadListener::Complete(
    int _transactionId,
    UrlResponseInfo* _response,
    eErrorCode _error )
    {
    // Notify the content downloader for data complete
    switch( m_CDType )
        {
        case CD_DIRECT_DOWNLOAD :
            {
            // Don't send the complete notification if the download is canceled
            if( ERR_CANCEL != _error )
                {
				//sfb_re_eng_cd
                //SFBCdWebkitDirectDnldDataCB( m_transportHandle, 
                //                            ( char* ) _response->Body(), 
                //                            _response->BodyLength(), 
                //                            true );
                }
            break;
            }
        case CD_OMA_DOWNLOAD :
            {
            // Send the collated data to content downloader
            if( 0 < m_collatedData.length() )
                {
				//sfb_re_eng_cd
                //SFBCdWebkitDescDownloadNotify( m_transportHandle, 
                //                                _response->HeaderInfo()->RequestUrl(), 
                //                                SFB_CD_WEBKIT_DESC_DD, 
                //                                ( char* ) m_collatedData.latin1().data(),
                //                                m_collatedData.length() );
                }
            break;
            }
        case CD_MIDP_DOWNLOAD :
            {
            // Send the collated data to content downloader
            if( 0 < m_collatedData.length() )
                {
				//sfb_re_eng_cd
                //SFBCdWebkitDescDownloadNotify( m_transportHandle, 
                //                                _response->HeaderInfo()->RequestUrl(), 
                //                                SFB_CD_WEBKIT_DESC_JAD, 
                //                                ( char* ) m_collatedData.latin1().data(),
                //                                m_collatedData.length() );
                }
            break;
            }
        }
    }

// -----------------------------------------------------------------------------
// ContentDownloadListener::HandleError
//
//
// -----------------------------------------------------------------------------
//
void ContentDownloadListener::HandleError(
    int _transactionId,
    int _error )
    {
    // Notify the content downloader for error
    // No error calback for MIDP and OMA???
    if( CD_DIRECT_DOWNLOAD == m_CDType )
        {
		//sfb_re_eng_cd
        //SFBCdWebkitDirectDnldErrorCB( m_transportHandle,  _error);
			;
        }
	}

#endif // SUPPORT_CONTENT_DOWNLOADER

// -----------------------------------------------------------------------------
// Notifications from the Content Downloader
//
//
// -----------------------------------------------------------------------------
//

extern "C" 
{
void SFBWebkitCdCancelDownload(void* pUserData)
    {
#ifdef SUPPORT_CONTENT_DOWNLOADER
    ContentDownloadListener* cdListener = (ContentDownloadListener*)pUserData;
    cdListener->GetLoader()->CancelTransaction( cdListener->GetTransactionId(), ERR_CANCEL );
#endif // SUPPORT_CONTENT_DOWNLOADER
    }

} // extern "C"

// End of File
