#ifndef REQUESTDISPATCHER_H
#define REQUESTDISPATCHER_H

#include "WebKitLoader.h"
#include "UrlRequestInfo.h"
#include "UrlLoader.h"
#include "HttpLoader.h"
#include "FileLoader.h"
#include "CMMDefs.h"

// CLASS DECLARATION

/**
*  This is the entry point for the clients.
*  Call dispatch to have the request processed.
*  @lib resLoader.lib
*/



class RequestDispatcher CMM_MODIFIED
    {
    public:  // Constructors and destructor

        /**
        * Constructor.
        */
        RequestDispatcher( WebKitLoader* _webKitLoader,
                            ContentDispatcher* _contentDispatcher );

        /**
        * Destructor.
        */
        virtual ~RequestDispatcher();

    public: // New functions

        /**
        * Dispatch a request.
        * @param _requestInfo Request info created by the client
        * @return result of the dispatching. OS wide erro code.
        */
        eErrorCode Dispatch( UrlRequestInfo* _requestInfo, IContentLoader** _urlLoader ,unsigned char _isMainResource, StateChangeInfo state_info);

        /**
        * CancelTransaction
        * Cancel a particular transaction for the given Transaction Id
        * @param _transactionId : Id of the transaction to be canceled
        * @param _error : Reason to cancel the transaction
        * @return void
        */
        void CancelTransaction( int _transactionId, eErrorCode _error );

        /**
        * Cancel all active resource loading operations.
        * @param _error This error code to pass on to the various UrlLoaders
        * @return KErrNotFound if there is not outstanding transaction - otherwise KErrNone.
        */
        void CancelAllLoaders(  eErrorCode _error );

        static void initCount() { m_transactionCount = 0; }

		/**
		* Redirect Transaction
		*
		*/
		eErrorCode RedirectTransaction( int transID, int *transportHandle );



    private: 

        /**
        * returns the url loader based on the scheme
        * @param _scheme 
        * @return The url loader
        */
        UrlLoader* IsSupportedScheme( const char* _scheme);

    private:    // Data

        // loader
        IContentLoader*         m_contentLoader;
        HttpLoader*             m_httpLoader;     //refcounted
        FileLoader*             m_fileLoader;     // refcounted

		WebKitLoader*			m_webkitloader;
        ContentDispatcher*      m_contentDispatcher; // not owned

        // unique transaction id
        static unsigned int     m_transactionCount;
	};

#endif      // REQUESTDISPATCHER_H

// End of File
