#ifndef TRANSACTIONPROGRESSSTATUS_H
#define TRANSACTIONPROGRESSSTATUS_H

// INCLUDES

// CONSTANTS

// MACROS

// DATA TYPES

// FUNCTION PROTOTYPES

// FORWARD DECLARATIONS

// CLASS DECLARATION

/**
*  This class provides progress information on the
*  corresponding transaction
*  @lib resLoader.lib
*/
class TransactionProgressStatus
    {
    public: // Constructors and destructor        

        /**
        * Construct.
        * @param
        * @return
        */
        TransactionProgressStatus() { m_contentMaxSize = -1; }

    public: // New functions

        /**
        * Set the size of the body. This information is coming
        * from the server and it could be false.
        * @param _contentMaxSize Size of the body
        * @return void
        */
        inline void SetContentMaxSize( double _contentMaxSize ) 
            { 
            m_contentMaxSize = _contentMaxSize;
            }
        
        /**
        * Get the size of the body. This information is coming
        * from the server and it could be false.
        * @return Size of the body.
        */
        inline double ContentMaxSize() const { return m_contentMaxSize; }

        /**
        * Set the size of the data we've received so far.
        * @param _receivedContentSize Size of the data received so far
        * @return void
        */
        inline void SetReceivedContentSize( double _receivedContentSize )
            {
            m_receivedContentSize = _receivedContentSize;
            }        

        /**
        * Get the size of the data we've received so far.
        * @return Size of the data received so far 
        */
        inline double ReceivedContentSize() const { return m_receivedContentSize; }        

        /**
        * Set wheter or not the corresponding 
        * transaction is a top level one.(page load)
        * @param _topLevel Indicates if the corresponding 
        * transaction is a top level one
        * @return void
        */
        inline void SetTopLevel( bool _topLevel ) { m_topLevel = _topLevel; }        

        /**
        * Get wheter or not the corresponding 
        * transaction is a top level one.(page load)
        * @return True if the corresponding 
        * transaction is a top level one 
        */
        inline bool TopLevel() const { return m_topLevel; }

        /**
        * Check if the progress informaction object 
        * has already been initialized. If not
        * the caller gets false information.
        * @return True if progress informaction has
        * been initialized.
        */
        inline bool Initialized() const 
            {
            return ( !( m_contentMaxSize == -1 || m_receivedContentSize == 0 ) );
            }

    private : // Data
        
        // the size of the body. This information is coming
        // from the server and it could be false.
        double m_contentMaxSize;
        // the size of the data we've received so far.
        double m_receivedContentSize;
        // True if the corresponding transaction is a top level one 
        bool m_topLevel;
    };

#endif      // TRANSACTIONPROGRESSSTATUS_H

// End of File
