/*
* ==============================================================================
*  Name        : UrlResponseHeaderInfo.cpp
*  Part of     : WebKit / ResourceLoader
*  Interface   :
*  Description : Implementation of UrlResponseHeaderInfo
*  Version     : %Version%
* ==============================================================================
*/

// INCLUDE FILES
#include "types.h"
#include "UrlResponseHeaderInfo.h"
#ifdef __CMM__
#include <AnsiCLib.h>
#include <allocs.h>
#else
#include "Shp.h"
#endif


# if defined(SHP_SUPPORT_WML)
# include "PlatformString.h"
#include "CString.h"
# endif


// EXTERNAL DATA STRUCTURES

// EXTERNAL FUNCTION PROTOTYPES

// CONSTANTS

// MACROS

// LOCAL CONSTANTS AND MACROS

// MODULE DATA STRUCTURES

// LOCAL FUNCTION PROTOTYPES

// FORWARD DECLARATIONS

// ============================= LOCAL FUNCTIONS ===============================


// ============================ MEMBER FUNCTIONS ===============================

// -----------------------------------------------------------------------------
// UrlResponseHeaderInfo::UrlResponseHeaderInfo
// C++ default constructor can NOT contain any code, that
// might leave.
// -----------------------------------------------------------------------------
//
UrlResponseHeaderInfo::UrlResponseHeaderInfo()
    {
	// <TODO> : Use initialization list
    // default values
    m_httpStatus = HTTP_OK;
    m_requestUrl = NULL;
    m_responseUrl = NULL;
    m_contentTypeString = (char *)malloc(AcStrlen("text/html")+1);
	AcStrcpy(m_contentTypeString, "text/html");
    m_contentLocation = NULL;
    m_boundary = NULL;
    // webcore does not like default charset. set it as empty
    m_charset = NULL;
	m_refresh = NULL;

	// Redirection
	m_isRedirected = false;
	m_redirectedUrl = NULL;
    // todo check what date it should be. mktime cannot handle dates beyond 2030.
    // overflow returned
    }

// Destructor
UrlResponseHeaderInfo::~UrlResponseHeaderInfo()
    {
    free(m_responseUrl);
    free(m_requestUrl);
    m_responseHeaders.clear();
    free(m_contentTypeString);
    free(m_contentLocation);
    free(m_boundary);
    free(m_charset);
    free(m_refresh);
	free(m_redirectedUrl);
    }

// -----------------------------------------------------------------------------
// UrlResponseHeaderInfo::SetRequestUrl
//
//
// -----------------------------------------------------------------------------
//
void UrlResponseHeaderInfo::SetRequestUrl(const char* _requestUrl )
    {
    // release old m_requestUrl
    free(m_requestUrl);
    if(_requestUrl)
        {
			m_requestUrl = (char *)malloc(AcStrlen(_requestUrl)+1);
			AcStrcpy(m_requestUrl, _requestUrl);
        }
    }

// -----------------------------------------------------------------------------
// UrlResponseHeaderInfo::SetResponseUrl
//
//
// -----------------------------------------------------------------------------
//
void UrlResponseHeaderInfo::SetResponseUrl(const char* _responseUrl )
    {
    free(m_responseUrl);
    if(_responseUrl)
        {
        m_responseUrl = (char *)malloc(AcStrlen(_responseUrl)+1);
		AcStrcpy(m_responseUrl, _responseUrl);
        }
    }

// -----------------------------------------------------------------------------
// UrlResponseHeaderInfo::SetContentType
//
//
// -----------------------------------------------------------------------------
//
void UrlResponseHeaderInfo::SetContentType(const char* _contentType )
    {
    // release old m_contentTypeString
    free(m_contentTypeString);
    if(_contentType)
        {
        m_contentTypeString = (char *)malloc(AcStrlen(_contentType)+1);
		AcStrcpy(m_contentTypeString, _contentType);
        }
    }

// -----------------------------------------------------------------------------
// UrlResponseHeaderInfo::SetCharset
//
//
// -----------------------------------------------------------------------------
//
void UrlResponseHeaderInfo::SetCharset(const char* _charset )
    {
    // release old charset
    free(m_charset);
    if(_charset)
        {
        m_charset = (char *)malloc(AcStrlen(_charset)+1);
		AcStrcpy(m_charset, _charset);
        }
    }

// -----------------------------------------------------------------------------
// UrlResponseHeaderInfo::SetRefresh
//
//
// -----------------------------------------------------------------------------
//
void UrlResponseHeaderInfo::SetRefresh(const char* _refresh )
    {
    // release old charset
    free(m_refresh);
    if(_refresh)
        {
        m_refresh = (char *)malloc(AcStrlen(_refresh)+1);
		AcStrcpy(m_refresh, _refresh);
        }
    }

// -----------------------------------------------------------------------------
// UrlResponseHeaderInfo::SetBoundary
//
//
// -----------------------------------------------------------------------------
//
void UrlResponseHeaderInfo::SetBoundary(const char* _boundary )
    {
    // release old m_boundary
    free(m_boundary);
    if(_boundary)
        {
        m_boundary = (char *)malloc(AcStrlen(_boundary)+1);
		AcStrcpy(m_boundary, _boundary);
        }
    }

// -----------------------------------------------------------------------------
// UrlResponseHeaderInfo::SetContentLocationL
//
//
// -----------------------------------------------------------------------------
//
void UrlResponseHeaderInfo::SetContentLocation(const char* _contentLocation )
    {
    // release old m_contentLocation
    free(m_contentLocation);
    if(_contentLocation)
        {
        m_contentLocation = (char *)malloc(AcStrlen(_contentLocation)+1);
		AcStrcpy(m_contentLocation, _contentLocation);
        }
    }

// -----------------------------------------------------------------------------
// UrlResponseHeaderInfo::SetResponseHeaderField
//
//
// -----------------------------------------------------------------------------
//
void UrlResponseHeaderInfo::SetResponseHeaderField( String _name, String _value )
    {
        m_responseHeaders.set(_name, _value);   
    }

// -----------------------------------------------------------------------------
// UrlResponseHeaderInfo::GetResponseHeaderField
//
//
// -----------------------------------------------------------------------------
//
String UrlResponseHeaderInfo::GetResponseHeaderField( String& _name ) const
    {
        return m_responseHeaders.get(_name);   
    }

// -----------------------------------------------------------------------------
// UrlResponseHeaderInfo::ResponseHeaders
//
//
// -----------------------------------------------------------------------------
//
const HTTPHeaderMap& UrlResponseHeaderInfo::ResponseHeaders() const
    {
    return  m_responseHeaders;
    }

// -----------------------------------------------------------------------------
// UrlResponseHeaderInfo::ResponseHeaderString
//
//
// -----------------------------------------------------------------------------
//
WebCore::String UrlResponseHeaderInfo::ResponseHeadersString() const
    {
    return m_httpHeaderString;
    }

// -----------------------------------------------------------------------------
// UrlResponseHeaderInfo::ResponseHeaderString
//
//
// -----------------------------------------------------------------------------
//
void UrlResponseHeaderInfo::AddToResponseHeadersString(WebCore::String _header)
    {
    m_httpHeaderString.append( _header );
    }

// -----------------------------------------------------------------------------
// UrlResponseHeaderInfo::SetRedirectedUrl
//
//
// -----------------------------------------------------------------------------
//
void UrlResponseHeaderInfo::SetRedirectedUrl( const char* _redirectedUrl )
	{
	free( m_redirectedUrl );
	if( _redirectedUrl )
		{
		m_redirectedUrl = (char *)malloc( AcStrlen( _redirectedUrl ) + 1 );
		AcStrcpy( m_redirectedUrl, _redirectedUrl );
		}
	}

//  End of File

