    /*
* ==============================================================================
*  Name        : HttpTransaction.cpp
*  Part of     : WebKit / ResourceLoader
*  Interface   : 
*  Description : Implementation of HttpTransaction
*  Version     : %Version%
*/

// INCLUDE FILES
#include "HttpTransaction.h"
#include "HttpLoader.h"
#include "UrlResponseHeaderInfo.h"
#include "TransactionManager.h"
#include "CString.h"
#ifdef __BROWSER_SDK
#include "webkitloader.h"
#endif

// EXTERNAL DATA STRUCTURES

// EXTERNAL FUNCTION PROTOTYPES  

// CONSTANTS

// MACROS

// LOCAL CONSTANTS AND MACROS

// MODULE DATA STRUCTURES

// LOCAL FUNCTION PROTOTYPES

// FORWARD DECLARATIONS

// ============================= LOCAL FUNCTIONS ===============================


// ============================ MEMBER FUNCTIONS ===============================

// -----------------------------------------------------------------------------
// HttpTransaction::HttpTransaction
// C++ default constructor 
// -----------------------------------------------------------------------------
//
HttpTransaction::HttpTransaction( 
    UrlRequestInfo* _urlRequestInfo,
    ITransactionObserver* _transactionObserver)
    : BaseTransaction( _urlRequestInfo, _transactionObserver ),
      m_transCompleted( FALSE ),
      m_isRedirected( FALSE ),
      m_handle( NULL )
    {
    }


// Destructor
HttpTransaction::~HttpTransaction()
    {    
    }

// -----------------------------------------------------------------------------
// HttpTransaction::DoSubmit
// 
// 
// -----------------------------------------------------------------------------
// 
eErrorCode HttpTransaction::DoSubmit()
    {
    // <TODO> : Do a cache check before submitting the transaction
    TransactionManager::sharedInstance()->add(this);
    return ERR_OK;
    }

// -----------------------------------------------------------------------------
// HttpTransaction::CancelTransaction
// 
// 
// -----------------------------------------------------------------------------
//
void HttpTransaction::CancelTransaction( 
    eErrorCode _error )
    {
    // cancel 
    TransactionManager::sharedInstance()->cancel(this);
    // 
    BaseTransaction::CancelTransaction( _error );
    }

// -----------------------------------------------------------------------------
// HttpTransaction::CreateHeaderResponse
// <TODO>
// 
// -----------------------------------------------------------------------------
//
void HttpTransaction::CreateHeaderResponse()
    {
    UrlResponseHeaderInfo* header = new UrlResponseHeaderInfo();      
    //
    header->SetTransId( m_transactionId );
    //
    header->SetTopLevel( m_urlRequestInfo->TopLevel() );    
    //
    header->SetChildFrameLoad( m_urlRequestInfo->ChildFrameLoad() );
    
    //add back the fragment if it exists
    if(header->ResponseUrl())
        {
        char* buf = strdup(header->ResponseUrl());
        if(m_fragment)
            {
            strcat(buf, "#");
            strcat(buf, m_fragment);
            }
        header->SetResponseUrl( buf );
        }
    header->SetRequestUrl( m_urlRequestInfo->Url() );
    
    //transfer the ownership to response info
    m_responseInfo->SetHeaderInfo(header);
    }
    
// -----------------------------------------------------------------------------
// HttpTransaction::HandleGotResponseHeaders
// 
// 
// -----------------------------------------------------------------------------
//
void HttpTransaction::HandleGotResponseHeaders( ) 
    {
    // create header object
    UrlResponseHeaderInfo* header = m_responseInfo->HeaderInfo();

    // set http status code
    header->SetHttpStatus( header->HttpStatus() );
  
    // set content type string
    String cT("Content-type");
    String contentType( header->GetResponseHeaderField(cT) );
    header->SetContentType( contentType.utf8().data() );

    // set the content location string    
    String cL("Content-Location");
    String contentLocation( header->GetResponseHeaderField(cL));
    header->SetContentLocation( contentLocation.utf8().data() );
    
    // set content length
    String cLen("Content-Length");
    String contentLength( header->GetResponseHeaderField(cLen));
    header->SetContentLength( contentLength.toInt() );
    
    //set redirection
    header->SetIsRedirected(IsRedirected());

    // get all the headers
    HTTPHeaderMap headers = header->ResponseHeaders();
    header->SetResponseHeaders(headers);

    bool handled = FALSE;
    if( ( m_responseInfo->HeaderInfo()->HttpStatus() == HTTP_UNAUTHORIZED ) || 
        ( m_responseInfo->HeaderInfo()->HttpStatus() == HTTP_PROXY_AUTH_REQUIRED ) )
        {
        handled = BaseTransaction::HandleError( m_responseInfo->HeaderInfo()->HttpStatus() );
        // trans is cancelled at this point. if 
        // authentication is failed (not handled) then
        // just complete this transaction
        if( !handled )
            {
            //
            TransactionComplete( ERR_CANCEL );
            return;
            }
        }
    }

// -----------------------------------------------------------------------------
// HttpTransaction::HandleResponseBodyData
// 
// 
// -----------------------------------------------------------------------------
//
void HttpTransaction::HandleResponseBodyData(void* _body, int _len)
    {    
    // get it from the transaction
    //
    AddBodyData( _body, _len );
    }

// -----------------------------------------------------------------------------
// HttpTransaction::AddRequestHeaders
// Add headers to the request as required by the WebKit.
// -----------------------------------------------------------------------------
//
void HttpTransaction::AddRequestHeaders()
    {
    // referer header
    if( m_urlRequestInfo->SendRefererHeaderEnabled() && m_urlRequestInfo->RefererUrl() )
        {
        String ref("Referer");
        String refUrl(m_urlRequestInfo->RefererUrl());
        m_urlRequestInfo->SetRequestHeaderField( ref, refUrl);
        }

    String acceptHeaders( m_urlRequestInfo->AcceptHeader() );
    // accept header
    if( acceptHeaders.length() )
        {
        String strAccept("Accept");
        m_urlRequestInfo->SetRequestHeaderField( strAccept, acceptHeaders);
        }
    }

//  End of File  

