#ifndef TransactionManager_h
#define TransactionManager_h

#include "Timer.h"
#include "BaseTransaction.h"
#include "CMMDefs.h"

class TransactionList  CMM_MODIFIED
{
public:
    TransactionList(BaseTransaction* job)
        : m_job(job)
        , m_next(0)
    {}
    void setNext(TransactionList* next) { m_next = next; }
    TransactionList* next() const { return m_next; }
    BaseTransaction* job() const { return m_job; }

private:
    BaseTransaction* m_job;
    TransactionList* m_next;
};

#ifndef USE_CURL
class TransactionManager  CMM_MODIFIED
{
public:
    static TransactionManager* sharedInstance();
    static void removeTransactionMgr();
    void add(BaseTransaction*);
    void cancel(BaseTransaction*);
    void setCookieJarFileName(const char* cookieJarFileName);

    void setupPOST(BaseTransaction*, void* _pReqInfo);
	void setupGET(BaseTransaction*, void* _pReqInfo);
    void setupPUT(BaseTransaction*);
	void AddReqHdrs(BaseTransaction*, void* _pReqInfo);

	static void StopTimer();

private:
    TransactionManager();
    ~TransactionManager();
    void downloadTimerCallback(Timer<TransactionManager>*);
    void stopRMTransation(BaseTransaction*);
    bool removeScheduledJob(BaseTransaction*);
    bool startJob(BaseTransaction*);
    bool startScheduledJobs();
    void parseDataUrl(BaseTransaction*);

	Timer<TransactionManager> m_downloadTimer;
    char* m_cookieJarFileName;
    TransactionList* m_TransactionListHead;
    TransactionList* m_TransactionListTail;

    double m_pollTimeSec;
};

#else //USE_CURL
#include <curl/curl.h>

class TransactionManagerCURL {
public:
    static TransactionManagerCURL* sharedInstance();
    void add(BaseTransaction*);
    void cancel(BaseTransaction*);
    void setCookieJarFileName(const char* cookieJarFileName);

    void setupPOST(BaseTransaction*);
    void setupPUT(BaseTransaction*);

private:
    TransactionManagerCURL();
    ~TransactionManagerCURL();
    void downloadTimerCallback(Timer<TransactionManagerCURL>*);
    void removeFromCurl(BaseTransaction*);
    bool removeScheduledJob(BaseTransaction*);
    void startJob(BaseTransaction*);
    bool startScheduledJobs();

    void TempHeaders(curl_slist** headers);
    
    Timer<TransactionManagerCURL> m_downloadTimer;
    CURLM* m_curlMultiHandle;
    CURLSH* m_curlShareHandle;
    char* m_cookieJarFileName;
    char m_curlErrorBuffer[CURL_ERROR_SIZE];
    TransactionList* m_TransactionListHead;
};
#endif //USE_CURL

#endif // TransactionManager_h

// End of File
