/*
 * Copyright (C) 2007 Holger Hans Peter Freyther
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1.  Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 * 2.  Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 * 3.  Neither the name of Apple, Inc. ("Apple") nor the names of
 *     its contributors may be used to endorse or promote products derived
 *     from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY APPLE AND ITS CONTRIBUTORS "AS IS" AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL APPLE OR ITS CONTRIBUTORS BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#include "config.h"

#include "webkitshppage.h"
#include "webkitshpprivate.h"


#include "NotImplemented.h"
#include "ChromeClientShp.h"
#include "ContextMenuClientShp.h"
#include "DragClientShp.h"
#include "EditorClientShp.h"
#include "EventHandler.h"
#include "HitTestRequest.h"
#include "HitTestResult.h"
#include "InspectorClientshp.h"
#include "FrameLoader.h"
#include "FrameView.h"
#include "PlatformKeyboardEvent.h"
#include "PlatformMouseEvent.h"
#include "PlatformWheelEvent.h"
#include "SubstituteData.h"
#include "NotImplemented.h"
#include "CString.h"
#include "DocumentType.h"
#include "WebCorePageData.h"
#include "Cache.h"
#include "CachedImage.h"
#include "RenderImage.h"

#ifdef SFB_RSS_LINK //vamsi Jan 28 Rss Link Tag Support for SFB.
#include "HTMLLinkElement.h"
#endif
#include "HTMLAnchorElement.h"
#include "HTMLImageElement.h"
#include "SharedTimer.h"

#ifdef USE_NATIVE_WIDGET_SUPPORT
#include "NativeEditField.h"
#include "NativeComboField.h"
#endif

#include "gwes.h"
//#include "BrAppEventTypes.h"//sfb_re_eng
#include "BrowserSettings.h"
#include "FileMgr.h"
#include "Page.h"

#ifdef __CMM__
#include <allocs.h>
#endif
#if defined ( SHP_SUPPORT_WML )
#include "WMLDocument.h"
#include "BrowserControl.hpp"
#include "BrowserControl_DataTypes.h"
//#include "platform.h"
#include "CMMDefs.h"
#endif

#include "Cursor.h"

#ifdef SUPPORT_VIRTUAL_CURSOR
#include "VirtualCursor.h"
#endif //SUPPORT_VIRTUAL_CURSOR

#include "MemoryManager.h"
#include "SFBalMain.h"
#include "kjs_proxy.h"

#include "TransactionManager.h"
#include "SFBCache.h"
#include "SFBCookie.h"

// <TODO> : replace with the new hearder once we get the place-holder image
// Also remove the include directory 
// "..\..\..\..\..\..\..\..\..\Target\C501\UI\WindowSet\Rsrc\H"
// from the Webkit project
#ifdef SFB_PROGRESSIVE_RENDERING
#include "WsRsrcCheck.h"
#endif // SFB_PROGRESSIVE_RENDERING


/**
 *   bs0425.lee  modified for touch events 
 */
#include "MouseEventWithHitTestResults.h"
#include "Node.h" 
//vprajan: below header is windows specific
#ifdef _SHP_SIMUL
#include "WebElementPropertyBag.h"
#endif
//#include "COMPtr.h"
#include "NativeEditField.h"
#include "HTMLDocument.h"
//#include "BrCoreControl.h"

using namespace WebKit;
using namespace WebCore;
// to remove duplicated error of WebControl
extern HWIN gHWin;

#ifdef _SFB_PAGE_PREVIEW	//vamsi Mar 03 - Page Overview
#define PAGE_PREVIEW_DIFF_SCROLLBAR	7
#endif

#define HTML_DOC_TYPE_STRING ("HTML Document")

#define MINIMUM_FONT_SIZE 12
#define DEFAULT_FONT_SIZE 16

// <TODO> - Get the resource for place-holder image from SISO
#ifdef SFB_PROGRESSIVE_RENDERING
#define SFB_RSRC_FILENAME "SFBrowserRsrc.rbm"
#endif // SFB_PROGRESSIVE_RENDERING

bool g_inFinalize = false;

namespace WebCore
{
  void finalizeMIMETypeRegistry();
}

extern "C" {

void ReInitDlState();

#ifdef USE_NATIVE_WIDGET_SUPPORT
static void destroyNativeWidgetsIfAny()
{
	NativeEditField::destroyNativeEditFieldInstance();
	NativeComboField::destroyNativeComboFieldInstance();
}
#endif



void webkit_shp_initialize(HWIN hWin)
{
	//create cursor window
	CPlatformCursorHelper::Initialize(static_cast<int>(hWin));
}

void webkit_shp_finalize()
{
	WebCore::stopSharedTimer();

#ifdef USE_NATIVE_WIDGET_SUPPORT
	destroyNativeWidgetsIfAny();
#endif
	//destroy cursor window
	CPlatformCursorHelper::Finalize();

#ifdef LOG_MEMORY_LEAKS
	_CrtDumpMemoryLeaks();
#endif // LOG_MEMORY_LEAKS

	WebCore::finalizeMIMETypeRegistry();
	TransactionManager::removeTransactionMgr();

	MemoryManager::PrepareForExit();

#ifdef USE_FAST_MALLOC
	ReInitDlState();
#endif
}

void webkit_shp_page_finalize(WebKitShpPage* page)
{
g_inFinalize = true;
	WebCore::stopSharedTimer();
    webkit_shp_page_stop_loading(page);

#ifdef USE_NATIVE_WIDGET_SUPPORT
	destroyNativeWidgetsIfAny();
#endif

    WebKitShpPagePrivate* pageData = page->privatePage;

	KJSProxy* scriptProxy = pageData->mainFrame->privateFrame->frame->scriptProxy();
	if(scriptProxy)
	{
		scriptProxy->clear();
	}

    delete pageData->page;
    delete pageData->settings;
    delete pageData->userAgent;

#ifdef SUPPORT_VIRTUAL_CURSOR
	CVirtualCursor::destroyInstance(pageData->pageVirtualCursor);
#endif //SUPPORT_VIRTUAL_CURSOR

    // Cleanup the place-holder image
#ifdef SFB_PROGRESSIVE_RENDERING
    if(pageData->placeholderImage)
    {
        __WsFreeBitmap(pageData->placeholderImage);
    }
#endif // SFB_PROGRESSIVE_RENDERING

    //cleanup
    if(INVALID_HOBJ != pageData->hGc)
    {
        WmDeleteGC(pageData->hGc);
        pageData->hGc = INVALID_HOBJ;
    }

#ifdef _SFB_PAGE_PREVIEW	//vamsi Mar 03 - Page Overview
    if(INVALID_HOBJ != pageData->hGc_restore)
    {
		// Delete the restore_hGc when Page preview mode is exited
        WmDeleteGC(pageData->hGc_restore);
        pageData->hGc_restore = INVALID_HOBJ;
    }
#endif

	//destroy cursor window
	CPlatformCursorHelper::Finalize();

    webkit_shp_frame_finalize(pageData->mainFrame);

	delete page->privatePage;


	//shkim
	// finalize RmClient
	SFBExitCache(SFBRmGetCacheStorage(page->m_hRmClient));
	SFBExitCookie(SFBRmGetCookieStorage(page->m_hRmClient));
	SFBRmUnregisterClient( page->m_hRmClient );
	

    delete page;

	KJS::JSLock lock;
	KJS::Collector::collect();

	cache()->RemoveResources(0);

#ifdef LOG_MEMORY_LEAKS
	_CrtDumpMemoryLeaks();
#endif // LOG_MEMORY_LEAKS

	WebCore::finalizeMIMETypeRegistry();
	TransactionManager::removeTransactionMgr();

	MemoryManager::PrepareForExit();

#ifdef USE_FAST_MALLOC
	ReInitDlState();
#endif
}

static void webkit_shp_page_init(WebKitShpPage* page)
{

	// shkim
	// create RmClient and set cache/cookie storage for this RmClient
	//
	page->m_hRmClient = SFBRmRegisterClient( "webkit", NULL );

	page->m_hRmCache = SFBInitCache("/User/Cache", NULL);
	page->m_hRmCookie  = SFBInitCookie("/User/Ar/Cookie", NULL); 

	SFBRmSetCookieStorage(page->m_hRmClient, page->m_hRmCookie);
	SFBRmSetCacheStorage(page->m_hRmClient, page->m_hRmCache);



    WebKitShpPagePrivate* pageData = page->privatePage;
    pageData->page = new Page(new WebKit::ChromeClient(page), new WebKit::ContextMenuClient, new WebKit::EditorClient(page), new WebKit::DragClient, new WebKit::InspectorClient);

#ifdef _SFB_PAGE_PREVIEW	//vamsi Mar 03 - Page Overview
	// initialising the border rect to zero position
	 pageData->m_border_rect = new IntRect(0,0,0,0);
#endif
    Settings* settings = pageData->page->settings();
    settings->setLoadsImagesAutomatically(true);
    settings->setMinimumFontSize(MINIMUM_FONT_SIZE);
    settings->setMinimumLogicalFontSize(MINIMUM_FONT_SIZE);
    settings->setShouldPrintBackgrounds(true);
#ifdef _SHP_SIMUL
    settings->setJavaScriptEnabled(true);
#else
	settings->setJavaScriptEnabled(false);
#endif
    settings->setDefaultFixedFontSize(DEFAULT_FONT_SIZE);
    settings->setDefaultFontSize(DEFAULT_FONT_SIZE);
    settings->setSerifFontFamily("Times New Roman");
    settings->setSansSerifFontFamily("Arial");
    settings->setFixedFontFamily("Courier");
    settings->setStandardFontFamily("Arial");
    settings->setPluginsEnabled(true);

    pageData->mainFrame = webkit_shp_frame_new(page);

#pragma message ("VDJ - TODO - Need to verify we do not need to initialize WebKitShpPagePrivate::settings")
    pageData->settings = NULL;
#pragma message ("VDJ - TODO - Need to verify we do not need to initialize WebKitShpPagePrivate::userAgent")
    pageData->userAgent = NULL;

#ifdef SUPPORT_VIRTUAL_CURSOR
	pageData->pageVirtualCursor = CVirtualCursor::createInstance();
#endif //SUPPORT_VIRTUAL_CURSOR

    // Initialize the place-holder image
#ifdef SFB_PROGRESSIVE_RENDERING
#pragma message ("DD - TODO - Need to get the place-holder image resource from SISO")
    pageData->placeholderImage = RsrcLoadBitmap( SFB_RSRC_FILENAME, IDB_CHECK_ON );
#endif // SFB_PROGRESSIVE_RENDERING
}

WebKitShpPage*
webkit_shp_page_new (void)
{
    WebKitShpPage* page = new WebKitShpPage();
    page->privatePage = new WebKitShpPagePrivate();

    webkit_shp_page_init(page);
    return page;
}

gboolean
webkit_shp_page_can_cut (WebKitShpPage* page)
{
    notImplemented();
    return false;
}

gboolean
webkit_shp_page_can_copy (WebKitShpPage* page)
{
    notImplemented();
    return false;
}


void
webkit_shp_page_paste (WebKitShpPage* page)
{
    notImplemented();
}

gboolean
webkit_shp_page_can_go_backward (WebKitShpPage* page)
{
    WebKitShpPagePrivate* pageData = page->privatePage;
    WebKitShpFramePrivate* frameData = pageData->mainFrame->privateFrame;

    return frameData->frame->loader()->canGoBackOrForward(-1);
}

 gboolean
webkit_shp_page_can_go_forward (WebKitShpPage* page)
{
    WebKitShpPagePrivate* pageData = page->privatePage;
    WebKitShpFramePrivate* frameData = pageData->mainFrame->privateFrame;

    return frameData->frame->loader()->canGoBackOrForward(1);
}

 void
webkit_shp_page_go_backward (WebKitShpPage* page)
{
    WebKitShpPagePrivate* pageData = page->privatePage;
    WebKitShpFramePrivate* frameData = pageData->mainFrame->privateFrame;

    //frameData->frame->loader()->goBackOrForward(-1);
	frameData->frame->page()->goBack() ;
}

 void
webkit_shp_page_go_forward (WebKitShpPage* page)
{
    WebKitShpPagePrivate* pageData = page->privatePage;
    WebKitShpFramePrivate* frameData = pageData->mainFrame->privateFrame;

    //frameData->frame->loader()->goBackOrForward(1);
	frameData->frame->page()->goForward() ;
}

 void
webkit_shp_page_stop_loading (WebKitShpPage* page)
{
    WebKitShpPagePrivate* pageData = page->privatePage;
    WebKitShpFramePrivate* frameData = pageData->mainFrame->privateFrame;

    if (FrameLoader* loader = frameData->frame->loader())
        loader->stopAllLoaders();
}

 void
webkit_shp_page_switch_tab (WebKitShpPage* page)
 {
	 //set cursor to this page
#ifdef SUPPORT_VIRTUAL_CURSOR
	 CVirtualCursor::switchPageCursor(page->privatePage->pageVirtualCursor);
#endif //SUPPORT_VIRTUAL_CURSOR
 }

 void
webkit_shp_page_open (WebKitShpPage* page, const gchar* url)
{
	MemoryManager::ResetOOMState();

    WebKitShpPagePrivate* pageData = page->privatePage;
    WebKitShpFramePrivate* frameData = pageData->mainFrame->privateFrame;

    DeprecatedString string = DeprecatedString::fromUtf8(url);
    frameData->frame->loader()->load(ResourceRequest(KURL(string)));

}

 void
webkit_shp_page_reload (WebKitShpPage *page)
{
	MemoryManager::ResetOOMState();

    WebKitShpPagePrivate* pageData = page->privatePage;
    WebKitShpFramePrivate* frameData = pageData->mainFrame->privateFrame;

    frameData->frame->loader()->reload();
}

 void
webkit_shp_page_load_string (WebKitShpPage* page, const gchar* content, const gchar* content_mime_type, const gchar* content_encoding, const gchar* base_url)
{
	MemoryManager::ResetOOMState();

    WebKitShpPagePrivate* pageData = page->privatePage;
    WebKitShpFramePrivate* frameData = pageData->mainFrame->privateFrame;

    KURL url(DeprecatedString::fromUtf8(base_url));

    char* _content = (char *)malloc(AcStrlen(content) + 1);
    AcStrcpy(_content, content);

    RefPtr<SharedBuffer> sharedBuffer = new SharedBuffer(_content, AcStrlen(content));
    SubstituteData substituteData(sharedBuffer.release(), String(content_mime_type), String(content_encoding), KURL("about:blank"), url);

    frameData->frame->loader()->load(ResourceRequest(url), substituteData);
}

 void
webkit_shp_page_load_html_string (WebKitShpPage* page, const gchar* content, const gchar* base_url)
{
	MemoryManager::ResetOOMState();

    webkit_shp_page_load_string( page, content, "text/html", "UTF-8", base_url);
}

WebKitShpFrame*
webkit_shp_page_get_main_frame (WebKitShpPage* page)
{
    return page->privatePage->mainFrame;
}

//tahseen
#ifdef SFB_VISUAL_SUPPORT
void webkit_shp_page_visual_history(WebKitShpPage* page, WmBitmap** pWmBitmap,void* pUserData)
{
	int Height,Width;
	int reqHeight,reqWidth;
	WmRect _rect;
	WebKitShpPagePrivate* pageData = page->privatePage;
	WebKitShpFramePrivate* frameData = pageData->mainFrame->privateFrame;
    Frame* frame = frameData->frame;
	WmRect* pRect = (WmRect *) pUserData;
	Frame* frame1 = core(getFrameFromPage(page));
	FrameView* view = frame1->view();

	//get the client rect
	WmGetWindowClientRect( pageData->hWin , &_rect );

	// get the actual width & height from the page
	Height = frame1->view()->contentsHeight();
	Width = frame1->view()->contentsWidth();

	if(Height >2*(_rect.h) || Width > 2*(_rect.w)) //if width and height are more than twice of the window rect then resize accordingly
	{		
		if(Width >  2*(_rect.w) && Height < 2*(_rect.h))
		{
			//resize to 2w,h
			reqHeight = Height;
			reqWidth = 2*(_rect.w);
		}
		else if(Width < 2*(_rect.w) && Height > 2*(_rect.h))
		{
			//resize w,2h
			reqHeight = 2*(_rect.h);
			reqWidth = Width ;
		}
		else
		{
			//resize 2w,2h
			reqHeight = 2*(_rect.h);
			reqWidth = 2*(_rect.w);
		}
	}
	else // resize to the actual height n width
	{
		reqHeight = Height;
		reqWidth = Width ;
	}

	/*CQ  SFB 246 tahseen 19 mar 08 */ 
	//resize
	frame->view()->resize(reqWidth,reqHeight);
	frame->view()->adjustViewSize();
	frame->sendResizeEvent();

	//create MemGC of the required height & width
	HGC hGC = WmCreateWindowSubMemGC(pageData->hWin,0,0,reqWidth,reqHeight);

	GraphicsContext ctx2(&hGC);

	//paint the GC
	if (frame->renderer()) 
	{
		frame->view()->layoutIfNeededRecursive();
		IntRect rect(pRect->x, pRect->y, reqWidth,reqHeight);
		frame->view()->paint(&ctx2, rect);
	}

	//capture Bitmap of the painted GC
	*pWmBitmap = WmCaptureBitmap( hGC, 0, 0,reqWidth,reqHeight );
	
	//destroy hGC
	WmDeleteGC(hGC);

	//resize to original GC
	GraphicsContext ctx1(&(pageData->hGc));
	
	frame->view()->resize(_rect.w,_rect.h);
	frame->view()->adjustViewSize();
	frame->sendResizeEvent();
    
	//paint original GC
	if (frame->renderer()) {
		frame->view()->layoutIfNeededRecursive();
		IntRect rect(pRect->x, pRect->y, _rect.w, _rect.h);
		frame->view()->paint(&ctx1, rect);
	}

	//get focused window n show it only if it the focused 
	HWIN hFocusedWnd = WmGetFocusWindow();

	//show original 
	if(pageData->hWin == WmGetParentWindow(hFocusedWnd))
	{
		WmShowGC( pageData->hGc );
	}
}
#endif

void
 webkit_shp_page_paint(WebKitShpPage* page, void* pUserData)
{
    WebKitShpPagePrivate* pageData = page->privatePage;

    WebKitShpFramePrivate* frameData = pageData->mainFrame->privateFrame;
    Frame* frame = frameData->frame;
#ifdef _SFB_PAGE_PREVIEW
	WmRect preview_Rect;
	int Height,Width;
	WmBitmap* 	pWBm3DEffect;
#endif

    WmRect* pRect = (WmRect *) pUserData;
#ifdef _SFB_PAGE_PREVIEW	//vamsi Mar 03 - Page Overview
	if(pageData->gPreview)
	{
		WmGetWindowClientRect( pageData->hWin , &preview_Rect );
		pRect->w = preview_Rect.w  * 2;
		pRect->h = preview_Rect.h  * 2;
	}

	HGC hGC_restore = pageData->hGc_restore;
#endif
    HGC hGC = pageData->hGc;

    IntRect viewRect = frame->view()->frameGeometry();

    GraphicsContext ctx(&hGC);

    // set the place-holder image
#ifdef SFB_PROGRESSIVE_RENDERING
    ctx.setPlaceholderImage(pageData->placeholderImage);
#endif // SFB_PROGRESSIVE_RENDERING

    if (frame->renderer()) {
        frame->view()->layoutIfNeededRecursive();
        IntRect rect(pRect->x, pRect->y, pRect->w, pRect->h);
        frame->view()->paint(&ctx, rect);
    }
    
    HWIN hFocusedWnd = WmGetFocusWindow();

#ifdef _SFB_PAGE_PREVIEW	//vamsi Mar 03 - Page Overview
	if(pageData->gPreview)
	{
		/* Getting the frame height and width */
		Height = frame->view()->contentsHeight();
		Width = frame->view()->contentsWidth();

		/* Check for height and width of the frame whether greater than twice 
		the client rect height and width respectively */
		if(Height >2*(preview_Rect.h ) || Width > 2*(preview_Rect.w))
		{
			/* If width of the frame is greater than twice the client rect width and 
			height of the frame less than twice the client rect height */
			if(Width >  2*(preview_Rect.w) && Height < 2*(preview_Rect.h))
			{
				/* Capturing the page as bitmap */
				pWBm3DEffect = WmCaptureBitmap( hGC, 0, 0, preview_Rect.w * 2, Height);
			}
			/* If width of the frame is less than twice the client rect width and 
			height of the frame greater than twice the client rect height */
			else if(Width < 2*(preview_Rect.w) && Height > 2*(preview_Rect.h))
			{
				/* Capturing the page as bitmap */
				pWBm3DEffect = WmCaptureBitmap( hGC, 0, 0, Width, preview_Rect.h * 2 );
			}
			/* If width and height of the frame are greater than twice the client rect width and height respectively */
			else
			{
				/* Capturing the page as bitmap */
				pWBm3DEffect = WmCaptureBitmap( hGC, 0, 0, preview_Rect.w * 2, preview_Rect.h * 2 );
			}
		}
		/* If width and height of the frame are less than twice the client rect width and height */
		else
		{
			/* Capturing the page as bitmap */
			pWBm3DEffect = WmCaptureBitmap( hGC, 0, 0, Width, Height );
		}

		/* Drawing the captured bitmap */
		WmDrawScaledBitmap( hGC_restore, 0, 0, preview_Rect.w  - PAGE_PREVIEW_DIFF_SCROLLBAR, preview_Rect.h - PAGE_PREVIEW_DIFF_SCROLLBAR, pWBm3DEffect );
		WmFreeBitmap(pWBm3DEffect);
	}
#endif

	if(hFocusedWnd == pageData->hWin)
    {
#ifdef _SFB_PAGE_PREVIEW	//vamsi Mar 03 - Page Overview
		if(pageData->gPreview)
		{
			GraphicsContext ctx(&hGC_restore);
			
			/* Setting of border rect's width and height */
			pageData->m_border_rect->setWidth(preview_Rect.w / 2);
			pageData->m_border_rect->setHeight(preview_Rect.h / 2);
			
			/* If in page preview, draw the border rect and 
			   show the GC to scroll to required position */
			ctx.drawBorderRect(*(pageData->m_border_rect));

			WmShowGC( hGC_restore );
		}
		else
		{
#endif
		WmShowGC( hGC );
#ifdef SUPPORT_VIRTUAL_CURSOR
        if(CVirtualCursor::isEnabled())
            CVirtualCursor::drawCursor();
#endif
#ifdef _SFB_PAGE_PREVIEW	//vamsi Mar 03 - Page Overview
		}
#endif
    }
	else
    {	
		if(pageData->hWin == WmGetParentWindow(hFocusedWnd))
		{
#ifdef _SFB_PAGE_PREVIEW	//vamsi Mar 03 - Page Overview
			if(pageData->gPreview)
				WmShowGC( hGC_restore );
			else
#endif
			WmShowGC( hGC );
		}

        WmDrawWindow(hFocusedWnd, TRUE );
        WmShowWindow(hFocusedWnd);

    }
}

void
 webkit_shp_page_set_hwnd(WebKitShpPage* page, HWIN hWin)
{
    WebKitShpPagePrivate* pageData = page->privatePage;

    gHWin = hWin;
    //cleanup
    if(INVALID_HOBJ != pageData->hGc)
    {
        WmDeleteGC(pageData->hGc);
        pageData->hGc = INVALID_HOBJ;
    }

	//create cursor window
	CPlatformCursorHelper::Initialize(static_cast<int>(hWin));

    pageData->hWin = hWin;
    pageData->hGc = WmCreateClientGC(hWin);

    /* Initializes the  GC attributes */
    WmInitGCAttrib ( pageData->hGc );

    WmRect          rect;
    Frame* frame = core(getFrameFromPage(page));

    WmGetWindowClientRect( hWin, &rect );
    frame->view()->resize(rect.w, rect.h);

    frame->view()->adjustViewSize();
    frame->sendResizeEvent();

#ifdef SUPPORT_VIRTUAL_CURSOR
	CVirtualCursor::setProperties(pageData->pageVirtualCursor, frame->view(), pageData->hGc);
	CVirtualCursor::switchPageCursor(pageData->pageVirtualCursor);
#endif //SUPPORT_VIRTUAL_CURSOR
}

#ifdef _SFB_PAGE_PREVIEW	//vamsi Mar 03 - Page Overview
/**
  * This function is used to restore to normal page 
  * after zooming the page to the selected region
  */
void
webkit_shp_page_preview_cancel(WebKitShpPage* page)
{
    Frame* frame = core(getFrameFromPage(page));
    WebKitShpPagePrivate* pageData = page->privatePage;
	WmRect preview_Rect;

	if(INVALID_HOBJ != pageData->hGc)
	{
		/* Delete the Page preview GC and restore the normal page GC */
		WmDeleteGC(pageData->hGc);
		pageData->hGc = pageData->hGc_restore;
		pageData->hGc_restore = INVALID_HOBJ;
								
	}

	WmGetWindowClientRect( pageData->hWin , &preview_Rect );
	/* resizing of the frame to screen size */
	frame->view()->resize(preview_Rect.w , preview_Rect.h );
	frame->forceLayout();
	frame->view()->adjustViewSize();
	frame->sendResizeEvent();

	/* Scrolling to the selected region in the page */
	IntRect viewRect = enclosingIntRect(frame->view()->visibleContentRectConsideringExternalScrollers());
	frame->view()->scroll(ScrollDown, ScrollByLine, (2 * pageData->m_border_rect->y())/LINE_STEP);
	frame->view()->scroll(ScrollRight, ScrollByLine, (2 * pageData->m_border_rect->x())/LINE_STEP);
	
	// Initialising the VC and setting the location to (0,0)
	webkit_shp_page_virtual_cursor_init(page);

	CVirtualCursor::setPointerLocation(0,0);

	/* Setting the properties of the Virtual Cursor for that page */
	CVirtualCursor::setProperties(pageData->pageVirtualCursor, frame->view(), pageData->hGc);
	CVirtualCursor::setFrameView(frame->view(), true);

	/* Disabling the Page preview mode */
	pageData->gPreview = FALSE;
	return;
}

/**
  * This function is used to check whether in the Page Preview mode
  * or not.
  */
void
webkit_shp_page_is_preview(WebKitShpPage* page, bool* bPreview)
{
    WebKitShpPagePrivate* pageData = page->privatePage;
	*bPreview = pageData->gPreview;
}

/**
  * This function is used to set the page preview mode
  * Virtual cursor is disabled
  * page "hGc" is stored in "hGc_restore"
  * Frame is resized
  * border rect is initialised to (0,0)
  */
void
webkit_shp_page_set_preview(WebKitShpPage* page)
{
    Frame* frame = core(getFrameFromPage(page));
    WebKitShpPagePrivate* pageData = page->privatePage;
	WmRect preview_Rect;
	int Height,Width;

	pageData->gPreview = true;

	webkit_shp_page_virtual_cursor_hide();
	
	WmGetWindowClientRect( pageData->hWin , &preview_Rect );

	Height = frame->view()->contentsHeight();
	Width = frame->view()->contentsWidth();

	pageData->hGc_restore = pageData->hGc;

	/* Create a MemGC of frame width and height */
	pageData->hGc = WmCreateWindowSubMemGC(pageData->hWin , 0, 0, 2*(preview_Rect.w), 2*(preview_Rect.h));
	
	/* Check for height and width of the frame whether greater than twice 
	the client rect height and width respectively */
	if(Height >2*(preview_Rect.h ) || Width > 2*(preview_Rect.w))
	{
		/* If width of the frame is greater than twice the client rect width
		and height of the frame less than twice the client rect height */
		if(Width >  2*(preview_Rect.w) && Height < 2*(preview_Rect.h))
		{
			frame->view()->resize(preview_Rect.w * 2, Height);
		}
		/* If width of the frame is less than twice the client rect width
		and height of the frame greater than height */
		else if(Width < 2*(preview_Rect.w) && Height > 2*(preview_Rect.h))
		{
			frame->view()->resize(Width, preview_Rect.h * 2);
		}
		/* If width and height of the frame are greater than twice the client rect width and height respectively */
		else
		{
			frame->view()->resize(preview_Rect.w * 2, preview_Rect.h * 2);
		}
	}
	/* If width and height of the frame are less than twice the client rect width and height respectively */
	else
	{
		frame->view()->resize(Width, Height);
	}


	frame->view()->adjustViewSize();
	frame->sendResizeEvent();

	pageData->m_border_rect->setY(0);
	pageData->m_border_rect->setX(0);

	return;
}

#define PAGE_MOVE   50	// Border rect moves by this count

/**
  * This function is used to calculate the position of the border rect
  * When Key up is pressed it should move up till the end point is reached
  * When Key down is pressed the rectangle should move down till the scrollbar position is reached
  * When Key right is pressed the rectangle should move right till the scrollbar position is reached
  * When Key left is pressed the rectangle should move left
  */
void
webkit_shp_page_preview_key_event(WebKitShpPage* page, int keyCode,  bool isKeyUp)
{
    Frame* frame = core(getFrameFromPage(page));
    WebKitShpPagePrivate* pageData = page->privatePage;
	FrameView* view = frame->view();
	int i, ii, ij;
	WmRect preview_Rect;
	IntRect viewRect = enclosingIntRect(frame->view()->visibleContentRectConsideringExternalScrollers());

	if(isKeyUp != false) return;

	WmGetWindowClientRect( pageData->hWin , &preview_Rect );

	switch(keyCode)
	{
		//case BR_APP_KEY_NAVI_UP: //14
		case VK_HP_UP:
			i = pageData->m_border_rect->y() - PAGE_MOVE;
			if(i > 0)
				pageData->m_border_rect->setY(i);
			else if(viewRect.y() == 0)
				pageData->m_border_rect->setY(0);
			else
				frame->view()->scroll(ScrollUp, ScrollByLine, 5);
			break;

		//case BR_APP_KEY_NAVI_DOWN: //15
		case VK_HP_DOWN:
			i = pageData->m_border_rect->y() + PAGE_MOVE;
			ii = i + pageData->m_border_rect->height();
			ij = viewRect.y() + viewRect.height();
			if(ii < preview_Rect.w)
				pageData->m_border_rect->setY(i);
			else if(ij == frame->view()->contentsHeight())
				pageData->m_border_rect->setY( (preview_Rect.h / 2) - PAGE_PREVIEW_DIFF_SCROLLBAR);
			else
				frame->view()->scroll(ScrollDown, ScrollByLine, 5);
			break;

		//case BR_APP_KEY_NAVI_LEFT: //12
		case VK_HP_LEFT:
			i = pageData->m_border_rect->x() - PAGE_MOVE;
			if(i > 0)
				pageData->m_border_rect->setX(i);
			else if(viewRect.x() == 0)
				pageData->m_border_rect->setX(0);
			else
				frame->view()->scroll(ScrollLeft, ScrollByLine, 5);
			break;

		//case BR_APP_KEY_NAVI_RIGHT: //12
		case VK_HP_RIGHT:
			i = pageData->m_border_rect->x() + PAGE_MOVE;
			ii = i + pageData->m_border_rect->width();
			ij = viewRect.x() + viewRect.width();
			if (ii < preview_Rect.h)
				pageData->m_border_rect->setX(i);
			else if(ij >= (frame->view()->contentsWidth() - PAGE_PREVIEW_DIFF_SCROLLBAR ))
				pageData->m_border_rect->setX((preview_Rect.w / 2) - PAGE_PREVIEW_DIFF_SCROLLBAR);
			else
				frame->view()->scroll(ScrollRight, ScrollByLine, 5);
			break;

	}

	WmRect wmRect;
	wmRect.x = 0;
	wmRect.y = 0;
	wmRect.w = preview_Rect.w;		
	wmRect.h = preview_Rect.h;		
	
	webkit_shp_page_paint(page, &wmRect);
}
#endif

void
webkit_shp_page_key_event(WebKitShpPage* page, int keyCode,  bool isKeyUp)
{
    Frame* frame = core(getFrameFromPage(page));
    WebKitShpPagePrivate* pageData = page->privatePage;

#ifdef _SFB_PAGE_PREVIEW	//vamsi Mar 03 - Page Overview
	//if((keyCode == BR_APP_KEY_OK) && pageData->gPreview)
	if((keyCode == VK_HP_OK) && pageData->gPreview)
	{
		if(isKeyUp == false) return;

		/* When Center key is pressed and page preview is enabled, zoom to the select region*/
		webkit_shp_page_preview_cancel(page);
		return;
	}
	/* When left easy key is pressed and page preview is enabled, zoom to the select region */
	//if((keyCode == BR_APP_KEY_LEFT_EASY) && pageData->gPreview)
	if((keyCode == VK_HP_LEFTEASY) && pageData->gPreview)
	{
		webkit_shp_page_preview_cancel(page);
		return;
	}

	if(pageData->gPreview)
	{
		/* When page preview is enabled, this function is called to 
		   move rectangle region to required position */
		webkit_shp_page_preview_key_event(page, keyCode, isKeyUp);
		return;
	}
#endif

#ifdef SUPPORT_VIRTUAL_CURSOR
    if(isKeyUp)
        CVirtualCursor::stopFastScrolling();
    else
        CVirtualCursor::startFastScrolling();
#endif //SUPPORT_VIRTUAL_CURSOR

    HWIN hFocusedWnd = WmGetFocusWindow();
    if ( hFocusedWnd != pageData->hWin )
    {
        frame->eventHandler()->keyEvent(PlatformKeyboardEvent(keyCode,  isKeyUp));
        return;
    }

    switch(keyCode)
    {
        //case BR_APP_KEY_NAVI_UP: //14
	case VK_HP_UP:
        {
#ifdef SUPPORT_VIRTUAL_CURSOR
            if(CVirtualCursor::isEnabled())
            {
                if(!isKeyUp)
                    CVirtualCursor::handleNavigationEvent(VCS_NAVIGATION_UP);
            }
            else
#endif //SUPPORT_VIRTUAL_CURSOR
            {
                frame->view()->scroll(ScrollUp, ScrollByLine);
            }

            break;
        }

        //case BR_APP_KEY_NAVI_DOWN: //15
	case VK_HP_DOWN:
        {
#ifdef SUPPORT_VIRTUAL_CURSOR
            if(CVirtualCursor::isEnabled())
            {
                if(!isKeyUp)
                    CVirtualCursor::handleNavigationEvent(VCS_NAVIGATION_DOWN);
            }
            else
#endif //SUPPORT_VIRTUAL_CURSOR
            {
                frame->view()->scroll(ScrollDown, ScrollByLine);
            }

            break;
        }

        //case BR_APP_KEY_NAVI_LEFT: //12
	case VK_HP_LEFT:
        {
#ifdef SUPPORT_VIRTUAL_CURSOR
            if(CVirtualCursor::isEnabled())
            {
                if(!isKeyUp)
                    CVirtualCursor::handleNavigationEvent(VCS_NAVIGATION_LEFT);
            }
            else
#endif //SUPPORT_VIRTUAL_CURSOR
            {
                frame->view()->scroll(ScrollLeft, ScrollByLine);
            }

            break;
        }

        //case BR_APP_KEY_NAVI_RIGHT: //13
	case VK_HP_RIGHT:
        {
#ifdef SUPPORT_VIRTUAL_CURSOR
            if(CVirtualCursor::isEnabled())
            {
                if(!isKeyUp)
                    CVirtualCursor::handleNavigationEvent(VCS_NAVIGATION_RIGHT);
            }
            else
#endif //SUPPORT_VIRTUAL_CURSOR
            {
                frame->view()->scroll(ScrollRight, ScrollByLine);
            }

            break;
        }

        //case BR_APP_KEY_OK:
	case VK_HP_OK:
        {
#ifdef SUPPORT_VIRTUAL_CURSOR
            if(CVirtualCursor::isEnabled())
            {
                if(isKeyUp)
                    CVirtualCursor::handleNavigationEvent(VCS_NAVIGATION_CENTER);
            }
#endif //SUPPORT_VIRTUAL_CURSOR
            break;
        }

        default:
        {
            frame->eventHandler()->keyEvent(PlatformKeyboardEvent(keyCode, isKeyUp));
            break;
        }
    }
}


/**
 *   bs0425.lee  modified for touch events 
 */

void
webkit_shp_page_mouse_event(WebKitShpPage* page, int recX, int recY, int in_MouseEventType)
{
    Frame* frame = core(getFrameFromPage(page));
    WebKitShpPagePrivate* pageData = page->privatePage;

	 HWIN hFocusedWnd = WmGetFocusWindow();

    if (webkit_shp_page_IsEditBox())
    	{
    		if(hFocusedWnd != pageData->hWin)
    		{
	
			  webkit_shp_page_ExitEditBox();
      
      			  return;
      		}
 	}
// SFB_Mouse_XXX has been removed in BrCoreControl.h
  //  switch(in_MouseEventType)
		//{
		//	case SFB_Mouse_Moved: // TouchPressed
		//	{   

		//		 //for hovered
		//	
		//		FrameView *frameview = frame->view();
		//		IntPoint ptContent = frameview->windowToContents( IntPoint(recX, recY));	
		//		PlatformMouseEvent pme(recX, recY,ptContent.x() , ptContent.y(), NoButton, MouseEventMoved,
		//					false, false, true, false, false, 0, 0);		
		//		frameview->frame()->eventHandler()->mouseMoved(pme);

		//	
		//		const HitTestRequest hitTest(false, true);
		//		IntPoint documentPoint(ptContent.x(), ptContent.y());
		//		MouseEventWithHitTestResults mev = frameview->frame()->document()->prepareMouseEvent(hitTest, documentPoint, pme); 

		//			
  //         			break;
		//	}

		//	case SFB_Mouse_Pressed: // TouchClicked
		//	{	

		//	    FrameView *frameview = frame->view();
		//		MouseEventType met = MouseEventPressed;

		//		// See : m_virtualCursor->sendVirtualMouseEvent(VIRTUAL_MOUSE_PRESSED);

		//		IntPoint ptContent = frameview->windowToContents(IntPoint(recX,recY));
		//		PlatformMouseEvent pme(recX,recY,ptContent.x() , ptContent.y(), LeftButton, met,
		//				   true, false, false, false, false, 0, 0);

 	//			 frameview->frame()->eventHandler()->handleMousePressEvent(pme);
		//
		//		// See :m_virtualCursor->sendVirtualMouseEvent(VIRTUAL_MOUSE_RELEASED);
		//		
		//		met = MouseEventReleased;
		//		PlatformMouseEvent pme_(recX, recY,ptContent.x() , ptContent.y(), LeftButton, met,
		//			   true, false, false, false, false, 0, 0);

		//		frameview->frame()->eventHandler()->handleMouseReleaseEvent(pme_);
		//			
		//		break;
		//	}

		//	case SFB_Mouse_Released: // TouchReleased
		//	{


		//		break;
		//	}
		//}

}

void
webkit_shp_page_ScrollBy(WebKitShpPage* page, int offsetX, int offsetY)
{
	Frame* frame = core(getFrameFromPage(page));
	frame->view()->scrollBy(offsetX, offsetY);
}


//  This function checks whether current point was already focused or not.

gboolean 
 webkit_shp_page_IsFocusedByPoint(WebKitShpPage* page, int pointX, int pointY)
{
	static Element* preElement;
	Frame* frame = core(getFrameFromPage(page));
	
     FrameView *frameview = frame->view();
	 IntPoint ptContent = frameview->windowToContents(IntPoint(pointX,pointY));
	 Document* NewDocument=frame->documentAtPoint(ptContent);


	 Element* element =NewDocument->elementFromPoint(ptContent.x(), ptContent.y());


	 if(preElement==element)
	 {
	 	preElement=element;
	 	return TRUE;
	 }

	 preElement=element;
	 return FALSE;
	
}

//  This function checks whether current element was linked or not.

gboolean 
 webkit_shp_page_IsLinkedElement(WebKitShpPage* page, int pointX, int pointY)
{
	 bool isLink=FALSE;
	 Frame* frame = core(getFrameFromPage(page));
	
        FrameView *frameview = frame->view();
	 IntPoint ptContent = frameview->windowToContents(IntPoint(pointX, pointY));
	 Document* NewDocument=frame->documentAtPoint(ptContent);


	 Element* element =NewDocument->elementFromPoint(ptContent.x(), ptContent.y());
	 isLink=element->isLink();
	
	 return isLink;
}

//  This function checks whether current element is Image or not. 

gboolean 
 webkit_shp_page_IsImageElement(WebKitShpPage* page, int pointX, int pointY)
{
	 Frame* frame = core(getFrameFromPage(page));
	
        FrameView *frameview = frame->view();
	 IntPoint ptContent = frameview->windowToContents(IntPoint(pointX,pointY));
	 Document* NewDocument=frame->documentAtPoint(ptContent);
	
	 Element* element =NewDocument->elementFromPoint(ptContent.x(), ptContent.y());
	 
	 String Isimage=element->nodeName();
	
	 if(Isimage=="IMG")
	 {
		 return TRUE;
	 }
	 else
	 {
		 return FALSE;
	 }
}



//  This function checks whether current point was Editbox focused or not.

gboolean 
 webkit_shp_page_IsEditBox()
{
    NativeEditField* pNativeEdit = NativeEditField::getNativeEditFieldInstance();
    ASSERT(pNativeEdit);

    BOOL	isEditBox=pNativeEdit->isActivated() ;
    return isEditBox;
}

// This function exit editbox. 

void
 webkit_shp_page_ExitEditBox()
{
	 NativeEditField* pNativeEdit = NativeEditField::getNativeEditFieldInstance();
        ASSERT(pNativeEdit);
	 //pNativeEdit->ExitFocusedNode();
}

//  This function gets current pointed element infomation . 

void 
webkit_shp_page_GetElementInfoByPoint( WebKitShpPage* mPage, EPageItemType aType, ElementInfoBase* aInfo, int pointX, int pointY)
{

     Frame* frame = core(getFrameFromPage(mPage));
	
     FrameView *frameview = frame->view();
     IntPoint ptContent = frameview->windowToContents(IntPoint(pointX, pointY));
     Document* NewDocument=frame->documentAtPoint(ptContent);
	
    Element* element =NewDocument->elementFromPoint(ptContent.x(), ptContent.y());
    WebCore::Node * curNode =element;
    
    if(curNode)
    {
        switch (aType)
        {
        case EAnchor:
            {
                WebCore::Node * parNode = curNode->parentNode();
                WebCore::String curNodeName = curNode->nodeName();

                if(parNode)
                {
                    if(equalIgnoringCase(parNode->nodeName(), "A"))
                    {
                        WebCore::HTMLAnchorElement *anchorElement = reinterpret_cast<WebCore::HTMLAnchorElement *>(parNode);
                        WebCore::String anchorUrl = anchorElement->href();
                        AnchorInfo * tempAnchorInfo = (AnchorInfo *) aInfo;
                        tempAnchorInfo->element_info_base.element_type = EAnchor;
                        tempAnchorInfo->access_key = 0;
                        if(anchorUrl.latin1().length() < tempAnchorInfo->length)
                        {
                            AcStrcpy(tempAnchorInfo->url,anchorUrl.latin1().data());
                        }
                        else
                        {
                            AcStrncpy(tempAnchorInfo->url ,anchorUrl.latin1().data(), tempAnchorInfo->length - 1);
                            tempAnchorInfo->length = anchorUrl.latin1().length();
                        }
                    }
                }
            }
            break;
        case EImages:
            if( equalIgnoringCase(curNode->nodeName(), "IMG") )
            {
                RenderImage *renderImage;
                renderImage = static_cast<RenderImage *>(curNode->renderer());
                if(renderImage)
                {
                    CachedImage* cachedImage;
                    cachedImage = renderImage->cachedImage();
                    if(cachedImage)
                    {
                        ImageInfo* tempImageInfo = (ImageInfo*) aInfo;
                        tempImageInfo->element_info_base.element_type = EImages;
                        Image* image = cachedImage->image();
                        int ImageSize = image->data()->size();
			   tempImageInfo->size=ImageSize;
                        WebCore::HTMLImageElement *imageElement = reinterpret_cast<WebCore::HTMLImageElement *>(curNode);
                        String url;
                        String extension;
                        url = imageElement->src();
                        if(url.latin1().length() < tempImageInfo->url_length)
                        {
                            AcStrcpy(tempImageInfo->url, url.latin1().data());
                        }
                        else
                        {
                            AcStrncpy(tempImageInfo->url ,url.latin1().data(), tempImageInfo->url_length - 1);
                            tempImageInfo->url_length = url.latin1().length();
                        }
                        int dotIndex = url.reverseFind('.');
                        if( -1 != dotIndex )
                        {
                            extension = url.substring(dotIndex + 1);
                            if(extension.latin1().length() < tempImageInfo->extension_length)
                            {
                                AcStrcpy(tempImageInfo->extension, extension.latin1().data());
                            }
                            else
                            {
                                AcStrncpy(tempImageInfo->extension ,extension.latin1().data(), tempImageInfo->extension_length - 1);
                                tempImageInfo->extension_length = extension.latin1().length();
                            }
                        }
                    }
                }
            }
            break;
        default:
            break;
        }
    }

}


// This function gets pointed element types. 

void webkit_shp_page_GetSelectedElementTypeByPoint( WebKitShpPage* mPage,EPageItemType* aType, int pointX, int pointY )
{
      *aType = ENone;


	 Frame* frame = core(getFrameFromPage(mPage));
	
       FrameView *frameview = frame->view();
	 IntPoint ptContent = frameview->windowToContents(IntPoint(pointX,pointY));
	 Document* NewDocument=frame->documentAtPoint(ptContent);
	
	 Element* element =NewDocument->elementFromPoint(ptContent.x(), ptContent.y());
	 

	 if(element)
    	{
       	 if(equalIgnoringCase(element->nodeName(), "IMG"))
        	{
            		*aType = EImages;
        	}
        	WebCore::Node* parNode = element->parentNode();
        	if(parNode)
        	{
           		 if(equalIgnoringCase(element->nodeName(), "A"))
            		{
                		*aType = (EPageItemType)(*aType | EAnchor);
            		}
        	}
    	}
}



/**
 *   bs0425.lee  modified for touch events 
 *  
 */




#if PLATFORM(SHP) && defined(SFB_SSR_RENDERING)
// CELUNITE_SSR_Change_Begin
void
webkit_shp_page_layoutChange(WebKitShpPage* page, WEBKIT_SSR_MODES sSRMode)
{
    Frame* frame = core(getFrameFromPage(page));
    if(frame && frame->document() && frame->document()->styleSelector())
    {
       SSR_MODES mode = SSR_MODE_OFF;

       if(WEBKIT_SSR_MODE_ON == sSRMode)
       {
           mode = SSR_MODE_ON;
       }

       frame->setSSRMode(mode);

#ifdef SUPPORT_VIRTUAL_CURSOR
	//update stats as page load is complete
	//update horizontal stats
	CVirtualCursor::updateHorizontalStats();
	//update vertical stats
	CVirtualCursor::updateVerticalStats();

	CVirtualCursor::setPointerLocation(0, 0);

#endif //SUPPORT_VIRTUAL_CURSOR

       frame->view()->scroll(ScrollUp, ScrollByDocument);
    }
}

// This function will check if the filename is a valid name or not
// returns true if its valid name else will return false
// NOTE: This functions logic can vary according to the platform.
bool is_valid_filename(const String& fileName)
{
	if(fileName.isEmpty() 
		|| fileName.contains(UChar('?')) || fileName.contains(UChar('*'))
		|| fileName.contains(UChar('>')) || fileName.contains(UChar('/'))
		|| fileName.contains(UChar('<')) || fileName.contains(UChar('\"'))
		|| fileName.contains(UChar(':')) || fileName.contains(UChar('|'))
		|| fileName.contains(UChar('\\'))|| !fileName.contains(UChar('.')))
		return false;
	return true;
}

// returns true if the image has been saved already else returns false
WEBKIT_SHP_RES_RESOLUTION is_already_saved(const Vector<WebCore::Node*>& imgDataList,const WebCore::Node* image)
{
	WEBKIT_SHP_RES_RESOLUTION result = WEBKIT_SHP_DIFF_RES;
	for(int i=0;i<imgDataList.size();i++)
	{
		String rPreOrgPath = imgDataList[i]->attributes()->getAttributeItem("src")->value();
		String rOrgPath = image->attributes()->getAttributeItem("src")->value();
		//if paths are same then do not compare the img attributes
		if(rPreOrgPath == rOrgPath)
		{
			result = WEBKIT_SHP_SAME_RES;
			break;
		}
		else
		{
	        int slashIndex = rPreOrgPath.reverseFind('/');
			String rPreFileName = ( -1 != slashIndex )?rPreOrgPath.substring(slashIndex + 1):rPreOrgPath;
			slashIndex = rOrgPath.reverseFind('/');
			String rFileName = ( -1 != slashIndex )?rOrgPath.substring(slashIndex + 1):rOrgPath;
			if(rPreFileName == rFileName)
			{
				Image* rPreImage = (static_cast<RenderImage *>(imgDataList[i]->renderer()))->cachedImage()->image();
				Image* rImage = static_cast<RenderImage *>(image->renderer())->cachedImage()->image();
				if(rPreImage->size() == rImage->size() && 
					rPreImage->width() == rImage->width() && 
					rPreImage->height() == rImage->height())
				{
					//path not same,just check that the img attributes are different.
					result = WEBKIT_SHP_SAME_RES;
					break;
				}
				else
				{
					result = WEBKIT_SHP_SAME_NAME_RES_DIFF;
					break;
				}
			}
		}
	}
	return result;
}


// CELUNITE_SSR_Change_End
#endif

void
webkit_shp_page_save(WebKitShpPage* page, const gchar* aPath)
{
    WebKitShpFramePrivate* frameData = page->privatePage->mainFrame->privateFrame;

    int itemTypeMask = 0x01 | 0x02 | 0x04;
    Vector<WebCorePageDataItem*> pdItems;
    pdItems = frameData->frame->getWebKitBridge()->GetWebCoreBridge()->GetPageData(itemTypeMask);
    WebCorePageDataItem* pd = pdItems[0];
    String markupData;
    markupData = pd->Data();

    PassRefPtr<HTMLCollection> imageCollection = frameData->frame->getWebKitBridge()->GetWebCoreBridge()->GetFrame()->document()->images();
    //Cached Image has the image bitmap buffer and also holds the fully qualified Url.
    CachedImage* cachedImage;
    RenderImage *renderImage;
    String orgPath("");
    String newPath("");
    String fileName("");
    String filesPath("");
    filesPath = aPath;
    filesPath += "_files/";

    FmCreateDir(filesPath.latin1().data());

    HFile hFile;
    //Node maintains the 'src' attribute as it appears in the HTML. This may be a relative path
    //w.r.t the page.
    PassRefPtr<WebCore::Node> tempNode;
	Vector<WebCore::Node*> imgDataList;
	int dupIndex = 0;
    for (int i=0; i< imageCollection->length() ;i++)
    {
        tempNode = imageCollection->item(i);
        if(!tempNode)
            continue;
        renderImage = static_cast<RenderImage *>(tempNode->renderer());
        if(!renderImage)
            continue;
        cachedImage = renderImage->cachedImage();
        if(!cachedImage)
            continue;

        orgPath = tempNode->attributes()->getAttributeItem("src")->value();

        int slashIndex = orgPath.reverseFind('/');
        if( -1 != slashIndex )
            fileName = orgPath.substring(slashIndex + 1);
        else
            fileName = orgPath;

		if(is_valid_filename(fileName))
		{
			switch(is_already_saved(imgDataList,tempNode.get()))
			{
				case WEBKIT_SHP_DIFF_RES:
					break;
				case WEBKIT_SHP_SAME_NAME_RES_DIFF:
				{
					int idx = fileName.find(UChar('.'));
					String dupIdxStr("");
					dupIdxStr = dupIdxStr.number(dupIndex++);
					dupIdxStr.insert(String("_"),0);
					fileName.insert(dupIdxStr,idx);
				}
				break;
				case WEBKIT_SHP_SAME_RES:
					continue;
				break;
			}
			newPath = filesPath + fileName;

			Image* image = cachedImage->image();
			imgDataList.append(tempNode.get());
			if(!image->data())
				continue;

			hFile = FmCreateFile (newPath.latin1().data(), FM_WRITE | FM_CREATE | FM_TRUNCATE);
			if (hFile == INVALID_HOBJ )
			{
				SysDebug((MID_EXCEPTION,"webkit_shp_page_save:FmCreateFile Failed\n"));
			}
			if (hFile != INVALID_HOBJ)
			{
				FmWriteFile(hFile, (void*)(image->data()->data()), image->data()->size());
				FmCloseFile(hFile);
			}
			markupData.replace(orgPath, newPath);
		}
    }

    for(int i = 1; i < pdItems.size(); i++)
    {
        orgPath = pdItems[i]->Url();
        int slashIndex = orgPath.reverseFind('/');
        if( -1 != slashIndex )
            fileName = orgPath.substring(slashIndex + 1);
        else
            fileName = orgPath;
		
		// if there is some data that needs to be posted which follows the
		// file name just remove it.
		int qIndex = fileName.find('?');
        if( -1 != qIndex )
            fileName = fileName.substring(0,qIndex);

		if(is_valid_filename(fileName))
		{
			if(fileName.length() > FM_FILENAME_LEN_MAX)
			{
				// Finding the dot and then the extension will not work
				// as most of these long names as some url's
				// so just adding a simple hack.
				fileName = fileName.substring(0,10);
			}

			newPath = filesPath + fileName;

			if(newPath.contains(UChar('%')))
				newPath.replace(UChar('%'),UChar('_'));
			if(newPath.contains(UChar('&')))
				newPath.replace(UChar('&'),UChar('_'));
			
			hFile = FmCreateFile(newPath.latin1().data(), FM_WRITE);

			if (hFile == INVALID_HOBJ )
			{
				SysDebug((MID_EXCEPTION,"webkit_shp_page_save:FmCreateFile Failed\n"));
			}
			if (hFile != INVALID_HOBJ)
			{
				//FmWriteFile(hFile, (void*)(pdItems[i]->Data()).characters(), pdItems[i]->Data().length()*2);
				FmWriteFile(hFile, (void*)(markupData.characters()), markupData.length()*2); /*Added by vijay to fix SFB00000389*/

				FmCloseFile(hFile);
			}

			markupData.replace(orgPath, newPath);
		}
	}
	// Adding .html extension to the save file name.
	unsigned int saveFileNameLength = sizeof(gchar) * AcStrlen(aPath) + 6;
	gchar* saveFileName = (gchar*)SFBalMemAlloc(saveFileNameLength);
	AcMemset(saveFileName,0x00,saveFileNameLength);
	AcStrncat(saveFileName,aPath,AcStrlen(aPath));
/*	gchar* saveFileNameExt = (gchar*)SFBalMemAlloc(6);
	AcMemset(saveFileNameExt,0x00,6);
	
	int extLength = 0;
	switch(frameData->frame->getWebKitBridge()->GetWebCoreBridge()->GetFrame()->document()->htmlMode())
	{
		case Document::Html3:
		case Document::Html4:
			extLength = 5;
			AcStrncat(saveFileNameExt,".html",extLength);
			break;
		case Document::XHtml:
			extLength = 6;
			AcStrncat(saveFileNameExt,".xhtml",extLength);
			break;
		case Document::Wml :
			extLength = 4;
			AcStrncat(saveFileNameExt,".wml",extLength);
			break;
	}
	AcStrncat(saveFileName + AcStrlen(aPath),saveFileNameExt,extLength);*/
    hFile = FmCreateFile(saveFileName, FM_WRITE);
    FmWriteFile(hFile, (void*)(markupData.characters()), markupData.length()*2);
    FmCloseFile(hFile);
	//SFBalMemFree(saveFileNameExt);
	SFBalMemFree(saveFileName);
}

void
webkit_shp_page_virtual_cursor_init(WebKitShpPage* page)
{
#ifdef SUPPORT_VIRTUAL_CURSOR
//    CVirtualCursor::setEnabled(true);
#endif //SUPPORT_VIRTUAL_CURSOR
}

void
webkit_shp_page_virtual_cursor_hide()
{
#ifdef SUPPORT_VIRTUAL_CURSOR
//    CVirtualCursor::setEnabled(false);
#endif //SUPPORT_VIRTUAL_CURSOR
}

void
webkit_shp_page_get_url(WebKitShpPage* page, gchar* aUrl, unsigned long* aSize)
{
    String url = page->privatePage->mainFrame->privateFrame->frame->document()->documentURI();
    if(*aSize <= url.length())
    {
        AcStrncpy(aUrl, url.latin1().data(), *aSize - 1);
        *aSize = url.length() + 1;
    }
    else
    {
        AcStrcpy(aUrl, url.latin1().data());
    }
}

void
webkit_shp_page_get_title(WebKitShpPage* page, gchar* aTitle, unsigned long* aSize)
{
    String title = page->privatePage->mainFrame->privateFrame->frame->document()->title();
    if(*aSize <= title.length())
    {
        AcStrncpy(aTitle, title.latin1().data(), *aSize - 1);
        *aSize = title.length() + 1;
    }
    else
    {
        AcStrcpy(aTitle, title.latin1().data());
    }
}

void
webkit_shp_page_get_mime_type(WebKitShpPage* page, gchar* aType, unsigned long* aSize)
{
    DocumentType* docType = page->privatePage->mainFrame->privateFrame->frame->document()->doctype();
    String type;
    if(docType)
        type = docType->name();
    else
        type = HTML_DOC_TYPE_STRING;
    if(*aSize <= type.length())
    {
        AcStrncpy(aType, type.latin1().data(), *aSize - 1);
        *aSize = type.length() + 1;
    }
    else
    {
        AcStrcpy(aType, type.latin1().data());
    }
}

void
webkit_shp_page_get_size(WebKitShpPage* page, unsigned long* aSize)
{
    String doc = page->privatePage->mainFrame->privateFrame->frame->document()->toString();
    *aSize = doc.length();
}

#ifdef SFB_RSS_LINK //vamsi Jan 28 Rss Link Tag Support for SFB.
void
webkit_shp_page_get_rss_links(WebKitShpPage* page, SRssLink** ppRssLink, int* rssCount, gchar** url)
{
	RefPtr<HTMLCollection> collection = new HTMLCollection(page->privatePage->mainFrame->privateFrame->frame->document(), HTMLCollection::LinkTag) ;
	
	//If no links are found, just return immediately
	int count = collection->length();
	if(count <=0 )
	{
		//delete collection;
		return;
	}

	*ppRssLink = (SRssLink*) SFBalMemCalloc(count, sizeof(SRssLink));
	SRssLink* pRssLink = *ppRssLink;

	WebCore::String szRss("application/rss+xml");
	WebCore::String szAtom("application/atom+xml");
	WebCore::String szAlternate("alternate");

	//Parse all link item nodes and see if they are Rss specific.
	for(WebCore::Node *node = collection->firstItem(); node; node = collection->nextItem())	
	{
		WebCore::HTMLLinkElement* linkElement =  reinterpret_cast<HTMLLinkElement *>(node) ;

		WebCore::String szType	= linkElement->type();
		WebCore::String szRel	= linkElement->rel();
		WebCore::String szHref	= linkElement->href();
		WebCore::String szTitle = linkElement->title();

		//The combination of rel="alternate" and type="application/rss+xml" defines this as a link to an RSS file.
		if(! ( ((szType == szRss) || (szType == szAtom)) && (szRel == szAlternate) ) )
			continue;

		//Copy Href
		if(szHref.length() > 0)
		{
			(pRssLink)->szHref = (gchar*) SFBalMemAlloc((szHref.length() * sizeof(gchar)) + 1);
			AcStrcpy((pRssLink)->szHref, (char *)szHref.latin1().data());
		}

		//Copy Title
		if(szTitle.length() > 0)
		{
			(pRssLink)->szTitle = (gchar*) SFBalMemAlloc((szTitle.length() * sizeof(gchar)) + 1);
			AcStrcpy((pRssLink)->szTitle, (char *)szTitle.latin1().data());
		}

		//Copy Type
		if(szType.length() > 0)
		{
			(pRssLink)->szType = (gchar*) SFBalMemAlloc((szType.length() * sizeof(gchar)) + 1);
			AcStrcpy((pRssLink)->szType, (char *)szType.latin1().data());
		}

		(*rssCount)++;
		pRssLink++;
	}

	//Url is required as Href can be a relative one
	if(*rssCount > 0)
	{
		String szUrl = page->privatePage->mainFrame->privateFrame->frame->document()->documentURI();
		*url = (gchar*) SFBalMemAlloc((szUrl.length() * sizeof(gchar)) + 1);
		AcStrcpy(*url, szUrl.latin1().data());
	}

	//delete collection;
}
#endif

void
webkit_shp_page_ZoomIn(WebKitShpPage* page, const int iZoom)
{
    Frame* frame = core(getFrameFromPage(page));
    frame->setZoomFactor(iZoom);
}

int
webkit_shp_page_get_ZoomFactor(WebKitShpPage* page )
{
    Frame* frame = core(getFrameFromPage(page));
    return frame->zoomFactor();
}

int
webkit_shp_page_GetProperty( WebKitShpPage* mPage,
                                const EBrowserProperty aBrowserProperty,
                                gboolean* aSettingValue )
{
    switch(aBrowserProperty)
    {
        case ESetSSR:
            {
                *aSettingValue = BrowserSettings::GetStatus_SSR();
            }
            break;

        case EVirtualCursor:
            {
                *aSettingValue = BrowserSettings::GetStatus_VirtualCursor();
            }
            break;
        case ESetSplashHTML:
            {
                *aSettingValue = BrowserSettings::GetStatus_SplashHTML();
            }
            break;
        case ESetCookies:
            {
                *aSettingValue = BrowserSettings::GetStatus_Cookies();
            }
            break;
        case ESetCache:
            {
                *aSettingValue = BrowserSettings::GetStatus_Cache();
            }
            break;
        case ESetSmartLink:
            {
                *aSettingValue = BrowserSettings::GetStatus_SmartLink();
            }
            break;
        case ESetAnimationDisplay:
            {
                *aSettingValue = BrowserSettings::GetStatus_AnimationDisplay();
            }
            break;
        case ESetImageSaveMode:
            {
                *aSettingValue = BrowserSettings::GetStatus_ImageSaveMode();
            }
            break;
        case EIsJavaEnabled:
            {
                *aSettingValue = ((struct _WebKitShpPage*)mPage)->privatePage->page->settings()->isJavaEnabled();
                break;
            }
        case ELoadsImagesAutomatically:
            {
                *aSettingValue = ((struct _WebKitShpPage*)mPage)->privatePage->page->settings()->loadsImagesAutomatically();
                break;
            }
        case EPrivateBrowsingEnabled:
            {
                *aSettingValue = ((struct _WebKitShpPage*)mPage)->privatePage->page->settings()->privateBrowsingEnabled();
                break;
            }
        case EArePluginsEnabled:
            {
            *aSettingValue = ((struct _WebKitShpPage*)mPage)->privatePage->page->settings()->arePluginsEnabled();
            break;
            }
        case EIsJavaScriptEnabled:
            {
                *aSettingValue = ((struct _WebKitShpPage*)mPage)->privatePage->page->settings()->isJavaScriptEnabled();
                break;
            }
        case EJavaScriptCanOpenWindowsAutomatically:
            {
                *aSettingValue = ((struct _WebKitShpPage*)mPage)->privatePage->page->settings()->JavaScriptCanOpenWindowsAutomatically();
                break;
            }
        case EShouldPrintBackgrounds:
            {
                *aSettingValue = ((struct _WebKitShpPage*)mPage)->privatePage->page->settings()->shouldPrintBackgrounds();
                break;
            }
        case ETextAreasAreResizable:
            {
                *aSettingValue = ((struct _WebKitShpPage*)mPage)->privatePage->page->settings()->textAreasAreResizable();
                break;
            }
        case EUsesDashboardBackwardCompatibilityMode:
            {
                *aSettingValue = ((struct _WebKitShpPage*)mPage)->privatePage->page->settings()->usesDashboardBackwardCompatibilityMode();
                break;
            }
        case ENeedsAdobeFrameReloadingQuirk:
            {
                *aSettingValue = ((struct _WebKitShpPage*)mPage)->privatePage->page->settings()->needsAcrobatFrameReloadingQuirk();
                break;
            }
        case EIsDOMPasteAllowed:
            {
                *aSettingValue = ((struct _WebKitShpPage*)mPage)->privatePage->page->settings()->isDOMPasteAllowed();
                break;
            }
        case EShrinksStandaloneImagesToFit:
            {
                *aSettingValue = ((struct _WebKitShpPage*)mPage)->privatePage->page->settings()->shrinksStandaloneImagesToFit();
                break;
            }
        case EUsesPageCache:
            {
                *aSettingValue = ((struct _WebKitShpPage*)mPage)->privatePage->page->settings()->usesDashboardBackwardCompatibilityMode();
                break;
            }
        case EShowsURLsInToolTips:
            {
                *aSettingValue = ((struct _WebKitShpPage*)mPage)->privatePage->page->settings()->showsURLsInToolTips();
                break;
            }
        case EForceFTPDirectoryListings:
            {
                *aSettingValue = ((struct _WebKitShpPage*)mPage)->privatePage->page->settings()->forceFTPDirectoryListings();
                break;
            }
        case EDeveloperExtrasEnabled:
            {
                *aSettingValue = ((struct _WebKitShpPage*)mPage)->privatePage->page->settings()->developerExtrasEnabled();
                break;
            }
        default:
            return EFail;

    }
    return ESuccess;
}

int
webkit_shp_page_SetProperty( WebKitShpPage* mPage,
                                const EBrowserProperty aBrowserProperty,
                                gboolean aSettingValue )
{

    switch(aBrowserProperty)
    {
        case ESetSSR:
            {
                WEBKIT_SSR_MODES gSSRMode = aSettingValue?WEBKIT_SSR_MODE_ON:WEBKIT_SSR_MODE_OFF;
                webkit_shp_page_layoutChange(mPage, gSSRMode);
                BrowserSettings::SetStatus_SSR(aSettingValue);
            }
            break;

        case EVirtualCursor:
            {
                if(aSettingValue)
                {
                    webkit_shp_page_virtual_cursor_init(mPage);
                }
                else
                {
                    webkit_shp_page_virtual_cursor_hide();
                }
                BrowserSettings::SetStatus_VirtualCursor(aSettingValue);
            }
            break;
        case ESetSplashHTML:
            {
                BrowserSettings::SetStatus_SplashHTML(aSettingValue);
            }
            break;
        case ESetCookies:
            {
                BrowserSettings::SetStatus_Cookies(aSettingValue);
            }
            break;
        case ESetCache:
            {
                BrowserSettings::SetStatus_Cache(aSettingValue);
            }
            break;
        case ESetSmartLink:
            {
                BrowserSettings::SetStatus_SmartLink(aSettingValue);
            }
            break;
        case ESetAnimationDisplay:
            {
                BrowserSettings::SetStatus_AnimationDisplay(aSettingValue);
            }
            break;
        case ESetImageSaveMode:
            {
                BrowserSettings::SetStatus_ImageSaveMode(aSettingValue);
            }
            break;
        case EIsJavaEnabled:
            {
                ((struct _WebKitShpPage*)mPage)->privatePage->page->settings()->setJavaEnabled(aSettingValue);
                break;
            }
        case ELoadsImagesAutomatically:
            {
                ((struct _WebKitShpPage*)mPage)->privatePage->page->settings()->setLoadsImagesAutomatically(aSettingValue);
                break;
            }
        case EPrivateBrowsingEnabled:
            {
                ((struct _WebKitShpPage*)mPage)->privatePage->page->settings()->setPrivateBrowsingEnabled(aSettingValue);
                break;
            }
        case EArePluginsEnabled:
            {
                ((struct _WebKitShpPage*)mPage)->privatePage->page->settings()->setPluginsEnabled(aSettingValue);
                break;
            }
        case EIsJavaScriptEnabled:
            {
                ((struct _WebKitShpPage*)mPage)->privatePage->page->settings()->setJavaScriptEnabled(aSettingValue);
                break;
            }
        case EJavaScriptCanOpenWindowsAutomatically:
            {
                ((struct _WebKitShpPage*)mPage)->privatePage->page->settings()->setJavaScriptCanOpenWindowsAutomatically(aSettingValue);
                break;
            }
        case EShouldPrintBackgrounds:
            {
                ((struct _WebKitShpPage*)mPage)->privatePage->page->settings()->setShouldPrintBackgrounds(aSettingValue);
                break;
            }
        case ETextAreasAreResizable:
            {
                ((struct _WebKitShpPage*)mPage)->privatePage->page->settings()->setTextAreasAreResizable(aSettingValue);
                break;
            }
        case EUsesDashboardBackwardCompatibilityMode:
            {
                ((struct _WebKitShpPage*)mPage)->privatePage->page->settings()->setUsesDashboardBackwardCompatibilityMode(aSettingValue);
                break;
            }
        case ENeedsAdobeFrameReloadingQuirk:
            {
                ((struct _WebKitShpPage*)mPage)->privatePage->page->settings()->setNeedsAdobeFrameReloadingQuirk(aSettingValue);
                break;
            }
        case EIsDOMPasteAllowed:
            {
                ((struct _WebKitShpPage*)mPage)->privatePage->page->settings()->setDOMPasteAllowed(aSettingValue);
                break;
            }
        case EShrinksStandaloneImagesToFit:
            {
                ((struct _WebKitShpPage*)mPage)->privatePage->page->settings()->setShrinksStandaloneImagesToFit(aSettingValue);
                break;
            }
        case EUsesPageCache:
            {
                ((struct _WebKitShpPage*)mPage)->privatePage->page->settings()->setUsesPageCache(aSettingValue);
                break;
            }
        case EShowsURLsInToolTips:
            {
                ((struct _WebKitShpPage*)mPage)->privatePage->page->settings()->setShowsURLsInToolTips(aSettingValue);
                break;
            }
        case EForceFTPDirectoryListings:
            {
                ((struct _WebKitShpPage*)mPage)->privatePage->page->settings()->setForceFTPDirectoryListings(aSettingValue);
                break;
            }
        case EDeveloperExtrasEnabled:
            {
                ((struct _WebKitShpPage*)mPage)->privatePage->page->settings()->setDeveloperExtrasEnabled(aSettingValue);
                break;
            }
        default:
            return EFail;
    }

    return ESuccess;
}


void webkit_shp_page_SetHttpProxy( WebKitShpPage* mPage,const SProxyInfo* aProxyInfo)
{
    BrowserSettings::SetHttpProxy(aProxyInfo);
}

void webkit_shp_page_SetIAP( WebKitShpPage* mPage,const IAPHANDLE aSelectedIapHandle)
{
    BrowserSettings::SetIAP( aSelectedIapHandle);

	SFBRmSetPDPAccount( mPage->m_hRmClient, aSelectedIapHandle);
}

void webkit_shp_page_SetCacheSize( WebKitShpPage* mPage,const guint aCacheSize)
{
    BrowserSettings::SetCacheSize( aCacheSize);
}
#if defined ( SHP_SUPPORT_WML )
int webkit_document_get_count(WebKitShpPage *pageData)
{
	WebKitShpFrame* frame = webkit_shp_page_get_main_frame(pageData);
	return (frame->privateFrame->page->privatePage->mainFrame->privateFrame->frame->document())->WML_ItemCount();
}
int webkit_document_wmldoEvent(WebKitShpPage *pageData, int OptionId)
{
	WebKitShpFrame* frame = webkit_shp_page_get_main_frame(pageData);
	(static_cast<WMLDocument *>(frame->privateFrame->page->privatePage->mainFrame->privateFrame->frame->document())->handleRequestedGoEvent(OptionId));
	return 1 ;
}
#endif//SHP_SUPPORT_WML
void webkit_shp_page_GetHttpProxy( WebKitShpPage* mPage,SProxyInfo* aProxyInfo)
{
    BrowserSettings::GetHttpProxy(aProxyInfo);
}

void webkit_shp_page_GetIAP( WebKitShpPage* mPage,IAPHANDLE* aSelectedIapHandle)
{
    BrowserSettings::GetIAP(aSelectedIapHandle);
}

void webkit_shp_page_GetCacheSize( WebKitShpPage* mPage,guint* aCacheSize)
{
    BrowserSettings::GetCacheSize(aCacheSize);
}


void webkit_shp_page_SetMinimumFontSize( WebKitShpPage* mPage,const int aSettingsValue)
{
    ((struct _WebKitShpPage *)mPage)->privatePage->page->settings()->setMinimumFontSize( aSettingsValue);
}
void webkit_shp_page_SetMinimumLogicalFontSize( WebKitShpPage* mPage,const int aSettingsValue)
{
    ((struct _WebKitShpPage*)mPage)->privatePage->page->settings()->setMinimumLogicalFontSize(aSettingsValue);
}
void webkit_shp_page_SetDefaultFontSize( WebKitShpPage* mPage,const int aSettingsValue)
{
    ((struct _WebKitShpPage*)mPage)->privatePage->page->settings()->setDefaultFontSize(aSettingsValue);
}
void webkit_shp_page_SetDefaultFixedFontSize( WebKitShpPage* mPage,const int aSettingsValue)
{
    ((struct _WebKitShpPage*)mPage)->privatePage->page->settings()->setDefaultFixedFontSize(aSettingsValue);
}
void webkit_shp_page_SetDefaultTextEncodingName( WebKitShpPage* mPage,const gchar* aSettingsValue)
{
    ((struct _WebKitShpPage*)mPage)->privatePage->page->settings()->setDefaultTextEncodingName(aSettingsValue);
}

void webkit_shp_page_SetFtpDirectoryTemplatePath( WebKitShpPage* mPage,const gchar* aSettingsValue)
{
    ((struct _WebKitShpPage*)mPage)->privatePage->page->settings()->setFTPDirectoryTemplatePath(aSettingsValue);
}

void webkit_shp_page_SetUserStyleSheetLocation( WebKitShpPage* mPage,const gchar* aSettingsValue)
{
    ((struct _WebKitShpPage*)mPage)->privatePage->page->settings()->setUserStyleSheetLocation(aSettingsValue);
}

void webkit_shp_page_SetStandardFontFamily( WebKitShpPage* mPage,const gchar* aSettingsValue)
{
    ((struct _WebKitShpPage*)mPage)->privatePage->page->settings()->setStandardFontFamily(aSettingsValue);
}

void webkit_shp_page_SetFixedFontFamily( WebKitShpPage* mPage,const gchar* aSettingsValue)
{
    ((struct _WebKitShpPage*)mPage)->privatePage->page->settings()->setFixedFontFamily(aSettingsValue);
}

void webkit_shp_page_SetSerifFontFamily( WebKitShpPage* mPage,const gchar* aSettingsValue)
{
   ((struct _WebKitShpPage*)mPage)->privatePage->page->settings()->setSerifFontFamily(aSettingsValue);
}

void webkit_shp_page_SetSansSerifFontFamily( WebKitShpPage* mPage,const gchar* aSettingsValue)
{
    ((struct _WebKitShpPage*)mPage)->privatePage->page->settings()->setSansSerifFontFamily(aSettingsValue);

}

void webkit_shp_page_SetCursiveFontFamily( WebKitShpPage* mPage,const gchar* aSettingsValue)
{
    ((struct _WebKitShpPage*)mPage)->privatePage->page->settings()->setCursiveFontFamily(aSettingsValue);

}

void webkit_shp_page_SetFantasyFontFamily( WebKitShpPage* mPage,const gchar* aSettingsValue)
{
    ((struct _WebKitShpPage*)mPage)->privatePage->page->settings()->setFantasyFontFamily(aSettingsValue);

}

void webkit_shp_page_SetEditableLinkBehavior( WebKitShpPage* mPage,const WebCore::EditableLinkBehavior aSettingsValue)
{
    ((struct _WebKitShpPage*)mPage)->privatePage->page->settings()->setEditableLinkBehavior(aSettingsValue);

}

void webkit_shp_page_GetMinimumFontSize( WebKitShpPage* mPage,int *  aSettingsValue)
{
    *aSettingsValue = ((struct _WebKitShpPage*)mPage)->privatePage->page->settings()->minimumFontSize();

}

void webkit_shp_page_GetMinimumLogicalFontSize( WebKitShpPage* mPage,int *  aSettingsValue)
{
    *aSettingsValue = ((struct _WebKitShpPage*)mPage)->privatePage->page->settings()->minimumLogicalFontSize();

}

void webkit_shp_page_GetDefaultFontSize( WebKitShpPage* mPage,int *  aSettingsValue)
{
    *aSettingsValue = ((struct _WebKitShpPage*)mPage)->privatePage->page->settings()->defaultFontSize();

}

void webkit_shp_page_GetDefaultFixedFontSize( WebKitShpPage* mPage,int *  aSettingsValue)
{
    *aSettingsValue = ((struct _WebKitShpPage*)mPage)->privatePage->page->settings()->defaultFixedFontSize();

}

void webkit_shp_page_GetDefaultTextEncodingName( WebKitShpPage* mPage,gchar* aSettingsValue,gulong *aSize)
{
    const WebCore::String & tempString =((struct _WebKitShpPage*)mPage)->privatePage->page->settings()->defaultTextEncodingName();

    if (aSettingsValue)
    {
        if( *aSize > tempString.latin1().length() )
        {
            AcStrcpy(aSettingsValue,tempString.latin1().data());
        }
        else
        {
            AcStrncpy(aSettingsValue,tempString.latin1().data(),*aSize-1);
            *aSize = tempString.latin1().length() + 1;
        }
    }
    else
    {
        *aSize = tempString.latin1().length() + 1;
    }
}

void webkit_shp_page_GetFtpDirectoryTemplatePath( WebKitShpPage* mPage,gchar* aSettingsValue,gulong *aSize)
{
    const WebCore::String & tempString =((struct _WebKitShpPage*)mPage)->privatePage->page->settings()->ftpDirectoryTemplatePath();

    if (aSettingsValue)
    {
        if( *aSize > tempString.latin1().length() )
        {
            AcStrcpy(aSettingsValue,tempString.latin1().data());
        }
        else
        {
            AcStrncpy(aSettingsValue,tempString.latin1().data(),*aSize-1);
            *aSize = tempString.latin1().length() + 1;
        }
    }
    else
    {
        *aSize = tempString.latin1().length() + 1;
    }
}

void webkit_shp_page_GetUserStyleSheetLocation( WebKitShpPage* mPage,gchar* aSettingsValue,gulong *aSize)
{
    WebCore::KURL kUrl;
    kUrl = ((struct _WebKitShpPage*)mPage)->privatePage->page->settings()->userStyleSheetLocation();

    if (aSettingsValue)
    {
        if( *aSize > kUrl.string().length() )
        {
            CString aTempURL = kUrl.string().latin1();
			AcStrcpy(aSettingsValue, aTempURL.data());
        }
        else
        {
            CString aTempURL = kUrl.string().latin1();
            AcStrncpy(aSettingsValue,aTempURL.data(),*aSize-1);
            *aSize =kUrl.string().length() + 1;
        }
    }
    else
    {
        *aSize = kUrl.string().length() + 1;
    }
}

void webkit_shp_page_GetStandardFontFamily( WebKitShpPage* mPage,gchar* aSettingsValue,gulong *aSize)
{
    WebCore::AtomicString tempString;
    tempString = ((struct _WebKitShpPage*)mPage)->privatePage->page->settings()->standardFontFamily();

    if (aSettingsValue)
    {
        if( *aSize > tempString.domString().latin1().length() )
        {
            AcStrcpy(aSettingsValue,tempString.domString().latin1().data());
        }
        else
        {
            AcStrncpy(aSettingsValue,tempString.domString().latin1().data(),*aSize-1);
            *aSize = tempString.domString().latin1().length() + 1;
        }
    }
    else
    {
        *aSize = tempString.domString().latin1().length() + 1;
    }
}

void webkit_shp_page_GetFixedFontFamily( WebKitShpPage* mPage,gchar* aSettingsValue,gulong *aSize)
{
    WebCore::AtomicString tempString;
    tempString = ((struct _WebKitShpPage*)mPage)->privatePage->page->settings()->fixedFontFamily();

    if (aSettingsValue)
    {
        if( *aSize > tempString.domString().latin1().length() )
        {
            AcStrcpy(aSettingsValue,tempString.domString().latin1().data());
        }
        else
        {
            AcStrncpy(aSettingsValue,tempString.domString().latin1().data(),*aSize-1);
            *aSize = tempString.domString().latin1().length() + 1;
        }
    }
    else
    {
        *aSize = tempString.domString().latin1().length() + 1;
    }

}

void webkit_shp_page_GetSerifFontFamily( WebKitShpPage* mPage,gchar* aSettingsValue,gulong *aSize)
{
    WebCore::AtomicString tempString;
    tempString = ((struct _WebKitShpPage*)mPage)->privatePage->page->settings()->serifFontFamily();

    if (aSettingsValue)
    {
        if( *aSize > tempString.domString().latin1().length() )
        {
            AcStrcpy(aSettingsValue,tempString.domString().latin1().data());
        }
        else
        {
            AcStrncpy(aSettingsValue,tempString.domString().latin1().data(),*aSize-1);
            *aSize = tempString.domString().latin1().length() + 1;
        }
    }
    else
    {
        *aSize = tempString.domString().latin1().length() + 1;
    }
}
void webkit_shp_page_GetSansSerifFontFamily( WebKitShpPage* mPage,gchar* aSettingsValue,gulong *aSize)
{
    WebCore::AtomicString tempString;
    tempString = ((struct _WebKitShpPage*)mPage)->privatePage->page->settings()->sansSerifFontFamily();

    if (aSettingsValue)
    {
        if( *aSize > tempString.domString().latin1().length() )
        {
            AcStrcpy(aSettingsValue,tempString.domString().latin1().data());
        }
        else
        {
            AcStrncpy(aSettingsValue,tempString.domString().latin1().data(),*aSize-1);
            *aSize = tempString.domString().latin1().length() + 1;
        }
    }
    else
    {
        *aSize = tempString.domString().latin1().length() + 1;
    }
}

void webkit_shp_page_GetCursiveFontFamily( WebKitShpPage* mPage,gchar* aSettingsValue,gulong *aSize)
{
    WebCore::AtomicString tempString;
    tempString = ((struct _WebKitShpPage*)mPage)->privatePage->page->settings()->cursiveFontFamily();

    if (aSettingsValue)
    {
        if( *aSize > tempString.domString().latin1().length() )
        {
            AcStrcpy(aSettingsValue,tempString.domString().latin1().data());
        }
        else
        {
            AcStrncpy(aSettingsValue,tempString.domString().latin1().data(),*aSize-1);
            *aSize = tempString.domString().latin1().length() + 1;
        }
    }
    else
    {
        *aSize = tempString.domString().latin1().length() + 1;
    }

}

void webkit_shp_page_GetFantasyFontFamily( WebKitShpPage* mPage,gchar* aSettingsValue,gulong *aSize)
{
    WebCore::AtomicString tempString;
    tempString = ((struct _WebKitShpPage*)mPage)->privatePage->page->settings()->fantasyFontFamily();

    if (aSettingsValue)
    {
        if( *aSize > tempString.domString().latin1().length() )
        {
            AcStrcpy(aSettingsValue,tempString.domString().latin1().data());
        }
        else
        {
            AcStrncpy(aSettingsValue,tempString.domString().latin1().data(),*aSize-1);
            *aSize = tempString.domString().latin1().length() + 1;
        }
    }
    else
    {
        *aSize = tempString.domString().latin1().length() + 1;
    }

}

void webkit_shp_page_GetEditableLinkBehavior( WebKitShpPage* mPage,WebCore::EditableLinkBehavior* aSettingsValue)
{
    *aSettingsValue = ((struct _WebKitShpPage*)mPage)->privatePage->page->settings()->editableLinkBehavior();
}
void webkit_shp_page_GetSelectedElementType( WebKitShpPage* mPage,EPageItemType* aType)
{
    *aType = ENone;
#ifdef SUPPORT_VIRTUAL_CURSOR
    WebCore::Node * curNode = WebCore::CVirtualCursor::getMouseHoveredNode();
    if(curNode)
    {
        if(equalIgnoringCase(curNode->nodeName(), "IMG"))
        {
            *aType = EImages;
        }
        WebCore::Node* parNode = curNode->parentNode();
        if(parNode)
        {
            if(equalIgnoringCase(parNode->nodeName(), "A"))
            {
                *aType = (EPageItemType)(*aType | EAnchor);
            }
        }
    }
#else
    ASSERT(!"Need to support virtual cursor for webkit_shp_page_GetSelectedElementType to work");
#endif
}

void webkit_shp_page_GetSelectedElementInfo( WebKitShpPage* mPage, EPageItemType aType, ElementInfoBase* aInfo)
{
#ifdef SUPPORT_VIRTUAL_CURSOR
    WebCore::Node * curNode = WebCore::CVirtualCursor::getMouseHoveredNode();

    if(curNode)
    {
        switch (aType)
        {
        case EAnchor:
            {
                WebCore::Node * parNode = curNode->parentNode();
                WebCore::String curNodeName = curNode->nodeName();

                if(parNode)
                {
                    if(equalIgnoringCase(parNode->nodeName(), "A"))
                    {
                        WebCore::HTMLAnchorElement *anchorElement = reinterpret_cast<WebCore::HTMLAnchorElement *>(parNode);
                        WebCore::String anchorUrl = anchorElement->href();
                        AnchorInfo * tempAnchorInfo = (AnchorInfo *) aInfo;
                        tempAnchorInfo->element_info_base.element_type = EAnchor;
                        tempAnchorInfo->access_key = 0;
                        if(anchorUrl.latin1().length() < tempAnchorInfo->length)
                        {
                            AcStrcpy(tempAnchorInfo->url,anchorUrl.latin1().data());
                        }
                        else
                        {
                            AcStrncpy(tempAnchorInfo->url ,anchorUrl.latin1().data(), tempAnchorInfo->length - 1);
                            tempAnchorInfo->length = anchorUrl.latin1().length();
                        }
                    }
                }
            }
            break;
        case EImages:
            if( equalIgnoringCase(curNode->nodeName(), "IMG") )
            {
                RenderImage *renderImage;
                renderImage = static_cast<RenderImage *>(curNode->renderer());
                if(renderImage)
                {
                    CachedImage* cachedImage;
                    cachedImage = renderImage->cachedImage();
                    if(cachedImage)
                    {
                        ImageInfo* tempImageInfo = (ImageInfo*) aInfo;
                        tempImageInfo->element_info_base.element_type = EImages;
                        Image* image = cachedImage->image();
/**
 *   bs0425.lee  modified for touch events   - to save image data size. 
 */
                        int ImageSize = image->data()->size();
						tempImageInfo->size=ImageSize;

                   //     int size = image->data()->size();
                        WebCore::HTMLImageElement *imageElement = reinterpret_cast<WebCore::HTMLImageElement *>(curNode);
                        String url;
                        String extension;
                        url = imageElement->src();
                        if(url.latin1().length() < tempImageInfo->url_length)
                        {
                            AcStrcpy(tempImageInfo->url, url.latin1().data());
                        }
                        else
                        {
                            AcStrncpy(tempImageInfo->url ,url.latin1().data(), tempImageInfo->url_length - 1);
                            tempImageInfo->url_length = url.latin1().length();
                        }
                        int dotIndex = url.reverseFind('.');
                        if( -1 != dotIndex )
                        {
                            extension = url.substring(dotIndex + 1);
                            if(extension.latin1().length() < tempImageInfo->extension_length)
                            {
                                AcStrcpy(tempImageInfo->extension, extension.latin1().data());
                            }
                            else
                            {
                                AcStrncpy(tempImageInfo->extension ,extension.latin1().data(), tempImageInfo->extension_length - 1);
                                tempImageInfo->extension_length = extension.latin1().length();
                            }
                        }
                    }
                }
            }
            break;
        default:
            break;
        }
    }
#else
    ASSERT(!"Need to support virtual cursor for webkit_shp_page_GetSelectedElementInfo to work");
#endif //#ifdef SUPPORT_VIRTUAL_CURSOR
}


int
webkit_shp_get_image_count(WebKitShpPage* page, bool bVisibleOnly)
{
	WebKitShpFramePrivate* frameData = page->privatePage->mainFrame->privateFrame;
	PassRefPtr<HTMLCollection> imageCollection = frameData->frame->getWebKitBridge()->GetWebCoreBridge()->GetFrame()->document()->images();
	FrameView* view = page->privatePage->mainFrame->privateFrame->frame->view();

	int contentsx = view->contentsX();				// view port x position
	int contentsy = view->contentsY();				// view port y position
	int visibleW = view->visibleWidth();			// width of the display
	int visibleH = view->visibleHeight();			// height of the display

	IntRect viewRect(contentsx, contentsy, visibleW, visibleH);
	IntRect contentRect = IntRect(view->visibleContentRect());

	CachedImage* cachedImage;
    RenderImage *renderImage;

	String orgPath;

    WebCore::Node * tempNode;
	int count = 0;
    for (int i=0; i< imageCollection->length() ;i++)
    {
        tempNode = imageCollection->item(i);
        if(!tempNode)
            continue;
        renderImage = static_cast<RenderImage *>(tempNode->renderer());
        if(!renderImage)
            continue;
        cachedImage = renderImage->cachedImage();
        if(!cachedImage)
            continue;

		int xPos, yPos;
		renderImage->absolutePosition(xPos, yPos, FALSE);

		Image* image = cachedImage->image();
		if(image->data())
		{
			IntRect imgRect = IntRect(  xPos, yPos, renderImage->width(),renderImage->height() );

			if(bVisibleOnly)
			{
				if(imgRect.intersects(viewRect))
				{
					count++;
				}
			}
			else
				count++;
		}
	}
	return count;
}


void
webkit_shp_get_images(WebKitShpPage* page, bool bVisibleOnly, ImageSaveInfo*** imageList)
{
	WebKitShpFramePrivate* frameData = page->privatePage->mainFrame->privateFrame;
	PassRefPtr<HTMLCollection> imageCollection = frameData->frame->getWebKitBridge()->GetWebCoreBridge()->GetFrame()->document()->images();
	FrameView* view = page->privatePage->mainFrame->privateFrame->frame->view();
	int contentsx = view->contentsX();				// view port x position
	int contentsy = view->contentsY();				// view port y position
	int visibleW = view->visibleWidth();			// width of the display
	int visibleH = view->visibleHeight();			// height of the display

	IntRect viewRect(contentsx, contentsy, visibleW, visibleH);

	CachedImage* cachedImage;
    RenderImage *renderImage;

	int count = webkit_shp_get_image_count(page, bVisibleOnly);
	if(count <= 0)
		return;
	// Allocate space for counted images
	// SFBalMalloc
	*(imageList) = (ImageSaveInfo**) SFBalMemAlloc(count * sizeof(ImageSaveInfo));

	String orgPath;
    WebCore::Node * tempNode;	
	int i=0, j=0;
    for (; i< imageCollection->length() ;i++)
    {
        tempNode = imageCollection->item(i);
        if(!tempNode)
            continue;
        renderImage = static_cast<RenderImage *>(tempNode->renderer());
        if(!renderImage)
            continue;
        cachedImage = renderImage->cachedImage();
        if(!cachedImage)
            continue;
        orgPath = tempNode->attributes()->getAttributeItem("src")->value();
        Image* image = cachedImage->image();

		int xPos, yPos;
		renderImage->absolutePosition(xPos, yPos, FALSE);

		IntRect imgRect = IntRect(  xPos, yPos, renderImage->width(),renderImage->height() );

		ImageSaveInfo* imageInfo;

		if (bVisibleOnly)
		{
			if(imgRect.intersects(viewRect))
			{
				(*(imageList))[j] = (ImageSaveInfo*) SFBalMemAlloc(sizeof(ImageSaveInfo));
				imageInfo = (*(imageList))[j];
				j++;
			}
			else 
				continue;
		}
		else
		{
			(*(imageList))[j] = (ImageSaveInfo*) SFBalMemAlloc(sizeof(ImageSaveInfo));
			imageInfo = (*(imageList))[j];
			j++;
		}
		if(image->data())
		{
			imageInfo->data = image->data()->data();
			imageInfo->size = image->data()->size();
			
			WebCore::HTMLImageElement *imageElement = (WebCore::HTMLImageElement *)(tempNode);

			String url = imageElement->src();
			imageInfo->url  =  (char*) SFBalMemAlloc(sizeof(char) * (AcStrlen(url.latin1().data()) + 1));
			AcStrcpy(imageInfo->url, url.latin1().data());

			String name;
			int dotIndex = url.reverseFind('/');
			if( -1 != dotIndex )
			{
				name = url.substring(dotIndex + 1);
			}
			else
			{
				dotIndex = url.reverseFind('\\');
				if( -1 != dotIndex )
				{
					name = url.substring(dotIndex + 1);
				}
			}
			if( -1 == dotIndex )
			{
				name = url;
			}

			if(name.latin1().length() > 0)
			{
				imageInfo->name = (char*) SFBalMemAlloc(sizeof(char) * (100));
				AcStrcpy(imageInfo->name, name.latin1().data());
			}
		}
	}
}

/**
 *   bs0425.lee  modified for touch events 
 */ 

//  This function saves the pointed image. 

void
webkit_shp_element_saveByPoint(WebKitShpPage* page, const gchar* aPath, int pointX, int pointY)
{

     Frame* frame = core(getFrameFromPage(page));
	
     FrameView *frameview = frame->view();
     IntPoint ptContent = frameview->windowToContents(IntPoint(pointX, pointY));
     Document* NewDocument=frame->documentAtPoint(ptContent);
	
    Element* element =NewDocument->elementFromPoint(ptContent.x(), ptContent.y());
    WebCore::Node * curNode =element;
    if( curNode && equalIgnoringCase(curNode->nodeName(), "IMG") )
    {
        RenderImage *renderImage;
        renderImage = static_cast<RenderImage *>(curNode->renderer());
        if(renderImage)
        {
            CachedImage* cachedImage;
            cachedImage = renderImage->cachedImage();
            if(cachedImage)
            {
                String orgPath;
                String newPath;
                String fileName;
                newPath = aPath;
                orgPath = curNode->attributes()->getAttributeItem("src")->value();
                int slashIndex = orgPath.reverseFind('/');
                if( -1 != slashIndex )
                    fileName = orgPath.substring(slashIndex + 1);
                else
                    fileName = orgPath;
					/* newPath += fileName; Commented by vijay to get the correct image Store Name*/
                Image* image = cachedImage->image();
                HFile hFile = FmCreateFile (newPath.latin1().data(), FM_WRITE);
                if (hFile != NULL)
                {
                    FmWriteFile(hFile, (void*)(image->data()->data()), image->data()->size());
                    FmCloseFile(hFile);
                }
           
		}

       }
    }
}

	
/**
 *   bs0425.lee  modified for touch events 
 */ 




void
webkit_shp_element_save(WebKitShpPage* page, const gchar* aPath)
{
#ifdef SUPPORT_VIRTUAL_CURSOR
    WebCore::Node * curNode = WebCore::CVirtualCursor::getMouseHoveredNode();
    if( curNode && equalIgnoringCase(curNode->nodeName(), "IMG") )
    {
        RenderImage *renderImage;
        renderImage = static_cast<RenderImage *>(curNode->renderer());
        if(renderImage)
        {
            CachedImage* cachedImage;
            cachedImage = renderImage->cachedImage();
            if(cachedImage)
            {
                String orgPath;
                String newPath;
                String fileName;
                newPath = aPath;
                orgPath = curNode->attributes()->getAttributeItem("src")->value();
                int slashIndex = orgPath.reverseFind('/');
                if( -1 != slashIndex )
                    fileName = orgPath.substring(slashIndex + 1);
                else
                    fileName = orgPath;
					/* newPath += fileName; Commented by vijay to get the correct image Store Name*/
                Image* image = cachedImage->image();
                HFile hFile = FmCreateFile (newPath.latin1().data(), FM_WRITE);
                if (hFile != NULL)
                {
                    FmWriteFile(hFile, (void*)(image->data()->data()), image->data()->size());
                    FmCloseFile(hFile);
                }
            }
        }
    }
#else
    ASSERT(!"Need to support virtual cursor for webkit_shp_element_save to work");
#endif //#ifdef SUPPORT_VIRTUAL_CURSOR
}

void
webkit_shp_page_GetItemCount(WebKitShpPage* page, const EPageItemType aType, guint* aCount)
{
    if(EImages == aType)
    {
        WebKitShpFramePrivate* frameData = page->privatePage->mainFrame->privateFrame;
        *aCount = frameData->frame->getWebKitBridge()->GetWebCoreBridge()->GetFrame()->document()->images()->length();
    }
}

void
webkit_shp_page_GetLastModifiedTime(WebKitShpPage* page, gchar* aTime, gulong* aSize)
{
    WebKitShpPagePrivate* pageData = page->privatePage;
    WebKitShpFramePrivate* frameData = pageData->mainFrame->privateFrame;

    String time = frameData->frame->lastModifiedTime();

    if(*aSize <= time.length())
    {
        AcStrncpy(aTime, time.latin1().data(), *aSize - 1);
        *aSize = time.length() + 1;
    }
    else
    {
        AcStrcpy(aTime, time.latin1().data());
    }
}

#if defined SHP_SUPPORT_WML
void webkit_page_send_new_fetch_request (WebKitShpPage* page)
{
	WebKitShpPagePrivate* pageData = page->privatePage;
	WebKitShpFramePrivate* frameData = pageData->mainFrame->privateFrame;
	frameData->frame->loader()->loadRequestedUrl () ;
}
#endif

#ifdef _SFB_SEARCH_STRING_IN_PAGE //Jan-16 Vamsi/Pramod
bool
webkit_shp_page_find(WebKitShpPage* page, const SFindInfo* pFindInfo)
{
	bool bstartInSelection = FALSE, bWrapFlag = TRUE;
	WebKitShpPagePrivate* pageData = page->privatePage;
    WebKitShpFramePrivate* frameData = pageData->mainFrame->privateFrame;

	bool bRet ;

    Frame* frame = frameData->frame;

	#ifdef SUPPORT_VIRTUAL_CURSOR
	//This is a temporary implementation as virtual cursor was giving some problem
	if(CVirtualCursor::isEnabled())
		webkit_shp_page_virtual_cursor_hide();
	#endif

	bRet = frameData->frame->findString(pFindInfo->text, pFindInfo->search_forward, pFindInfo->match_case, bWrapFlag, bstartInSelection);
	
	//initialise the vitual cursor and set frame properties
	#ifdef SUPPORT_VIRTUAL_CURSOR
	webkit_shp_page_virtual_cursor_init(page);
	CVirtualCursor::setProperties(pageData->pageVirtualCursor, frame->view(), pageData->hGc);
    CVirtualCursor::setFrameView(frame->view(), true);
	#endif

	return bRet;
}
#endif

}

//vamsi relayout
void webkit_shp_page_relayout(WebKitShpPage* page)
{
    Frame* frame = core(getFrameFromPage(page));
    WebKitShpPagePrivate* pageData = page->privatePage;
	WmRect wm_Rect;

	#ifdef SUPPORT_VIRTUAL_CURSOR
	if(CVirtualCursor::isEnabled())
		webkit_shp_page_virtual_cursor_hide();
	#endif

	WmGetWindowClientRect( pageData->hWin , &wm_Rect );
	/* resizing of the frame to screen size */
	frame->view()->resize(wm_Rect.w , wm_Rect.h );
	frame->forceLayout();
	frame->view()->adjustViewSize();
	frame->sendResizeEvent();

    if(INVALID_HOBJ != pageData->hGc)
    {
        WmDeleteGC(pageData->hGc);
        pageData->hGc = INVALID_HOBJ;
    }

    pageData->hGc = WmCreateClientGC(pageData->hWin);

		//initialise the vitual cursor and set frame properties
	#ifdef SUPPORT_VIRTUAL_CURSOR
	webkit_shp_page_virtual_cursor_init(page);
	CVirtualCursor::setProperties(pageData->pageVirtualCursor, frame->view(), pageData->hGc);
    CVirtualCursor::setFrameView(frame->view(), true);
	#endif
}