/*
 * Samsung Mobile Browser
 * Copyright (c) 2007 SISO
 * All rights reserved.
 *
 * This software is the confidential and proprietary information
 * of Samsung Electronics, Inc. ("Confidential Information").  You
 * shall not disclose such Confidential Information and shall use
 * it only in accordance with the terms of the license agreement
 * you entered into with Samsung Electronics.
 */

/** 
 * This file contains HTTP RFC2617 implementation
 *
 * @author          Vamsi  vamsi.y@samsung.com
 */

/* Version History
Author		Date		BugID		BugDescription

*/

#ifndef _SFB_AUTH_H_
#define _SFB_AUTH_H_

#ifdef __cplusplus
extern "C" {
#endif // __cplusplus

#define SFB_AUTH_ERROR			 0
#define SFB_AUTH_USER_PROMPT	 1
#define SFB_AUTH_USER_INPUT_ERR	 "Authentication Failed. Retry?"
#define SFB_AUTH_BASIC			 "Login information is being sent as plain text, Do you want to continue?"	
	/*FUNCTION DECLARATIONS*/

	/*FUNCTION DECLARATIONS*/
/**
  * This function is called when 200 OK response comes.
  * The SFBAuthData is added to the HASHTABLE for further use.
  * Before adding SFBAuthData to the HASHTABLE for DIGEST case,the Authentication-Info
  * header should be parsed and nextnonce is updated. Called from RM
  * @param  : SFBRmEntity
  * @return : void
  */
void SFBAuthProcessOkStatus( SFBRmEntity *pEntity );

/**
  * This function is processed for 401 and 407 response status codes
  * In this function, the SFBAuthData is checked in the HASHTABLE
  * and if present, request is made using the data.
  * if not present, user is asked to fill Username and Password
  * and then a request is made. Called from RM
  * @param  : SFBRmEntity
  * @return : int	---- SFB_AUTH_ERROR			if Error
  *					---- SFB_AUTH_USER_PROMPT	if User is prompted for username and password
  */
int SFBProcessUnAuthHeader(SFBRmEntity* pEntity);

/**
  * This function is used to initialise the AuthContext and 
  * also to read the data from the filesystem and store it in 
  * the HASHTABLE. Called from Control or RM
  * @param  : NONE
  * @return : void
  */
void SFBAuthInit();

/**
  * This function is used to generate authentication credentials
  * if the data is already present in the Storage without going to
  * 401 response status code. Called from RM
  * @param  : SFBRmEntity
  * @return : char*		A String is returned
  *			  				
  */
char* SFBGenerateAuthCredentials ( SFBRmEntity* pEntity);

/*
 * This function receives authentication information from UI
 * Should be called from UI
 * @param	rId [in]  Reqest entity  id  stores inforamtion .
 * @param	szUser [in]  User Id information from UI.
 * @param	szPass [in] Password information from UI.
 * @return	BOOL	Return
 *			 
 * @exception	None
 */
void 
SFBGetPasswdCallback ( SFBRmHClient hClient, SFBRmReqId reqID, BOOL bCancel, char* szUser, char* szPass);

/**
  * This function is to write the data into the filesystem
  * from the HASHTABLE and to deinitialise all the components that were initiliased in 
  * SFBAuthInit. Can be called from RM or Control.
  * @param  : NONE
  * @return : NONE
  */
void SFBAuthExit();

/**
  * This function is used to clear SFBAuthData if user wants to clear
  * the stored entries in both FS and HashTable. Can be called in SIM change scenario
  * as per some operators requirement. Can be called from Control or UI
  * @param  : NONE
  * @return : NONE
  */
void SFBClearAuth();


#ifdef __cplusplus
}
#endif // __cplusplus

#endif //_SFB_AUTH_H_