/*
 * Samsung Mobile Browser
 * Copyright (c) 2007 SISO
 * All rights reserved.
 *
 * This software is the confidential and proprietary information
 * of Samsung Electronics, Inc. ("Confidential Information").  You
 * shall not disclose such Confidential Information and shall use
 * it only in accordance with the terms of the license agreement
 * you entered into with Samsung Electronics.
 */
             
/** 
 * This file contains HTTP RFC2616 cache implementation
 *
 * @author          Gnanasekar S  gnanasekar.s@samsung.com
 */

/* Version History
Author		Date		BugID		BugDescription

*/
 
#ifndef _SFB_CACHE_H_
#define _SFB_CACHE_H_

#ifdef __cplusplus
extern "C" {
#endif // __cplusplus

/*INCLUDES*/
#include "SFBRmCoreTypes.h" // For RmEntity, and Cache headers in Cache file

#define SFB_L1_L2_CACHE

#ifndef SFB_L1_L2_CACHE
//#define SFB_FILE_HANDLE
#endif
                    
                          
/*MACROS*/ 
#define SFB_CACHE_DIRECTORY_PATH					"/User/Br/SFB/Cache"/* Cache directory path */
#define SFB_NORMAL_CACHE_FILE_NAME					"Cache.dat"	/* normal cache file path */
#define SFB_CACHE_DELIMITER_STRING					"\r\n"

/*---------FOR internal Cache operation-----------*/
#define SFB_OK										32 /* Server response 200 OK */
#define SFB_NOT_MODIFIED							52 /* server response 304 not modified */
#define	SFB_YEAR_BASE_1900_ADJUST					2208988800UL	/* Adjustment value for time in 1900 Base*/
#define SFB_SECONDS_FOR_ONE_DAY						86400 /* seconds for a day*/
/*------------------------------------------------*/

                            
/*DATA TYPES*/

/**
* Enum SFBCacheRequestStatus
* Status of cache request
*/
typedef enum _SFBCacheRequestStatus
{
	SFB_CACHE_NO_HANDLE=0,
	SFB_CACHE_NOT_FOUND,		/* When cache is not found*/				
	SFB_CACHE_FOUND_VALID,		/* when cache is found and valid*/				
	SFB_CACHE_FOUND_REVALID,	/* when cache is found, but not valid*/					
	SFB_CACHE_FOUND_UPDATE		/* when cache is found, but need to refresh or update*/				
} SFBCacheRequestStatus;


/**
* Enum _SFBCacheStoragePolicy
* Status of cache storage Policy
*/
typedef enum _SFBCacheStoragePolicy
{
	SFB_CACHE_UNKNOWN=0,		/* Cache Default storage policy */				
	SFB_CACHE_MEMORY = 1,		/* Cache is stored in memory*/				
	SFB_CACHE_PERSISTENT = 2,	/* Cache is stored in persistent */					
	SFB_CACHE_BOTH = SFB_CACHE_MEMORY |	SFB_CACHE_PERSISTENT/* Cache is stored in both memory and persistent */				
} SFBCacheStoragePolicy;


/**
* Enum SFBCacheEntryState
* State of the cache
*/
typedef enum _SFBCacheEntryState
{
	SFB_CACHE_UNINITIALISED,		/* Cache is uninitialised */
    SFB_CACHE_INITIALISED,			/* Cache is initialised */
    SFB_CACHE_REQUESTING,			/* Rm requesting cache*/
    SFB_CACHE_RESPONDING,			/* when response received from Rm to add cache */
    SFB_CACHE_COMPLETE,				/* when cache operation is complete*/
    SFB_CACHE_DESTROYED				/* when cache is destroyed or corrupted*/	
} SFBCacheEntryState;


/**
* enum SFBCacheType
* type of cache
*/
typedef enum _SFBCacheType
{
	SFB_CACHETYPE_NIL,
	SFB_CACHETYPE_CACHE,
	SFB_CACHETYPE_HISTORY
} SFBCacheType;


/**
* Struct SFBCacheMultipart
* State of the multipart cache
*/
typedef struct _SFBCacheMultipart
{
      char*	szUrl;				/* Multipart cache Url*/
      char* pResponseheader;	/* multipart cache header*/
      int	iResponseHeaderLen;	/* length of the header*/
      char* pResponseBody;		/* multipart body content*/
	  int	iResponseBodyLen;	/* length of body content*/
}SFBCacheMultipart;



/**
* Struct SFBCacheWriteBuffer;
* State of the multipart cache
*/
typedef struct
{
	int			iCurBufferLen;	// Curr Buffer
	char*		pBuffer ;	// File Reading and writing 

} SFBCacheReadWriteBuffer;



/**
* This struct defines one instance of cache entry
*/
typedef struct _SFBCacheEntry
{
	char*	szUrl;					/* URL attribute */
	char*	szFileName;    			/* Filename attribute */
	int		iRef;					/* reference count attribute */
	
	UINT32 	iExpires;	           	/* Expiry time based on iExpires or maxage */

	UINT32	iLast_accessed; 		/* To record the latest access time */
	int 	iState;					/* Cache State */
	
	char*	szLastModified;          /* Last modified date */
	char*	szEtag;					/* ETag attribute */
	
	int		iHeaderSize;			/* Stores headersize for the entry*/ 	
	int		iFileRead;				/* Stores position upto which the file is read for the entry*/

#ifdef SFB_L1_L2_CACHE
	int		iTotalEntrySize;				/* stores total cache size for the entry */
	BOOL    bEntryStorageState;     /* shows the status on where it is present in memory or filesystem cache. TRUE- in memory,FALSE- in file system */
#endif

	BOOL	bExpiresStatus;			/* Give status of expire value came from headers or a default value*/
	BOOL	bCacheControlStatus;	/* Give status if cache control is present*/
	BOOL	bMaxAgeStatus;			/* gives status of maxage in http header or  meta tags*/

	UINT32  iDate;					/* this stores date when server generated the response*/

	BOOL    bMultiPartStatus;		/* this differentiates whether the content is multipart or normal cache*/

#ifdef __SFB_LRU_SP
	BOOL	bEvictionVictim;		/* this is used to check whether the victim is proper or coprrupted*/
#endif

#ifdef SFB_FILE_HANDLE
	HFILE   hFileHandle;			/* File handle is saved here to reduce number of file open and close operations*/	
#endif

#ifdef SFB_L1_L2_CACHE
    SFBCacheReadWriteBuffer pRWBuffer; /* saves in a buffer before til entire data is received. also acts as memory cache*/
#endif

	char*	szIndexBuffer;

} SFBCacheEntry;



/*This structure defines the handler for the Cache*/
typedef struct _SFBCacheHandler
{
	void*   iLookUpTable;			/* LookUpTable for HashStable memorycpy*/
	void*   iEvictionVector;         /* Queue for cache Eviction */
	char*   szIndexFile;			/* Index File */ 
	int     iCurNumCache;			/* Current Number of Caches in LookupTable*/
	int		iCurFileNameIndex;		/* stores current file name index */

#ifndef SFB_L1_L2_CACHE
        int      iNormalCacheSize;               /*  Normal Cache size*/
#else
	int		iCurMemCacheSize;		/* stores current memory cache size for the handle */
	int		iCurDiskCacheSize;		/* stores current file system cache size */	

#endif
} SFBCacheHandler;



/*This structure gets the configuration settings for the cache*/
typedef struct _SFBCacheConfig
{
	int     iMaxTotalCaches;		/* Maximum number of cache */
	int     iMaxTotalSizeCaches;	/* Maximum size for caches */	
	int     iMaxSizePerCache;		/* maximum size per cache */
	int     iDefaultChunkSize;      /* chunksize for data transfer b/w RM and Cache */
	int		iMode;					/* 0- cache, 1- history, 2- both */
#ifdef SFB_L1_L2_CACHE
//	SFBCacheStoragePolicy iStorageState; /* Provides the storge policy for cache */
#endif				
} SFBCacheConfig;



/*This structure defines the settings for handling cache*/
typedef struct _SFBCacheManager
{
	SFBCacheHandler*  pNormalCache;  /* handler for Normal Cache*/ 
	SFBCacheHandler*  pNormalHistory; /* Handler for Histroy */
    BOOL    bCacheEnabled;           /* cache enable for both normal cache and widget cache*/
	BOOL	bHistoryEnabled;		 /* history Enable */
   	int		iRef;				     /* Number of instances of CacheManager*/
	SFBCacheConfig	tSFBCacheConfig; /* Cache Configurations*/
	char*	  pCurCacheDir;		/* Directory for stroing caches of each application.*/


} SFBCacheManager;

//typedef SFBCacheManager* SFBHCache;

/*This structure defines the handler and the entry point for the Cache Request*/

typedef struct _SFBTCacheEntry
{
 	SFBCacheHandler*  pCacheHandler; /* Handler for the RmEntity Request */
    SFBCacheEntry*    pCacheEntry;   /* Entry for the RmEntity Request */
} SFBTCacheEntry;
	


// FUNCTION PROTOTYPES
//Init-Exit APIs

/** Used to initialize the cache manager. 
 *  This Function Initializes the global data like max size of caches,
 *  total size of cache and Maximum number of caches.
 *  Reads Cache data from FileSystem to LookUp Table.
 *  Should be called when initializing browser object. 
 *  Acts as Initilization API. Can be called from multiple objects. 
 *  Increases the reference count when Called if its already initialized.
 *  @param tSFBCacheConfig- tSFBCacheConfig contain config information.
 *  @return None.
 */
SFBHCache 
SFBCreateCache (char* pCurCacheDir, SFBCacheConfig* pTSFBCacheConfig);

//INIT API
/** Used to initialize the cache manager. 
 *  Reads Cache data from FileSystem to LookUp Table.
 *  Param SFBHCache
 *  @return None.
 */
void 
SFBInitCache (SFBHCache pCurCache);

/* Acts as Destructor API. Can be called from multiple objects.
 * Decreases the reference count when Called. When count reaches zero,
 * writes Caches in memory to File System. 
 * Frees the memory used for Lookup Table.
 * @param none
 * @return None.
 */
void  
SFBExitCache (SFBHCache hCacheHandle);

//Enable-Disable APIs
/* This function gets cache enable status.
 * @param none
 * @return TRUE, if cache is enable, FALSE otherwise. 
 */
BOOL 
SFBGetCacheEnable();

//Enable-Disable APIs
/* This function sets cache enable status.
 * @param bEnable -input to enable or Disable.
 * @return none.
 */
void 
SFBSetCacheEnable(BOOL bEnable);

//Clear-Clear ALL APIs
/* This Function clear all the Caches in LookUp Table 
 * and also persistent Storage.
 * @param none
 * @return None.
 */
void  
SFBClearAllCache (SFBHCache hCacheHandle);

/* This Function clears the passed URL from Cache in LookUp Table 
 * and also persistent Storage.
 * @param szUrl- szUrl contains Url information
 * @return None.
 */
void  
SFBClearCache(char* szUrl);

//Cache Request API's
/* This function will take SFBRmEntity and Cachemode.
 * It checks the transaction URL in the cache LookUp Table. 
 * When found in the Cache LookUp Table, 
 * it checks for complete URL and Validates the content. 
 * And returns status as found after appending header to rmentity. 
 * If found incomplete or corrupt in cache lookup table, returns status as not found.  
 * If Found Expired, Returns Found Revalidate.
 * @param pRmEntity- Structure variable of type SFBRmEntity.
 * @Param tCacheMode- Gives the mode in which cache operates.
 * @return CACHE_FOUND_VALID	Cache entry is found, and its Valid.
 *		   CACHE_FOUND_REVALID	Cache entry found, but expired.
 *         CACHE_NOT_FOUND	    Cache entry not found, found but incomplete or corrupt. 
 */
SFBResult 
SFBFindCacheRequest(SFBRmEntity* pRmEntity, SFBCacheMode tCacheMode);

/* This function will take SFBRmEntity and alastchunk status.
 * It gets next chunk of data added to SFBRMEntity. 
 * When end of the content is reached, alastchunk is made TRUE.
 * @param pRmEntity- Structure variable of type SFBRmEntity.
 * @param bLastChunk- Gives the status of last chunk.
 * @Return  1 -if  next chunk is passed sucessfully else returns 0.         
 */
SFBResult
SFBCacheRequestNextChunk(SFBRmEntity *pRmEntity, BOOL *bLastChunk);

/* This function will take SFBRmEntity.
 * It normally closes on a request - when the content is loaded from the cache.
 * When transaction is closed without being completed, 
 * it removes uncompleted/destroyed entry.
 * And this function also checks if the stream is released.
 * @param pRmEntity- Structure variable of type SFBRmEntity.
 * @Return none
 */
void 
SFBCacheRequestClosed (SFBRmEntity* pRmEntity);

//Cache Response API's

/* This function will take SFBRmEntity. 
 * It checks if the item is cacheable and 
 * item is not bigger than the 1/3 of the cache size. 
 * And then checks if the entry is already in the cache.
 * Else creates a new cache item in the lookup table 
 * and saves the header content based on the status code (200/304).
 * @param pRmEntity- Structure variable of type SFBRmEntity.
 * @Return none
 */
void 
SFBCacheReceivedResponseHeaders (SFBRmEntity* pRmEntity);

/* This function takes SFBRmEntity. 
 * It takes the ResponseBodybuffer content from the pRmEntity
 * and appends it to the cache file.
 * @param pRmEntity- Structure variable of type SFBRmEntity.
 * @param aLastChunk- Gives the status of last chunk.
 * @Return none
 */
void 
SFBCacheReceivedResponseBodyData (SFBRmEntity* pRmEntity, BOOL bLastchunk);

/* This function will take SFBRmEntity. 
 * It checks if we have cached this resource and marks the entry as complete.
 * @param pRmEntity- Structure variable of type SFBRmEntity.
 * @Return none
 */
void 
SFBCacheResponseComplete (SFBRmEntity* pRmEntity); 

//Meta Data Cache
/* This function will take Url and Header.
 * It updates the cache based on the webcore- metadata.
 * @param szUrl- String variable ends with null character contain URL information.
 * @param szHeader-Character pointer contains the Cache header information to be parsed.
 *                 szHeader format should be "Filedname1: FieldValue1;Filedname1: FieldValue1;"  
 * @Return none
 */
//void 
//SFBUpdateCacheMetaData ( SFBHCache pCurCache,char* szUrl,  char* szMetaField , char* szMetaFieldContent);

//Cache 304 Not-Modified Request API
/* This function will take SFBRmEntity. 
 * It handles 304 not modified request.
 * @param pRmEntity- Structure variable of type SFBRmEntity.
 * @Return none
 */
SFBResult 
SFBCacheProcessNotModified(SFBRmEntity* pRmEntity);

//Multipart Cache 
/* This function will add Multipart content to cache. 
 * @param pMultipartCache- Structure variable of type SFBRmEntity.
 * @Return none
 */
void 
SFBCacheAddMultipartCache(SFBHCache hCacheHandle,SFBCacheMultipart *pMultipartCache);
/**
*This function create cachehandler.
* @param		aUrl and headerString
* @return		SFBCacheHandler
*/
SFBCacheHandler* 
SFBGetCacheHandler( char* szUrl,  char* szHttpHeaderString, SFBCacheMode aCacheMode );

/**
* This Function stops current cache operation 
* @param		reqId
* @return		none.
*/
void
SFBCacheStopOperation(SFBHCache hCacheHandle,SFBRmReqId reqId);

void SFBCacheCancelResponse(SFBRmEntity* pRmEntity);

/**
* This Function checks whether the rmentity is cacheableeration 
* @param		pRmEntity
* @return		none.
*/
BOOL SFBCacheIsCacheable(SFBRmEntity* pRmEntity);

//Global Cache context
extern SFBCacheManager *gpSFBCacheManager;

#ifdef __cplusplus
}
#endif // __cplusplus

//Meta Data Cache
/* This function will take Url and Header.
 * It updates the cache based on the webcore- metadata.
 * @param szUrl- String variable ends with null character contain URL information.
 * @param szHeader-Character pointer contains the Cache header information to be parsed.
 *                 szHeader format should be "Filedname1: FieldValue1;Filedname1: FieldValue1;"  
 * @Return none
 */
void 
SFBUpdateCacheMetaData ( SFBHCache pCurCache,char* szUrl,  char* szMetaField , char* szMetaFieldContent);

#endif //_SFB_CACHE_H_