/*
 * Samsung Mobile Browser
 * Copyright (c) 2007 SISO
 * All rights reserved.
 *
 * This software is the confidential and proprietary information
 * of Samsung Electronics, Inc. ("Confidential Information").  You
 * shall not disclose such Confidential Information and shall use
 * it only in accordance with the terms of the license agreement
 * you entered into with Samsung Electronics.
 */

/**
 * This file contains HTTP RFC2965/2109/Netscape compliant cookie implementation
 *
 * @author          Vamsi  vamsiy@samsung.com
 */

/* Version History
Author		Date		BugID		BugDescription

*/

#ifndef _SFB_COOKIE_H_
#define _SFB_COOKIE_H_

#ifdef __cplusplus
extern "C" {
#endif // __cplusplus

	/*INCLUDES*/
#include "SFBRmCoreTypes.h"

//#include "ULHTTPCookieStorage.h"

	/*MACROS*/

#define SFB_COOKIE_DEFAULT_MAX_SIZE_COOKIE			2048	/*2 KB - default size per cookie entry */
#define SFB_COOKIE_DEFAULT_MAX_TOTAL_SIZE_COOKIE	65536	/*64 KB- default max size for cookie */
#define SFB_COOKIE_DEFAULT_MAX_NUM_COOKIE			256		/*256 Maximum number of cookie */

#define SFB_COOKIE_FILE_NAME						"cookie1.bin"	/*cookie file name */


#define SFB_COOKIE_SET_COOKIE_STRING				"Set-Cookie"
#define SFB_COOKIE_SET_COOKIE_COMMA_SEP_STRING		","

#define SFB_COOKIE_VERSION							0x0001  /* cookie version */
#define SFB_COOKIE_IS_SECURE						0x0002  /* cookie secure */

#define	SFB_COOKIE_MAX_STORAGE						10		/* How many cookie storage */

	/*DATA TYPES*/


/**
 * The client call this callback response in order to continue to process the cookie.

 *
 * @param szCurrentUrl The actual cookie information to be sotred
 * @param szMainDocUrl The actual cookie information to be sotred
 * @param szHeaders The actual cookie information to be sotred
 * @param bCancel TRUE if the client want to save cookie, otherwise FALSE
 * @param hNewCookieStorage[in] 	If client wish to use a different cookie storage instead of the default storage specified by SFBRmSetCookieStorage,
 *						the client must set the handle of the different cookie storage. If client want to use the default storage, set SFB_INVALID_COOKIE_HANDLE
 * 						For example, If szHeaders include the operator specific header, client can specify the handle of the operator cookie storage.
 */

typedef void (*SFBCookieCallbackResponse)( SFBRmHClient hClient,
											SFBRmReqId reqID,
											unsigned int iConfirm,
											void* pCookie
											);


/**
 * Called before storing the response cookies to cookie storage in order to give the client a chance to change it.
 * In case the client does not specify this callback, SFBRm will save the cookies using the default cookie storages.
 * If the client implements this method, the request will suspend until the client call fpCallbackResponse.
 *
 * @param szCurrentUrl[in]
 * @param szMainDocUrl[in]
 * @param szHeaders[in] The http headers
 */
typedef void (*SFBCookieCallback)( SFBRmHClient hClient,
											SFBRmReqId reqID,
											void* appData,
											char* szHostName,
											BOOL bCookieExist,
											char* szCookieString,
											void* pCookie,
											SFBCookieCallbackResponse fpCallbackResponse
											);
/**
Cookie Map
**/
typedef struct _SFBCookieHandleMap
{
	int			iDirectoryCount;	
	void*	hDirectoryHandle;	
	char* szDirectory;
} SFBCookieHandleMap;
							
/**
Cookie Map
**/
typedef struct _SFBCookieDirectoryMap
{
	SFBCookieHandleMap* handleMap;
	void* next;
} SFBCookieDirectoryMap;

/**
* Enum SFBCookieList
* Cookie LIst written in Memory
*/
typedef struct _SFBCookieList
{
	char*	szDomain;					/* Domain attribute */
	char*	szPath	;					/* Path attribute */

	char*	szName;						/* NAME attribute */
	char*	szValue;					/* VALUE attribute */

	int		iFlag;						/* Secure and Version Here */
	UINT32	expires;					/* Expiry time based on expires or maxage.
										   Discard also would be taken care here*/

	UINT32	last_accessed; 				/* To record the latest access time */

	int		length;						/* Cookie length */

	SFBCookieCallback	pCookieCallback; /* UI callback*/

	struct _SFBCookieList* prev;		/* link to previous node */
	struct _SFBCookieList* next;		/* link to next node */
} SFBCookieList;


/**
* Enum SFBCookieConfig
* Cookie configuration
*/
typedef struct _SFBCookieConfig
{
	int		iMaxTotalCookies;			/* Maximum number of cookies */
	int		iMaxTotalSizeCookies;		/* Maximum size of cookies */
	int		iMaxSizePerCookie;			/* Maximum size per cookie */

} SFBCookieConfig;

/**
* Enum SFBCookieContext
* Cookie Handler
*/
typedef struct _SFBCookieContext
{
	SFBCookieList*	Head;				/* Head of cookie list*/
	SFBCookieList*	Tail;				/* Tail of cookie list*/

	int				iCurNumCookies;		/* Current no. of cookies*/
	BOOL			bCookieActivated;	/* Cookie enable/disable*/
	int				iRef;				/* Reference count- no of instance*/

	char*			pCurDirectory;		/* Directory for storing cookies */

	//SFBSetCookieConfirmCallback	pCookieConfirmCallback;/* UI confirm callback*/

	SFBCookieConfig	tSFBCookieConfig;	/* cookie configuration*/

} SFBCookieContext;


typedef struct _SFBCookieHandleList
{
	SFBHCookie				curCookieHandle;
	int						iRef;

	struct _SFBCookieHandleList*	next;				/* Head of cookie handle list*/
	struct _SFBCookieHandleList*	prev;				/* Tail of cookie handle list*/

} SFBCookieHandleList;

	/*FUNCTION DECLARATIONS*/

//Set-Get APIs
/*
 * This function will take the Current URL, Main Document URL and the HTTP header string
 * as parameter and parse the cookie header and stores the Current URL and the cookie information
 * into the cookie list. Before adding, it checks with User preference for adding cookies.
 * Based on the preference it takes decision regarding addition/rejection/prompt.
 * If added, then the number of cookies in the table is incremented by one.
 * If the maximum number of cookie in the cookie list is reached, then some cleanup will be done
 * before storing new cookie..
 * @param		szCurrentUrl,szMainDocUrl,HTTPheader
 * @return		true/ false
 */
BOOL  SFBSetCookie (SFBCookieCallback fpCookieCallBack,SFBRmReqId reqId,SFBRmHClient hClient, void* appData, SFBHCookie hHandle, const char* szCurrentUrl, const char* szMainDocUrl, const char* HTTPheader);

/*
 * API for JavaScript/MetaData Cookie
 * This function will take the Current URL, Cookie Header string
 * as parameter and parse the cookie header and stores the Current URL and the cookie information
 * into the cookie list. Before adding, it checks with User preference for adding cookies.
 * Based on the preference it takes decision regarding addition/rejection/prompt.
 * If added, then the number of cookies in the table is incremented by one.
 * If the maximum number of cookie in the cookie list is reached, then some cleanup will be done
 * before storing new cookie..
 * @param		szCurrentUrl,szCookieHeader
 * @return		true/ false
 */
BOOL SFBSetJavaScriptCookie (SFBCookieCallback fpCookieCallBack,SFBRmReqId reqId,SFBRmHClient hClient,void* appData, SFBHCookie hHandle,const char* szCurrentUrl, const char* szCookieHeader);

/*
 *This function takes the URL as the input and it will make the Cookie?HTTP response header
 *by searching cookie for the given host and domain in URL.
 * @param		szHostUrl
 * @return		cookie header string.
 */
char* SFBGetCookie (SFBHCookie hHandle, const char* szHostUrl);

/*
 * API for JavaScript/MetaData Cookie
 * This function takes the URL as the input and it will make the Cookie HTTP response header
 * by searching cookie for the given host and domain in URL.
 * @param		szHostUrl
 * @return		cookie header string.
 */
char* SFBGetJavaScriptCookie (SFBHCookie hHandle,const char* szHostUrl);

//Init-Exit APIs
/**
 *This Function Initializes the global data like max number of cookies, max size of cookies etc.
 *Reads Cookie data from FileSystem to LookUp Table. Should be called when initializing browser object.
 *Acts as Initilization API. Can be called from multiple objects. Increases the reference count
 *when Called if its already initialized.
 *
 * @param		pSFBCookieConfig [in]	Cookie configuration pointer
 * @return		None
 */
SFBHCookie  SFBInitCookie (char* pCurDir, SFBCookieConfig* tSFBCookieConfig);

/**
 * Acts as Destructor API. Can be called from multiple objects. Decreases the reference count when Called.
 * When count reaches zero, writes Cookies in memory to File System. Frees the memory used for Lookup Table.
 *
 * @param		none
 * @return		None
 */
void  SFBExitCookie (SFBHCookie hHandle);

//Enable-Disable APIs
//Get Enable status API
/**
 * This function will return Cookie Activation status.
 *
 * @param			None
 * @return                    bEnable, used to enable the SFBGetCookie function
 */
BOOL SFBGetCookieEnable(SFBHCookie hHandle);

//Set Enable status API
/**
 * This function will set Cookie Activation status.
 *
 * @param		bEnable
 * @return		None
 */
void SFBSetCookieEnable(SFBHCookie hHandle,BOOL bEnable);

//Remove-Remove ALL APIs
//Remove API
/**
 * This Function removes the cookie related to that Url. (Domain + path)
 *
 * @param			szHostUrl [in] The URL of the server
 * @return                    None
 */
void  SFBRemoveCookie(SFBHCookie hHandle,const char* szHostUrl);

//Remove All API
/**
 * This Function removes all the cookies in storage.
 *
 * @param			None
 * @return          None
 */
void  SFBRemoveAllCookie (SFBHCookie hCookieHandle);

//Confirm-Callback
/**
 * TThis function will act as callback function for the browser control/UI.
 * This will add/reject cookie based on user choice.
 *
 * @param			bDecision [in]
 * @return                    None
 */
void  SFBSetCookieUIConfirmCallback( SFBRmHClient hClient,SFBRmReqId reqID,	unsigned int iConfirm, void* pCookie );

extern SFBCookieContext *gpSFBCookieContext;

extern SFBCookieHandleList	*gszCookieStorage; /* This need to manage cookie handle list */

extern SFBCookieDirectoryMap *gszCookieDirectoryMap;

#ifdef __cplusplus
}
#endif // __cplusplus

#endif //_SFB_COOKIE_H_