/* 
 * Copyright (c) 2007 SISO, Samsung Electronics, Inc.
 * All rights reserved.
 *
 * This software is the confidential and proprietary information
 * of Samsung Electronics, Inc. ("Confidential Information").  You
 * shall not disclose such Confidential Information and shall use
 * it only in accordance with the terms of the license agreement
 * you entered into with Samsung Electronics. 
 /* FILE :
 * This file provides external API's to the Webkit for making a request
 *
 * @name		SFBalRmCore.c [SHP HTTP interface]
 * @author		Ancil K George (ancilgeorge@samsung.com)
				Basavaraj P S (basavaraj.ps@samsung.com)
				Anshul Saxena (a.saxena@samsung.com)
				Rohit P Shetty (rohit.shetty@samsung.com)
  */

#ifndef _SFB_RMCORE_H_
#define _SFB_RMCORE_H_

#ifdef __cplusplus
extern "C" {
#endif /* __cplusplus */


	/*INCLUDES*/
#include <SFBRmCoreTypes.h>
#include <SFBRmHttp.h>

	/*MACROS*/ 
#ifdef MULTIPLE_INSTANCE_SUPPORT //Pramod - Multiple instance support
#define SFBRmSendEvent(w, x, y, z) 	SFBalSendEvent(BAL_APP_RM, w, x, y, z, NULL)
#else
#define SFBRmSendEvent(w, x, y, z) 	SFBalSendEvent(BAL_APP_RM, w, x, y, z)
#endif

#define SFBRm_MAX_SIZE_FILE 1024*50//rohit
#define SFB_CONT_LEN 30
#define SFB_CONT_TYPE 6

#define SFB_RM_REMOVE_HASH //SFB00000694 
#define SFB_RM_PERF
#define SFB_MEM_1KB 1024

/* GLOBALS*/

//rohit
typedef enum
{
	SFB_RM_HTTP_REQ_NONE,
	SFB_RM_HTTP_REQ_IND,
	SFB_RM_HTTP_CHUNKED_REQ_IND,
	SFB_RM_HTTP_ABORT_REQ_IND,
	SFB_RM_HTTP_ADD_CHUNK_IND,
	SFB_RM_WSP_CONNECT_CNF_IND,
	SFB_RM_WSP_DISCONNECT_IND,
	SFB_RM_WSP_METHOD_INVOKE_CNF_IND,
	SFB_RM_WSP_METHOD_RESULT_CNF_IND,
	SFB_RM_WSP_METHOD_ABORT_IND,
	SFB_RM_WSP_SUSPEND_IND
}SFBRmInd;

/* Function Declarations */

typedef struct 
{
	char pExt [SFB_CONT_TYPE];
	char cont_Name [SFB_CONT_LEN];
}RMExtTable;


static const RMExtTable extensionTable[] = {

	{ "js",			"text/javascript"},
	{ "css",			"text/css" },
};

/* Queue Operation Functions */

/**
Fucntion: SFBRmPutRmEntityintoQ

* This function is called to to allocate memory for the RmEntity and Put the request and 
* append that node at the end of the list, inside this function 
* next request Id is returned
* this function used for both HTTP and WSP connection type
* @param	pReqInfo [in] pointer to request Info structure of the application
* @param	appId [in] Specifies the module who is responsible for the request
* @param	appData [in] application Data needs to be passed  back in the response callbacks
* @param	SFBRmRequestStatus [in] specifies the enum value of the request status
* @param	headerCallback [in] header callback function to give response Header when Response comes
* @param	contentCallback [in] content callback function to give response body when Response comes
* @param	errorCallback [in] error callback function to give error response other than 400 500 status
* @param	SFBRmReqId [out] specifies identifier of the request
*/

SFBRmReqId SFBRmPutRmEntityintoQ(SFBRmRequestInfo * pReqInfo, 
								SFBRmHClient hClient, 
								void * appData, 
								SFBRmHeaderCallback headerCallback, 
								SFBRmContentCallback contentCallback,
								SFBRmErrorCallback errorCallback,
								SFBRmRedirectCallback redirectCallback,
								SFBRmCacheCallback cacheCallback,
						        SFBRmCookieCallback cookeCallback,
								SFBRmAuthenticationCallback authenticationCallback,
								SFBRmSSLConfirmationCallback sslConfirmationCallback,
								SFBRmPostCallback postCallback,
								SFBRmSendNextPostChunkCallback	sendNextPostChunkCallback//Anshul July 3
								);


/**
Fucntion: SFBRmGetRmEntityByReqId

* This function is called to get the RmEntity Node by reqId, inside this function 
* coresponding (RmEntity) pointer is returned
* this function used for both HTTP and WSP connection type
* @param	reqId [in]request id
* @param	pRmEntity [out] specifies the pointer to a node RmEntity which has to be processed

* @return	Returns the valid pointer to a node if success or SFB_NULL if fail
*/

SFBRmEntity *SFBRmGetRmEntityByReqId(int reqId);
SFBRmEntity *SFBRmGetRmEntityByTrId(int trId);

/**
Fucntion: SFBRmDeleteRequestFromQ

* This function is called to update the request status
* this function used for both HTTP and WSP connection type

* @param	pRmEntity [in] reqId, Identifier of the request to be deleted

* @return	Returns the SFB_RM_OK if success or any other failure if fail
*/

SFBResult SFBRmDeleteRequestFromQ (int reqId);

/**
Fucntion: SFBRmDeleteAllRequestsFromQ

* This function is called to update the request status
* this function used for both HTTP and WSP connection type


* @return	Returns the SFB_RM_OK if success or any other failure if fail
*/

SFBResult SFBRmDeleteAllRequestsFromQ ();


/**
Fucntion: SFBRmHandleHTTPResponse

* This function is called from underlying HTTP/WSP Layer resposne callback
* this function used for both HTTP and WSP connection type

* @param	connType [in] specifies the Type of the response (WSP/HTTP)
* @param	trId [in] specifies transaction Id of the response
* @param	status [in] specifies response status code
* @param	pResponseHeader [in] specifies response header buffer
* @param	responseHeaderLen [in] specifies response header length
* @param	pResponseBody [in] specifies response body buffer
* @param	responseBodyLen [in] specifies response body length

* @return	Returns void
*/

void SFBRmHandleHTTPResponse (SFBRmInd httpInd, 
							 int trId, 
							 int respStatus, 
							 char * pResponseHeader, 
							 char * pResponseBody, 
							 int responseBodyLen);


/**
Fucntion: SFBRmHandleResponseStatus

* This function is called to update the request status
* this function used for both HTTP and WSP connection type

* @param	status [in] status code of the response
* @param	pRmEntity [in] pointer to the RmEntity

* @return	Returns the SFB_RM_OK if success or any other failure if fail
*/
SFBResult SFBRmHandleResponseStatus(int status, SFBRmEntity * pRmEntity, SFBRmInd httpInd);

/**
Fucntion: SFBRmGetRmEntityHead

* This function is called to get the global RmEntity Head Pointer
* this function used for both HTTP and WSP connection type
* @return[out] SFBRmEntity,	Returns the global valid RmEntity Herad pointer 
*/
SFBRmEntity * SFBRmGetRmEntityHead (void);

/**
Fucntion: SFBRmMakeRequest

* This function is called from the Bearer stated Notification for making request
* this function used for both HTTP and WSP connection type
* @return	Returns the SFB_RM_OK if success or any other failure if fail
*/
SFBResult SFBRmMakeRequest(SFBRmHClient hClient);

void SFBRmFreeRequestInfo(SFBRmRequestInfo* pRequestInfo);
void SFBRmResetReqCount(void);
void SFBRmReduceRunningReqCount(void);
void SFBRmResetRunningReqCount(void);
SFBResult SFBRmMakeRequest(int account);
void SFBRmAuthString(char* szAuthString, int rId);
void SFBRmAbortAllReqFromQ(SFBStatus err, SFBRmHClient hRmClient);
SFBResult SFBRmMakeRequestHeader (SFBRmEntity * pRmEntity, char ** pRequestHeader, BOOL proto);
SFBResult SFBRmRedirectResponse( SFBRmHClient hClient, SFBRmReqId reqID,SFBRmRequestInfo* pRedirectedReq, BOOL userConfirmation );//Anshul
SFBResult SFBRmSetRedirectionInfo( SFBRmHClient hClient,SFBRmReqId reqID,SFBRmRequestInfo* pRedirectedReq );//Anshul
void SFBRmRetreiveFromCache(SFBRmEntity * pRmEntity);
SFBResult SFBRmRemoveRedundantHdrs (char *pRequestHeader, char ** pTempReqHdr, int status);
#ifdef SFB_RM_REMOVE_HASH
void SFBRmRemoveFragmentFromUrl(char * pUrl);
#endif

SFBResult SFBRmMakeHeaderCallBack(int status, SFBRmEntity* pRmEntity);
void SFBRmMakeContentCallBack( SFBRmEntity* pRmEntity, char* respBody,int respBodyLen,BOOL bLastChunk);
void SFBRmMakeErrorCallBack(SFBRmEntity* pRmEntity, int err);

SFBalHProtoClient __SFBRmGetHProtoClient( SFBRmHClient hClient );
void SFBRmEventHandler(SFBalEventDataList * evt);
#ifdef SFB_RM_UNZIP
SFBStatus SFBRmUnzip(SFBRmEntity *pRmEntity, char **pUnzipData, int *pUnzipDataLen);
#endif
#ifdef __cplusplus
}
#endif /* __cplusplus */

#endif //_SFB_RMCORE_H_