/*
 * Copyright (c) S/W Platform, Samsung Electronics, Inc.
 * All rights reserved.
 *
 * This software is the confidential and proprietary information
 * of Samsung Electronics, Inc. ("Confidential Information").  You
 * shall not disclose such Confidential Information and shall use
 * it only in accordance with the terms of the license agreement
 * you entered into with Samsung Electronics. 
 */
/** 
 * @file		IChannelEventListener.h
 * @brief		Define interface of URILoader IChannelEventListener
 * @author	Sangheon Kim (sangheon77.kim@samsung.com)
 * @version	1.0
 * @date		2008.03.05	v1.0		Created
 */
#ifndef __IChannelEventListener_Defined
#define __IChannelEventListener_Defined

#include "URILoader.h"
#include "IChannelEventSender.h"

namespace URILoader
{

class IChannel;

class IChannelEventListener : public IUnknown
{
public:
	 /* This is a temporary redirect. New requests for this resource should
	 * continue to use the URI of the old channel */
	static const unsigned long REDIRECT_TEMPORARY = 1 << 0;

	/**
	 * This is a permanent redirect. New requests for this resource should use
	 * the URI of the new channel (This might be an HTTP 301 reponse).
	 */
	static const unsigned long REDIRECT_PERMANENT = 1 << 1;

	/**
	 * This is an internal redirect, i.e. it was not initiated by the remote
	 * server.
	 */
	static const unsigned long REDIRECT_INTERNAL = 1 << 2;

	/**
	 * Called when a redirect occurs. This may happen due to an HTTP 3xx status
	 * code.
	 */
	virtual Result OnRedirect( IChannel *oldChannel, 
					IChannel *newChannel,
					unsigned long flags,
					IChannelEventSender *aSender ) = 0;



	virtual Result OnAuthentication( IChannel *aChannel,
						void* aContext,
						unsigned int level,
						IAuthInformation* aAuthInfo,
						IChannelEventSender *aSender ) = 0;


	/**
	* Called when a error occures during https handshaking.
	*/
	virtual Result OnCertificateError (
						unsigned long type,
						String *aString,
						IChannelEventSender* aSender ) = 0;

	/**
	 * Called to notify the event sink that progress has occurred for the given request.
	 */
	virtual void OnProgress( IChannel* aChannel,
	                void* aContext,
	                unsigned long long aProgress,
	                unsigned long long aProgressMax) = 0;



	virtual Result OnProcessCookie(
			String* aHostname,
			ICookie* aCookie,
			BOOL	bSameCookieExist,
			IChannelEventSender* aSender ) = 0;
	


};


} // namespace 

#endif //__IChannelEventListener_Defined
