/*
 * Copyright (c) S/W Platform, Samsung Electronics, Inc.
 * All rights reserved.
 *
 * This software is the confidential and proprietary information
 * of Samsung Electronics, Inc. ("Confidential Information").  You
 * shall not disclose such Confidential Information and shall use
 * it only in accordance with the terms of the license agreement
 * you entered into with Samsung Electronics. 
 */
/** 
 * @file		ILoadGroup..h
 * @brief		Define interface of URILoader ILoadGroup
 * @author	Sangheon Kim (sangheon77.kim@samsung.com)
 * @version	1.0
 * @date		2008.03.05	v1.0		Created
 */
#ifndef __ILoadGroup_Defined
#define __ILoadGroup_Defined

#include "URILoader.h"
#include "IPDPContextListener.h"
#include "IChannelEventListener.h"
#include "IChannelEventSender.h"
#include "ICookieStorage.h"

namespace URILoader
{

typedef int AccessPoint;

class IHttpChannel;
class ICacheStorage;
/**
* ILoadGroup is a manager of several IChannels.
* All IChannels in ILoadGroup will share same PDP context.
* Client can receive events related to the PDP context through IPDPContextListener.
* 
*/
class ILoadGroup : public IUnknown
{
public:

	
	/***************************************************************************
	*
	* Properties
	*
	****************************************************************************/
	/**
	* Informations for activating PDP context
	* 
	*/
	virtual Result SetAccessPoint( AccessPoint aValue ) = 0;
	virtual Result GetAccessPoint( AccessPoint *pValue ) = 0;

	/**
	* Event listener related to PDP context
	*/
	virtual Result GetPDPContextListener( IPDPContextListener **ppValue ) = 0;
	virtual Result SetPDPContextListener( IPDPContextListener* pListener ) = 0;




	virtual Result GetChannelEventListener( IChannelEventListener ** pValue ) = 0;
	virtual Result SetChannelEventListener( IChannelEventListener* aValue ) = 0;


	virtual Result GetCacheStorage( ICacheStorage **pValue ) = 0;
	virtual Result SetCacheStorage( ICacheStorage *aValue ) = 0;

	
	virtual Result GetCookieStorage( ICookieStorage **pValue ) = 0;
	virtual Result SetCookieStorage( ICookieStorage *aValue ) = 0;



	/***************************************************************************
	*
	* Funtions
	*
	****************************************************************************/
	/**
	* Create a new channel
	* All IChannels created by this ILoadGroup will share same PDP context.
	*/
	virtual Result CreateChannel( String URL, IChannel **ppChannel  ) = 0;

	virtual Result GetClientID(int *pValue ) = 0;


	

};

/**
* Factory funtion to create ILoadGroup
*
* @param name the name of LoadGroup
*/
Result CreateLoadGroup( String name, ILoadGroup **ppLoadGroup ) ;


void  DestroyLoadGroup( ILoadGroup *ppLoadGroup ) ;


} // namespace 

#endif //__ILoadGroup_Defined
