/*
 * Copyright (c) S/W Platform, Samsung Electronics, Inc.
 * All rights reserved.
 *
 * This software is the confidential and proprietary information
 * of Samsung Electronics, Inc. ("Confidential Information").  You
 * shall not disclose such Confidential Information and shall use
 * it only in accordance with the terms of the license agreement
 * you entered into with Samsung Electronics. 
 */
/** 
 * @file		URILoader..h
 * @brief		Configuration file for URILoader
 * @author	Sangheon Kim (sangheon77.kim@samsung.com)
 * @version	1.0
 * @date		2008.03.05	v1.0		Created
 */
#ifndef __URILoader_Defined
#define __URILoader_Defined


//vprajan: wint_t already defined in ShpTypes.h
//#define wint_t XYZ
#ifdef STLPORT_ENABLED
#include "map"
#include "vector"
#include "string"
#else
#include <map>
#include <vector>
#include <string>
#include <algorithm>
#define stlport std
#endif

namespace URILoader
{

#define	UVector stlport::vector<void>	
typedef stlport::string	String; 

typedef	stlport::map<stlport::string, stlport::string>	HeaderMap;

typedef 		int			Result;
#undef BOOL
typedef			int			BOOL;


const int RESULT_OK = 1;
const int RESULT_ERR_NOT_IMPLEMENTED = -1;
const int RESULT_ERR_NO_MEMORY = -2;
const int RESULT_ERR_UNKNOWN = -3;
const int RESULT_ERR_INVALID_PARAMS = -4;

#define		RESULT_ISFAIL(x)	(x < 0)

const int BASE_ERROR_PROTO = 1;
const int URI_ERROR_PROTO_NO_SERVICE = BASE_ERROR_PROTO + 1;		
const int URI_ERROR_PROTO_NOT_SUBSCRIBED = BASE_ERROR_PROTO + 2;	
const int URI_ERROR_PROTO_INVALID_APN = BASE_ERROR_PROTO + 3;
const int URI_ERROR_PROTO_TIME_OUT = BASE_ERROR_PROTO + 4;
const int URI_ERROR_PROTO_INVALID_PARAM = BASE_ERROR_PROTO + 5;
const int URI_ERROR_PROTO_AUTH_FAIL = BASE_ERROR_PROTO + 6;
const int URI_ERROR_PROTO_FDN_NOT_ALLOWED = BASE_ERROR_PROTO + 7; 
const int URI_ERROR_PROTO_NOT_SUPPORT_3GCSD = BASE_ERROR_PROTO + 8; 
const int URI_ERROR_PROTO_NETWORK_FAILURE = BASE_ERROR_PROTO + 9; 
const int URI_ERROR_PROTO_UNKNOWN = BASE_ERROR_PROTO + 10; 
const int URI_ERROR_PROTO_NETWORK_USED = BASE_ERROR_PROTO + 11; 
const int URI_ERROR_PROTO_MAX_CONNECTED = BASE_ERROR_PROTO + 12;

// HTTP Error- Gnana
const int BASE_ERROR_HTTPERR = 20;
const int URI_ERROR_HTTPERR_INVALID_PARAM = BASE_ERROR_HTTPERR + 1;	
const int URI_ERROR_HTTPERR_UNKNOWN = BASE_ERROR_HTTPERR + 2;			
const int URI_ERROR_HTTPERR_INVALID_PROXY = BASE_ERROR_HTTPERR + 3;	
const int URI_ERROR_HTTPERR_OUT_OF_MEMORY = BASE_ERROR_HTTPERR + 4;				
const int URI_ERROR_HTTPERR_NO_RESPONSE = BASE_ERROR_HTTPERR + 5;				
const int URI_ERROR_HTTPERR_INIT = BASE_ERROR_HTTPERR + 6;					
const int URI_ERROR_HTTPERR_NETDOWN = BASE_ERROR_HTTPERR + 7;					
const int URI_ERROR_HTTPERR_TIMEOUT = BASE_ERROR_HTTPERR + 8;				
const int URI_ERROR_HTTPERR_HOST_UNREACH = BASE_ERROR_HTTPERR + 9;
const int URI_ERROR_HTTPERR_CONN_RESET = BASE_ERROR_HTTPERR + 10;			
const int URI_ERROR_HTTPERR_INTERNAL = BASE_ERROR_HTTPERR + 11;				
const int URI_ERROR_HTTPERR_CHUNKEDTR = BASE_ERROR_HTTPERR + 12;				
const int URI_ERROR_HTTPERR_USER = BASE_ERROR_HTTPERR + 13;				
const int URI_ERROR_HTTPERR_TOO_BIG = BASE_ERROR_HTTPERR + 14;
const int URI_ERROR_HTTPERR_NOT_SUPPORTED_SVC = BASE_ERROR_HTTPERR + 15;
const int URI_ERROR_HTTPERR_NO_CONTENT = BASE_ERROR_HTTPERR + 16;		
const int URI_ERROR_HTTPERR_WRITEFAIL = BASE_ERROR_HTTPERR + 17;				
const int URI_ERROR_HTTPERR_AUTHFAIL = BASE_ERROR_HTTPERR + 18;
const int URI_ERROR_HTTPERR_DNSFAIL = BASE_ERROR_HTTPERR + 19;				
const int URI_ERROR_HTTPERR_UAGENT_NOT_ALLOWED = BASE_ERROR_HTTPERR + 20;

//-------------

const int BASE_ERROR_NOCONTENT = 50;
const int URI_ERROR_NOCONTENT_DEFAULT = BASE_ERROR_NOCONTENT + 0;
const int URI_ERROR_NOCONTENT_HTTP400 = BASE_ERROR_NOCONTENT + 1;
const int URI_ERROR_NOCONTENT_HTTP401 = BASE_ERROR_NOCONTENT + 2;
const int URI_ERROR_NOCONTENT_HTTP402 = BASE_ERROR_NOCONTENT + 3;
const int URI_ERROR_NOCONTENT_HTTP403= BASE_ERROR_NOCONTENT + 4;
const int URI_ERROR_NOCONTENT_HTTP404= BASE_ERROR_NOCONTENT + 5;
const int URI_ERROR_NOCONTENT_HTTP405= BASE_ERROR_NOCONTENT + 6;
const int URI_ERROR_NOCONTENT_HTTP406= BASE_ERROR_NOCONTENT + 7;
const int URI_ERROR_NOCONTENT_HTTP407= BASE_ERROR_NOCONTENT + 8;
const int URI_ERROR_NOCONTENT_HTTP408= BASE_ERROR_NOCONTENT + 9;
const int URI_ERROR_NOCONTENT_HTTP409= BASE_ERROR_NOCONTENT + 10;
const int URI_ERROR_NOCONTENT_HTTP410= BASE_ERROR_NOCONTENT + 11;
const int URI_ERROR_NOCONTENT_HTTP411= BASE_ERROR_NOCONTENT + 12;
const int URI_ERROR_NOCONTENT_HTTP412= BASE_ERROR_NOCONTENT + 13;
const int URI_ERROR_NOCONTENT_HTTP413= BASE_ERROR_NOCONTENT + 14;
const int URI_ERROR_NOCONTENT_HTTP414= BASE_ERROR_NOCONTENT + 15;
const int URI_ERROR_NOCONTENT_HTTP415= BASE_ERROR_NOCONTENT + 16;
const int URI_ERROR_NOCONTENT_HTTP500= BASE_ERROR_NOCONTENT + 17;
const int URI_ERROR_NOCONTENT_HTTP501= BASE_ERROR_NOCONTENT + 18;
const int URI_ERROR_NOCONTENT_HTTP502= BASE_ERROR_NOCONTENT + 19;
const int URI_ERROR_NOCONTENT_HTTP503= BASE_ERROR_NOCONTENT + 20;
const int URI_ERROR_NOCONTENT_HTTP504= BASE_ERROR_NOCONTENT + 21;
const int URI_ERROR_NOCONTENT_HTTP505= BASE_ERROR_NOCONTENT + 22;

const int URI_ERROR_AUTH_INVALID_FORMAT = 41;

class IUnknown
{
public:
	virtual ~IUnknown() {}
};

}

#endif //__URILoader_Defined
