/*
 * Copyright (c) S/W Platform, Samsung Electronics, Inc.
 * All rights reserved.
 *
 * This software is the confidential and proprietary information
 * of Samsung Electronics, Inc. ("Confidential Information").  You
 * shall not disclose such Confidential Information and shall use
 * it only in accordance with the terms of the license agreement
 * you entered into with Samsung Electronics. 
 */
/** 
 * @file		IContentsSender.h
 * @brief		Define interface ISetting of WebControl
 * @author	Chaehwan Li (chaehwan.li@samsung.com)
 * @version	1.0
 * @date		2008.03.14	v1.0		Created
 */

#ifndef __IContentsSender_Defined
#define __IContentsSender_Defined

#include "WebDefs.h"

namespace	WebControl
{

class IContentsSender
{
public:

	/*
	 * @desc sender function called when sends result about authentication.
	 * @param in_pszID pointer to ID
	 * @param in_pszPassword pointer to password
	 * @return
	 */
	virtual HRESULT auth( BOOL bCancel, char* in_pszID, char* in_pszPassword ) = 0;

	/*
	 * @desc sender function called when sends result about redirect.
	 * @param in_bOK : HRESULT_NOK / HRESULT_OK
	 * @return
	 */
	virtual HRESULT redirect( HRESULT in_bOK ) =0;

	/*
	 * @desc sender function called when sends result about cookie.
	 * @param in_bAllow : HRESULT_NOT_ALLOW / HRESULT_ALLOW
	 * @return
	 */
	virtual HRESULT cookie( HRESULT in_bAllow, void* ) = 0;

	/*
	 * @desc sender function called when sends result about ExScheme
	 * @param
	 * @return
	 */
	virtual HRESULT exScheme( void ) = 0;

	/*
	 * @desc sender function called when sends result about upload file
	 * @param in_pszFilePath : selected file and path to upload
	 * @return
	 */
	virtual HRESULT upload( char* in_pszFilePath ) = 0;

	/*
	 * @desc sender function called when sends result about Upload State
	 * @param
	 * @return
	 */
	virtual HRESULT uploadState( void ) = 0;

	/*
	 * @desc sender function called when sends result about forward
	 * @param in_YesNo : HRESULT_NOK / HRESULT_OK
	 * @return
	 */
	virtual HRESULT forwardHistory( HRESULT in_OK ) = 0;

	/*
	 * @desc sender function called when sends result about backward
	 * @param in_YesNo : HRESULT_NOK / HRESULT_OK
	 * @return
	 */
	virtual HRESULT backwardHistory( HRESULT in_OK ) = 0;

	/*
	 * @desc sender function called when sends result about notify
	 * @param in_YesNo : HRESULT_NOK / HRESULT_OK
	 * @return
	 */
	virtual HRESULT notify( HRESULT in_YesNo ) = 0;

	/*
	 * @desc sender function called when sends result about confirm
	 * @param in_YesNo : HRESULT_NOK / HRESULT_OK
	 * @return
	 */
	virtual HRESULT confirm( ConfirmType in_ConfirmType, HRESULT in_YesNo ) = 0;

	/*
	 * @desc sender function called when sends result about prompt
	 * @param in_YesNo : YES / NO
	 * @return
	 */
	virtual HRESULT prompt( HRESULT in_YesNo, char* in_pszReturn ) = 0;

	/*
	 * @desc sender function called when sends result about alert
	 * @param in_OK : HRESULT_NOK / HRESULT_OK
	 * @return
	 */
	virtual HRESULT alert( HRESULT in_OK ) = 0;

};

} // namespace WebControl

#endif // __IContentsSender_Defined