/*
 * Copyright (c) S/W Platform, Samsung Electronics, Inc.
 * All rights reserved.
 *
 * This software is the confidential and proprietary information
 * of Samsung Electronics, Inc. ("Confidential Information").  You
 * shall not disclose such Confidential Information and shall use
 * it only in accordance with the terms of the license agreement
 * you entered into with Samsung Electronics. 
 */
/** 
 * @file		WebContentsSender.h
 * @brief		
 * @author	Sangheon Kim (sangheon77.kim@samsung.com)
 * @version	1.0
 * @date		2008.04.14	v1.0		Created
 */
#ifndef __WebContentsSender_Defined
#define __WebContentsSender_Defined

#include "WebControlConfig.h"
#include "IContentsSender.h"

#include "SFBRmMgr.h"
#include "WebControl.h"


namespace	WebControl
{

class WebContentsSender : public IContentsSender
{
public:
	WebContentsSender( WebControl *pWebControl );

	void SetAuthenticationInfo(  SFBRmHClient hClient,
												SFBRmReqId reqID,
												int transID,
												SFBRmAuthenticationCallbackResponse fpRespCallback );

	void SetCookieInfo( SFBRmHClient hClient,SFBRmReqId reqID, SFBRmCookieCallbackResponse fpCallbackResponse );

	void SetRedirectionInfo(  SFBRmHClient hClient,SFBRmReqId reqID, SFBRmRequestInfo* pRedirectedReq, SFBRmRedirectCallbackResponse fpCallbackResponse );

	void SetSSLInfo(  SFBRmHClient hClient, SFBRmReqId reqID, int transID,SFBalRmSSLCallbackResponse fpCallbackResponse );

	void SetPDPEventInfo(  SFBRmHClient hClient, SFBRmPDPContextCallbackResponse fpCallbackResponse );


private:
	// authentication info
	SFBRmHClient m_hClient;
	SFBRmReqId m_reqID;
	int m_transID;
	SFBRmRequestInfo*	m_pRedirectReqInfo;

	SFBRmAuthenticationCallbackResponse m_fpAuthResp;
	SFBRmCookieCallbackResponse			m_fpCookieResp;
	SFBRmRedirectCallbackResponse		m_fpRedirectResp;
	SFBalRmSSLCallbackResponse			m_fpSSLResp;
	SFBRmPDPContextCallbackResponse		m_fpPDPEventResp;

public:
	/*
	 * @desc sender function called when sends result about authentication.
	 * @param in_pszID pointer to ID
	 * @param in_pszPassword pointer to password
	 * @return
	 */
	virtual HRESULT auth( BOOL bCancel, char* in_pszID, char* in_pszPassword );

	/*
	 * @desc sender function called when sends result about redirect.
	 * @param in_bOK : HRESULT_NOK / HRESULT_OK
	 * @return
	 */
	virtual HRESULT redirect( HRESULT in_bOK );

	/*
	 * @desc sender function called when sends result about cookie.
	 * @param in_bAllow : HRESULT_NOT_ALLOW / HRESULT_ALLOW
	 * @return
	 */
	virtual HRESULT cookie( HRESULT in_bAllow, void* pCookie );

	/*
	 * @desc sender function called when sends result about ExScheme
	 * @param
	 * @return
	 */
	virtual HRESULT exScheme( void );

	/*
	 * @desc sender function called when sends result about upload file
	 * @param in_pszFilePath : selected file and path to upload
	 * @return
	 */
	virtual HRESULT upload( char* in_pszFilePath );

	/*
	 * @desc sender function called when sends result about Upload State
	 * @param
	 * @return
	 */
	virtual HRESULT uploadState( void );

	/*
	 * @desc sender function called when sends result about forward
	 * @param in_YesNo : HRESULT_NOK / HRESULT_OK
	 * @return
	 */
	virtual HRESULT forwardHistory( HRESULT in_OK );

	/*
	 * @desc sender function called when sends result about backward
	 * @param in_YesNo : HRESULT_NOK / HRESULT_OK
	 * @return
	 */
	virtual HRESULT backwardHistory( HRESULT in_OK );

	/*
	 * @desc sender function called when sends result about notify
	 * @param in_YesNo : HRESULT_NOK / HRESULT_OK
	 * @return
	 */
	virtual HRESULT notify( HRESULT in_YesNo );

	/*
	 * @desc sender function called when sends result about confirm
	 * @param in_YesNo : HRESULT_NOK / HRESULT_OK
	 * @return
	 */
	virtual HRESULT confirm( ConfirmType in_ConfirmType, HRESULT in_YesNo );

	/*
	 * @desc sender function called when sends result about prompt
	 * @param in_YesNo : YES / NO
	 * @return
	 */
	virtual HRESULT prompt( HRESULT in_YesNo, char* in_pszReturn ) ;

	/*
	 * @desc sender function called when sends result about alert
	 * @param in_OK : HRESULT_NOK / HRESULT_OK
	 * @return
	 */
	virtual HRESULT alert( HRESULT in_OK );

public:



};



} // namespace  WebControl

#endif // __WebContentsSender_Defined