/*
 * Copyright (C) 2006 Don Gibson <dgibson77@gmail.com>
 * Copyright (C) 2006 Zack Rusin <zack@kde.org>
 * Copyright (C) 2006, 2007 Apple Computer, Inc.  All rights reserved.
 * Copyright (C) 2007 Trolltech ASA
 * Copyright (C) 2007 Alp Toker <alp.toker@collabora.co.uk>
 * Copyright (C) 2007 Holger Hans Peter Freyther
 *
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY APPLE COMPUTER, INC. ``AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL APPLE COMPUTER, INC. OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
//#include "config.h"
#include "WebFrameLoaderClient.h"
#include "WebControl.h"
#include "WebDownload.h"
#if defined(_SFB_BACKINGSTORE)
#include "WebBackingStore.h"
#endif


#include "DocumentLoader.h"
#include "FrameLoader.h"
#include "FrameView.h"
#include "FrameTree.h"
#include "HTMLFormElement.h"
#include "HTMLFrameElement.h"
#include "HTMLFrameOwnerElement.h"
#include "HTMLNames.h"
#include "MIMETypeRegistry.h"
//#include "NotImplemented.h"
#include "PlatformString.h"
#include "ResourceRequest.h"
#include "CString.h"
#include "ProgressTracker.h"
#include "MainResourceLoader.h"

#ifdef USE_NATIVE_WIDGET_SUPPORT
#include "NativeEditField.h"
#include "NativeComboField.h"
#endif


#ifdef SFB_SMARTLINK_FEATURE 
#include "SmartLinkMap.h"
#endif

#ifdef SUPPORT_NP_PLUGIN
#include "PluginView.h"
#include "RenderPlugin.h"
#include "PluginDatabase.h"
#endif // SUPPORT_NP_PLUGIN

#include "Navigation.h"
#include "RenderFrameSet.h"

#include "WebDefs.h" // WebControl Error Code

#include "SharedTimer.h"

// to identify policy change error
#define POLICYCHANGE_ERROR "PolicyChangeError"

#if 0 
#if !defined (_SFB_PAGE_PILOT_FEATURE) && !defined(_SFB_BACKINGSTORE) //Pramod - Page pilot feature flag
#define _SFB_PAGE_PILOT_FEATURE
#endif
#endif

#define MAX_CONTROL_CHAR_CHECK              (10)

using namespace WebCore;



namespace WebControl {

//using WebCore::String;


class WebFrameDecisionSender : public IDecisionSender
{
public:
	WebFrameDecisionSender( WebFrameLoaderClient *pFrameLoaderClient, WebCore::FramePolicyFunction policyFunction );

public:
	/*
	 * @desc This sender function called when UI decided download.
	 * @param in_pIStreamListener pointer to IStreamListener
	 * @return HRESULT TRUE/FALSE
	 */
	virtual HRESULT download( );

	/*
	 * @desc This sender function called when UI decided ignore
	 * @param
	 * @return HRESULT TRUE/FALSE
	 */
	virtual HRESULT ignore( void );

	/*
	 * @desc This sender function called when UI decided use
	 * @param
	 * @return HRESULT TRUE/FALSE
	 */
	virtual HRESULT use( void );
private:
	WebFrameLoaderClient *m_pFrameLoaderClient;
	WebCore::FramePolicyFunction m_policyFuntion;

};


WebFrameDecisionSender::WebFrameDecisionSender( WebFrameLoaderClient *pFrameLoaderClient, FramePolicyFunction policyFunction )
{
	m_pFrameLoaderClient = pFrameLoaderClient;
	m_policyFuntion = policyFunction;
}


HRESULT WebFrameDecisionSender::download()
{
	m_pFrameLoaderClient->DecidedPolicyForMIMEType( PolicyDownload );
    (m_pFrameLoaderClient->GetWebFrame()->GetFrame()->loader()->*m_policyFuntion)(PolicyDownload);
	delete this;
	return HRESULT_OK;
}

HRESULT WebFrameDecisionSender::ignore( void )
{
	m_pFrameLoaderClient->DecidedPolicyForMIMEType( PolicyIgnore );
	(m_pFrameLoaderClient->GetWebFrame()->GetFrame()->loader()->*m_policyFuntion)(PolicyIgnore);
	delete this;
	return HRESULT_OK;
}

HRESULT WebFrameDecisionSender::use( void )
{
	m_pFrameLoaderClient->DecidedPolicyForMIMEType( PolicyUse );
	(m_pFrameLoaderClient->GetWebFrame()->GetFrame()->loader()->*m_policyFuntion)(PolicyUse);
	delete this;
	return HRESULT_OK;
}







WebFrameLoaderClient::WebFrameLoaderClient( WebFrame *pWebFrame )
{
	m_pWebFrame = pWebFrame;
	m_pluginView = NULL;
	m_hasSentResponseToPlugin = false;
	m_bFrameLoaderMimeTypeDecision = false;
}

WebFrameLoaderClient::~WebFrameLoaderClient()
{
}

WebCore::String WebFrameLoaderClient::userAgent(const KURL&)
{

    __FuncEntered();
/*
 *  Returning the user agent used by RM as per SISO requirements. This will lead to issue like bug 
 *  SFB00000429 & SFB00000431. The issue is some webpages refer to Javascript which user agent string
 *  to selectively modify the page contents. The script look for MSIE, opera, Gecko, safari borwser 
 *  User agents to do the page content modifications. Now with returning the Jasmine browser useragent
 *  the page contents will not be modified as on the desktop browers.
*/
//  return "Mozilla/5.0 (X11; U; Linux i686; en-US) AppleWebKit/420+ (KHTML, like Gecko) Safari/420+";
    char* pszUserAgent = SFBalGetUserAgent();
    ASSERT(pszUserAgent);
    
    WebCore::String usrAgent(pszUserAgent);
	/* Don't Call webcore free , its allocated by SFBal Layer using SFBalMalloc */
    SFBalMemFree(pszUserAgent);
    pszUserAgent = NULL;

    return usrAgent;
}

#ifdef PLATFORM(SHP)
WebCore::String WebFrameLoaderClient::appName() const
{                     
	char *pszAppName = SFBalGetAppName();
	ASSERT(pszAppName);

	WebCore::String appName(pszAppName);
	/* Don't Call webcore free , its allocated by SFBal Layer using SFBalMalloc */
    SFBalMemFree(pszAppName);
    pszAppName = NULL;

    return appName;
}

WebCore::String WebFrameLoaderClient::appVersion() const
{
	char *pszAppVersion = SFBalGetAppVersion();
	ASSERT(pszAppVersion);

	WebCore::String appVersion(pszAppVersion);
	/* Don't Call webcore free , its allocated by SFBal Layer using SFBalMalloc */
    SFBalMemFree(pszAppVersion);
    pszAppVersion = NULL;

    return appVersion;
}
#endif
WTF::PassRefPtr<WebCore::DocumentLoader> WebFrameLoaderClient::createDocumentLoader(const WebCore::ResourceRequest& request, const SubstituteData& substituteData)
{
	__FuncEntered();
    RefPtr<DocumentLoader> loader = new DocumentLoader(request, substituteData);
    return loader.release();
}

void WebFrameLoaderClient::dispatchWillSubmitForm(FramePolicyFunction policyFunction,  PassRefPtr<FormState>)
{
	__FuncEntered();
    // FIXME: This is surely too simple
    ASSERT(policyFunction);
    if (!policyFunction)
        return;
    (core(m_pWebFrame)->loader()->*policyFunction)(PolicyUse);
}


void WebFrameLoaderClient::committedLoad(DocumentLoader* loader, const char* data, int length)
{
	 // FIXME: This should probably go through the data source.
	__FuncEntered();

	if(!m_pluginView)
	{
	    // Check if it is a valid text/plain content
        // according to IETF RFC 2046 (Section 4.1.2)
        if(m_response.mimeType() == "text/plain" && !isValidTextContent(data, length))
        {
            // Currently ignore the content
            // TODO : Need to pass it to the content downloader for further action
            cannotShowMIMETypeError(m_response);
            loader->mainResourceLoader()->cancel(interruptForPolicyChangeError(loader->request()));
            return;
        }

		FrameLoader *fl = loader->frameLoader();
		fl->setEncoding(m_response.textEncodingName(), false);
		fl->addData(data, length);

		// Set the doc size
		WebControl* iControl = kit(loader->frame()->page());
		if(loader->isLoadingMainResource() == true && fl->isLoadingMainFrame())
		{
			iControl->m_docSize = loader->mainResourceLoader()->getDocSize();
		}
	}
   
	if( !m_pluginView ||  (m_pluginView && m_pluginView->status() != PluginStatusLoadedSuccessfully))
        return;
	
    if (!m_hasSentResponseToPlugin) 
	{
        m_pluginView->didReceiveResponse(core(m_pWebFrame)->loader()->documentLoader()->response());

        // didReceiveResponse sets up a new stream to the plug-in. on a full-page plug-in, a failure in
        // setting up this stream can cause the main document load to be cancelled, setting m_pluginView
        // to null
        if (!m_pluginView)
            return;

        m_hasSentResponseToPlugin = true;
    }

    m_pluginView->didReceiveData(data, length);
}

bool WebFrameLoaderClient::isControlChar(const char c)
{
    // according to IETF RFC 2046 (Section 4.1.2)
    if((0 <= c && 9 >= c) || (14 <= c && 31 >= c))
        return true;

    return false;
}

bool WebFrameLoaderClient::isValidTextContent(const char* data, int length)
{
    bool retVal = true;
    for(int i = 0;  i < MAX_CONTROL_CHAR_CHECK && i < length; i++)
    {
        if(isControlChar(data[i]))
        {
            retVal = false;
            break;
        }
    }

    return retVal;
}

void WebFrameLoaderClient::dispatchDidReceiveAuthenticationChallenge(DocumentLoader*, unsigned long  identifier, const AuthenticationChallenge&)
{
    __NotImplemented();
}

void WebFrameLoaderClient::dispatchDidCancelAuthenticationChallenge(DocumentLoader*, unsigned long  identifier, const AuthenticationChallenge&)
{
    __NotImplemented();
}

void WebFrameLoaderClient::dispatchWillSendRequest(DocumentLoader*, unsigned long reqId, ResourceRequest& request, const ResourceResponse&)
{
	WebControl *pWebControl = m_pWebFrame->GetWebControl();
	if( pWebControl  )
	{
		IContentsListener *pContentsListener = pWebControl->GetContentListener();


		if( pContentsListener )
		{
			KURL url = request.url();
			WebCore::String sheme( url.protocol() );
			pContentsListener->willSendRequest( (char*)(sheme.latin1().data()), (char*)(url.string().latin1().data()), reqId );
		}
	}
}

void WebFrameLoaderClient::assignIdentifierToInitialRequest(unsigned long identifier, DocumentLoader*, const ResourceRequest&)
{
    __NotImplemented();
}

void WebFrameLoaderClient::postProgressStartedNotification()
{
	__FuncEntered();
 //   WebKitShpPage* page = getPageFromFrame(m_frame);
	//if(page->m_progressHandler)
	//	(page->m_progressHandler)(0);

	WebControl *pWebControl = m_pWebFrame->GetWebControl();
	
	if (pWebControl->isExitInstanceCalled())
		return;

	if( pWebControl && m_pWebFrame->IsMainFrame())
	{
//#if defined(_SFB_BACKINGSTORE)
//		pWebControl->GetBackingStore()->PageLoadStarted();
//#endif

		IContentsListener *pContentsListener = pWebControl->GetContentListener();
		if( pContentsListener )
		{
		//	pContentsListener->state( STATE_FINISH, 0, NULL );
			pContentsListener->state( STATE_START, 0, NULL );
			m_loadingComplete=false;
		}
	}

	core(m_pWebFrame)->view()->setPageLoadingComplete(false);
	m_pWebFrame->GetFrame()->page()->setPageLoadingComplete(false);

	m_pWebFrame->GetFrame()->page()->navigationInstance()->setFrameView(core(m_pWebFrame)->view());
	m_pWebFrame->GetFrame()->page()->navigationInstance()->resetPage();

#if( PLATFORM(SHP) && SFB_SMARTLINK_FEATURE )
	Frame* pFrame = m_pWebFrame->GetFrame();

	while(pFrame)
	{
		pFrame->GetSmartLinkMap()->StopIndexingSmartLinks();
		pFrame = pFrame->tree()->traverseNext();
	}
#endif

}

void WebFrameLoaderClient::postProgressEstimateChangedNotification()
{
	__FuncEntered();
 //   WebKitShpPage* kitPage = getPageFromFrame(m_frame);
	//Page* corePage = core(kitPage);
	//if(kitPage->m_progressHandler)
	//	(kitPage->m_progressHandler) (lround(corePage->progress()->estimatedProgress()*100));
	WebControl *pWebControl = m_pWebFrame->GetWebControl();

	if (pWebControl->isExitInstanceCalled())
		return;

	if( pWebControl && m_pWebFrame->IsMainFrame() )
	{
		IContentsListener *pContentsListener = pWebControl->GetContentListener();
		if( pContentsListener )
		{
			Page* pPage = pWebControl->GetPage();
			int progress = lround( pPage->progress()->estimatedProgress() * 100 );
			pContentsListener->state( STATE_PROGRESS, progress , NULL );
			m_loadingComplete=false;
			
//#ifdef _SFB_PAGE_PREVIEW - Pramod 23/July/08
#if defined (_SFB_PAGE_PILOT_FEATURE)
			//Added for Page pilot to launch during page loading - Pramod 22/July/'08
			if(true == core(m_pWebFrame)->loader()->isLoadingMainFrame())
			{
			if(pPage->mainFrame()->view()->contentsWidth() > (2 * pPage->mainFrame()->view()->visibleWidth()) && (pWebControl->getPagePilotFlagFirstRendering() == FALSE))
			{
				if(pWebControl->isMapViewMode() == FALSE)
				{
//					pWebControl->pagePilotScroll(0,0);
					
					if(pWebControl->setMapViewMode(TRUE))
						pWebControl->setPagePilotFlagFirstRendering( TRUE );
				}

			}
	}
#endif // _SFB_PAGE_PILOT_FEATURE
//#endif
		}
	}

}

void WebFrameLoaderClient::stopLingerTimerAfterAJAX()
{
	WebControl *pWebControl = m_pWebFrame->GetWebControl();
	pWebControl->stopLingerTimerAfterAjaxRequest();
	return;
}

void WebFrameLoaderClient::postProgressFinishedNotification()
{
	__FuncEntered();
 //   WebKitShpPage* page = getPageFromFrame(m_frame);
	//page->isLoadingDone = true;
	WebControl *pWebControl = m_pWebFrame->GetWebControl();

	if (pWebControl->isExitInstanceCalled())
		return;

	if( pWebControl && m_pWebFrame->IsMainFrame() )
	{
#if defined(_SFB_BACKINGSTORE)
		if( m_policyActionFormMimeType == PolicyUse && m_bFrameLoaderMimeTypeDecision )
		pWebControl->GetBackingStore()->PageLoadFinished();
#endif

		IContentsListener *pContentsListener = pWebControl->GetContentListener();
		if( pContentsListener )
		{
			pContentsListener->state( STATE_FINISH, 0, NULL );
			m_loadingComplete=true;

//#ifdef _SFB_PAGE_PREVIEW - Pramod 23/July/08
#if defined (_SFB_PAGE_PILOT_FEATURE)
			//Added for Page pilot to launch during page loading - Pramod 22/July/'08
			pWebControl->setPagePilotFlagFirstRendering( FALSE );
			pWebControl->setResetPagePilotFlag( FALSE );
#endif // _SFB_PAGE_PILOT_FEATURE
//#endif
		}
	}

	Page * page = m_pWebFrame->GetWebControl()->GetPage() ;

	m_pWebFrame->GetWebControl()->SetIsLoadingDone( true );
	page->setPageLoadingComplete(true);
	FrameView* fView = static_cast<FrameView*>(core(m_pWebFrame)->view());
	Document* doc = 0 ;
	if (fView)
		fView->frame()->document();
	if (doc)
	{
		WebCore::Node* body = doc->body();
        if (body && body->renderer()) 
		{
			RenderObject* renderer = body->renderer();
			if (renderer->isFrameSet())
			{
				fView->setNeedsLayout();
			}
		}
	}

	if (fView)	{
	core(m_pWebFrame)->view()->setPageLoadingComplete(true);
	core(m_pWebFrame)->view()->update();
	}
	core(m_pWebFrame)->loadOnUserGesture(false);

	//update stats as page load is complete
	//update horizontal stats
	if (m_pWebFrame->GetFrame()->page())	{
	m_pWebFrame->GetFrame()->page()->navigationInstance()->updateHorizontalStats();
	//update vertical stats
	m_pWebFrame->GetFrame()->page()->navigationInstance()->updateVerticalStats();
	
	//inform virtual cursor that page loading has been finished
	m_pWebFrame->GetFrame()->page()->navigationInstance()->pageLoadingFinished();
	}

#ifdef SFB_SMARTLINK_FEATURE 

	Frame* pFrame = m_pWebFrame->GetFrame();

	while(pFrame)
	{
#ifdef MULTIPLE_INSTANCE_SUPPORT
		SFBalSendEvent(BAL_APP_BROWSER,BAL_EVENT_BROWSER_SMARTLINK, NULL,(int)pFrame, NULL, m_pWebFrame->GetWebControl());
#else
		SFBalSendEvent(BAL_APP_BROWSER,BAL_EVENT_BROWSER_SMARTLINK, NULL, (int)pFrame, NULL);
#endif//MULTIPLE_INSTANCE_SUPPORT
		pFrame = pFrame->tree()->traverseNext();
	}

#endif //SFB_SMARTLINK_FEATURE

}

void WebFrameLoaderClient::frameLoaderDestroyed()
{
	__FuncEntered();
    m_pWebFrame = NULL;
    delete this;
}

void WebFrameLoaderClient::dispatchDidReceiveResponse(DocumentLoader* loader, unsigned long, const ResourceResponse& response)
{
	__FuncEntered();
    m_response = response;
    //m_firstData = true;
	FrameLoader *fl = loader->frameLoader();
	WebCore::String scheme(response.url().protocol()) ;
	WebControl* iControl = kit(fl->frame()->page());
	EHttpState isHttps = EState_HTTP;
	bool stateChanged = false ;
	
	StateChangeInfo state_info = iControl->GetStateChangeInfo();
    if(scheme.startsWith("http", false) || scheme.startsWith("https", false ))
	{
		if( scheme.startsWith("https", false ))
			isHttps = EState_HTTPS;
		if (loader->isLoadingMainResource() == true && fl->isLoadingMainFrame())	{
			state_info.m_stateChange = false ;
		}
		if(iControl->GetContentListener())
		{
			if(state_info.presentMainFrame != isHttps && loader->isLoadingMainResource() == true && fl->isLoadingMainFrame())
			{
				state_info.m_prevMainFrame = state_info.presentMainFrame;
				if (isHttps)
					state_info.presentMainFrame = EState_HTTPS ;
				else
					state_info.presentMainFrame = EState_HTTP ;
				state_info.m_stateChange = false ;
				stateChanged = true ;
				iControl->SetStateChangeInfo(state_info);
			}
			//else if(state_info.m_prevHttpInclusionState!= isHttps && state_info.m_stateChange == false )
			else if(state_info.presentMainFrame!= isHttps && state_info.m_stateChange == false )
			{
				state_info.presentMainFrame = EState_MIXED ;
				state_info.m_stateChange = true;
				stateChanged = true ;
				iControl->SetStateChangeInfo(state_info);
			}
		}
	}
	if (stateChanged)	{
		switch (state_info.presentMainFrame)	{
			case EState_HTTPS:
				iControl->GetContentListener()->state(STATECHANGE_SECURE,1,NULL);
				break ;
			case EState_HTTP:
				if (EState_MIXED == state_info.m_prevMainFrame)
					iControl->GetContentListener()->state(STATECHANGE_SECURE_INCLUSION,0,NULL);
				else // this means previous was Secure
					iControl->GetContentListener()->state(STATECHANGE_SECURE,0,NULL);
				break ;
			case EState_MIXED:
				if (EState_HTTPS == state_info.m_prevMainFrame)
					iControl->GetContentListener()->state(STATECHANGE_SECURE,1,NULL);
				else // this means previous was Secure
					iControl->GetContentListener()->state(STATECHANGE_SECURE_INCLUSION,1,NULL);
				break ;
		}
	}
	m_pWebFrame->GetWebControl()->SetIsLoadingDone( false );

}


WebFrameDecisionSender* 
WebFrameLoaderClient::setupDecisionSender( FramePolicyFunction policyFuntion, bool bMimeTypeDecision )
{
	m_bFrameLoaderMimeTypeDecision = bMimeTypeDecision;
	return new WebFrameDecisionSender( this, policyFuntion );
}

void WebFrameLoaderClient::dispatchDecidePolicyForMIMEType(FramePolicyFunction policyFunction, const WebCore::String& mime, const ResourceRequest& request)
{
	__FuncEntered();
 //    FIXME: we need to call directly here (comment copied from Qt version)
    //ASSERT(policyFunction);
    //if (!policyFunction)
    //    return;
    //(core(m_pWebFrame)->loader()->*policyFunction)(PolicyUse);


	WebControl *pWebControl = m_pWebFrame->GetWebControl();
	IDecisionListener *pDecisionListener = pWebControl->GetDecisionListener();
	if( !pDecisionListener )
		// [todo] implement DefaultDecisionListener
		//pDecisionListener = DefaultDecisionListener::sharedInstance(); 
		(core(m_pWebFrame)->loader()->*policyFunction)(PolicyUse);
	else
	{	
		bool isMainFrame = m_pWebFrame->IsMainFrame();
		pDecisionListener->DecisionMimeListener( (char*)mime.latin1().data(), NULL, 0,  setupDecisionSender( policyFunction, true  ), isMainFrame  );
	}

}

void WebFrameLoaderClient::DecidedPolicyForMIMEType( WebCore::PolicyAction action )
{
	m_policyActionFormMimeType = action;
	
	if( m_policyActionFormMimeType == WebCore::PolicyUse && m_bFrameLoaderMimeTypeDecision)
	{
		WebControl *pWebControl = m_pWebFrame->GetWebControl();
		if( pWebControl && m_pWebFrame->IsMainFrame())
		{
	#if defined(_SFB_BACKINGSTORE)
		if( m_policyActionFormMimeType == PolicyUse && m_bFrameLoaderMimeTypeDecision)
			pWebControl->GetBackingStore()->PageLoadStarted();
	#endif
		}
	}
}

void WebFrameLoaderClient::dispatchDecidePolicyForNewWindowAction(FramePolicyFunction policyFunction, const NavigationAction&, const ResourceRequest&, const WebCore::String&)
{
	__FuncEntered();
    ASSERT(policyFunction);
    if (!policyFunction)
        return;
    // FIXME: I think Qt version marshals this to another thread so when we
    // have multi-threaded download, we might need to do the same
#if defined(JS_MULTIPLE_WINDOW_SUPPORT)
	(core(m_pWebFrame)->loader()->*policyFunction)(PolicyUse);
#else
    (core(m_pWebFrame)->loader()->*policyFunction)(PolicyIgnore);
#endif
}

void WebFrameLoaderClient::dispatchDecidePolicyForNavigationAction(FramePolicyFunction policyFunction, const NavigationAction&, const ResourceRequest& request)
{
	__FuncEntered();
    ASSERT(policyFunction);
    if (!policyFunction)
        return;

//#if defined(SHP_SUPPORT_WML)
//	KURL url = request.url();
//	DeprecatedString dProtocol = url.protocol();
//	WebCore::String protocol = dProtocol;
//	if((equalIgnoringCase("mailto",protocol)) ||
//	   (equalIgnoringCase("tel",protocol)) ||
//	   (equalIgnoringCase("wtai",protocol)))
//	{
//		//sfb_re_eng
//		//BrAppWTAIHandleScheme(url.url().ascii());
//		(core(m_pWebFrame)->loader()->*policyFunction)(PolicyIgnore);
//	}
//	else
//	{
//		(core(m_pWebFrame)->loader()->*policyFunction)(PolicyUse);
//	}
//#else
//    (core(m_pWebFrame)->loader()->*policyFunction)(PolicyUse);
//#endif

	WebControl *pWebControl = m_pWebFrame->GetWebControl();
	IDecisionListener *pDecisionListener = pWebControl->GetDecisionListener();
	if( !pDecisionListener )
		// [todo] implement DefaultDecisionListener
		//pDecisionListener = DefaultDecisionListener::sharedInstance(); 
		(core(m_pWebFrame)->loader()->*policyFunction)(PolicyUse);
	else
	{
		KURL url = request.url();

#if defined( JS_MULTIPLE_WINDOW_SUPPORT )
		if (!pWebControl->isWebControlCloseWindowFired())
		{
		bool isMainFrame = core(m_pWebFrame)->isLoadOnUserGesture();

		//use the existing policy when we have already fired a timer for deleting the current Window (Scenario: <a onclick="window.close" href="#close" >)
			if( isMainFrame)
		pDecisionListener->DecisionUrlListener( (char*)(url.string().latin1().data()), setupDecisionSender( policyFunction, false )  );
		else
			(core(m_pWebFrame)->loader()->*policyFunction)(PolicyUse);
		}
		else
			//Ignore load requests after a window.close() is fired.
			(core(m_pWebFrame)->loader()->*policyFunction)(PolicyIgnore);
#else
		pDecisionListener->DecisionUrlListener( (char*)(url.string().latin1().data()), setupDecisionSender( policyFunction, false )  );
#endif



	}
	
}

Widget* WebFrameLoaderClient::createPlugin(const IntSize& pluginSize, Element* e, const KURL& url, const Vector<WebCore::String>& paramNames, const Vector<WebCore::String>& paramValues, const WebCore::String& mimeType, bool loadManually)
{
	__FuncEntered();
#ifdef SUPPORT_NP_PLUGIN

	PluginView* pluginView = PluginView::create(core(m_pWebFrame), pluginSize, e, url, paramNames, paramValues, mimeType, loadManually);

	return pluginView;

#else // SUPPORT_NP_PLUGIN
    __NotImplemented();
    return 0;
#endif // SUPPORT_PLUGIN
}

PassRefPtr<Frame> WebFrameLoaderClient::createFrame(const KURL& url, const WebCore::String& name, HTMLFrameOwnerElement* ownerElement,
                                        const WebCore::String& referrer, bool allowsScrolling, int marginWidth, int marginHeight)
{
	__FuncEntered();
    Frame* coreFrame = core(m_pWebFrame);

	
    ASSERT(core(m_pWebFrame->GetWebControl()) == coreFrame->page());

	WebFrame *pNewWebFrame = NULL;
	HRESULT hResult = WebFrame::New( m_pWebFrame->GetWebControl(), ownerElement, &pNewWebFrame );
	if( HRESULT_ISFAIL(hResult) ) 
		return NULL;


    Frame* childFrame = core(pNewWebFrame);

    /* http://trac.webkit.org/changeset/35088
	 * return if page is empty
	 */
	if (!childFrame->page())
		return 0;


    // Frames are created with a refcount of 1. Release this ref, since we've assigned it to d->frame.
    childFrame->deref();
    childFrame->tree()->setName(name);
    childFrame->init();
    childFrame->loader()->load(url, referrer, FrameLoadTypeRedirectWithLockedHistory, WebCore::String(), 0, 0);

    // The frame's onload handler may have removed it from the document.
    if (!childFrame->tree()->parent())
        return 0;

    // Propagate the marginwidth/height and scrolling modes to the view.
    if (ownerElement->hasTagName(HTMLNames::frameTag) || ownerElement->hasTagName(HTMLNames::iframeTag)) {
        HTMLFrameElement* frameElt = static_cast<HTMLFrameElement*>(ownerElement);
        if (frameElt->scrollingMode() == ScrollbarAlwaysOff)
            childFrame->view()->setScrollbarsMode(ScrollbarAlwaysOff);
        int marginWidth = frameElt->getMarginWidth();
        int marginHeight = frameElt->getMarginHeight();
        if (marginWidth != -1)
            childFrame->view()->setMarginWidth(marginWidth);
        if (marginHeight != -1)
            childFrame->view()->setMarginHeight(marginHeight);
    }

    return childFrame;
}

void WebFrameLoaderClient::deleteWebFrame()
{
    if(m_pWebFrame != m_pWebFrame->GetWebControl()->GetMainWebFrame())
	delete m_pWebFrame;
}

void WebFrameLoaderClient::redirectDataToPlugin(Widget* pluginWidget)
{
	m_pluginView = static_cast<PluginView*>(pluginWidget);
}

Widget* WebFrameLoaderClient::createJavaAppletWidget(const IntSize&, Element*, const KURL& baseURL,
                                                    const Vector<WebCore::String>& paramNames, const Vector<WebCore::String>& paramValues)
{
    __NotImplemented();
    return 0;
}

ObjectContentType WebFrameLoaderClient::objectContentType(const KURL& url, const WebCore::String& mimeType)
{
	WebCore::String type = mimeType;
	
	if (type.isEmpty())
	{
		type = MIMETypeRegistry::getMIMETypeForExtension(url.string().substring(url.string().reverseFind('.') + 1));		
	}

	if (type.isEmpty())
	{
		// Go ahead and hope that we can display the content.
		return ObjectContentFrame;
	}

    if (MIMETypeRegistry::isSupportedImageMIMEType(type))
    {
        return ObjectContentImage;
    }

	if (PluginDatabase::installedPlugins()->isMIMETypeRegistered(type))
	{
		return ObjectContentNetscapePlugin;
	}

	if (MIMETypeRegistry::isSupportedNonImageMIMEType(type))
	{
		return ObjectContentFrame;
	}

    return ObjectContentType();
}

WebCore::String WebFrameLoaderClient::overrideMediaType() const
{
    __NotImplemented();
    return WebCore::String();
}

void WebFrameLoaderClient::windowObjectCleared() 
{
	__NotImplemented();
}

void WebFrameLoaderClient::setMainFrameDocumentReady(bool)
{
    // this is only interesting once we provide an external API for the DOM
	__NotImplemented();
}

bool WebFrameLoaderClient::hasWebView() const
{
    __NotImplemented();
    return true;
}

bool WebFrameLoaderClient::hasFrameView() const
{
    __NotImplemented();
    return true;
}

void WebFrameLoaderClient::dispatchDidFinishLoad()
{
    __NotImplemented();
}

void WebFrameLoaderClient::frameLoadCompleted()
{
    __NotImplemented();
}

void WebFrameLoaderClient::saveViewStateToItem(HistoryItem* item )
{
#if defined(_SFB_BACKINGSTORE)
	WebControl *pWebControl = m_pWebFrame->GetWebControl();
    //__NotImplemented();
	if( pWebControl /*&& m_pWebFrame->IsMainFrame()*/ )
	{
		if( m_policyActionFormMimeType == PolicyUse && m_bFrameLoaderMimeTypeDecision )
		pWebControl->GetBackingStore()->SaveViewState(item);
	} 
#endif
}

void WebFrameLoaderClient::restoreViewState()
{
#if defined(_SFB_BACKINGSTORE)
	WebControl *pWebControl = m_pWebFrame->GetWebControl();
   // __NotImplemented();
	if( pWebControl /*&& m_pWebFrame->IsMainFrame()*/ )
	{
		if( m_policyActionFormMimeType == PolicyUse && m_bFrameLoaderMimeTypeDecision )
		pWebControl->GetBackingStore()->RestoreViewState();
	}
#endif
}

#ifdef SFB_UPDATE_URL_BAR  //Pramod - Bresson Model - update Url ComboBox bar
WebCore::String WebFrameLoaderClient::setLinkUrl(const WebCore::KURL& url ) 
{
	WebControl *pWebControl = m_pWebFrame->GetWebControl();
	if( pWebControl )
	{
		IContentsListener *pContentsListener = pWebControl->GetContentListener();
		if( pContentsListener )
		{
			pContentsListener->SetLinkInBrAppState((char*)url.string().latin1().data());
			return url.string();
		}
	}
}
#endif

bool WebFrameLoaderClient::shouldGoToHistoryItem(HistoryItem* item) const
{
    // FIXME: This is a very simple implementation. More sophisticated
    // implementation would delegate the decision to a PolicyDelegate.
    // See mac implementation for example.
    return item != 0;
}

bool WebFrameLoaderClient::privateBrowsingEnabled() const { __NotImplemented(); return false; }
void WebFrameLoaderClient::makeDocumentView() { __NotImplemented(); }
void WebFrameLoaderClient::makeRepresentation(DocumentLoader*) { __NotImplemented(); }
void WebFrameLoaderClient::forceLayout() { __NotImplemented(); }
void WebFrameLoaderClient::forceLayoutForNonHTML() { __NotImplemented(); }
void WebFrameLoaderClient::setCopiesOnScroll() { __NotImplemented(); }
void WebFrameLoaderClient::detachedFromParent1() { __NotImplemented(); }
void WebFrameLoaderClient::detachedFromParent2() { __NotImplemented(); }
void WebFrameLoaderClient::detachedFromParent3() { __NotImplemented(); }
void WebFrameLoaderClient::detachedFromParent4() { __NotImplemented(); }
void WebFrameLoaderClient::loadedFromCachedPage() { __NotImplemented(); }
void WebFrameLoaderClient::dispatchDidHandleOnloadEvents() {__NotImplemented(); }
void WebFrameLoaderClient::dispatchDidReceiveServerRedirectForProvisionalLoad() { __NotImplemented(); }
void WebFrameLoaderClient::dispatchDidCancelClientRedirect() { __NotImplemented(); }
void WebFrameLoaderClient::dispatchWillPerformClientRedirect(const KURL&, double, double) { __NotImplemented(); }
void WebFrameLoaderClient::dispatchDidChangeLocationWithinPage() { __NotImplemented(); }
void WebFrameLoaderClient::dispatchWillClose() { __NotImplemented(); }

void WebFrameLoaderClient::dispatchDidReceiveIcon()
{
	__NotImplemented();
}

void WebFrameLoaderClient::dispatchDidStartProvisionalLoad()
{
	__FuncEntered();
    //WebKitShpPage* page = getPageFromFrame(m_frame);
	//page->isLoadingDone = false;
	m_pWebFrame->GetWebControl()->SetIsLoadingDone( false );
}

void WebFrameLoaderClient::dispatchDidReceiveTitle(const WebCore::String& title)
{
    __NotImplemented();
}

void WebFrameLoaderClient::dispatchDidCommitLoad() 
{
	WebControl *pWebControl = m_pWebFrame->GetWebControl();

	if(pWebControl)
		pWebControl->updateFocusedAndActiveState();

#if defined(_SFB_BACKINGSTORE)
	//__NotImplemented(); 
	if( pWebControl && m_pWebFrame->IsMainFrame() )
	{
		if( m_policyActionFormMimeType == PolicyUse && m_bFrameLoaderMimeTypeDecision )
		pWebControl->GetBackingStore()->ResetViewState();
	}
#endif

 //#ifdef _SFB_PAGE_PREVIEW - Pramod 23/July/08
#if defined (_SFB_PAGE_PILOT_FEATURE)
	WebControl *pWebControl = m_pWebFrame->GetWebControl();
	if( pWebControl && !pWebControl->getResetPagePilotFlag())
	{
		if(true == core(m_pWebFrame)->loader()->isLoadingMainFrame())
		{
		pWebControl->pagePreviewCancel(FALSE);
		pWebControl->setResetPagePilotFlag( TRUE );
		}
	}
#endif //_SFB_PAGE_PILOT_FEATURE
//#endif
 }

void WebFrameLoaderClient::dispatchDidFinishDocumentLoad() { __NotImplemented(); }
void WebFrameLoaderClient::dispatchDidFirstLayout() 
{

#if defined(_SFB_BACKINGSTORE)
	if( m_pWebFrame->IsMainFrame() )
	{
		WebControl *pWebControl = m_pWebFrame->GetWebControl();
		if (pWebControl->isExitInstanceCalled())
			return;
		if( pWebControl &&  m_policyActionFormMimeType == PolicyUse && m_bFrameLoaderMimeTypeDecision)
			pWebControl->GetBackingStore()->DispatchDidFirstLayout();

	}
#endif

}
void WebFrameLoaderClient::dispatchShow() { __NotImplemented(); }
void WebFrameLoaderClient::cancelPolicyCheck() { __NotImplemented(); }
void WebFrameLoaderClient::dispatchDidLoadMainResource(DocumentLoader* loader) 
{ 
#if defined(_SFB_BACKINGSTORE)
	if( m_pWebFrame->IsMainFrame() )
	{
		WebControl *pWebControl = m_pWebFrame->GetWebControl();

		if( pWebControl && m_policyActionFormMimeType == PolicyUse && m_bFrameLoaderMimeTypeDecision )
			pWebControl->GetBackingStore()->DispatchDidLoadMainResource(loader);

	}
#endif
}
void WebFrameLoaderClient::revertToProvisionalState(DocumentLoader*) { __NotImplemented(); }
void WebFrameLoaderClient::clearUnarchivingState(DocumentLoader*) { __NotImplemented(); }
void WebFrameLoaderClient::willChangeTitle(DocumentLoader*) { __NotImplemented(); }
void WebFrameLoaderClient::didChangeTitle(DocumentLoader *l) { setTitle(l->title(), l->url()); }
void WebFrameLoaderClient::finalSetupForReplace(DocumentLoader*) { __NotImplemented(); }
void WebFrameLoaderClient::setDefersLoading(bool) { __NotImplemented(); }
bool WebFrameLoaderClient::isArchiveLoadPending(ResourceLoader*) const { __NotImplemented(); return false; }
void WebFrameLoaderClient::cancelPendingArchiveLoad(ResourceLoader*) { __NotImplemented(); }
void WebFrameLoaderClient::clearArchivedResources() { __NotImplemented(); }
bool WebFrameLoaderClient::canHandleRequest(const ResourceRequest&) const { __NotImplemented(); return true; }
bool WebFrameLoaderClient::canShowMIMEType(const WebCore::String& mimeType, const KURL& m_Url ) const 
{ 
    if (MIMETypeRegistry::isUnSupportedMIMEType(mimeType))
    {
        WebControl *pWebControl = m_pWebFrame->GetWebControl();
        if( pWebControl )
        {
            IContentsListener *pContentsListener = pWebControl->GetContentListener();
	        if( pContentsListener )
            {
		        pContentsListener->error( ERROR_CONTENTDOWNLOAD_UNSUPPORTED_MIMETYPE,
										  (char*)m_Url.string().latin1().data(),
										  NULL );
            }

        }
        return false;
    }
    return true;
}
bool WebFrameLoaderClient::representationExistsForURLScheme(const WebCore::String&) const { __NotImplemented(); return false; }
WebCore::String WebFrameLoaderClient::generatedMIMETypeForURLScheme(const WebCore::String&) const { __NotImplemented(); return WebCore::String(); }

void WebFrameLoaderClient::finishedLoading(DocumentLoader* loader)
{
	// Telling the frame we received some data and passing 0 as the data is our
    // way to get work done that is normally done when the first bit of data is
    // received, even for the case of a document with no data (like about:blank)
    if (!m_pluginView)
	{
        committedLoad(loader, 0, 0);
        return;
    }

    if (m_pluginView->status() == PluginStatusLoadedSuccessfully)
        m_pluginView->didFinishLoading();

    m_pluginView = 0;
    m_hasSentResponseToPlugin = false;
}

void WebFrameLoaderClient::provisionalLoadStarted()
{
    __NotImplemented();
}

void WebFrameLoaderClient::didFinishLoad() {
    __NotImplemented();
}

void WebFrameLoaderClient::prepareForDataSourceReplacement() { __NotImplemented(); }

void WebFrameLoaderClient::setTitle(const WebCore::String& title, const KURL& url)
{
    __NotImplemented();
 //   WebKitShpPage* page = getPageFromFrame(m_frame);

 //   CString titleString = title.utf8();
 //   DeprecatedCString urlString = url.prettyURL().utf8();
	//if(page->m_titleChangedHandler)
	//	(page->m_titleChangedHandler)(titleString.data(), urlString.data());
}

void WebFrameLoaderClient::setDocumentViewFromCachedPage(WebCore::CachedPage*) { __NotImplemented(); }
void WebFrameLoaderClient::dispatchDidReceiveContentLength(DocumentLoader*, unsigned long  identifier, int lengthReceived) { __NotImplemented(); }
void WebFrameLoaderClient::dispatchDidFinishLoading(DocumentLoader* loader, unsigned long  identifier)
{
	WebControl *pWebControl = m_pWebFrame->GetWebControl();
	if( pWebControl  )
	{
		IContentsListener *pContentsListener = pWebControl->GetContentListener();
		if( pContentsListener )
		{
			pContentsListener->didFinishLoading(identifier);
		}
	}
}
void WebFrameLoaderClient::dispatchDidFailLoading(DocumentLoader*, unsigned long  identifier, const ResourceError&) { __NotImplemented(); }
bool WebFrameLoaderClient::dispatchDidLoadResourceFromMemoryCache(DocumentLoader*, const ResourceRequest&, const ResourceResponse&, int length) { __NotImplemented(); return false; }

void WebFrameLoaderClient::dispatchDidFailProvisionalLoad(const ResourceError& error)
{
	__FuncEntered();

	// in case of policychangeerror, do not notify error to the listener
	if( error.domain() == POLICYCHANGE_ERROR )
		return;

	if( error.errorCode() == USER_CANCELLED_ERROR)
		return;

	// skip passing this error to UI as this is a success after handling the external URL Scheme.
	if( error.errorCode() == HRESULT_URL_SCHEME_HANDLED )
		return;
		
	WebControl *pWebControl = m_pWebFrame->GetWebControl();
    
    // Fix for Bug SFB00001910 
    // Ignore errors if not main frame
    if (m_pWebFrame->GetFrame() != pWebControl->GetPage()->mainFrame())
        return;

	if( pWebControl )
	{
		IContentsListener *pContentsListener = pWebControl->GetContentListener();
		if( pContentsListener )
			pContentsListener->error( error.errorCode(), (char*)(error.failingURL().latin1().data()), NULL );
	}
}

void WebFrameLoaderClient::dispatchDidFailLoad(const ResourceError& error )
{
	__FuncEntered();

	// in case of policychangeerror, do not notify error to the listener
	if( error.domain() == POLICYCHANGE_ERROR )
		return;

	if( error.errorCode() == USER_CANCELLED_ERROR)
		return;

	WebControl *pWebControl = m_pWebFrame->GetWebControl();

    // Fix for Bug SFB00001910 
    // Ignore errors if not main frame
    if (m_pWebFrame->GetFrame() != pWebControl->GetPage()->mainFrame())
        return;

	if( pWebControl )
	{
		IContentsListener *pContentsListener = pWebControl->GetContentListener();
		if( pContentsListener )
			pContentsListener->error( error.errorCode(), (char*)(error.failingURL().latin1().data()), NULL );
	}
}

void WebFrameLoaderClient::download(IWebClientBridge* bridge, ResourceLoader* resLoader, const ResourceRequest& request, const ResourceRequest& request2, const ResourceResponse& response ) 
{
	__FuncEntered();

	// redirect data stream to download engine
	WebDownload *pWebDownload = new WebDownload( m_pWebFrame->GetWebControl(), request, response );

	if( pWebDownload == NULL )
	{
		SysDebug(( MID_EXCEPTION, "WebFrameLoaderClient::download :  pWebDownload == NULL\n" ));
		return;
	}

    // Stop the shared timer before downloading since it might fire if the user delays the click
    // on download message box
    WebCore::stopSharedTimer();

	HRESULT hr = pWebDownload->RedirectStream( bridge, resLoader );
    
    // Restart the timer
    WebCore::setSharedTimerFireTime(0);
	if( HRESULT_ISFAIL(hr) )
	{
		SysDebug(( MID_EXCEPTION, "WebFrameLoaderClient::download :  RedirectStream failed\n" ));
		delete pWebDownload;
		return;
	}

}

void WebFrameLoaderClient::download(ResourceHandle*, const ResourceRequest&, const ResourceRequest&, const ResourceResponse&) { __NotImplemented(); }
ResourceError WebFrameLoaderClient::cancelledError(const ResourceRequest&) 
{ 
	WebCore::String domain ;
	WebCore::String url ;
	WebCore::String desc ;
	ResourceError resource(domain, USER_CANCELLED_ERROR, url, desc); 
	return resource ;
}

ResourceError WebFrameLoaderClient::blockedError(const ResourceRequest&) { __NotImplemented(); return ResourceError(); }
ResourceError WebFrameLoaderClient::cannotShowURLError(const ResourceRequest&) { __NotImplemented(); return ResourceError(); }
ResourceError WebFrameLoaderClient::interruptForPolicyChangeError(const ResourceRequest& request) 
{ 
		__FuncEntered();
	// [todo] need to define error codes
	return ResourceError(WebCore::String(POLICYCHANGE_ERROR), -1, request.url().string(), WebCore::String());
}

ResourceError WebFrameLoaderClient::cannotShowMIMETypeError(const ResourceResponse& r)
{
    WebControl *pWebControl = m_pWebFrame->GetWebControl();
    if( pWebControl )
    {
        IContentsListener *pContentsListener = pWebControl->GetContentListener();
        if( pContentsListener )
        {
	        pContentsListener->error( ERROR_CONTENTDOWNLOAD_UNSUPPORTED_MIMETYPE,
									  (char*)r.url().string().latin1().data(),
									  NULL );
        }

    }

    return ResourceError();
}
ResourceError WebFrameLoaderClient::fileDoesNotExistError(const ResourceResponse&) { __NotImplemented(); return ResourceError(); }
bool WebFrameLoaderClient::shouldFallBack(const ResourceError&) { __NotImplemented(); return false; }
bool WebFrameLoaderClient::willUseArchive(ResourceLoader*, const ResourceRequest&, const KURL& originalURL) const { __NotImplemented(); return false; }
void WebFrameLoaderClient::saveDocumentViewToCachedPage(CachedPage*) { __NotImplemented(); }
bool WebFrameLoaderClient::canCachePage() const { __NotImplemented(); return false; }
Frame* WebFrameLoaderClient::dispatchCreatePage() 
{ 
#if defined(JS_MULTIPLE_WINDOW_SUPPORT)
	/*WebFrameLoaderClient::dispatchCreatePage is for formsubmission using Get/Post method.	
	 * If framename is _blank we have to create new window, otherwise if _self or something is 
	 * present we have to load in same window. Control comes here when we are creating a new window without having to load anything.
	 * Note: for window.open control doesnt come here. Rather it goes to WebChromeClient::createWindow */
	WebControl *pWebControl = m_pWebFrame->GetWebControl();
	
	// Partho - CQ 3561 n 3604 Fix :: Downloading 
	bool bUserGesture = m_pWebFrame->GetFrame()->isLoadOnUserGesture();
	
	if( pWebControl->GetContentListener() )
	{
		WebControl *pControl =  (WebControl*)pWebControl->GetContentListener()->createWebControl();
		if (pControl )
		{
			pControl->GetMainWebFrame()->GetFrame()->loadOnUserGesture(bUserGesture);			// Partho - CQ 3561 n 3604 Fix :: Downloading 
			return pControl->GetMainWebFrame()->GetFrame();
		}
		else 
			return 0 ;
	}
	else
	{
		// by default, do nothing
		return 0; 
	}
#else
	__NotImplemented(); 
	return 0; 
#endif //JS_MULTIPLE_WINDOW_SUPPORT
}
void WebFrameLoaderClient::dispatchUnableToImplementPolicy(const ResourceError&) { __NotImplemented(); }
void WebFrameLoaderClient::setMainDocumentError(DocumentLoader*, const ResourceError&) { __NotImplemented(); }
void WebFrameLoaderClient::startDownload(const ResourceRequest&) { __NotImplemented(); }
void WebFrameLoaderClient::updateGlobalHistoryForStandardLoad(const KURL&) { __NotImplemented(); }
void WebFrameLoaderClient::updateGlobalHistoryForReload(const KURL&) { __NotImplemented(); }

void WebFrameLoaderClient::savePlatformDataToCachedPage(WebCore::CachedPage*)
{
    __NotImplemented();
}

void WebFrameLoaderClient::transitionToCommittedFromCachedPage(WebCore::CachedPage*)
{
    __NotImplemented();
}
void WebFrameLoaderClient::transitionToCommittedForNewPage()
{
    __NotImplemented();
}
void WebFrameLoaderClient::didPerformFirstNavigation() const
{
    __NotImplemented();
}

void WebFrameLoaderClient::registerForIconNotification(bool)
{
    __NotImplemented();
}



#if defined ( SHP_SUPPORT_WML )
void WebFrameLoaderClient::delegateDoElementToUserMenu ( int id, WebCore::String displayString)
{
	    __NotImplemented();
	//sfb_re_eng
	//BrCoreControl_WMLButtonData ( id, (MCHAR *)displayString.charactersWithNullTermination(), displayString.length() ) ; 
}
void WebFrameLoaderClient::removeDoElementinUserMenu (int id )
{
	    __NotImplemented();
	//sfb_re_eng
	//BrAppWmlTextData ( id ) ;
}
//NATARAJ 16th MAY SFB00001083 
int WebFrameLoaderClient::GetClientId ()
{
	WebControl *pWebControl = m_pWebFrame->GetWebControl();
	if(pWebControl)
		return (int)pWebControl->GetLoadGroup();
	else
		return -1;
}
void WebFrameLoaderClient::errorMsg()
{
	WebControl *pWebControl = m_pWebFrame->GetWebControl();
	if(pWebControl)
	{
		//To do : the string should be obtained from the native resources strings
		pWebControl->pFuncHandler->getWmlAlertFuncHandler()("No Homepage URL") ;
	}	
}
 
void WebFrameLoaderClient::errorMsg(const char *errMsg)
{
	WebControl *pWebControl = m_pWebFrame->GetWebControl();
	if(pWebControl)
	{
		//To do : the string should be obtained from the native resources strings
		pWebControl->pFuncHandler->getWmlAlertFuncHandler()(errMsg) ;
	}	
}

 
void WebFrameLoaderClient::alertMsg()
{
	WebControl *pWebControl = m_pWebFrame->GetWebControl();
	if(pWebControl)
	{
		//To do : the string should be obtained from the native resources strings
		pWebControl->pFuncHandler->getWmlAlertFuncHandler()("No Pages Backward") ;
	}	
}
# endif// SHP_SUPPORT_WML


void WebFrameLoaderClient::showErrorMessageBox(const char *errMsg)
{
	    __NotImplemented();
	//sfb_re_eng
	//BrAppWindowsCreateErrorMsgbox(errMsg);
}
# if SFB_WIDGET
void WebFrameLoaderClient::DeActivateNativeInstance()
{
#ifdef USE_NATIVE_WIDGET_SUPPORT
	NativeEditField* pNativeEdit = NativeEditField::getNativeEditFieldInstance();
	NativeEditField::getNativeEditFieldInstance()->deactivate();
	pNativeEdit->m_isEditFeild = false;
	NativeComboField::getNativeComboFieldInstance()->deactivate();
#endif
}
# endif


#ifdef PLATFORM(SHP) // Don't Remove this flag , to make it consistent with its prototype definition
int	 WebFrameLoaderClient::getProtoAccountHandle()
{
	WebControl *pWebControl = m_pWebFrame->GetWebControl();
	if(pWebControl)
	{
		SFBRmHClient hClient = reinterpret_cast<SFBRmHClient>(pWebControl->GetLoadGroup());
		if(hClient != INVALID_HOBJ)
		{
			return SFBRmGetHProtoAccountHandle( hClient );
		}
	}
	return INVALID_HOBJ;
}
#endif

# if PLATFORM(SHP)
# ifdef SFB_WIDGET
// Fix B0100142435 amitg@samsung.com
 void WebFrameLoaderClient::SetExposeEvent(bool bSet )
 {
	if (m_pWebFrame)
	{
		WebControl *pWebControl = m_pWebFrame->GetWebControl();
		if (pWebControl)
			pWebControl->setExposeEvent(bSet);
	}
 }
 bool WebFrameLoaderClient::GetExposeEvent()
 {
	if (m_pWebFrame)
	{
		WebControl *pWebControl = m_pWebFrame->GetWebControl();
		if (pWebControl)
			return pWebControl->isExposeEventPresent();
	}
	return false;
 }

# endif

# endif

}
