/*
 * Copyright (c) S/W Platform, Samsung Electronics, Inc.
 * All rights reserved.
 *
 * This software is the confidential and proprietary information
 * of Samsung Electronics, Inc. ("Confidential Information").  You
 * shall not disclose such Confidential Information and shall use
 * it only in accordance with the terms of the license agreement
 * you entered into with Samsung Electronics. 
 */
/** 
 * @file		WebDOMCore.h
 * @brief		implement DOM level 2 Core
 * @author	Sangheon Kim (sangheon77.kim@samsung.com)
 * @version	1.0
 * @date		2008.03.25	v1.0		Created
 */
#ifndef __WebDOMCore_Defined
#define __WebDOMCore_Defined


#include "WebControl.h"

namespace	WebControl
{

class DOMNode : public IDOMNode
{
public:
	static IDOMNode* New( WebCore::Node* pNode );
	DOMNode( WebCore::Node* pNode );

private:
	WebCore::Node* m_pNode;

public:
	/*
	 * @desc gets valud of node
	 * @param out_pszBuffer
	 * @param in_nBufferLen
	 * @return HRESULT TRUE/FALSE
	 */
	virtual HRESULT getValue( char* out_pszBuffer, int in_nBufferLen );

	/*
	 * @desc gets name of node
	 * @param out_pszBuffer
	 * @param in_nBufferLen
	 * @return HRESULT TRUE/FALSE
	 */
	virtual HRESULT getName( char* out_pszBuffer, int in_nBufferLen );

	/*
	 * @desc gets the type of node
	 * @param
	 * @return DomNodeType
	 */
	virtual HRESULT getNodetype( DomNodeType *out_type );

	/*
	 * @desc checks whether the node has attribute
	 * @param
	 * @return HRESULT TRUE/FALSE
	 */
	virtual HRESULT hasAttribute();
	
        /*
	 * @desc gets the Webcore node
	 * @param
	 * @return WebCore::Node*
	 */
	WebCore::Node* getWebCoreNode();
};

class DOMElement : public IDOMElement
{
public:
	static IDOMElement* DOMElement::New(WebCore::Element* pElement );
	DOMElement( WebCore::Element* pElement );

private:
	WebCore::Element* m_pElement;
	DOMNode m_DOMNode; // use embedded DOMNode not to use mutiple inheritance

public:

	// DOMNode funtions
	virtual HRESULT getValue( char* out_pszBuffer, int in_nBufferLen );
	virtual HRESULT getName( char* out_pszBuffer, int in_nBufferLen );
	virtual HRESULT getNodetype( DomNodeType *out_type );
	virtual HRESULT hasAttribute();


	/*
	 * @desc gets the tag name
	 * @param out_pszBuffer
	 * @param in_nBufferLen
	 * @return HRESULT TRUE/FALSE
	 */
	virtual HRESULT getTagName( char* out_pszBuffer, int in_nBufferLen );
	/*
	 * @desc gets the DOM attribute
	 * @param out_pszBuffer
	 * @param in_nBufferLen
	 * @param in_pszName
	 * @return HRESULT TRUE/FALSE
	 */
	virtual HRESULT getAttribute( char* out_pszBuffer, int in_nBufferLen, char* in_pszName );
	/*
	 * @desc get IDOMAttr
	 * @param in_pszName
	 * @return HRESULT TRUE/FALSE
	 */
	virtual HRESULT getAttributeNode( char* in_pszName, IDOMAttr **out_ppDOMAttr );

};

class DOMDocument :public IDOMDocument
{
public:
	static IDOMDocument* New( WebCore::Document* pDoc );
	DOMDocument( WebCore::Document* pDoc );

protected:
	WebCore::Document* m_pDoc;
	DOMNode m_DOMNode;

public:
	// DOMNode funtions
	virtual HRESULT getValue( char* out_pszBuffer, int in_nBufferLen );
	virtual HRESULT getName( char* out_pszBuffer, int in_nBufferLen );
	virtual HRESULT getNodetype( DomNodeType *out_type );
	virtual HRESULT hasAttribute();

	virtual HRESULT getDocType( IDOMDocumentType **out_ppDocType );

};

} // namespace  WebControl



#endif // __WebDOMCore_Defined