/*
 * Copyright (c) S/W Platform, Samsung Electronics, Inc.
 * All rights reserved.
 *
 * This software is the confidential and proprietary information
 * of Samsung Electronics, Inc. ("Confidential Information").  You
 * shall not disclose such Confidential Information and shall use
 * it only in accordance with the terms of the license agreement
 * you entered into with Samsung Electronics. 
 */
/** 
 * @file		WebDownload.cpp
 * @brief		
 * @author	Sangheon Kim (sangheon77.kim@samsung.com)
 * @version	1.0
 * @date		2008.04.10	v1.0		Created
 */

#include "WebDownload.h"
#include "WebControl.h"

#include "PlatformString.h"
#include "CString.h"

#include "UrlResponseInfo.h"

namespace WebControl {





WebDownload::WebDownload( WebControl *pControl,
						 const WebCore::ResourceRequest& request, 
						 const WebCore::ResourceResponse& response)
{
	m_pWebControl = pControl;

	HTTPHeaderMap headersMap = response.httpHeaderFields();
	HTTPHeaderMap::iterator iter = headersMap.begin();
	HTTPHeaderMap::iterator endIter = headersMap.end();

	for( ; iter != endIter; ++iter )
	{
		m_headers += iter->first;
		m_headers += ": ";
		m_headers += iter->second;
		m_headers += "\r\n";
	}

	m_statusCode = response.httpStatusCode();
#if defined(_JASMINE_ECOM)
	using namespace URILoader;
	CreateIURI( (char*)(request.url().string().latin1().data()), &m_pURI );
#else
	URILoader::CreateIURI( (char*)(request.url().string().latin1().data()), &m_pURI );
#endif
	m_totalReceived = 0;
}

WebDownload::~WebDownload()
{
	if( m_pURI )
		delete m_pURI;
}

void WebDownloadContentCallback(
									SFBRmHClient hClient,
									SFBRmReqId reqID, 
									  void * appData, 
									  char* respBody, 
									  int respBodyLen, 
									  BOOL bLastChunk)
{

	WebDownload *pWebDownload = (WebDownload*)appData;

	SFBRmResetLingerTimer((SFBRmHClient)hClient); //AKG Reset Linger to avoid timeout during large content download. May have to be moved to CDDownloadMgr.cpp via IHttpChannel

	pWebDownload->OnData( respBody, respBodyLen );

	if( bLastChunk )
			pWebDownload->OnComplete();

}

void WebDownloadErrorCallback (
									SFBRmHClient hClient,
									SFBRmReqId reqID, 
									 void * appData, 
									 SFBStatus errorCode)
{
		WebDownload *pWebDownload = (WebDownload*)appData;

		pWebDownload->OnError( errorCode );
}


HRESULT WebDownload::RedirectStream( IWebClientBridge* bridge, ResourceLoader* resLoader )
{
	int transportHandle = 0;

	if( bridge->RedirectTransaction( resLoader, &transportHandle ) != ERR_OK )
	{
		SysDebug(( MID_EXCEPTION, "WebDownload::RedirectStream : RedirectTransaction failed\n" ));
		return HRESULT_NOK;
	}

	SFBRmResetAllCallbacks( (int)(m_pWebControl->GetLoadGroup()), 
							transportHandle); // B0100141049

	// redirect SFBRm's stream to WebDownload
	SFBRmSetAppData( (int)(m_pWebControl->GetLoadGroup()), 
							transportHandle, this );

	SFBRmSetContentCallback( (int)(m_pWebControl->GetLoadGroup()), 
							transportHandle,
							WebDownloadContentCallback );

	SFBRmSetErrorCallback( (int)(m_pWebControl->GetLoadGroup()), 
							transportHandle,
							WebDownloadErrorCallback );


	URILoader::IStreamListener* pStreamListener = m_pWebControl->GetDownloadListener( );

	if( pStreamListener == NULL )
	{
		SysDebug(( MID_EXCEPTION, "WebDownload::Response : RedirectStream == NULL\n" ));
		SFBRmRequestStop( (int)(m_pWebControl->GetLoadGroup()), 
							transportHandle );
		return HRESULT_NOK;
	}
	//stop cache operation for this download
	SFBCacheStopOperation(SFBRmGetCacheStorage((int)(m_pWebControl->GetLoadGroup())), transportHandle);

	m_bridge = bridge;
	m_resLoader = resLoader;
	m_transportHandle = transportHandle;

	pStreamListener->OnStartRequest(  this, NULL );

                    
	return HRESULT_OK;
}




URILoader::Result  WebDownload::GetName( URILoader::String *pValue ) 
{
	__NotImplemented();
	return HRESULT_NOTSUPPORT;
}
URILoader::Result  WebDownload::GetStatus( URILoader::Result *pValue  )
{
	__NotImplemented();
	return HRESULT_NOTSUPPORT;
}
URILoader::Result  WebDownload::GetLoadFlags( URILoader::LoadFlags *pValue ) 
{
	__NotImplemented();
	return HRESULT_NOTSUPPORT;
}
URILoader::Result  WebDownload::SetLoadFlags( URILoader::LoadFlags aValue )
{
	__NotImplemented();
	return HRESULT_NOTSUPPORT;
}
URILoader::Result  WebDownload::IsPending( BOOL *pIsPending )
{
	__NotImplemented();
	return HRESULT_NOTSUPPORT;
}
URILoader::Result  WebDownload::Cancel( URILoader::Result aStatus )
{	
	SFBRmPDPContextEventParams	szPDPContext;

	// For stopping 3G indicator - sumdra 
	szPDPContext.nEvent = SFB_PDPCTX_NO_TRANSACTION;

	m_pWebControl->ReceivedHandlePDPevent((SFBRmHClient)m_pWebControl->GetLoadGroup(), &szPDPContext, NULL );

	SFBRmRequestStop( (int)(m_pWebControl->GetLoadGroup()), 
							m_transportHandle );

	return HRESULT_OK;
}

URILoader::Result  WebDownload::Suspend()
{
	__NotImplemented();
	return HRESULT_NOTSUPPORT;
}

URILoader::Result  WebDownload::Pause()
{
	if(SFBRmPauseReq((int)(m_pWebControl->GetLoadGroup()), 
							m_transportHandle, SFB_HTTP_DOWNLOAD_PAUSE) == FALSE)
	{
		SysDebug(( MID_EXCEPTION, "WebDownload::Pause : SFBRmPauseReq Fail \n" ));
		return URILoader::RESULT_ERR_UNKNOWN;
	}

	return URILoader::RESULT_OK;
}
URILoader::Result  WebDownload::Resume()
{

	if(SFBRmResumeReq((int)(m_pWebControl->GetLoadGroup()), 
							m_transportHandle, SFB_HTTP_DOWNLOAD_RESUME) == FALSE)
	{
		SysDebug(( MID_EXCEPTION, "WebDownload::Resume : SFBRmResumeReq Fail \n" ));
		return URILoader::RESULT_ERR_UNKNOWN;
	}

	return URILoader::RESULT_OK;
}
URILoader::Result  WebDownload::GetOriginalURI( URILoader::IURI **ppValue )
{
	__NotImplemented();
	return HRESULT_NOTSUPPORT;
}
URILoader::Result  WebDownload::SetOriginalURI( URILoader::IURI *pURI ) 
{
	__NotImplemented();
	return HRESULT_NOTSUPPORT;
}
URILoader::Result  WebDownload::GetURI(URILoader::IURI **ppURI )
{
	*ppURI = m_pURI;

	return URILoader::RESULT_OK;
}
URILoader::Result  WebDownload::GetChannelEventListener( URILoader::IChannelEventListener ** pValue )
{
	__NotImplemented();
	return HRESULT_NOTSUPPORT;
}
URILoader::Result  WebDownload::SetChannelEventListener( URILoader::IChannelEventSender* aValue )
{
	__NotImplemented();
	return HRESULT_NOTSUPPORT;
}
URILoader::Result  WebDownload::GetCacheStorage( URILoader::ICacheStorage **pValue )
{
	__NotImplemented();
	return HRESULT_NOTSUPPORT;
}
URILoader::Result  WebDownload::SetCacheStorage( URILoader::ICacheStorage *aValue )
{
	__NotImplemented();
	return HRESULT_NOTSUPPORT;
}
URILoader::Result  WebDownload::GetCookieStorage( URILoader::ICookieStorage **pValue )
{
	__NotImplemented();
	return HRESULT_NOTSUPPORT;
}
URILoader::Result  WebDownload::SetCookieStorage( URILoader::ICookieStorage *aValue )
{
	__NotImplemented();
	return HRESULT_NOTSUPPORT;
}
URILoader::Result  WebDownload::GetContentType( URILoader::String *pValue )
{
	__NotImplemented();
	return HRESULT_NOTSUPPORT;
}
URILoader::Result  WebDownload::SetContentType( URILoader::String *aValue ) 
{
	__NotImplemented();
	return HRESULT_NOTSUPPORT;
}
URILoader::Result  WebDownload::GetContentCharset( URILoader::String *pValue ) 
{
	__NotImplemented();
	return HRESULT_NOTSUPPORT;
}
URILoader::Result  WebDownload::SetContentCharset(  URILoader::String *aValue )
{
	__NotImplemented();
	return HRESULT_NOTSUPPORT;
}
URILoader::Result  WebDownload::GetContentLength( long *pValue ) 
{
	__NotImplemented();
	return HRESULT_NOTSUPPORT;
}
URILoader::Result  WebDownload::SetContentLength( long aValue )
{
	__NotImplemented();
	return HRESULT_NOTSUPPORT;
}
URILoader::Result  WebDownload::AsyncOpen( URILoader::IStreamListener *aListener, void* aContext)
{
	__NotImplemented();
	return HRESULT_NOTSUPPORT;
}

URILoader::Result WebDownload::GetRequestMethod( String *pValue ) 
{
	__NotImplemented();
	return HRESULT_NOTSUPPORT;
}
URILoader::Result  WebDownload::SetRequestMethod( String aValue ) 
{
	__NotImplemented();
	return HRESULT_NOTSUPPORT;
}
URILoader::Result  WebDownload::GetReferrer( URILoader::IURI **ppValue ) 
{
	__NotImplemented();
	return HRESULT_NOTSUPPORT;
}
URILoader::Result  WebDownload::SetReferrer( URILoader::IURI *aValue ) 
{
	__NotImplemented();
	return HRESULT_NOTSUPPORT;
}
URILoader::Result  WebDownload::GetResponseStatus( unsigned long *pValue ) 
{

	*pValue = m_statusCode;
	return HRESULT_OK;
}
URILoader::Result  WebDownload::GetRequestSucceeded( BOOL *pValue ) 
{
	__NotImplemented();
	return HRESULT_NOTSUPPORT;
}
URILoader::Result  WebDownload::GetRequestHeader( URILoader::String *aHeader, URILoader::String *pValue )
{
	__NotImplemented();
	return HRESULT_NOTSUPPORT;
}
URILoader::Result  WebDownload::SetRequestHeader( URILoader::String *aHeader,
                      URILoader::String *aValue,
                      BOOL aMerge)
{
	__NotImplemented();
	return HRESULT_NOTSUPPORT;
}
URILoader::Result  WebDownload::GetResponseHeader(  URILoader::String *aHeader, URILoader::String *pValue )
{
	__NotImplemented();
	return HRESULT_NOTSUPPORT;
}
URILoader::Result  WebDownload::SetResponseHeader( URILoader::String *aHeader,
                       URILoader::String aValue,
                       BOOL aMerge) 
{
	__NotImplemented();
	return HRESULT_NOTSUPPORT;
}

URILoader::Result  WebDownload::GetResponseHeaderMap( URILoader::HeaderMap* aVisitor)
{
		__NotImplemented();
	return HRESULT_NOTSUPPORT;
}

URILoader::Result  WebDownload::GetResponseHeaderString( char **pszHeaders )
{
	*pszHeaders = (char*)MemAlloc( m_headers.length() + 1 );
	if( *pszHeaders == NULL )
		return HRESULT_NO_MEMORY;
	AcStrcpy( *pszHeaders, m_headers.latin1().data() );
	
	return HRESULT_OK;
}

URILoader::Result WebDownload::SetUploadStream( const char* pData, int len, URILoader::String contentType )
{
		__NotImplemented();
	return HRESULT_NOTSUPPORT;
}
URILoader::Result WebDownload::SetCallerClientID(int clientID )
{
			__NotImplemented();
	return HRESULT_NOTSUPPORT;	
}

URILoader::Result WebDownload::GetCallerClientID(int* clientID)
{
	*clientID = (int)(m_pWebControl->GetLoadGroup());
	return HRESULT_OK;
}

URILoader::Result  WebDownload::IsNoStoreResponse( BOOL *pValue )
{
	__NotImplemented();
	return HRESULT_NOTSUPPORT;
}
URILoader::Result  WebDownload::IsNoCacheResponse( BOOL *pValue ) 
{
	__NotImplemented();
	return HRESULT_NOTSUPPORT;
}



void WebDownload::OnData( char* pData, int len )
{
	URILoader::IStreamListener* pStreamListener = m_pWebControl->GetDownloadListener( );

	if( pStreamListener == NULL )
	{
		SysDebug(( MID_EXCEPTION, "WebDownload::Response : pStreamListener == NULL\n" ));
		return;
	}


	pStreamListener->OnDataAvailable( this, NULL, pData, m_totalReceived, len );

	m_totalReceived += len;
}

void WebDownload::OnError( int _error )
{
	SFBRmPDPContextEventParams	szPDPContext;

	SysDebug(( MID_EXCEPTION, "WebDownload::OnError : called\n" ));

	URILoader::IStreamListener* pStreamListener = m_pWebControl->GetDownloadListener( );

	if( pStreamListener == NULL )
	{
		SysDebug(( MID_EXCEPTION, "WebDownload::OnError : pStreamListener == NULL\n" ));
		return;
	}

	// For stopping 3G indicator - sumdra 
	szPDPContext.nEvent = SFB_PDPCTX_NO_TRANSACTION;

	m_pWebControl->ReceivedHandlePDPevent((SFBRmHClient)m_pWebControl->GetLoadGroup(), &szPDPContext, NULL );

	pStreamListener->OnStopRequest( this, NULL, URILoader::RESULT_ERR_UNKNOWN );

}

void WebDownload::OnComplete()
{
	SFBRmPDPContextEventParams	szPDPContext;
	
	URILoader::IStreamListener* pStreamListener = m_pWebControl->GetDownloadListener( );

	if( pStreamListener == NULL )
	{
		SysDebug(( MID_EXCEPTION, "WebDownload::OnComplete : pStreamListener == NULL\n" ));
		return;
	}

	// For stopping 3G indicator - sumdra 
	szPDPContext.nEvent = SFB_PDPCTX_NO_TRANSACTION;

	m_pWebControl->ReceivedHandlePDPevent((SFBRmHClient)m_pWebControl->GetLoadGroup(), &szPDPContext, NULL );

	pStreamListener->OnStopRequest( this, NULL, URILoader::RESULT_OK );
}








} // namespace WebControl