/*
 * Copyright (c) S/W Platform, Samsung Electronics, Inc.
 * All rights reserved.
 *
 * This software is the confidential and proprietary information
 * of Samsung Electronics, Inc. ("Confidential Information").  You
 * shall not disclose such Confidential Information and shall use
 * it only in accordance with the terms of the license agreement
 * you entered into with Samsung Electronics. 
 */
/** 
 * @file		WebElementPropertyMap.cpp
 * @brief		implement WebElementPropertyMap
 * @author	Sangheon Kim (sangheon77.kim@samsung.com)
 * @version	1.0
 * @date		2008.04.01	v1.0		Created
 */

#include "WebElementPropertyMap.h"
#include "WebDOMCore.h"
#include "WebImage.h"

#include <WebCore/HitTestResult.h>

using namespace WebCore;

namespace WebControl {

WebElementPropertyMap::WebElementPropertyMap( HitTestResult& result )
{
	m_pHitResult = new HitTestResult( result );
}


WebElementPropertyMap::~WebElementPropertyMap()
{
	delete m_pHitResult;
}

HRESULT WebElementPropertyMap::getValue( int key, void **pValue )
{
	if( key == ELEMENT_PROPERTY_DOM_NODE_KEY )
	{
		IDOMNode* pDOMNode = DOMNode::New(m_pHitResult->innerNonSharedNode());
		*pValue = pDOMNode;
		if(pDOMNode)
		{
        return HRESULT_OK;
	}
		else
		{
			return HRESULT_NOK;
		}
	}
	else if( key == ELEMENT_PROPERTY_IMAGE_KEY )
	{
		WebImage* pImage = new WebImage( *m_pHitResult );
		*pValue = pImage;

		return HRESULT_OK;
	}
	else
	{
		return HRESULT_NOTSUPPORT;
	}
}



} // namespace WebFrame