/*
 * Copyright (c) S/W Platform, Samsung Electronics, Inc.
 * All rights reserved.
 *
 * This software is the confidential and proprietary information
 * of Samsung Electronics, Inc. ("Confidential Information").  You
 * shall not disclose such Confidential Information and shall use
 * it only in accordance with the terms of the license agreement
 * you entered into with Samsung Electronics. 
 */
/** 
 * @file		WebPageInfo.h
 * @brief		implement IPageInfo of WebControl
 * @author	Sangheon Kim (sangheon77.kim@samsung.com)
 * @version	1.0
 * @date		2008.03.25	v1.0		Created
 */
#ifndef __WebPageInfo_Defined
#define __WebPageInfo_Defined


#include "WebControl.h"

namespace	WebControl
{

class WebPageInfo : public IPageInfo
{
public:
	WebPageInfo( WebControl *pWebControl );

public:
	/*
	 * @desc gets the title of page
	 * @param out_pszBuffer : pointer to buffer
	 * @param in_nBufferLen : length of buffer
	 * @return HRESULT TRUE/FALSE
	 */
	virtual HRESULT getTitle( char* out_pszBuffer, int in_nBufferLen );

	/*
	 * @desc gets the url of page
	 * @param out_pszBuffer : pointer to buffer
	 * @param in_nBufferLen : length of buffer
	 * @return HRESULT TRUE/FALSE
	 */
	virtual HRESULT getURL( char* out_pszBuffer, int in_nBufferLen );

	/*
	 * @desc gets the size of page
	 * @param out_pszBuffer : pointer to buffer
	 * @param in_nBufferLen : length of buffer
	 * @return HRESULT TRUE/FALSE
	 */
	virtual HRESULT getSize( int *out_size );

	/*
	 * @desc gets last modified date of page
	 * @param out_pszBuffer : pointer to buffer
	 * @param in_nBufferLen : length of buffer
	 * @return HRESULT TRUE/FALSE
	 */
	virtual HRESULT getLastModified( char* out_pszBuffer, int in_nBufferLen );

	/*
	 * @desc gets the mime type of page
	 * @param out_pszBuffer : pointer to buffer
	 * @param in_nBufferLen : length of buffer
	 * @return HRESULT TRUE/FALSE
	 */
	virtual HRESULT getMimetype( char* out_pszBuffer, int in_nBufferLen );

private:
	WebControl*			m_pWebControl;
};



} // namespace  WebFrame

#endif // __WebFrame_Defined