/*
 * Copyright (c) S/W Platform, Samsung Electronics, Inc.
 * All rights reserved.
 *
 * This software is the confidential and proprietary information
 * of Samsung Electronics, Inc. ("Confidential Information").  You
 * shall not disclose such Confidential Information and shall use
 * it only in accordance with the terms of the license agreement
 * you entered into with Samsung Electronics. 
 */
/** 
 * @file		WebPreferences.h
 * @brief		implement interface ISetting of WebControl
 * @author	Sangheon Kim (sangheon77.kim@samsung.com)
 * @version	1.0
 * @date		2008.03.21	v1.0		Created
 */
#ifndef __WebPreferences_Defined
#define __WebPreferences_Defined

#include "ISetting.h"
#include "WebControlConfig.h"
#include "vector"
#include "ShpConfig.h"

namespace	WebControl
{



class WebPreferences : public ISetting
{

#ifdef STLPORT_ENABLED
typedef stlport::map<int, int> IntMap;
typedef stlport::map<int, stlport::string> StrMap;
typedef stlport::pair<int, int> IntMapPair;
typedef stlport::pair<int, stlport::string> StrMapPair;
typedef stlport::vector<::WebControl::ISettingListener*> ListenerVector;
#else
typedef std::map<int, int> IntMap;
typedef std::map<int, std::string> StrMap;
typedef std::pair<int, int> IntMapPair;
typedef std::pair<int, std::string> StrMapPair;
#ifdef _SHP_SIMUL 
typedef std::vector<::WebControl::ISettingListener*> ListenerVector;
#else
typedef std::vector<ISettingListener*> ListenerVector;
#endif
#endif

public:
	static WebPreferences* GetDefaultPreferences();
	static HRESULT New( String name, WebPreferences** pWebPreferences );
	~WebPreferences();


private:
	HRESULT Construct( String name );
	HRESULT Destruct();
	HRESULT LoadDefaults();

	IntMap			m_IntMap;
	StrMap			m_StrMap;
	String			m_strName;
	ListenerVector	m_listenerVector;

public:
		/*
	 * @desc sets a property of Browser Engine.
	 * @param in_PropertyID : Property ID to setting
	 * @param in_PropertyValue : pointer to Property Value to setting
	 * @return bool TRUE / FALSE
	 */
	virtual HRESULT setProperty( int in_PropertyID, void* in_PropertyValue );

	/*
	 * @desc gets a property of Browser Engine.
	 * @param in_PropertyID : Property ID to setting
	 * @param out_PropertyValue : pointer to Property Value to setting
	 * @return bool TRUE / FALSE
	 */
	virtual HRESULT getProperty( int in_PropertyID, void* out_PropertyValue );


	/**
	* Add a listener to the listener list
	* All listeners in the list will be notified whenever the setting value change
	*/
	virtual HRESULT addListener( ::WebControl::ISettingListener* pListener );

	/**
	* Remove the listener from the listener list
	*/
	virtual HRESULT removeListener( ::WebControl::ISettingListener* pListener );
};

} // namespace  WebControl

#endif // __IControl_Defined
